###########################################################################
# These are the library routines developed for DOME
# 
# Subroutine list:
#  1. database_login(),
#  1a. database_login2(),
#  2. TimeStamp(),
#  3. createSampleId($dbh, $sampling_repl_id, $tissue_type_id);
#  4. createSampleExtractId($dbh, $sample_id, $extract_id);
#  5. createLabeledExtractId($dbh, $sample_extract_id, $sample_labeling_id,
#                            $technique_id);
#  6. createFileLinkId($dbh, $filename, $file_type_id);
#  7. createRawDataId($dbh, $hybrid_id, $scan_prot_id, $channel1_file_id,
#                     $channel2_file_id);
#  8. createImageAnalysisId($dbh, $raw_data_id, $file_link_id);
#  9. insertMicroArrayData($dbh, $gpr_file_name, $image_analysis_id, 
#                          $array_batch_id);
# 10. createHybridizationId($dbh, $labeled_extract_1_id, 
#               $labeled_extract_2_id, $array_id, $hybrid_protocol_id,
#               $person_id);
# 11. getSamplingPointId($dbh, $exp_set_name, $control, $exp_time, $replicate),
# 12. parseFileName($filename);
# 13. getPersonId($dbh, $personname);
# 14. createArrayId($dbh, $array_batch_id, $array_name);
# 15. getBatchAndArrayId($dbh, $file_name);
# 16. createMetaboliteDerivativeId($dbh, $name);
# 17. createMetaboliteMeasurementId_flink($dbh, $sample_extract_id;
#                 $technique_id, $replicate, $file_link_id);
# 17a. createMetaboliteMeasurementId($dbh, $sample_extract_id;
#                 $technique_id, $protocol_id, $replicate, $person_id);
# 17b. createMetaboliteMeasurementIdWithProcInputFile($dbh, $sample_extract_id;
#                 $technique_id, $protocol_id, $replicate, $person_id, 
#                 $proc_input_file);
# 18. insertMetaboliteData($dbh, $file, $metabolite_measurement_id);
# 19. getSamplingReplicateId($dbh, $exp_set_name, $control, $exp_time, 
#                            $replicate),
# 20. getMaxId($dbh, $table_name),
# 21. getSampleId($dbh, $sampling_replicate_id);
# 22. getSampleExtractId($dbh, $sample_id, $extract_id);
# 23. getMetaboliteMeasurementId($dbh, $sample_extract_id, $technique_id,
#         $replicate);
# 24. getKwTableId($dbh, $protocol_id);
# 25. createGeneId($dbh, $specie_id);
# 26. createEvidenceId($dbh, $evidence_code_id, $reference_id, $kwid, $gene_id);
# 27. createExtranalLinkId($dbh, $external_db_id, $kwid, $record_id, 
#         $internal_field_name, $external_id);
# 28. createAllnameId($dbh, $aname, $kwid, $record_id, is_primary, $unikey);
# 29. createCompositeSequenceId($dbh, $probe_name, $annotation, $gene_id);
# 30. createReporterId($dbh, $csid);
# 31. getSearchName($name);
# 32. getFileId($dbh, $file_name)
# 33. getFileTypeId($dbh, $filename);
# 34. createFileTypeId($dbh, $galfile, $desc);
# 35. createFileIds($dbh, $file_home, $filename, $file_type_id, $person_id);
# 36. createFileLocationId($dbh, $file_home);
#
#  NOTE:    
#  If you need to add a subroutine to this library, please do the following 
#  steps:
#  1. Add the subroutine name to the subroutine list above together with the 
#     parameters,
#  2. Add the subroutine name at the end of @EXPORT array,
#  3. Paste the subroutine at the end of this package but above the line of
#     End of package.
#
# Author: X. Jing Li
# Data: October 3, 2003
# Reversions:
#  10/04/2005: change compoite_sequence table tc field to "probe_name" in 
#    9: insertMicroArrayData.
#
###########################################################################
package DOMELIB;

require(Exporter);
@ISA = qw(Exporter);
@EXPORT = qw(database_login database_login2 TimeStamp createSampleId createSampleExtractId createLabeledExtractId createFileLinkId createRawDataId createImageAnalysisId insertMicroArrayData createHybridizationId getSamplingPointId parseFileName getPersonId createArrayId getBatchAndArrayId createMetaboliteDerivativeId_flink createMetaboliteMeasurementId createMetaboliteMeasurementIdWithProcInputFile insertMetaboliteData getSamplingReplicateId getMaxId getSampleId getSampleExtractId getMetaboliteMeasurementId getKwTableId createGeneId createEvidenceId createExternalLinksId createAllnameId createCompositeSequenceId createReporterId getSearchName getFileId getFileTypeId createFileTypeId createFileIds createFileLocationId);

###########################################################################
# 1. Subroutine database_login(): This routine will return the database name, 
#   username and password for the database.
#             
# parameters in: none,
#           out: $database --- The database name,
#                $username --- database username,
#                $password --- database password.
#
# last update: 9_25_03, X. Jing Li
###########################################################################
sub database_login {
    my ($database, $server, $username, $password);
    $param_file = "paramSettings.php";
    open (IN, "<$param_file") or die "Can not open parametersetting file $param_file for database parameters.\n";
    @contents = <IN>;
    close IN;

    $database = "";
    $username = "";
    $password = "";
    $server = "";
    foreach $line (@contents) {
        $line =~ s/\n|\r//g;
        if ($database ne "" && $server ne "" && $username ne "" 
            && $password ne "") {
            last;
        }
        if ($line =~ /\$database/ ) {
            ($temp, $database) = split /\ =\ /, $line;
            $database =~ s/\'|\;//g;
        } elsif ($line =~ /\$server/ ) {
            ($temp, $server) = split /\ =\ /, $line;
            $server =~ s/\'|\;//g;
        }elsif ($line =~ /\$username/ ) {
            ($temp, $username) = split /\ =\ /, $line;
            $username =~ s/\'|\;//g;
        } elsif ($line =~ /\$password/ ) {
            ($temp, $password) = split /\ =\ /, $line;
            $password =~ s/\'|\;//g;
        } else {
            next;
        }
    }
    $database = "dbi:Pg:dbname=" . $database . ";host=" . $server;
    return $database, $username, $password;
}

###########################################################################
# 1a. Subroutine database_login2(): This routine will return the database name, 
#   username and password for the database.
#             
# parameters in: none,
#           out: $database --- The database name,
#                $username --- database username,
#                $password --- database password.
#                $bnet_DB --- the BNET schema name,
#                $specieDB --- the specie database name,
#
# last update: 3/21/2005, X. Jing Li
###########################################################################
sub database_login2 {
    my ($database, $server, $username, $password);
    $param_file = "paramSettings.php";
    open (IN, "<$param_file") or die "Can not open parametersetting file $param_file for database parameters.\n";
    @contents = <IN>;
    close IN;

    $database = "";
    $username = "";
    $password = "";
    $server = "";
    $bnet_DB = "";
    $specieDB = "";
    foreach $line (@contents) {
        $line =~ s/\n|\r//g;
        #print "$line\n";
        if ($database ne "" && $server ne "" && $username ne "" 
            && $password ne "" && $bnet_DB ne "" && $specieDB ne "") {
            last;
        }
        if ($line =~ /\$database/ ) {
            ($temp, $database) = split /\ =\ /, $line;
            $database =~ s/\'|\;//g;
        } elsif ($line =~ /\$server/ ) {
            ($temp, $server) = split /\ =\ /, $line;
            $server =~ s/\'|\;//g;
        }elsif ($line =~ /\$username/ ) {
            ($temp, $username) = split /\ =\ /, $line;
            $username =~ s/\'|\;//g;
        } elsif ($line =~ /\$password/ ) {
            ($temp, $password) = split /\ =\ /, $line;
            $password =~ s/\'|\;//g;
        } elsif ($line =~ /\$BNetSchemaName/ ) {
            ($temp, $bnet_DB) = split /\ =\ /, $line;
            $bnet_DB =~ s/\"|\'|\;//g;
        }elsif ($line =~ /\$SpeciesSchemaName/ ) {
            ($temp, $specieDB) = split /\ =\ /, $line;
            $specieDB =~ s/\"|\'|\;//g;
        }else {
            next;
        }
    }
    $database = "dbi:Pg:dbname=" . $database . ";host=" . $server;
    return $database, $username, $password, $bnet_DB, $specieDB;
}

###########################################################################
# 2. sub TimeStamp():  
#  Get the current date and time of the server in the form accepted by Oracle
#  database.  
#  
#  parameter out: $time --- in the form of "MON-DD-YYYY HH:MM".
###########################################################################
sub TimeStamp {
    my (@months, $year, $mon, $yday, $mday, $wday, $hour, $min, $sec, $isdst);
    my ($day, $month, $time, $date, $datetime);

    @months = ("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");
    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    ($mday < 10) && ($mday = "0" . $mday);
    ($min < 10) && ($min = "0" . $min);
    ($hour < 10) && ($hour = " " . $hour);
    $datetime = "$months[$mon] $mday $year $hour:$min:$sec";


    # this writes the date in the format acceptable by Oracle and it is stored
    # in $date
    $day = substr($datetime,4,2);
    $month = substr($datetime,0,3);
    $year = substr($datetime,8,2);
    $time = substr($datetime,11,8);
    $date = "$month-$day-20$year $time-04";
    return $date;
}

###########################################################################
# 3. subroutine: createSampleId($dbh, $sampling_replicate_id):
#  this subroutine create a sample_id in SAMPLE table
# 
# parameters in: $dbh --- dtabase handle;
#                $sampling_replicate_id --- sampling replicate id;
# parameter out: $sample_id --- a new sample_id 
###########################################################################
sub createSampleId {
    my ($dbh, $smpl_repl, $tissue_type_id) = @_;
    my ($sql, $sth, $rc);
    my $sample_id;

    $sth = $dbh->prepare("select nextval('sample_id_seq')") 
         or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $sample_id = $sth->fetchrow;

    $sql = "insert into sample (sample_id, sampling_replicate_id, label_id) values ";
    $sql .= "(\'$sample_id\', \'$smpl_repl\', $tissue_type_id)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $sample_id;

}
   
###########################################################################
# 4. subroutine: createSampleExtractId($dbh, $sample_id, $extract_id):
#  this subroutine create a sample_id in SAMPLE_EXTRACT table
# 
# parameters in: $dbh --- database handle;
#                $sample_id --- sample id;
#                $extract_id --- extract id;
# parameter out: $sample_extract_id --- a new sample_extract_id 
###########################################################################
sub createSampleExtractId {
    my ($dbh, $sample_id, $extract_id, $technical_repl) = @_;
    my ($sql, $sth, $rc, $sample_extract_id);

    $sth = $dbh->prepare("select nextval('sample_extract_id_seq')") 
         or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $sample_extract_id = $sth->fetchrow;

    $sql = "insert into sample_extract (sample_extract_id, sample_id, extract_id, technical_replicate)";
    $sql .= " values ($sample_extract_id, $sample_id, $extract_id, $technical_repl)";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $sample_extract_id;
}
 

###########################################################################
# 5. subroutine: createLabeledExtractId($dbh, $sample_extract_id, 
#               $sample_labeling_id, $technique_id):
#  this subroutine create a labeled_extract_id in LABELED_EXTRACT table
# 
# parameters in: $dbh --- database handle;
#                $sample_id --- sample_extract id;
#                $sample_labeling_id --- sample_labeling id;
#                $technique_id -- technique id;
# parameter out: $labeled_extract_id --- a new labeled_extract_id 
###########################################################################
sub createLabeledExtractId {
    my ($dbh, $sample_extract_id, $sample_labeling_id, $technique_id) = @_;
    my ($sql, $sql2, $sth, $rc);
    my ($labeled_extract_id);

    $sth = $dbh->prepare("select nextval('labeled_extract_id_seq')") 
         or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $labeled_extract_id = $sth->fetchrow;

    $sql = "insert into labeled_extract (labeled_extract_id, sample_extract_id, technique_id";
    $sql2 = ") values ($labeled_extract_id, $sample_extract_id, $technique_id";
    if ($sample_label_id ne "") {
        $sql .= ", sample_labeling_id";
        $sql2 .= ", $sample_labeling_id";
    }
    $sql .= $sql2 . ")";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $labeled_extract_id;
}

###########################################################################
# 6. subroutine: createFileLinkId($dbh, $filename, $file_type_id):
#  this subroutine parse the file name and create records in file_location_link,
# File and File_location tables
# 
# parameters in: $dbh --- database handle;
#                $filename --- the source file name with full path;
#                $file_type_id --- file type id for different source file type;
# parameter out: $file_link_id ---the file_link_id for link file_id and
#                         $file_location_id
#
###########################################################################
sub createFileLinkId {
    my ($dbh, $filename, $type_id) = @_;
    my ($sql, $sth, $rc, $file_id, $file_location_id, $file_link_id);
    my ($fileIdIsNew, $locationIdIsNew, $path, $name, @dirs);  
              
    #== first check if the file exist:
    if (! (-e $filename) ) {
        return "$filename does not exist!\n";
    }

    @dirs = split(/\//, $filename);
    $path = "";                                     
    for( my $i=0; $i<@dirs-1; $i++) {
        $path .= $dirs[$i] . "\/";
    }
    $name = $dirs[@dirs-1];

    $fileIdIsNew = "N";
    $locationIdIsNew = "N";
    #== 1. create a new id for file:
    #-- 1.1 check for existance of hte file name:
    $sql = "select file_id from file where file_name = \'$name\'";
    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    $file_id = $sth->fetchrow;
    if ($file_id eq "") {
        $fileIdIsNew = "Y";
        #--1.2 get the next file_id
        $sth = $dbh->prepare("select nextval('file_id_seq')")
            or die "Can not prepare select nextval from dual: $DBI::errstr";
        $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
        $file_id = $sth->fetchrow;
        #--1.3 insert a new record into file table
        $sql = "insert into file values (\'$file_id\', \'$name\', \'$type_id\')";
        $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    }

    #== 2. check if this location is already in FILE_LOCATION table. If yes, use
    #== it for file_link table. Otherwise create a new id for file_location
    #-- 2.1 check if htis location is already in DB:
    $sql = "select file_location_id from file_location where location=\'$path\'";
    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";    
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";        
    $file_location_id = $sth->fetchrow;         
    if ($file_location_id eq ""){
        $locationIdIsNew = "Y";          
        #--2.2 get the next file_location id
        $sth = $dbh->prepare("select nextval('file_location_id_seq')")
            or die "Can not prepare select nextval from dual: $DBI::errstr";    
        $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
        $file_location_id = $sth->fetchrow;
        #--2.3 insert a new record into file table
        $sql = "insert into file_location values (\'$file_location_id\', ";
        $sql .= "\'$file_id\', \'$path\')";
        $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    }

    #== 3. check if the link is already exists:
    $sql = "select file_location_link_id from file_location_link where ";
    $sql .= "file_id = $file_id and file_location_id = $file_location_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    $file_link_id = $sth->fetchrow;
    if ($file_link_id eq "") {
        #== 4. create the link for file and file _location:                          
        #--4.1 get the next file_link_id
        $sth = $dbh->prepare("select nextval('file_location_link_id_seq')")
            or die "Can not prepare select nextval from dual: $DBI::errstr";
        $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
        $file_link_id = $sth->fetchrow;
        #--4.2 insert a new record into file_link table
        $sql = "insert into file_location_link values (\'$file_link_id\', \'$file_id\', ";
        $sql .= "\'$file_location_id\')";                                       
        $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    }

    $sth->finish;
    return $file_link_id;
}

###########################################################################
# 7. subroutine: 
# createRawDataId($dbh, $hybrid_id, $scan_prot_id, $channel1_file_id,
#                 $channel2_file_id):
#  this subroutine create a raw_data id in Raw_data table
# 
# parameters in: $dbh --- database handle;
#                $scan_prot_id --- the scanning_protocol_id;
#                $cy3_link_id --- the file location link id for Cy3;
#                $cy5_link_id --- the file location link id for Cy5;
# parameter out: $raw_data_id --- the new raw_data_id; 
#
###########################################################################
sub createRawDataId {
    my ($dbh, $hybrid_id, $scan_prot_id, $cy3_link_id, $cy5_link_id) = @_;
    my ($sql, $sth, $rc, $sql1, $sql2);
    my $raw_data_id;

    $sth = $dbh->prepare("select nextval('raw_data_id_seq')") 
         or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $raw_data_id = $sth->fetchrow;

    $sql1 = "insert into raw_data (raw_data_id, scanning_protocol_id, hybridization_id";
    $sql2 = "values (\'$raw_data_id\', \'$scan_prot_id\', \'$hybrid_id\'";
    if ($cy3_link_id ne '') {
        $sql1 .= ", cy3_file_location_link_id";
        $sql2 .= ", \'$cy3_link_id\'";
    }
    if ($cy5_link_id ne '') {
        $sql1 .= ", cy5_file_location_link_id";
        $sql2 .= ", \'$cy5_link_id\'";
    }
    $sql1 .= ") ";
    $sql2 .= ")";
    $sql = $sql1 . $sql2;
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $raw_data_id;
}

###########################################################################
# 8. subroutine: createImageAnalysisId($dbh, $raw_data_id, $file_link_id):
#  this subroutine create a image_analysis_id in Iamge _Analysis table
# 
# parameters in: $dbh --- database handle;
#                $raw_data_id --- ;
#                $file_link_id --- the link id for .gpr file
# parameter out: $image_analysis_id --- the new image_analysis_id; 
#
###########################################################################
sub createImageAnalysisId {
    my ($dbh, $raw_data_id, $file_link_id) = @_;
    my ($sql, $sth, $rc);
    my $image_analy_id;

    $sth = $dbh->prepare("select nextval('image_analysis_id_seq')") 
         or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $image_analy_id = $sth->fetchrow;

    $sql = "insert into image_analysis (image_analysis_id, raw_data_id, ";
    $sql .= "file_location_link_id) values (\'$image_analy_id\', \'$raw_data_id\', ";
    $sql .= "\'$file_link_id\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $image_analy_id;

}

###########################################################################
# subroutine: insertMicroArrayData($dbh, $gpr_file_name, $image_analysis_id, 
#                        $array_batch_id):
#  this subroutine opens the data file andpopulate the date into QUANTiFICATION
# table
# 
# parameters in: $dbh --- database handle;
#                $gpe_file_name --- the original date file for micro-array;
#                $image_analysis_id --- the image_analysis_id for this data file
#                $array_batch_id --- the id for the micro-array batch. There 
#                    are three batchs.
# parameter out: $success --- "SUCCESS" or "FAULT" for populating data from this
#                    file; 
# X. Jing Li
# 5/3/2005: Add code for feature_id is not in DOME.
###########################################################################
sub insertMicroArrayData {
    my ($dbh, $gpr_file_name, $image_analysis_id, $array_batch_id) = @_;
    my ($sth, $rc, $sql, $success, @contents, $start_read, $line_num, $line,
        @fields, $name, $ref_feature_id, %feature_ids, $ref_rid_fids, %rid_fids,
        @record, $key, @data, 
        $block, $column, $row, $feature_id, $tc,
        $f532_median, $f532_mean,$f532_sd, $b532_median, $b532_mean, $b532_sd,
        $gt_b532_1sd, $gt_b532_2sd, $f532_sat, $snr532, $snr635, $f635_median,
        $f635_mean, $f635_sd, $b635_median, $b635_mean, $b635_sd, $gt_b635_1sd,
        $gt_b635_2sd, $f635_sat, $f_pixels, $b_pixels, $ratio_median, 
        $ratio_mean, $f532_total_intens, $f635_total_intens);
 
    $success = "FAILED";

    #== get all feature_id, reporter_ids and there corresponding TCs:
    $ref_feature_id = ();
    %feature_ids = %$ref_feature_id;
    $ref_fid_rids = ();
    $fid_rids = %$ref_fid_rids;
    $sql = "select feature_id, block, \"row\", \"column\", reporter_id from feature where array_batch_id = $array_batch_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    while ( @record = $sth->fetchrow ) {
        $feature_id = $record[0];
        $block = $record[1];
        $row = $record[2];
        $column = $record[3];
        $rid = $record[4];
        $key = $block . "_" . $row . "_" . $column;
        $feature_ids{$key} = $feature_id;
        $fid_rids{$feature_id} = $rid;
    }

    $ref_rid_tcs = ();
    %rid_tcs = %$ref_rid_tcs;
    $sql = "select R.reporter_id, CS.probe_name from reporter R, composite_sequence CS where R.composite_sequence_id = CS.composite_sequence_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    while (($rid, $tc) = $sth->fetchrow()) {
       $rid_tcs{$rid} = $tc;
    }

    #== 1. read the data from the source file:
    open (datafile, "<$gpr_file_name") or die "Can not open GPR file $gpr_file_name\n";
    @contents = <datafile>;
    close datafile;
    #print "there are " . @contents . " lines in $gpr_file_name\n";

    #== 2. find where to start to read the data. Check for the column order is
    #== indead the date type it suposs to be.
    $start_read = "N";
    $line_num = 0;
    $sql = "";
    foreach $line (@contents) {
        $line_num++;
        chomp $line;
        $line =~ s/\"//g;      # delete double quotes
        $line =~ s /\
//g;

        if ($line =~ /^Block/){
            $start_read = "Y";
            @fields = split (/\t/, $line);
            #for (my $i=0; $i<@fields; $i++) {
            #    print "fields[$i] = $fields[$i]\n";
            #}
            
            #$index = 0;
            #foreach $field (@fields) {
            #    $field_index{$field} = $index;
            #    $index++;
            #}
                
            if ($fields[0] eq "Block" && $fields[1] eq "Column" && $fields[2] eq "Row"
                && $fields[8] eq "F532 Median" && $fields[9] eq "F532 Mean"
                && $fields[10] eq "F532 SD" && $fields[11] eq "B532 Median" 
                && $fields[12] eq "B532 Mean" && $fields[13] eq "B532 SD"
                && $fields[14] eq "% > B532+1SD" && $fields[15] eq "% > B532+2SD"
                && $fields[16] eq "F532 % Sat." 
                && $fields[17] eq "F635 Median" && $fields[18] eq "F635 Mean"
                && $fields[19] eq "F635 SD" && $fields[20] eq "B635 Median" 
                && $fields[21] eq "B635 Mean" && $fields[22] eq "B635 SD"
                && $fields[23] eq "% > B635+1SD" && $fields[24] eq "% > B635+2SD"
                && $fields[25] eq "F635 % Sat." 
                && $fields[26] eq "Ratio of Medians (532/635)" 
                && $fields[27] eq "Ratio of Means (532/635)"
                && $fields[33] eq "F Pixels" && $fields[34] eq "B Pixels"
                && $fields[42] eq "F532 Total Intensity" 
                && $fields[43] eq "F635 Total Intensity"
                && $fields[44] eq "SNR 532" && $fields[45] eq "SNR 635") {
                next;
            } else {
                #print "goto last line\n";
                print  "Data fields are in wrong order\n";
                return $success;
            }
        }
        #== 3. read the data if this is a data line:
        if ($start_read eq "N") {
            next;
        } else {
            @data = split(/\t/, $line);
            $block = $data[0];
            $column = $data[1];
            $row = $data[2];
            $tc = $data[4];
            $f532_median  = $data[8];
            $f532_mean = $data[9];
            $f532_sd = $data[10];
            $b532_median = $data[11];
            $b532_mean = $data[12];
            $b532_sd = $data[13];
            $gt_b532_1sd = $data[14];
            $gt_b532_2sd = $data[15];
            $f532_sat = $data[16];
            $f635_median = $data[17];
            $f635_mean = $data[18];
            $f635_sd = $data[19];
            $b635_median = $data[20];
            $b635_mean = $data[21];
            $b635_sd = $data[22];
            $gt_b635_1sd = $data[23];
            $gt_b635_2sd = $data[24];
            $f635_sat = $data[25];
            $ratio_median = $data[26];
            $ratio_mean = $data[27];
            $f_pixels = $data[33];
            $b_pixels = $data[34];
            $f532_total_intens = $data[42];
            $f635_total_intens = $data[43];
            $snr532 = $data[44];
            $snr635 = $data[45];
            $key = $block . "_" . $row . "_" . $column;
            $feature_id = $feature_ids{$key};

            if ($feature_id eq "") {
                print "feature_id for $tc is not in DOME. You may have not uploaded the GAL file yet.\n";
                return $success;
            }

            if ($rid_tcs{$fid_rids{$feature_id}} ne $tc) {
                print "feature_id $feature_id for Block $block, row $row and column $column is not for $tc. You may have the wrong array batch.\n";
                return $success;
            }

            #-- insert data into quantification:
            $sql .= "insert into quantification (quantification_id, ";
            $sql .= "image_analysis_id, feature_id, ";
            $sql .= "\"F532_Median\", \"F532_Mean\", \"F532_SD\", \"B532_Median\", ";
            $sql .= "\"B532_Mean\", \"B532_SD\", \"percent_gt_B532_plus_1SD\", ";
            $sql .= "\"percent_gt_B532_plus_2SD\", \"F532_percent_Sat\", ";
            $sql .= "\"F635_Median\", \"F635_Mean\", \"F635_SD\", ";
            $sql .= "\"B635_Median\", \"B635_Mean\", \"B635_SD\", ";
            $sql .= "\"percent_gt_B635_plus_1SD\", \"percent_gt_B635_plus_2SD\", ";
            $sql .= "\"F635_percent_Sat\", \"Ratio_of_Medians\", ";
            $sql .= "\"Ratio_of_Means\", \"F_Pixels\", \"B_Pixels\", ";
            $sql .= "\"F532_Total_Intensity\", \"F635_Total_Intensity\", ";
            $sql .= "\"SNR_532\", \"SNR_635\") values ";
            $sql .= "(nextval('quantification_id_seq'), $image_analysis_id, ";
            $sql .= "$feature_id, $f532_median, $f532_mean, $f532_sd, $b532_median, ";
            $sql .= "$b532_mean, $b532_sd, $gt_b532_1sd, $gt_b532_2sd, $f532_sat, ";
            $sql .= "$f635_median, $f635_mean, $f635_sd, $b635_median, $b635_mean, ";
            $sql .= "$b635_sd, $gt_b635_1sd, $gt_b635_2sd, $f635_sat, ";
            $sql .= "$ratio_median, $ratio_mean, $f_pixels, $b_pixels, ";
            $sql .= "$f532_total_intens, $f635_total_intens, $snr532, $snr635);\n";
        }
    }

    #== print out the sql statement before execute
    #my $tempfile = "/export/home/xingjing/data/dome/temp/temp_sql.sql";
    #open (TMP, ">$tempfile");
    #print TMP "$sql";
    #close TMP;

    if ($sql ne "") {
        $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
        $sth->finish;
        #print "finished insert data for $gpr_file_name\n";

        #== delete the temp file
        #unlink ($tempfile) or die "couldn't delete $tempfile: $!";

        if ($line_num == @contents) {
            $success = "SUCCESS";
            #print "All done!\n";
        }
    }

    return $success;
}

###########################################################################
# 10. subroutine: createHybridizationId($dbh, $labeled_extract_1_id, 
#               $labeled_extract_2_id, $array_id, $hybrid_protocol_id,
#               $person_id):
#  this subroutine create a hybridization_id in HYBRIDIZATION table
# 
# parameters in: $dbh --- database handle;
#                $labeled_extract_1_id --- labeled_extract id for Cy3;
#                $labeled_extract_2_id --- labeled_extract id for Cy5;
#                $array_id --- array id;
# parameter out: $hybridization_id --- a new hybridization_id 
###########################################################################
sub createHybridizationId {
    my ($dbh, $lb_ext_id_1, $lb_ext_id_2, $aid, $hybrid_prot_id, $person) = @_;
    my ($sql, $sth, $rc, $sql1, $sql2);
    my $hybridization_id;
    #print "$lb_ext_id_1, $lb_ext_id_2, $aid, $hybrid_prot_id, $person\n";

    $sth = $dbh->prepare("select nextval('hybridization_id_seq')") 
        or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $hybridization_id = $sth->fetchrow;

    $sql1 = "insert into hybridization (hybridization_id, ";
    $sql2 = "values (\'$hybridization_id\', ";
    if ($lb_ext_id_1 ne "") {
        $sql1 .= "labeled_extract1_id, ";
        $sql2 .= "\'$lb_ext_id_1\', ";
    }
    if ($lb_ext_id_2 ne "") {
        $sql1 .= "labeled_extract2_id, ";
        $sql2 .= "\'$lb_ext_id_2\', ";
    }
    $sql1 .= "array_id, hybridization_protocol_id, person_id) ";
    $sql2 .= "\'$aid\', \'$hybrid_prot_id\', \'$person\')";
    $sql = $sql1 . $sql2;
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $hybridization_id;
}

###########################################################################
# 11. subroutine: getSamplingPointId($dbh, $exp_set_name, $control, $exp_time, 
#             $replicate):
#    this subroutine get the sampling_point_id from Dome.
#
# parameters in: $dbh --- the database handle,
#                $exp_set_name --- the experiment set name,e.g Yest, 
#                                  Methyl Jasmonate or UV iradiation,
#                $control --- either Control or Elicity for experiment
#                $exp_time --- the experiment_time,
#                $replicate --- the replicate number,
# parameter out: $sampling_point_id --- the sampling_point_id correspoding to
#                the parameters.
#
###########################################################################
sub getSamplingPointId {
    my ($dbh, $exp_set_name, $control, $exp_time, $replicate) = @_;
    my ($sql, $sth, $rc);
    my ($label, $exp_set_id, $sampling_point_id, $exp_id);

    $label = $exp_set_name . " " . $control;

    $sql = "select smpl_point_id from sampling_point where label_id = ";
    $sql .= "(select label_id from label where name = \'$exp_time\')";
    $sql .= " and replicate = \'$replicate\' and experiment_id =(";
    $sql .= "select experiment_id from experiment where exp_name = \'$label\' ";
    $sql .= "and exp_set_id = (select exp_set_id from experiment_set where ";
    $sql .= "exp_set_name = \'$exp_set_name\') )";
    $sth = $dbh->prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    $sampling_point_id = $sth->fetchrow;
    #print "sampling_point_id = $sampling_point_id\n";

    $sth->finish;
    return $sampling_point_id;

}

###########################################################################
# 12. subroutine: parseFileName($filename):
#    this subroutine parse the gpr file name and get the exp_set_name (Yeast 
# Methyl Jasmonate or UV irradiation), exp_time (0 min, 5 min, ... 1 h, ..., 
# 48 h), replicate number (1, 2, 3) and experiment type (Control or Elicity).
#
# parameters in: $filename --- the filename which needs be processed,
# parameter out: ($exp_set_name, $time, $time_unit, $control, $replicate) 
#               --- the parameters from the filename.
#
###########################################################################
sub parseFileName {
    my ($filename) = @_;
    my ($exp_set_name, $exp_name, $smpl_point, $replicate, $control);
    my ($file, $temp_name, $type, $time_unit, $time, $len, $temp, $char);
    
    ($file, $type) = split(/\./, $filename);
    $temp_name = $file;
    #print "file = $file, temp_name = $temp_name\n";
    #$temp_name = substr($temp_name, 1);
    #print "file = $file, temp_name = $temp_name\n";

    $time = "";
    $replicate = 0;
    while ($temp_name ne "") {
        $char = chop $temp_name;
        if ($char eq "I"){
            $replicate++;
        } elsif ($char eq "C") {
            $control = "Control";
        } elsif ($char eq "E") {
            $control = "Elicited";
        } elsif ($char eq "m") {
            $time_unit = "min";
        } elsif ($char eq "H") {
            $time_unit = "h";
        } elsif ($char =~ /\d/) {
            $time = $char . $time;
        } elsif ($char eq "Y") {
            $exp_set_name = "Yeast";;
        } else {
            print "Undefined letter in file name: $filename\n";
            die;
        }
    }
    return ($exp_set_name, $time, $time_unit, $control, $replicate);
}

###########################################################################
# 13. subroutine: getPersonId($dbh, $personname):
#    this subroutine get the person'sid from Dome.
#
# parameters in: $dbh --- the database handle,
#                $personname --- the person's name
# parameter out: $person_id --- the person's id.
#
###########################################################################
sub getPersonId {
    my ($dbh, $name) = @_;
    my ($sql, $sth, $rc, $id);

    $sql = "select person_id from person where person_name=\'$name\'";
    $sth = $dbh->prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    $id = $sth->fetchrow;

    $sth->finish;
    return $id;
}

###########################################################################
# 14. subroutine: createArrayId($dbh, $array_batch_id, $file_name):
#    this subroutine create array_id for the array table.
#
# parameters in: $dbh --- the database handle,
#                $array_batch_id --- the array_batch_id,
#                $array_name --- the array name,usually the file name w/o ext.
# parameter out: $id --- the new array id.
#
###########################################################################
sub createArrayId {
    my ($dbh, $array_batch_id, $array_name) = @_;
    my ($sql, $sth, $rc, $id);

    $sth = $dbh->prepare("select nextval('array_id_seq')") 
        or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $id = $sth->fetchrow;

    $sql = "insert into \"array\" (array_id, array_batch_id, array_name) values (";
    $sql .="$id, $array_batch_id, \'$array_name\')";
    $sth = $dbh->prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $id;
}

###########################################################################
# 15. subroutine: getArrayId($dbh, $file_name):
#    this subroutine get array_id for the file $afile_name.
#
# parameters in: $dbh --- the database handle,
#                $file_name --- the file name,
# parameter out: $id --- the new array id.
#
###########################################################################
sub getBatchAndArrayId {
    my ($dbh, $file_name) = @_;
    my ($sql, $sth, $rc, $array_id, $batch_id);

    $sql = "select array_id, array_batch_id from array where array_name = ";
    $sql .= "\'$file_name\'";
    $sth = $dbh->prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    ($array_id, $batch_id) = $sth->fetchrow;

    $sth->finish;
    return ($array_id, $batch_id);
}

###########################################################################
# 16. subroutine: createMetaboliteDerivativeId($dbh, $name):
#  this subroutine create a metabolite_derivative_id in metabolite_derivative
#  table
# 
# parameters in: $dbh --- database handle;
#                $name --- the name of the metabolite;
# parameter out: $metabolite_derivative_id --- the new raw_data_id; 
#
###########################################################################
sub createMetaboliteDerivativeId {
    my ($dbh, $name) = @_;
    my ($sql, $sth, $rc);
    my $metabolite_derivative_id;

    $name =~ s/\'/\'\'/g;

    $sth = $dbh->prepare("select nextval('metabolite_derivative_id_seq')") 
         or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $metabolite_derivative_id = $sth->fetchrow;

    $sql = "insert into metabolite_derivative (metabolite_derivative_id, name)";
    $sql .= " values (\'$metabolite_derivative_id\', \'$name\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $metabolite_derivative_id;
}
###########################################################################
# 17. subroutine: createMetaboliteMeasurementId_flink($dbh, $sample_extract_id, 
#                 $technique_id, $replicate, $file_link_id):
#  this subroutine create a metabolite_measurement_id in metabolite_measurement
#  table
# 
# parameters in: $dbh --- database handle;
#                $sample_extract_id --- ;
#                $technique_id ---
#                $replicate --- the replicate number;
#                $file_link_id --- the link id for data file
# parameter out: $metabolite_measurement_id --- the new metabolite_measurement
#                                               _id; 
#
###########################################################################
sub createMetaboliteMeasurementId_flink {
    my ($dbh, $sample_extract_id, $technique_id, $replicate, $file_link_id) = @_;
    my ($sql, $sth, $rc);
    my $metabolite_measure_id;

    $sth = $dbh->prepare("select nextval('metabolite_measurement_id_seq')") 
         or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $metabolite_measurement_id = $sth->fetchrow;

    $sql = "insert into metabolite_measurement (metabolite_measurement_id,";
    $sql .= "sample_extract_id, technique_id, replicate, file_location_link_id";
    $sql .= ") values (\'$metabolite_measurement_id\', \'$sample_extract_id\', ";
    $sql .= "\'$technique_id\', \'$replicate\', \'$file_link_id\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $metabolite_measurement_id;
}

###########################################################################
# 17a. subroutine: createMetaboliteMeasurementId($dbh, $sample_extract_id, 
#                 $technique_id, $protocol_id, $replicate, $person_id):
#  this subroutine create a metabolite_measurement_id in metabolite_measurement
#  table
# 
# parameters in: $dbh --- database handle;
#                $sample_extract_id --- ;
#                $technique_id ---
#                $protocol_id ---
#                $replicate --- the replicate number;
#                $person_id --- the person who did the metabolite extraction
# parameter out: $metabolite_measurement_id --- the new metabolite_measurement
#                                               _id; 
# X. Jing Li, 9/2/2004.
###########################################################################
sub createMetaboliteMeasurementId {
    my ($dbh, $sample_extract_id, $technique_id, $protocol_id, $replicate, $person_id) = @_;
    my ($sql, $sth, $rc);
    my $metabolite_measure_id;

    $sth = $dbh->prepare("select nextval('metabolite_measurement_id_seq')") 
         or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $metabolite_measurement_id = $sth->fetchrow;

    $sql = "insert into metabolite_measurement (metabolite_measurement_id,";
    $sql .= "sample_extract_id, technique_id, protocol_id, replicate, person_id) ";
    $sql .= "values (\'$metabolite_measurement_id\', \'$sample_extract_id\', ";
    $sql .= "\'$technique_id\', \'$protocol_id\', \'$replicate\', \'$person_id\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $metabolite_measurement_id;
}

###########################################################################
# 17b. subroutine: createMetaboliteMeasurementIdWithProcInputFile
# ($dbh, $sample_extract_id, $technique_id, $protocol_id, $replicate, 
# $person_id, $proc_input_file):
#  this subroutine create a metabolite_measurement_id in metabolite_measurement
#  table
# 
# parameters in: $dbh --- database handle;
#                $sample_extract_id --- ;
#                $technique_id ---
#                $protocol_id ---
#                $replicate --- the replicate number;
#                $person_id --- the person who did the metabolite extraction
#                $proc_input_file --- the file name on user's local machine
# parameter out: $metabolite_measurement_id --- the new metabolite_measurement
#                                               _id; 
# X. Jing Li, 11/2/2004.
###########################################################################
sub createMetaboliteMeasurementIdWithProcInputFile {
    my ($dbh, $sample_extract_id, $technique_id, $protocol_id, $replicate, $person_id, $proc_input_file) = @_;
    my ($sql, $sth, $rc);
    my $metabolite_measure_id;

    $sth = $dbh->prepare("select nextval('metabolite_measurement_id_seq')") 
         or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $metabolite_measurement_id = $sth->fetchrow;

    $sql = "insert into metabolite_measurement (metabolite_measurement_id,"
        . "sample_extract_id, technique_id, protocol_id, replicate, person_id, "
        . "proc_input_file) values (\'$metabolite_measurement_id\', "
        . "\'$sample_extract_id\', \'$technique_id\', \'$protocol_id\', "
        . "\'$replicate\', \'$person_id\', \'$proc_input_file\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $metabolite_measurement_id;
}

###########################################################################
# 18. subroutine: insertMetaboliteData($dbh, $file, $metabolite_measurement_id):
#  this subroutine insert the data from metabolite to DOME
# 
# parameters in: $dbh --- database handle;
#                $file --- the file which contains the data;
#                $metabolite_measurement_id --- th
# parameter out: $success --- "SUCCESS" or "FAIL" for populating data from this
#                    file; 
#
###########################################################################
sub insertMetaboliteData {
    my ($dbh, $file, $metabolite_measurement_id) = @_;
    my ($sth, $rc, $sql, @record);
    my ($success, @contents, $start_read, $line_num, $line, @fields, $name);
    my ($ref_derivative_ids, %derivative_ids, $intensity, $derivative_id);
    my (@data, $tempfile);
 
    $success = "FAILT";

    #== get all metabolite_dervative_id and save in a hash. If a new name come
    #== up create a new metabolite_derivative_id and save it in the hash
    $ref_derivative_ids = ();
    %derivative_ids = %$ref_derivative_ids;
    $sql = "select metabolite_derivative_id, name from metabolite_derivative";
    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    while ( @record = $sth->fetchrow ) {
        $derivative_id = $record[0];
        $name = $record[1];
        $derivative_ids{$name} = $derivative_id;
    }

    #== 1. read the data from the source file:
   #print "$file\n";
    open (datafile, "<$file");
    @contents = <datafile>;
    close datafile;

    #== 2. find where to start to read the data. Check for the column order is
    #== indead the date type it suposs to be.
    $start_read = "N";
    $line_num = 0;
    $sql = "";
   #print "there are " . @contents . " lines\n";
    foreach $line (@contents) {
        $line_num++;
        chomp $line;
        $line =~ s/\"//g;      # delete double quotes
        $line =~ s /\
//g;

        if ($line =~ /^name/){
            $start_read = "Y";
            @fields = split (/\t/, $line);
            for (my $i=0; $i<@fields; $i++) {
               #print "fields[$i] = $fields[$i]\n";
            }
            if ($fields[0] eq "name" && $fields[1] eq "intensity") {
                next;
            } else {
                #print "goto last line\n";
               #print  "Data fields are in wrong order\n";
                last;
            }
        }
        #== 3. read the data if this is a data line:
        if ($start_read eq "N") {
            next;
        } else {
            @data = split(/\t/, $line);
            $name = $data[0];
            $intensity = $data[1];

            #== 4. get the derivative_id for this name. If it is not in the hash
            #== %derivative_ids then we need to create a new id.
            if ( !exists($derivative_ids{$name}) ){
                $derivative_id = createMetaboliteDerivativeId($dbh, $name);
                $derivative_ids{$name} = $derivative_id;
            } else {
                $derivative_id = $derivative_ids{$name};
            }
           
            #==5. Create the sql for insert data into metabolite_prossed_stage2:
            $sql .= "insert into metabolite_processed_stage2 (\"metabolite_";
            $sql .= "processed_stage2_id\", \"metabolite_measurement_id\", ";
            $sql .= "\"metabolite_derivative_id\", intensity) values ";
            $sql .= "(nextval('metabolite_processed_stage2_id_seq'), ";
            $sql .= "$metabolite_measurement_id, $derivative_id, $intensity);\n";
        }

        #for (my $i=1; $i<=20; $i++) {
        #    if ($line_num > (@contents/20)*$i && $line_num < (@contents/20)*$i+1) {
        #        print "$gpr_file_name: Finished " . 5*$i . "\% \n";
        #    }
        #}
    }

    #== print out the sql statement before execute
    #my $tempfile = "/export/home/xingjing/data/dome/upload_log/temp_sql.sql";
    #open (TMP, ">$tempfile");
    #print TMP $sql;
    #close TMP;

    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $sth->finish;

    #== delete the temp file
    #unlink ($tempfile) or die "couldn't delete $tempfile: $!";

    if ($line_num == @contents) {
        $success = "SUCCESS";
        #print "All done!\n";
    }

    return $success;
}

###########################################################################
# 19. subroutine: getSamplingReplicateId($dbh, $exp_set_name, $control, 
#                    $exp_time, 
#             $replicate):
#    this subroutine get the sampling_replicate_id from Dome.
#
# parameters in: $dbh --- the database handle,
#                $exp_set_name --- the experiment set name,e.g Yest, 
#                                  Methyl Jasmonate or UV iradiation,
#                $control --- either Control or Elicity for experiment
#                $exp_time --- the experiment_time,
#                $replicate --- the replicate number,
# parameter out: $sampling_replicate_id --- the sampling_point_id correspoding to
#                the parameters.
#
###########################################################################
sub getSamplingReplicateId {
    my ($dbh, $exp_set_name, $control, $exp_time, $replicate) = @_;
    my ($sql, $sth, $rc);
    my ($label, $exp_set_id, $sampling_replicate_id, $exp_id);

    if ($control eq "C") {
        $control = "Control";
    } elsif ($control eq "E") {
        $control = "Elicited";
    }
    $label = $exp_set_name . " " . $control;

    $sql = "select sampling_replicate_id from sampling_replicate where ";
    $sql .= "replicate = $replicate and sampling_point_id = ";
    $sql .= "(select sampling_point_id from sampling_point where label_id = ";
    $sql .= "(select label_id from label where name = \'$exp_time\')";
    $sql .= " and experiment_id =";
    $sql .= "(select experiment_id from experiment where exp_name = \'$label\'";
    $sql .= "and exp_set_id = (select exp_set_id from experiment_set where ";
    $sql .= "exp_set_name = \'$exp_set_name\') ) )";
    #print "In getSamplingReplicateId\nsql = $sql\n";
    $sth = $dbh->prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    $sampling_replicate_id = $sth->fetchrow;

    $sth->finish;
    return $sampling_replicate_id;

}

###########################################################################
# 20. subroutine: getMaxId($dbh, $table_name):
#    this subroutine get the maxium id (last id) in table $table_name.
#
# parameters in: $dbh --- the database handle,
#                $table_name --- Te table name,
# parameter out: $id --- the last id of the table.
#
###########################################################################
sub getMaxId {
    my ($dbh, $table_name) = @_;
    my ($sql, $sth, $rc, $id);

    $sql = "select max(" . $table_name . "_id) from $table_name";
    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    $id = $sth->fetchrow();

    $sth->finish;
    return $id;
}

###########################################################################
# 21. subroutine: getSampleId($dbh, $sampling_replicate_id):
#    this subroutine get the sample_id from sample.
#
# parameters in: $dbh --- the database handle,
#                $sampling_replicate_id --- the sampling_replicate_id,
# parameter out: $sampleid --- the sample_id.
#
# X. Jing Li, 8/31/2004
###########################################################################
sub getSampleId {
    my ($dbh, $sampling_replicate_id, $extract_id, $tissue_type_label_id) = @_;
    my ($sql, $sth, $rc, $id);

    $sql = "select sample_id from sample where sampling_replicate_id = "
        . "$sampling_replicate_id and label_id = $tissue_type_label_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    $id = $sth->fetchrow();

    $sth->finish;
    return $id;
}

###########################################################################
# 22. getSampleExtractId($dbh, $sample_id, $extract_id):
#    this subroutine get the sample_extract_id from sample_extract.
#
# parameters in: $dbh --- the database handle,
#                $sample_id --- the sample_id,
#                $extract_id --- the extract_id,
# parameter out: $sample_extract_id --- the sample_extract_id.
#
# X. Jing Li, 8/31/2004
###########################################################################
sub getSampleExtractId {
    my ($dbh, $sample_id, $extract_id, $technical_replicate) = @_;
    my ($sql, $sth, $rc, $id);

    $sql = "select sample_extract_id from sample_extract where sample_id = "
        . "$sample_id and extract_id = $extract_id and technical_replicate "
        . "= $technical_replicate";
    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    $id = $sth->fetchrow();

    $sth->finish;
    return $id;
}

###########################################################################
# 23. getMetaboliteMeasurementId($dbh, $seid, $technique_id, $replicate)
#    this subroutine get the metabolite_measurement_id from .
#
# parameters in: $dbh --- the database handle,
#                $seid --- the sample_extract_id,
#                $technique_id -- the technique_id,
#                $replicate --- the replicate number,
# parameter out: $mmid --- the metabolite_measurement_id.
#
# X. Jing Li, 9/2/2004
###########################################################################
sub getMetaboliteMeasurementId {
    my ($dbh, $seid, $technique_id, $replicate) = @_;
    my ($sql, $sth, $rc, $id);

    $sql = "select metabolite_measurement_id from  metabolite_measurement "
        . "where sample_extract_id = $seid and replicate = $replicate and "
        . "technique_id = $technique_id";
   #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    $id = $sth->fetchrow();

    $sth->finish;
    return $id;
}

##########################################################################
# 24. getKwTableId($dbh, $protocol_id)
#  this subroutine get the kw_table_id by given extract_id;
#
# parameters in: $dbh --- the database handle,
#                $protocol_type_id --- the protocol_type_id,
# parameter out: $kw_table_id --- the metabolite_measurement_id.
#
# X. Jing Li, 10/11/2005
###########################################################################
sub getKwTableId {
    my ($dbh, $protocol_type_id) = @_;
    my ($sql, $sth, $rc, $technique_type, $kw_table, $kw_table_id);

    #$sql = "select technique_type from technique where technique_id = (select technique_id from protocol_type where protocol_type_id = (select protocol_type_id from protocol where protocol_id = $protocol_id))";
    $sql = "select technique_type from technique where technique_id = (select technique_id from protocol_type where protocol_type_id = $protocol_type_id)";
#    print "getKwTable: $sql\n";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $technique_type = $sth->fetchrow();
    if (uc($technique_type) eq "MRNA") {
        $kw_table = "REPORTER";
    } elsif (uc($technique_type) eq "METABOLITE") {
        $kw_table = "COMPOUND";
    } elsif (uc($technique_type) eq "PROTEIN") {
        $kw_table = "PROTEIN";
    } else {
        $kw_table = "UNKNOWN";
    }
    $sql = "select kw_table_id from ref_kw_table where kw_table = \'$kw_table\'";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $kw_table_id = $sth->fetchrow();

    $sth->finish;
    return $kw_table_id;
}
##########################################################################
# 24a. getKwTableIdByProtocolId($dbh, $protocol_id)
#  this subroutine get the kw_table_id by given extract_id;
#
# parameters in: $dbh --- the database handle,
#                $protocol_id --- the protocol_id,
# parameter out: $kw_table_id --- the metabolite_measurement_id.
#
# X. Jing Li, 10/11/2005
###########################################################################
sub getKwTableIdByProtocolId {
    my ($dbh, $protocol_id) = @_;
    my ($sql, $sth, $rc, $technique_type, $kw_table, $kw_table_id);

    $sql = "select technique_type from technique where technique_id = (select technique_id from protocol_type where protocol_type_id = (select protocol_type_id from protocol where protocol_id = $protocol_id))";
    #$sql = "select technique_type from technique where technique_id = (select technique_id from protocol_type where protocol_type_id = $protocol_type_id)";
#    print "getKwTable: $sql\n";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $technique_type = $sth->fetchrow();
    if (uc($technique_type) eq "MRNA") {
        $kw_table = "REPORTER";
    } elsif (uc($technique_type) eq "METABOLITE") {
        $kw_table = "COMPOUND";
    } elsif (uc($technique_type) eq "PROTEIN") {
        $kw_table = "PROTEIN";
    } else {
        $kw_table = "UNKNOWN";
    }
    $sql = "select kw_table_id from ref_kw_table where kw_table = \'$kw_table\'";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $kw_table_id = $sth->fetchrow();

    $sth->finish;
    return $kw_table_id;
}

##########################################################################
# 25. createGeneId($dbh, $specie_id);
#  this subroutine create gene_id in gene table in BNET;
#
# parameters in: $dbh --- the database handle,
#                $specie_id --- the specieid,
# parameter out: $gene_id --- the gene_id.
#
# X. Jing Li, 10/12/2005
###########################################################################
sub createGeneId {
    my ($dbh, $specie_id) = @_;
    my ($sql, $sth, $rc, $id);

    $sql = "select nextval('gene_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $id = $sth->fetchrow();

    $sql = "insert into gene (gene_id, specie_id) values ($id, $specie_id)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;

    return $id;
}

##########################################################################
# 26. createEvidenceId($dbh, $evidence_code_id, $reference_id, $kwid, $record_id)
#  this subroutine create evidence_id in evidence table in BNET;
#
# parameters in: $dbh -- the database handle,
#                $evidence_code_id -- the evidence_code id,
#                $reference_id -- 
#                $kwid -- the kw_table_id,
#                $record_id -- the record_id,
# parameter out: $id -- the gene_id.
#
# X. Jing Li, 10/12/2005
###########################################################################
sub createEvidenceId {
    my ($dbh, $evidence_code_id, $reference_id, $kwid, $record_id) = @_;
    my ($sql, $sth, $rc, $id);

    $sql = "select nextval('evidence_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $id = $sth->fetchrow();

    $sql = "insert into evidence (evidence_id, evidence_code_id, reference_id, kw_table_id, record_id) values ($id, \'$evidence_code_id\', \'$reference_id\', \'$kwid\', \'$record_id\')";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;

    return $id;
}

##########################################################################
# 27. createExternalLinksId($dbh, $external_db_id, $kwid, $record_id, 
#            $internal_field_name, $external_id);
#  this subroutine create external_link_id in externail_link table in BNET;
#
# parameters in: $dbh -- the database handle,
#                $external_db__id -- the evidence_code id,
#                $kwid -- the kw_table_id,
#                $record_id -- the record_id,
#                $internal_field_name --
#                $external_id -- the id for external_database,
# parameter out: $id -- the external_link_id.
#
# X. Jing Li, 10/12/2005
###########################################################################
sub createExternalLinksId {
    my ($dbh, $external_db_id, $kwid, $record_id, $internal_field_name, $external_id) = @_;
    my ($sql1, $sql2, $sth, $rc, $id);

    $sql1 = "select nextval('external_links_id_seq')";
    $sth = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $id = $sth->fetchrow();

    $sql1 = "insert into external_links (external_links_id, external_database_id, kw_table_id, record_id";
    $sql2 = " values ($id, $external_db_id, $kwid, $record_id";
    if ($internal_field_name ne "") {
        $sql1 .= ", internal_field_name";
        $sql2 .= ", \'$internal_field_name\'";
    }
    if ($external_id ne "") {
        $sql1 .= ", external_id";
        $sql2 .= ", \'$external_id\'";
    }
    $sql1 .= ")";
    $sql2 .= ")";
    $sql1 = $sql1 . $sql2;
    #print "$sql1\n";
    $sth = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;

    return $id;
}

##########################################################################
# 28. createAllnameId($dbh, $aname, $kwid, $record_id, $is_primary)
#  this subroutine create evidence_id in evidence table in BNET;
#
# parameters in: $dbh -- the database handle,
#                $aname -- the entity name,
#                $kwid -- kw_table_id,
#                $record_id -- the entity id,
#                $is_primary -- "Y" for primary, "N" for synonym, 
#                               "A" for annotation,
#                $unikey -- the unchangable keyname, it is the 
#                    "taxon_id: + "_" + "primary_name"
# parameter out: $id -- the name_id.
#
# X. Jing Li
# 10/12/2005
# 10/31/2005: add unikey.
# 3/21/2006: add checking unique aname for allnames.  
###########################################################################
sub createAllnameId {
    my ($dbh, $aname, $kwid, $record_id, $is_primary, $unikey) = @_;
    my ($sql, $sth, $rc, $id);

    $search_name = getSearchName($aname);
    $aname =~ s/\'/\'\'/g;
    $search_name =~ s/\'/\'\'/g;
    $unikey =~ s/\'/\'\'/g;

    #-- check to see if this name is already in bnet:
    $sql = "select record_id from allnames where kw_table_id = $kwid and upper(search_name) = upper(\'$search_name\') and unikey = \'$unikey\'";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $id = $sth->fetchrow();
    if ($id ne "") {
        return $id;
    }

    $sql = "select nextval('allnames_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $id = $sth->fetchrow();

    $sql = "insert into allnames (name_id, aname, kw_table_id, record_id, is_primary, search_name, unikey) values ($id, \'$aname\', $kwid, $record_id, \'$is_primary\', \'$search_name\', \'$unikey\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;

    return $id;
}

###########################################################################
# 29. createCompositeSequenceId($dbh, $annotation, $probe_name, $gene_id):
#             
# parameters in: $dbh -- The database handle,
#                $annotation -- the gene name or gene description
#                $pname -- the probe_name, i.e. the TC number,
#                $gene_id -- the gene id
#           out: $csid_id --- The composite_sequence_id,
#
# last update: 5/11/2005, X. Jing Li
###########################################################################
sub createCompositeSequenceId {
    my ($dbh, $annotation, $probe_name, $gene_id) = @_;
    my ($sql, $sql2, $sth, $rc, $csid);

    $probe_name =~ s/\'/\'\'/g;
    $annotation =~ s/\'/\'\'/g;
    $sql = "select nextval('composite_sequence_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $csid = $sth->fetchrow();
    $sql = "insert into composite_sequence (composite_sequence_id";
    $sql2 = "values ($csid";
    if ($probe_name ne "") {
        $sql .= ", probe_name";
        $sql2 .= ", \'$probe_name\'";
    }
    if ($annotation ne "") {
        $sql .= ", annotation";
        $sql2 .= ", \'$annotation\'";
    }
    if ($gene_id ne "") {
        $sql .= ", gene_id";
        $sql2 .= ", \'$gene_id\'";
    }
    $sql .= ") " . $sql2 . ")";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;

    return $csid;
}

###########################################################################
# 30. createReporterId($dbh, $csid): 
#             
# parameters in: $dbh -- The database handle,
#                $csid -- the composite_sequence_id,
#           out: $reporter_id --- The reporter_id,
#
# last update: 5/11/2005, X. Jing Li
###########################################################################
sub createReporterId {
    my ($dbh, $csid) = @_;
    my ($sql, $sth, $rc, $id);

    $sql = "select nextval('reporter_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $id = $sth->fetchrow();

    $sql = "insert into reporter (reporter_id, composite_sequence_id) values ($id, $csid)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;

    return $id;
}

###########################################################################
# 31. getSearchName($name): 
#             
# parameters in: $name -- The name in allnames,
#           out: $search_name --- The name stripped ",", " ", "-" and """;
#
# last update: 10/12/2005, X. Jing Li
###########################################################################
sub getSearchName {
    my ($name) = @_;
#    my $search_name;

#    $search_name = $name;
#    $search_name =~ s/\,+|\s+|\-+|\"+//g;
#    print "in sub: name = $name, search_name = $search_name\n";
#    return $search_name;
    $name =~ s/\,+|\s+|\-+|\"+//g;
    return $name;
}

###########################################################################
# 32. getFileId($dbh, $file_name)
#
# parameters in: $dbh --- the database handle,
#                $file_name --- the file name
# parameter out: $fid --- the file_id.
#
# X. Jing Li, 9/14/2005
###########################################################################
sub getFileId {
    my ($dbh, $file_name) = @_;
    my ($sql, $sth, %rc, $fid);

    $sql = "select file_id from \"file\" where file_name = \'$file_name\'";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $fid = $sth->fetchrow();
    $sth->finish;

    return $fid;
}


###########################################################################
# 33. getFileTypeId($dbh, $file_name)
#
# parameters in: $dbh --- the database handle,
#                $file_name --- the file home,
# parameter out: $ftid --- the new file_type_id;
#
# X. Jing Li, 7/20/2005
###########################################################################
sub getFileTypeId {
    my ($dbh, $file_name) = @_;
    my ($sql, $sth, $rc, @temp, $file_type, $file_type_id);

    @temp = split /\./, $file_name;
    $file_type = "." . pop @temp;
    $sql = "select file_type_id from file_type where upper(\"file_suffix\") = \'" . (uc $file_type) . "\'";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $file_type_id = $sth->fetchrow();

    $sth->finish;
    return $file_type_id;
}
###########################################################################
# 34.createtFileTypeId($dbh, $file_name, $desc)
#
# parameters in: $dbh --- the database handle,
#                $file_name --- the file home,
#                $desc -- description of the file type;
# parameter out: $ftid --- the new file_type_id;
#
# X. Jing Li, 7/20/2005
###########################################################################
sub createFileTypeId {
    my ($dbh, $file_name, $desc) = @_;
    my ($sql, $sth, $rc, @temp, $file_type, $ftid);

    @temp = split /\./, $file_name;
    $file_type = "." . pop @temp;
    $sql = "select nextval('file_type_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $ftid = $sth->fetchrow();

    $sql = "insert into\"file_type\"  (file_type_id, \"type\", \"file_suffix\") values ($ftid, \'$desc\', \'$file_type\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $sth->finish;
    return $ftid;
}

###########################################################################
# 35. createFileIds($dbh, $file_home, $file_name, $file_type_id, person_id)
#  this subroutine create file_id, file_location_link_id, and copy the file to 
# its final resident location. If file_home is new then create file_location_id
# as well.
#
# parameters in: $dbh -- the database handle,
#                $file_home -- the final location which the file will be copied,
#                $file_name -- the file name,
#                $file_type_id -- file_type_id,
#                $person_id -- the person who did upload the file,
# parameter out: $id -- the file_location_link_id.
#
# X. Jing Li, 4/12/2005
###########################################################################
sub createFileIds {
    my ($dbh, $file_home, $file_name, $file_type_id, $person_id) = @_;
    my ($sql, $sth, $rc, $flid, $fid, $fllid, @temp, $file_suffix, $fname,
        $file_type, $new_fname, $cmd);

    #== get file_location_id for file_home:
    $sql = "select file_location_id from file_location where location = \'$file_home\'";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $flid = $sth->fetchrow();
    if ($flid eq "") {
        $flid = createFileLocationId($dbh, $file_home);
    }

    #== create file_id:
    @temp = split "/", $file_name;
    $fname = pop @temp;
    $fname =~ s/\'/\'\'/g;

    $sql = "select nextval('file_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $fid = $sth->fetchrow;

    $sql = "insert into \"file\" (file_id, file_name, file_type_id, time_uploaded, person_id) values ($fid, \'$fname\', $file_type_id, (SELECT abstime(timestamp 'now')), $person_id)";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    #== create file_location_link_id:
    $sql = "select nextval('file_location_link_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $fllid = $sth->fetchrow;

    $sql = "insert into file_location_link (file_location_link_id, file_id, file_location_id) values ($fllid, $fid, $flid)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    if ($fid ne "" && $flid ne "" && $fllid ne "") {
        umask (0033);
        @temp = split "/", $file_name;
        $fname = pop @temp;
        $fname =~ s/\'/\\\'/g;
        $new_fname = $file_home . $fname;
        $cmd = "cp \"$file_name\" \"$new_fname\"\n";
        #print "$cmd\n";
        system $cmd;
        $cmd = "chmod 644 \"$new_fname\"";
        system $cmd;
        #unlink ($file_name) or warn "Can not delete file $file_name from holding diretory. $!\n";
        umask();
    }
    $sth->finish;
    return $fllid;
}

###########################################################################
# 36. createFileLocationId($dbh, $file_home)
#
# parameters in: $dbh --- the database handle,
#                $file_home --- the file home name
# parameter out: $flid --- the file_location_id.
#
# X. Jing Li, 9/12/2005
###########################################################################
sub createFileLocationId {
    my ($dbh, $file_home) = @_;
    my ($sql, $sth, $rc, $flid);

    $sql = "select nextval('file_location_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $flid = $sth->fetchrow();

    $sql = "insert into file_location (file_location_id, \"location\") values ($flid, \'$file_home\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;

    return $flid;
}

###########################################################################
# End of the package. Paste your subroutine above this line and do not write 
# anything below. Otherwise this package won't compile.
###########################################################################
1;
