<?php
/**
* This script is a page for adding Metabolomics data to the DOME database.
* Date: 9 Sept 2004
* @author Aejaaz Kamal <aekamal@vt.edu>
*/

include_once("common.php");
include_once("compoundFrame.php");
include_once("workFlow.php");
//Parse Get or Post step number and set the current step number.
setStepNumber();

//Sets Environment Variable
setVariables($_SESSION['STEP_NUMBER']);
doControlBarRedirection(1);
?>

<?php
$stepCount = count(split(':', $_SESSION['STEP_NUMBER']));
$title = "Add Metabolomics Data";
include("header.php");
if (!(strtoupper($_SESSION['role']) == 'EDITOR' OR strtoupper($_SESSION['role']) == 'MANAGER'))	
{	printError("You are not authorized to view this page. Your role level should be 'editor' or 'manager' to view this page.");	exit();}
?>
<p class="help-tips">Please provide some more information about the data. If the drop down menu does not contain any options then please ask your manager to add the corresponding information in the Meta Data section in the database before you can load the data. If there is no reference compound used then please leave the field blank.</p>
<?php

if ($errMsgs)
{  
	foreach ($errMsgs as $msg)
	{	printError($msg);	}
}

if (isset($_REQUEST['technique_id']))
{
	$currPageNameUpper = strtoupper($currPageName);
	$_SESSION[$currPageNameUpper]['technique_id'] = $_REQUEST['technique_id'];
}


fillForm();

include_once("footer.php");
?>

<?php
//Functions
function fillForm()
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	$self = $_SERVER[PHP_SELF];
	print "<form name=\"init_form\" method=\"post\" action=\"$self\" enctype=\"multipart/form-data\">\n";
	
	
	//This MAX_SIZE is an advisory to the browser. (So that the user does not find
	// that his file is rejected by the php on the server after uploading it).
	print '<input type="hidden" name="MAX_FILE_SIZE" value="30000000" />';
	//Processed Data File
	print "<p class=\"help-tips\">Please enter the processed data file. (Note: The filename cannot contain space or single/double quotes. Also you cannot have same file names for more than one sample) One file for each technical replicate in a tab-delimited format:</p>";
	print "<p><label>Processed Data File: </label>";
	print '<input name="uploadFile" size="48" type="file" value="Select File using browse"/>';
	print "</p>";
	
	//Raw Data File
	print "<br><p class=\"help-tips\">Please enter the raw data file. If you are unable to provide the raw data file at this time then although you can still proceed with data insertion but remember that you will not be able to retrieve this file using DOME at a later instance. (Note: The filename cannot contain space or single/double quotes. Also you cannot have same file names for more than one sample):</p>";
	print "<p><label>Raw Data File: </label>";
	print '<input name="RawDataFile" size="48" type="file" value="Select File using browse"/>';
	print "</p>";
	print "<br><p class=\"help-tips\">Please enter the proper Meta Data information:</p>";
	printMetaDataSelections("", $_SESSION[$currPageNameUpper], true, true);

	
	//Print Extract.
	$sql = "SELECT extract_id, name FROM extract WHERE technique_id = '{$_SESSION[$currPageNameUpper]['technique_id']}' ORDER BY name";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Extracts. " . $sth->getMessage());	return;	}
	print "<p><label>Extraction Method: </label>";
	print "<select name=\"extract_id\">";
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['extract_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";

	//Print (Experimental) Protocol. 
	$sql = "SELECT p.protocol_id, p.protocol_name FROM protocol p, protocol_type pt
		WHERE p.protocol_type_id = pt.protocol_type_id 
		AND pt.technique_id = '{$_SESSION[$currPageNameUpper]['technique_id']}'
		AND UPPER(pt.method) = 'EXPERIMENTAL'
		ORDER BY protocol_name";
	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving (experimental) Protocols. " . $sth->getMessage());	return;	}
	print "<p><label>Experimental Protocol: </label>";
	print "<select name=\"protocol_id\">";
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['protocol_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
	
	//Print Normalization Method
	$sql = "SELECT \"protocol_type_id\", \"protocol_type_name\" FROM \"protocol_type\" 
			WHERE \"technique_id\" = '{$_SESSION[$currPageNameUpper]['technique_id']}' 
			AND UPPER(\"method\") = 'STATISTICAL' ORDER BY UPPER(\"protocol_type_name\")";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Protocol Types. " . $sth->getMessage());	return;	}
	print "\n<p><label>Normalization Method: </label>";
	print "<select name=\"protocol_type_id\">";
	
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['protocol_type_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
	
		//Now Print reference Compound Name
	print "<p><label>Reference Compound: </label><input type=\"text\" name=\"reference_cpd\" value=\"{$_SESSION[$currPageNameUpper]['reference_cpd']}\" size=\"25\" />";
	//Print Normalize Data checkbox.
	print "\n<input type=\"checkbox\" name=\"normalize_data\"";
	if ($_SESSION[$currPageNameUpper]['normalize_data'])
	{	print " checked";	}
	print ">Normalize Data</input>";
	print "</p>";
	
	//Print Researchers. 
	$sql = "SELECT person_id, person_name FROM person ORDER BY person_name";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Researchers. " . $sth->getMessage());	return;	}
	print "<p><label>Researcher: </label>";
	print "<select name=\"researcher_id\">";
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['researcher_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
	
	printControlBar($_SESSION['STEP_NUMBER']);
	print "\n</form>";
	
}

/**============================
* Request Functions
*============================*/
function processForm($currPageName)
{
	global $errMsgs, $currPageName, $prevPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	$_SESSION[$currPageNameUpper]['experiment_set_id'] = $_REQUEST['experiment_set_id'];
	$_SESSION[$currPageNameUpper]['exp_name'] = $_REQUEST['exp_name'];
	$_SESSION[$currPageNameUpper]['sampling_point_label_id'] = $_REQUEST['sampling_point_label_id'];
	$_SESSION[$currPageNameUpper]['tissue_type_label_id'] = $_REQUEST['tissue_type_label_id'];
	$_SESSION[$currPageNameUpper]['biological_replicate'] = $_REQUEST['biological_replicate'];
	$_SESSION[$currPageNameUpper]['technical_replicate'] = $_REQUEST['technical_replicate'];
	$_SESSION[$currPageNameUpper]['researcher_id'] = $_REQUEST['researcher_id'];
	$_SESSION[$currPageNameUpper]['extract_id'] = $_REQUEST['extract_id'];
	$_SESSION[$currPageNameUpper]['protocol_id'] = $_REQUEST['protocol_id'];
	$_SESSION[$currPageNameUpper]['protocol_type_id'] = $_REQUEST['protocol_type_id'];
	$_SESSION[$currPageNameUpper]['normalize_data'] = $_REQUEST['normalize_data'];
	$_SESSION[$currPageNameUpper]['reference_cpd'] = $_REQUEST['reference_cpd'];
	
	if (!$_SESSION[$currPageNameUpper]['technique_id'])
	{	$errMsgs[] = "technique_id not set";	}
	
	if (!$_SESSION[$currPageNameUpper]['experiment_set_id'])
	{	$errMsgs[] = "Please Select an Experiment Set.";	}
	
	if (!$_SESSION[$currPageNameUpper]['exp_name'])
	{	$errMsgs[] = "Please Select an Experiment.";	}
	
	if (!$_SESSION[$currPageNameUpper]['sampling_point_label_id'])
	{	$errMsgs[] = "Please Select a Sampling Point";	}
	
	if (!$_SESSION[$currPageNameUpper]['tissue_type_label_id'])
	{	$errMsgs[] = "Please Select a Tissue Type";	}
	
	if (!$_SESSION[$currPageNameUpper]['biological_replicate'])
	{	$errMsgs[] = "Please Select the biological replicate number.";	}
	
	if (!$_SESSION[$currPageNameUpper]['technical_replicate'])
	{	$errMsgs[] = "Please Select the technical replicate number.";	}
		
	if ($_REQUEST['extract_id'] == 0)
	{	$errMsgs[] = "Please select an Extraction Method";	}
	
	if ($_REQUEST['protocol_id'] == 0)
	{	$errMsgs[] = "Please select a Protocol.";	}
	
	if ($_REQUEST['protocol_type_id'] == 0)
	{	$errMsgs[] = "Please select a Normalization Method.";	}
	
	if ($_REQUEST['normalize_data'] AND strlen($_REQUEST['reference_cpd'])== 0)
	{	$errMsgs[] = "You must specify a Reference Compound if you want the data to be normalized.";	}
	
	if (!$_SESSION[$currPageNameUpper]['researcher_id'])
	{	$errMsgs[] = "Please Select the name of the Researcher.";	}
	
	if ($errMsgs)
	{	return 0;	}

	if ($_FILES['uploadFile']['name'] == "")
	{
		$errMsgs[] = "Please select the file to upload using the browse button.";		
	}
	if (preg_match('/\'/', $_FILES['uploadFile']['name']))
	{
		$errMsgs[] = "Name of the processed file contains an invalid character.";		
	}
	if (preg_match('/\'/', $_FILES['RawDataFile']['name']))
	{
		$errMsgs[] = "Name of the Raw Data file contains an invalid character.";		
	}
	
	if ($errMsgs)
	{	return 0;	}
	
	$uploaddir = $_SESSION['BROWSER_TMP_FULL'];
	$uploadfile = $uploaddir . $_FILES['uploadFile']['name'];
	
	if (move_uploaded_file($_FILES['uploadFile']['tmp_name'], $uploadfile)) 
	{
	   //File is valid, and was successfully uploaded
	   $_SESSION[$currPageNameUpper]['uploadFile'] = $uploadfile;
	   $_SESSION[$currPageNameUpper]['uploadFileName'] = $_FILES['uploadFile']['name'];
	   system("chmod 644 $uploadfile");
	} 
	else
	{
		$errMsgs[] = "Could not move the Processed Data File to the Destination on the server";
	}
	if ($_FILES['RawDataFile']['name'] == "")
	{
		unset($_SESSION[$currPageNameUpper]['RawDataFile']);
		unset($_SESSION[$currPageNameUpper]['RawDataFileName']);
	}
	else
	{
		$uploadfile = $uploaddir . $_FILES['RawDataFile']['name'];
		if (move_uploaded_file($_FILES['RawDataFile']['tmp_name'], $uploadfile)) 
		{
		   //File is valid, and was successfully uploaded
		   $_SESSION[$currPageNameUpper]['RawDataFile'] = $uploadfile;
		   $_SESSION[$currPageNameUpper]['RawDataFileName'] = $_FILES['RawDataFile']['name'];
		   system("chmod 644 $uploadfile");
		} 
		else 
		{	$errMsgs[] = "Could not move the Raw Data File to the Destination on the server.";	}
	}
	
	//Validate Software if it exists
	//First Open File
	$headerLine = trim(array_shift(file($_SESSION[$currPageNameUpper]['uploadFile'])));
	$headers = split("\t", $headerLine);
	if (count($headers) < 4)
	{	$errMsgs[] = "The header (first line) of the uploaded file has less than 4 colums.";	}
	elseif (count($headers) > 4) //Validate software if more than 4 columns.
	{
		for ($i = 4; $i < count($headers); $i++)
		{
			$header = $headers[$i];
			$software = array_shift(split(':', $header));
			if (strlen($software) == 0)
			{	$errMsgs[] = "No Software specified in header for column $i.";	}
			else
			{
				$sql = "SELECT \"software_id\", \"software_name\" FROM \"software\"
					WHERE UPPER(\"software_name\") = UPPER('$software')";
					
				$sth = $_SESSION['dbh']->query($sql);
				if ( DB::isError($sth) ) 
				{	
					$errMsgs[] = "Error checking for software name for column $i. " . $sth->getMessage();	
					return 0;
				}
				
				if ($sth->numrows() == 0)
				{	$errMsgs[] = "Software '$software' not found in the database for header column $i.";	}
	
			}
		}
	}
	if ($errMsgs)
	{	return 0;	}
	
		
	//Now find out sampling_replicate_id from the given info.
	$sql = "SELECT sr.\"sampling_replicate_id\" FROM \"sampling_replicate\" sr
					, \"sampling_point\" sp
					, \"experiment\" e
				WHERE e.\"exp_set_id\" = '{$_SESSION[$currPageNameUpper]['experiment_set_id']}'
				AND e.\"exp_name\" = '{$_SESSION[$currPageNameUpper]['exp_name']}'
				AND sp.\"experiment_id\" = e.\"experiment_id\"
				AND sp.\"label_id\" = '{$_SESSION[$currPageNameUpper]['sampling_point_label_id']}'
				AND sr.\"sampling_point_id\" = sp.sampling_point_id
				AND sr.\"replicate\" = '{$_SESSION[$currPageNameUpper]['biological_replicate']}'";
	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	
		$errMsgs[] = "Error retrieving Sampling Replicate. " . $sth->getMessage();	
		return 0;
	}
	
	if ($sth->numrows() > 0)
	{
		$row = $sth->fetchRow();
		$_SESSION[$currPageNameUpper]['sampling_replicate_id'] = $row[0];
		//print "Set sampling_replicate_id = {$_SESSION[$currPageNameUpper]['sampling_replicate_id']} ($n)";
	}
	else
	{	$errMsgs[] = "Sampling Replicate not found in the database for the information provided";	}
	
	if ($errMsgs)
	{	return 0;	}
		
	$retVal = setCpdsNotFound($_SESSION['ADD_CPD_OR_DER']['NOTFOUND'], $errMsgs);
	if ($errMsgs)
	{	return 0;	}

	if ($_SESSION['ADD_CPD_OR_DER']['NOTFOUND'][0] == 'ALL COMPOUNDS DONE')
	{
		//redirect
    	print "<meta http-equiv=\"refresh\" content=\"0; URL=intro.php?stepno=1\">";
	}
	else
	{
		//redirect
		print "<meta http-equiv=\"refresh\" content=\"0; URL=addCpdOrDer.php\">";
	}
	exit();
}
?>
