<?php
/**
* This script is a Front-End for adding Metadata for Experiment
* directly to the DOME database.
* Date: 31 Mar 2005
* @author Aejaaz Kamal <aekamal@vt.edu>
*/

include_once("common.php");

$currPageName = 'ADD_EXPERIMENT';
?>

<?php
$title = "DOME : Add Experiment";
include("header.php");
if (strtoupper($_SESSION['role']) != 'MANAGER')
{	printError("You are not authorized to view this page. Your role level should be 'manager' to view this page.");	exit();}
?>

<?php
if ($_REQUEST['submit'])
{	processForm();	}
if ($errMsgs)
{  
	foreach ($errMsgs as $msg)
	{	printError($msg);	}
}

fillForm();
include_once("footer.php");
?>

<?php
//Functions
function fillForm()
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	$self = $_SERVER[PHP_SELF];
	print "<form name=\"init_form\" method=\"post\" action=\"$self\">\n";
	
	//Print Experiment Sets. 
	$sql = "SELECT \"exp_set_id\", \"exp_set_name\"  FROM \"experiment_set\" ORDER BY \"exp_set_name\"";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Experiment Sets. " . $sth->getMessage());	return;	}
	print "<p><label>Experiment Set: </label>";
	print "<select name=\"experiment_set_id\">";
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['experiment_set_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
	
	
	//Print Experiment. 
	print "<p><label>Experiment: </label>";
	print "<input type=\"text\" name=\"exp_name\" size=\"75\" value=\"{$_SESSION[$currPageNameUpper]['exp_name']}\"/>";
	print "<input type=\"submit\" name=\"submit\" value=\"Add Experiment\" />";
	print "\n</form>";
}

/**============================
* Request Functions
*============================*/
function processForm()
{
	global $errMsgs, $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	$_SESSION[$currPageNameUpper]['experiment_set_id'] = $_REQUEST['experiment_set_id'];
	$_SESSION[$currPageNameUpper]['exp_name'] = $_REQUEST['exp_name'];

	if (!trim($_SESSION[$currPageNameUpper]['exp_name']))
	{
		$errMsgs[] = "Please enter name of the experiment.";	
		return 0;			
	}
	
	if ($_SESSION[$currPageNameUpper]['experiment_set_id'] == 0)
	{	$errMsgs[] = "Please Select an Experiment Set.";	}
	
	if ($errMsgs)
	{	return 0;	}
	
	insertExperiment($_SESSION[$currPageNameUpper]['experiment_set_id'], $_SESSION[$currPageNameUpper]['exp_name']);
	
	if ($errMsgs)
	{	return 0;	}
	return 1;
}


function insertExperiment($experiment_set_id, $exp_name)
{
	global $errMsgs, $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	$exp_name = $_SESSION['dbh']->quote($exp_name);
	
	//Check in Experiment
	$sql = "SELECT \"exp_set_id\", \"exp_name\" FROM \"experiment\" 
		WHERE UPPER(\"exp_name\") = " . strtoupper($exp_name) .
		" AND \"exp_set_id\" = $experiment_set_id";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	
		$errMsgs[] = "Error retrieving Experiments. " . $sth->getMessage();	
		return 0;	
	}

	if ($sth->numRows() == 0)	
	{
		//Begin Tranx
		$sth = $_SESSION['dbh']->query("BEGIN TRANSACTION");
		if ( DB::isError($sth) ) 
		{	
			$errMsgs[] = "Error starting transaction. " . $sth->getMessage();	
			return 0;	
		}
		//Check in label
		$sql = "SELECT \"label_id\", \"name\" FROM \"label\" 
			WHERE UPPER(\"name\") = " . strtoupper($exp_name) . 
			" AND kw_table_id = {$_SESSION['TableID']['EXPERIMENT']}";
		$sth = $_SESSION['dbh']->query($sql);
		if ( DB::isError($sth) ) 
		{	
			$errMsgs[] = "Error retrieving labels. " . $sth->getMessage();	
			$_SESSION['dbh']->query("ROLLBACK");
			return 0;	
		}
		if ($sth->numRows() > 0)
		{
			$row = $sth->fetchRow();
			$label_id = $row[0];
		}
		else
		{
			//Get next label_id
			$sth = $_SESSION['dbh']->query("SELECT nextval('label_id_seq')");
			if ( DB::isError($sth) ) 
			{	
				$errMsgs[] = "Error getting nextval from 'label_id_seq'. " . $sth->getMessage();	
				$_SESSION['dbh']->query("ROLLBACK");
				return 0;	
			}
			$row = $sth->fetchRow();
			$label_id = $row[0];
			
			//Insert into label
			$sql = "INSERT into \"label\" (\"label_id\", \"kw_table_id\", \"name\") 
				values ($label_id, {$_SESSION['TableID']['EXPERIMENT']}, $exp_name)";
			$sth = $_SESSION['dbh']->query($sql);
			if ( DB::isError($sth) ) 
			{	
				$errMsgs[] = "Error inserting into 'label'. " . $sth->getMessage();	
				$_SESSION['dbh']->query("ROLLBACK");
				return 0;	
			}
			
		}
		
		//Insert in Experiment		
		$sql = "INSERT INTO experiment (\"experiment_id\", \"exp_set_id\", 
			\"exp_name\", \"label_id\") values 
			(nextval('experiment_id_seq'), $experiment_set_id, 
			$exp_name, $label_id)";
		$sth = $_SESSION['dbh']->query($sql);
		//print $sql;
		if ( DB::isError($sth) ) 
		{	
			$errMsgs[] = "Error inserting Experiment $exp_name. " . $sth->getMessage();	
			$_SESSION['dbh']->query("ROLLBACK");
			return 0;	
		}
		$_SESSION['dbh']->query("COMMIT");
		printMessage("Experiment $exp_name successfully inserted in the database");
		//unset($_SESSION[$currPageNameUpper]);
	}
	else
	{	
		$errMsgs[] = "Experiment named $exp_name already exists in the database.";
	}

	if ($errMsgs)
	{	return 0;	}
	return 1;
}

?>
