<?php
/**
* This script is a Front-End for adding Metadata for Sampling
* Point directly to the DOME database.
* Date: 31 Mar 2005
* @author Aejaaz Kamal <aekamal@vt.edu>
*/

include_once("common.php");
$currPageName = 'ADD_SAMPLING_POINT';
?>

<?php
$title = "DOME : Add Sampling Point";
include("header.php");
if (strtoupper($_SESSION['role']) != 'MANAGER')
{	printError("You are not authorized to view this page. Your role level should be 'manager' to view this page.");	exit();}
?>

<?php
if ($_REQUEST['submit'])
{	processForm();	}
if ($errMsgs)
{  
	foreach ($errMsgs as $msg)
	{	printError($msg);	}
}

fillForm();

include_once("footer.php");
?>

<?php
//Functions
function fillForm()
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	$self = $_SERVER[PHP_SELF];
	print "<form name=\"init_form\" method=\"post\" action=\"$self\">\n";
	
	//Print Experiment Sets. 
	$sql = "SELECT \"exp_set_id\", \"exp_set_name\"  FROM \"experiment_set\" ORDER BY \"exp_set_name\"";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Experiment Sets. " . $sth->getMessage());	return;	}
	print "<p><label>Experiment Set *: </label>";
	print "<select name=\"experiment_set_id\">";
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['experiment_set_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
	
	//Print Experiments. 
	$sql = "SELECT DISTINCT \"exp_name\", UPPER(\"exp_name\") FROM \"experiment\" ORDER BY UPPER(\"exp_name\")";
    $sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Experiments. " . $sth->getMessage());	return;	}
	print "<p><label>Experiment *: </label>";
	print "<select name=\"exp_name\">";
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['exp_name'] == $row[0])
		{	print " selected";	}
		print ">$row[0]</option>";
	}
	print "\n</select></p>";
	

	print "\n<table><tr><td>";
	//Print Sampling Points. 
	if (!isset($_SESSION[$currPageNameUpper]['sampling_point_labels']))
	{	$_SESSION[$currPageNameUpper]['sampling_point_labels'] = array();	}
	$sql = "SELECT \"label_id\", \"name\", \"order\" FROM \"label\" 
		WHERE kw_table_id = '{$_SESSION['TableID']['SAMPLING_POINT']}'
		ORDER BY \"order\"";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Sampling Points. " . $sth->getMessage());	return;	}
	print "<p><label>Sampling Point: </label>";
	print "<select name=\"sampling_point_labels[]\" size=\"10\" multiple>";
	print "\n\t<option value=\"0\">--None--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option";
		if (in_array($row[1], $_SESSION[$currPageNameUpper]['sampling_point_labels']))
		{	print " selected";	}
		print " value=\"$row[1]\">$row[1] ($row[2])</option>";
	}
	print "\n</select>";
	print "</td><td>";
	print "&nbsp;&nbsp;&nbsp;OR&nbsp;&nbsp;&nbsp;";
	print "</td><td>";
	//Print New Sampling Point Fieldset
	print "<fieldset class=\"add-new-sampling-point\"><legend> Add New </legend>\n";
	//print Sampling Point Text Box
	print "<p><label>New Sampling Point: </label>";
	print "<input type=\"text\" name=\"sp_name\" size=\"4\" value=\"{$_SESSION[$currPageNameUpper]['sp_name']}\"/>";
	//print the unit drop-down
	$unitsArray = array('0'=>'--Select One--', 'min'=>'Minutes', 'hr'=>'Hours', 
		'day'=>'day', 'wk'=>'Week', 'custom'=>'Custom Order');
	print "\n<select name=\"sp_name_units\">";
	foreach ($unitsArray as $key=>$value)
	{
		print "\n\t<option value=\"$key\"";
		if ($_SESSION[$currPageNameUpper]['sp_name_units'] == $key)
		{	print " selected";	}
		print ">$value</option>";
	}
	print "\n</select>";
	print "</p>";
	
	//Print Order. 
	print "<p><label>Order : </label>";
	print "<input type=\"text\" name=\"order\" size=\"3\" value=\"{$_SESSION[$currPageNameUpper]['order']}\"/>";
	print "</p>";
	print "</fieldset></legend>";
	print "</td></tr></table>";	
	print "\n<br /><input type=\"submit\" name=\"submit\" value=\"Add Sampling Point\" />";
	print "\n</form>";
	
	print "<p><a href=\"./addMetaDataMenu.php\">< Back to Add Metadata Menu</a></p>";
}

/**============================
* Request Functions
*============================*/

function processForm()
{
	global $errMsgs, $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
		
	$_SESSION[$currPageNameUpper]['experiment_set_id'] = $_REQUEST['experiment_set_id'];
	$_SESSION[$currPageNameUpper]['exp_name'] = $_REQUEST['exp_name'];
	$_SESSION[$currPageNameUpper]['sampling_point_labels'] = $_REQUEST['sampling_point_labels'];
	$_SESSION[$currPageNameUpper]['sp_name'] = trim($_REQUEST['sp_name']);
	$_SESSION[$currPageNameUpper]['sp_name_units'] = $_REQUEST['sp_name_units'];
	$_SESSION[$currPageNameUpper]['order'] = trim($_REQUEST['order']);
	
	if (!$_SESSION[$currPageNameUpper]['experiment_set_id'])
	{	$errMsgs[] = "Please Select an Experiment Set.";	}
	if (!$_SESSION[$currPageNameUpper]['exp_name'])
	{	$errMsgs[] = "Please Select an Experiment.";	}

	//If sampling point not specified in either of the two ways.
	if (strlen($_SESSION[$currPageNameUpper]['sp_name']) > 0)
	{	
		if (!$_SESSION[$currPageNameUpper]['sp_name_units'])
		{	$errMsgs[] = "Please Select the units for the new sampling point.";	}
		
		if ($_SESSION[$currPageNameUpper]['sp_name_units'] == 'custom')
		{
			if (strlen($_SESSION[$currPageNameUpper]['order']) == 0)
			{		$errMsgs[] = "Please enter order for the sampling point.";		}
			elseif (!is_numeric($_SESSION[$currPageNameUpper]['order']))
			{		$errMsgs[] = "Please enter a numeric value for order of the sampling point.";		}
		}
		else
		{
			if (!is_numeric($_SESSION[$currPageNameUpper]['sp_name']))
			{	$errMsgs[] = "Please select or enter a numeric value for the new sampling point.";	}
		}
	

		if ($errMsgs)
		{	return 0;	}
	
	
		$sampling_point_label = $_SESSION[$currPageNameUpper]['sp_name'];
		switch($_SESSION[$currPageNameUpper]['sp_name_units'])
		{
		case 'min':
			$sampling_point_label .= ' min';
			$order = $_SESSION[$currPageNameUpper]['sp_name'];
			break;
		case 'hr':
			$sampling_point_label .= ' h';
			$order = $_SESSION[$currPageNameUpper]['sp_name'] * 60;
			break;
		case 'day':
			$sampling_point_label .= ' d';
			$order = $_SESSION[$currPageNameUpper]['sp_name'] * 60 * 24;
			break;
		case 'wk':
			$sampling_point_label .= ' wk';
			$order = $_SESSION[$currPageNameUpper]['sp_name'] * 60 * 24 * 7;
			break;
		case 'custom':
			$order = $_SESSION[$currPageNameUpper]['order'];
			break;
		}
		insertSamplingPoint($_SESSION[$currPageNameUpper]['experiment_set_id'], $_SESSION[$currPageNameUpper]['exp_name'], $sampling_point_label, $order);
	}
	elseif ($_SESSION[$currPageNameUpper]['sampling_point_labels'])
	{
		foreach ($_SESSION[$currPageNameUpper]['sampling_point_labels'] as $sampling_point_label)
		{
			insertSamplingPoint($_SESSION[$currPageNameUpper]['experiment_set_id'], $_SESSION[$currPageNameUpper]['exp_name'], $sampling_point_label, $_SESSION[$currPageNameUpper]['order']);
		}
	}
	else
	{	$errMsgs[] = "Please select or enter a sampling point.";	}
	

	if ($errMsgs)
	{	return 0;	}
	unset($_SESSION[$currPageNameUpper]['sp_name']);
	unset($_SESSION[$currPageNameUpper]['sp_name_units']);
	unset($_SESSION[$currPageNameUpper]['order']);
	return 1;
}


function insertSamplingPoint($experiment_set_id, $exp_name, $sp_name, $order)
{
	global $errMsgs, $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	$sp_name = $_SESSION['dbh']->quote($sp_name);
	$exp_name = $_SESSION['dbh']->quote($exp_name);
	//Check Valid Experiment
	$sql = "SELECT e.\"experiment_id\", es.\"exp_set_name\", e.\"exp_name\" FROM \"experiment_set\" es, \"experiment\" e
		WHERE es.\"exp_set_id\" = $experiment_set_id
		AND e.\"exp_set_id\" = es.\"exp_set_id\"
		AND UPPER(e.\"exp_name\") = " . strtoupper($exp_name);
	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	
		$errMsgs[] = "Error retrieving Experiments. " . $sth->getMessage();	
		return 0;	
	}
	
	if ($sth->numRows() == 0)	
	{
		$errMsgs[] = "No experiment or experiment_set found in the database for your selections. 
		<br />Please enter experiment before attempting to enter the sampling point";
		return 0;	
	}
	else
	{
		$row = $sth->fetchRow();
		$experiment_id = $row[0];
	}
	//Check in Sampling Point
	$sql = "SELECT sp.\"sampling_point_id\", es.\"exp_set_name\", 
		e.\"exp_name\" FROM \"sampling_point\" sp, 
		\"label\" l, \"experiment\" e, \"experiment_set\" es
		WHERE UPPER(l.\"name\") = " . strtoupper($sp_name) .
		" AND sp.\"label_id\" = l.\"label_id\"
		AND sp.\"experiment_id\" = e.\"experiment_id\"
		AND e.\"exp_set_id\" = es.\"exp_set_id\"
		AND sp.\"experiment_id\" = $experiment_id";
	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	
		$errMsgs[] = "Error retrieving sampling point. " . $sth->getMessage();	
		return 0;	
	}
	if ($sth->numRows() == 0)	
	{
		//Begin Tranx
		$sth = $_SESSION['dbh']->query("BEGIN TRANSACTION");
		if ( DB::isError($sth) ) 
		{	
			$errMsgs[] = "Error starting transaction. " . $sth->getMessage();	
			return 0;	
		}
		//Check in label
		$sql = "SELECT \"label_id\", \"name\" FROM \"label\" 
			WHERE UPPER(\"name\") = " . strtoupper($sp_name) . 
			" AND kw_table_id = {$_SESSION['TableID']['SAMPLING_POINT']}";
		$sth = $_SESSION['dbh']->query($sql);
		if ( DB::isError($sth) ) 
		{	
			$errMsgs[] = "Error retrieving labels. " . $sth->getMessage();	
			$_SESSION['dbh']->query("ROLLBACK");
			return 0;	
		}
		if ($sth->numRows() > 0)
		{
			$row = $sth->fetchRow();
			$label_id = $row[0];
		}
		else
		{
			//Get next label_id
			$sth = $_SESSION['dbh']->query("SELECT nextval('label_id_seq')");
			if ( DB::isError($sth) ) 
			{	
				$errMsgs[] = "Error getting nextval from 'label_id_seq'. " . $sth->getMessage();	
				$_SESSION['dbh']->query("ROLLBACK");
				return 0;	
			}
			$row = $sth->fetchRow();
			$label_id = $row[0];
			
			//Insert into label
			$sql = "INSERT into \"label\" (\"label_id\", \"kw_table_id\", \"name\", \"order\") 
				values ($label_id, {$_SESSION['TableID']['SAMPLING_POINT']}, $sp_name, $order)";
			$sth = $_SESSION['dbh']->query($sql);
			if ( DB::isError($sth) ) 
			{	
				$errMsgs[] = "Error inserting into 'label'. " . $sth->getMessage();	
				$_SESSION['dbh']->query("ROLLBACK");
				return 0;	
			}
			
		}
		
		//Insert in Sampling Point
		$sql = "INSERT INTO sampling_point (\"sampling_point_id\", \"experiment_id\", \"label_id\") 
			values (nextval('sampling_point_id_seq'), $experiment_id, $label_id)";
		$sth = $_SESSION['dbh']->query($sql);
		//print $sql;
		if ( DB::isError($sth) ) 
		{	
			$errMsgs[] = "Error inserting Sampling Point $sp_name. " . $sth->getMessage();	
			$_SESSION['dbh']->query("ROLLBACK");
			return 0;	
		}
		$_SESSION['dbh']->query("COMMIT");
		printMessage("Sampling Point $sp_name successfully inserted in the database");
		//unset($_SESSION[$currPageNameUpper]);
	}
	else
	{	
		$row = $sth->fetchRow();
		$errMsgs[] = "Sampling Point $sp_name for experiment_set '$row[1]' and experiment '$row[2]' already exists in the database.";
	}

	if ($errMsgs)
	{	return 0;	}
	return 1;
}

?>
