#!/usr/local/bin/perl

# X. Jing Li
# Nov. 10, 2005

#*** INPUT DATA: **********************************************************
# batchrun_readDataFromMetabolite_LCMS
# ($indir, $file_home, $exp_set_id, $tissue_type_label_id, $protocol_id, 
#  $extract_id, $file_header, $data_file_type, $raw_file_type, $reference,
#  $person_id)
# $indir = /array2/xingjing/metabolites/LCMS/
#     -- isoflavonoids/,
#     -- Media/,
#     -- saponins
# $file_home = /array2/xingjing/metabolites/upload/,
# $tissue_type_label_id = 23 -- root, 22 -- Media,
# $protocol_id = 203 -- LS/MS flavonoid
# $extract_id = 4 -- LC/MS flavonoid, 5 -- LC/MS saponins
# $file_header = LC1_ -- root, LC1_MEDIA -- media
# $data_file_type = ".txt",
# $raw_file_type = ".cdf",
# $reference = "Umbelliferone" -- LC/MS flavonoids, "" -- LC/MS saponins,
# $person_id = 40 -- Mohammed Farag, LC/MS flavonoid,
#              10 -- David Huhman, LC/MS saponins
#
##############################################################################

#== isoflavonoids -- root
$cmd = 'perl batchrun_readDataFromMetabolite_LCMS.pl "/array2/xingjing/metabolites/LCMS/isoflavonoid/" "/array1/dome-mt/" 23 203 4 "LC1_" ".txt" ".cdf" "Umbelliferone" 40 > /array2/xingjing/metabolites/upload/input_data_LCMS_flavonoid_root.log';
print "$cmd\n";
system $cmd;

#== isoflavonoids -- Media 
$cmd = 'perl batchrun_readDataFromMetabolite_LCMS.pl "/array2/xingjing/metabolites/LCMS/Media/" "/array1/dome-mt/" 22 203 4 "LC1_MEDIA_" ".txt" ".cdf" "Umbelliferone" 40 > /array2/xingjing/metabolites/upload/input_data_LCMS_flavonoid_Media.log';
print "$cmd\n";
system $cmd;

#*** get summary *******************************************************
#
# batchrun_getMeanStdevForMetaboliteBySPID.pl
# ($exp_set_id, $protocol_type_id, $tissue_type_label_id, $extract_id, $file_h)
# $exp_set_id = 
#    1 -- MJ,
#    2 -- YE,
#    3 -- UV,
#    4 -- MIX,
# $protocol_type_id:
#    203 -- Normalized by Internal Reference -- LC/MS flavonoids
#    202 -- Measured Intensity -- LC/MS saponins
# $tissue_type_label_id = 23 -- root, 22 -- media,
# $extract_id = 4 --  LC/MS flavonoids, 5 -- LC/MS saponins
#
###########################################################################

#== MeJa root
$cmd = 'perl batchrun_getMeanStdevForMetaboliteBySPIDs.pl 1 203 23 4 > /array2/xingjing/metabolites/upload/meanStdev_MeJa_root.log';
print "$cmd\n";
system $cmd;

#== Yeast root
$cmd = 'perl batchrun_getMeanStdevForMetaboliteBySPIDs.pl 2 203 23 4 > /array2/xingjing/metabolites/upload/meanStdev_Yeast_root.log';
print "$cmd\n";
system $cmd;

#== UV root
$cmd = 'perl batchrun_getMeanStdevForMetaboliteBySPIDs.pl 3 203 23 4 > /array2/xingjing/metabolites/upload/meanStdev_UV_root.log';
print "$cmd\n";
system $cmd;

#== Mixed root
$cmd = 'perl batchrun_getMeanStdevForMetaboliteBySPIDs.pl 4 203 23 4 > /array2/xingjing/metabolites/upload/meanStdev_Mixed_root.log';
print "$cmd\n";
system $cmd;

#== Yeast Media
$cmd = 'perl batchrun_getMeanStdevForMetaboliteBySPIDs.pl 2 203 22 4 > /array2/xingjing/metabolites/upload/meanStdev_Yeast_media.log';
print "$cmd\n";
system $cmd;

#*** comparison: **********************************************************

@elicited_ids = (1, 3, 5,     # MeJa elicitors
                 8, 9, 11,    # Yeast elicitors
                 13, 15, 17,  # UV elicitors
                 20, 22, 23   # Mixed elicitors
                 );
@control_ids = (2, 4, 6,      # MeJa control
                7, 10, 12,    # Yeast controls
                14,16, 18,    # UV controls
                21, 19, 24    # Mixed controls
                );
#== flavonoid -- root: extract_id = 4, protocol_type_id = 295, ttlid = 23
$ptid = 295;
for($i=0; $i<@elicited_ids; $i++) {
    $e_id = $elicited_ids[$i];
    $c_id = $control_ids[$i];
    $cmd = "perl batchrun_getRatioPvaluesBySPIDS.pl $e_id $c_id 23 23 3 $ptid > /array2/xingjing/metabolites/upload/ratio_E" . $e_id . "_C" . $c_id . "_PTID" . $ptid . ".log";
    print "$cmd\n";
    system $cmd;
}

#== flavonoid -- media: for Yeast_yeast only. e_id = 8, c_id = 7.
#== extract_id = 4, protocol_type_id = 295, ttlid = 22
$e_id = $elicited_ids[3];  
$c_id = $control_ids[3];
$cmd = "perl batchrun_getRatioPvaluesBySPIDS.pl $e_id $c_id 22 22 4 $ptid > /array2/xingjing/metabolites/upload/ratio_E" . $e_id . "_C" . $c_id . "_PTID" . $ptid . ".log";
print "$cmd\n";
system $cmd;

exit;
