<?php
/**
* This script is a Front-End for calculating Mean and
* standard deviation for data already added 
* in the DOME database.
* Date: 3 May 2005
* @author Aejaaz Kamal <aekamal@vt.edu>
*/


include_once("common.php");

$currPageName = 'CALCULATE_MEAN_AND_STD_DEV';
$currPageNameUpper = strtoupper($currPageName);
?>

<?php
if (isset($_REQUEST['technique_ids']))
{	$_SESSION[$currPageNameUpper]['technique_ids'] = strtoupper($_REQUEST['technique_ids']);	}

$title = "Calculate Mean And Std Dev";
if (isset($_REQUEST['title_suffix']))
{	$title .= " - {$_REQUEST['title_suffix']}";	}
include("header.php");
if (!(strtoupper($_SESSION['role']) == 'MANAGER' OR strtoupper($_SESSION['role']) == 'EDITOR'))
{	printError("You are not authorized to view this page. Your role level should be 'editor' or 'manager' to view this page.");	exit();}
?>

<?php

if ($_REQUEST['submit'] == 'Calculate')
{	processForm();	}

if ($errMsgs)
{  
	foreach ($errMsgs as $msg)
	{	printError($msg);	}
}

if ($_SESSION['Message'])
{  
	foreach ($_SESSION['Message'] as $msg)
	{	printMessage($msg);	}
	unset($_SESSION['Message']);
}
fillForm();
include_once("footer.php");
?>

<?php
//Functions
function fillForm()
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	$self = $_SERVER[PHP_SELF];
	print "<form name=\"init_form\" method=\"post\" action=\"$self\">\n";
	
	print '<p class="help-tips">This page assists in summarising the expression value data across different replicates. It calculates the mean value and standard deviation between replicates and stores this information into the database for further quering. Please provide information about which data point you want to calculate mean and standard deviation for. Please select a method and data only for that method will be used. You will have to choose methods one at a time if you want to calculate means and standard deviations for more than one method.</p>';
	printMetaDataSelections("", $_SESSION[$currPageNameUpper], false);
	
	//Print Normalization Method
	$sql = "SELECT \"protocol_type_id\", \"protocol_type_name\" FROM \"protocol_type\" pt, \"technique\" t 
			WHERE UPPER(pt.\"method\") = 'STATISTICAL' 
			AND pt.\"technique_id\" = t.\"technique_id\"
			AND t.\"technique_id\" IN ({$_SESSION[$currPageNameUpper]['technique_ids']})
			ORDER BY UPPER(\"protocol_type_name\")";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Protocol Type. " . $sth->getMessage());	return;	}
	print "\n<p><label>Normalization Method: </label>";
	print "<select name=\"protocol_type_id\">";
	
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['protocol_type_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
	
	printExtractMethod($_SESSION[$currPageNameUpper]['technique_ids']);
	
	print "<input type=\"submit\" name=\"submit\" value=\"Calculate\" />";
	print "\n</form>";
}


/**============================
* Request Functions
*============================*/
function processForm()
{
	global $errMsgs, $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	$_SESSION[$currPageNameUpper]['experiment_set_id'] = $_REQUEST['experiment_set_id'];
	$_SESSION[$currPageNameUpper]['exp_name'] = $_REQUEST['exp_name'];
	$_SESSION[$currPageNameUpper]['sampling_point_label_id'] = $_REQUEST['sampling_point_label_id'];
	$_SESSION[$currPageNameUpper]['tissue_type_label_id'] = $_REQUEST['tissue_type_label_id'];
	$_SESSION[$currPageNameUpper]['protocol_type_id'] = $_REQUEST['protocol_type_id'];
	$_SESSION[$currPageNameUpper]['extract_id'] = $_REQUEST['extract_id'];
		
	
	if (!$_SESSION[$currPageNameUpper]['experiment_set_id'])
	{	$errMsgs[] = "Please Select an Experiment Set.";	}
	
	if (!$_SESSION[$currPageNameUpper]['exp_name'])
	{	$errMsgs[] = "Please Select an Experiment.";	}
	
	if (!$_SESSION[$currPageNameUpper]['sampling_point_label_id'])
	{	$errMsgs[] = "Please Select a Sampling Point";	}
	
	if (!$_SESSION[$currPageNameUpper]['tissue_type_label_id'])
	{	$errMsgs[] = "Please Select a Tissue Type";	}
	
	if (!$_SESSION[$currPageNameUpper]['protocol_type_id'])
	{	$errMsgs[] = "Please Select a Normalization Method.";	}
	
	if (!$_SESSION[$currPageNameUpper]['extract_id'])
	{	$errMsgs[] = "Please Select an Extraction Method.";	}
	
	if ($errMsgs)
	{	return 0;	}
	
	//Find Sampling Point ID
	$sql = 	"SELECT sp.\"sampling_point_id\" FROM 
		\"sampling_point\" sp, \"experiment\" e
		WHERE e.\"exp_set_id\" = '{$_SESSION[$currPageNameUpper]['experiment_set_id']}'
		AND e.\"exp_name\" = '{$_SESSION[$currPageNameUpper]['exp_name']}'
		AND sp.\"experiment_id\" = e.\"experiment_id\"
		AND sp.\"label_id\" = '{$_SESSION[$currPageNameUpper]['sampling_point_label_id']}'";
			
	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	$errMsgs[] = "Error retrieving sampling_point_id. " . $sth->getMessage();	return 0;	}
	
	if ($sth->numrows() > 0)
	{
		$row = $sth->fetchRow();
		$_SESSION[$currPageNameUpper]['sampling_point_id'] = $row[0];
	}
	else
	{	$errMsgs[] = "Sampling Point not found in the database for the selections.";	}
	
	//Find extract_id and technique_type
	$sql = 	"SELECT t.\"technique_type\" FROM 
		\"extract\" e, \"protocol_type\" pt, \"technique\" t
		WHERE pt.\"protocol_type_id\" = '{$_SESSION[$currPageNameUpper]['protocol_type_id']}'
		AND e.\"extract_id\" = '{$_SESSION[$currPageNameUpper]['extract_id']}'
		AND pt.\"technique_id\" = e.\"technique_id\"
		AND t.\"technique_id\" = e.\"technique_id\"";
			
	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	$errMsgs[] = "Error retrieving technique type. " . $sth->getMessage();	return 0;	}
	
	if ($sth->numrows() > 0)
	{
		$row = $sth->fetchRow();
		$_SESSION[$currPageNameUpper]['technique_type'] = $row[0];
	}
	else
	{	$errMsgs[] = "Method not found in the database for the protocol selected.";	}

	if ($errMsgs)
	{	return 0;	}
		
	switch (strtoupper(trim($_SESSION[$currPageNameUpper]['technique_type'])))
	{
		//==Hard Coding Here.
		case 'MRNA':
			$perlScriptFile = "getSummaryForMicroarray.pl";
			break;
		case 'METABOLITE':
			$perlScriptFile = "getSummaryForMetabolite.pl";
			break;
		case 'PROTEIN':
			$perlScriptFile = "getSummaryForProteomics.pl";
			break;
	}
	
	$sampling_point_id = $_SESSION[$currPageNameUpper]['sampling_point_id'];
	$protocol_type_id = $_SESSION[$currPageNameUpper]['protocol_type_id'];
	$tissue_type_label_id = $_SESSION[$currPageNameUpper]['tissue_type_label_id'];
	$extract_id = $_SESSION[$currPageNameUpper]['extract_id'];
	
	$com = "{$_SESSION['PERL_DIR']}perl $perlScriptFile $sampling_point_id $protocol_type_id $extract_id $tissue_type_label_id 2>&1;";
	
	$retString = exec($com, $retLines, $retValue);
	
	if ($retLines)
  	{
		foreach ($retLines as $retLine)
		{	$msg .= "$retLine <br />";	}
  	}
	if ($retValue == 0)	//success
  	{	
	  	if (!$msg)
	  	{	$msg = "system command to '$perlScriptFile' returned a successful value.";	}
	  	
	  	$_SESSION['Message'][] = $msg;
		//redirect
		print "<meta http-equiv=\"refresh\" content=\"0; URL=intro.php?stepno=1\">";
		exit();
	}
	else
	{	
	  	if (!$msg)
	  	{	$msg = "system command to '$perlScriptFile' failed. <br />Command was \"$com\"\nReturn Value = $retValue";	}
	  	$errMsgs[] = $msg;
	}
	
  	if ($errMsgs)
	{	return 0;	}

	return 1;
}

?>
