#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use Math::Trig;
use POSIX;
use strict "vars";

#===============================================================================
# This script reads the data files from DOME front-end, copy the .png file from
# BromeHome to outdir, read .dat file from BromeHome and recreate .dat file in 
# outdir and call Brome with the corresponding data files.
# 
# Author: X. Jing Li
# Date: Dec. 14, 2004
# Reversions: 
#  Jan. 21, 2005: add options for actual data only, normalized by mean only 
#    or normalized by std_dev only. also add ARGV parameter for dynanmically
#    created brome configueration file name.
#
#    $normalize_code == 1: actual data only
#
#    $normalize_code == 2: normalized by mean only;
#      v(i,j) = v(i,j)/Vj where Vj = average(v(1,j), v(2,j), ..., v(n,j));
#      j = 1, 2, ..., m;
#      n = number of columns, m = number of rows;
# 
#    $normalize_code == 3: normalized by standard diviation only;
#      v(i,j) = (v(i,j)-Vmj)/Vj where 
#      Vmj = average(v(1,j), v(2,j), ..., v(n,j));
#      Vj = std_dev(v(1,j), v(2,j), ..., v(n,j));
#      j = 1, 2, ..., m;
#      n = number of columns, m = number of rows;
#===============================================================================

my ($input_file, $output_dir, $map_id, $rc_name, $normalize_code,
    $brome_home, $out_file, @contents, $line, @items, @titles, $option, $ok,
    $r_type1_array, @type1_array, $r_type1_names, @type1_names, $r_type1_idens,
    @type1_idens, $r_type2_array, @type2_array, $r_type2_names, @type2_names, 
    $r_type2_idens, @type2_idens, $i, $j, $type, $name, $identifier, $ref_value,
    $ref_new_files, $title, $error_code, 
    $orig_dir, $png_file, $dat_file, $new_png, 
    $new_dat, $num_images, $brome_cmd, $script_str, $script_file, 
    $r_temp_files, @temp_files, $file);

#($input_file, $output_dir, $map_id) = @ARGV;
($input_file, $output_dir, $map_id, $rc_name, $normalize_code) = @ARGV;
#print "input parameters: $input_file, $output_dir, $map_id, $rc_name, $normalize_code\n";

$brome_home = $ENV{'BROMEDIR'};
#print "brome_home = $brome_home\n";
$brome_cmd = "Brome ";
#$brome_cmd = "/export/home/xingjing/development/Brome/Brome ";

#== 1. open the input file and reformat it to Brome data files:
open (IN, "<$input_file") or die "can not open input file $input_file\n";
@contents = <IN>;
close IN;

#=1.1 read the first line and decide how many files for each type:
$line = $contents[0];
$line =~ s/\n|\r//g;        #delete the new line char
@items = split /\t/, $line;
shift @items;               # delete TYPE
shift @items;               # delete NAME
shift @items;               # delete IDENTIFIER
@titles = @items;
#print "titles = @titles\n";

#= check header to decide which option for coloring. "r" for ratio 
#= or "a" for absolute_value:
if ($line =~ /SAMPLE_1/ && $line =~ /REFER_1/) {
    $option = "r";
} elsif (!($line =~ /SAMPLE_1/) || !($line =~ /REFER_1/)) {
    $option = "a";
}

#= seperate the data, save data into array of arrays for each type( 1 and 2), 
#= names and identifiers are saved in arrays which have the same order as the 
#= array of arrays.
$r_type1_array;
@type1_array = @$r_type1_array;
$r_type1_names;
@type1_names = @$r_type1_names;
$r_type1_idens;
@type1_idens = @$r_type1_idens;
$r_type2_array;
@type2_array = @$r_type2_array;
$r_type2_names;
@type2_names = @$r_type2_names;
$r_type2_idens;
@type2_idens = @$r_type2_idens;
for ($i=1; $i<@contents; $i++) {
    $line = $contents[$i];
    #print $line;
    $line =~ s/\n|\r//g;
    @items = split /\t/, $line;
    $type = shift @items;
    $name = shift @items;
    $identifier = shift @items;
    #print "type = $type, name = $name, iden = $identifier\n";
    
    if ($type eq "TYPEI") {
        push @type1_names, $name;
        push @type1_idens, $identifier;
        $ref_value = createArrayRef(@items);
        push @type1_array, $ref_value;
    } elsif ($type eq "TYPEII") {
        push @type2_names, $name;
        push @type2_idens, $identifier;
        $ref_value = createArrayRef(@items);
        push @type2_array, $ref_value;
    }
}

#print "type2_names = @type2_names\ntype2_idens = @type2_idens\n";

#= print data into different files:
$r_temp_files = [];
@temp_files = @$r_temp_files;
if (@type1_idens > 0) {
    #= type_1 is not empty, print it to file(s):
    $type = ".type1";
    ($ref_new_files, $error_code) = createNewTypeDatFile($output_dir, \@titles, \@type1_idens, \@type1_array, $normalize_code, $type);
    push @temp_files, @$ref_new_files;
} 
if (@type2_idens > 0) {
    #= type_2 is not empty, print it to file(s):
    $type = ".type2";
    $r_type2_idens = \@type2_idens;
    ($ref_new_files, $error_code) = createNewTypeDatFile($output_dir, \@titles, $r_type2_idens, \@type2_array, $normalize_code, $type);
    push @temp_files, @$ref_new_files;
}        

#== 2. copy map_id.png from BromeHome to output_dir and recreate map_id.dat:
$orig_dir = $brome_home . "share/Brome/maps/";
$png_file = $orig_dir . $map_id . ".png";
$dat_file = $orig_dir . $map_id . ".dat";
$new_png = $output_dir . $map_id . ".png";
system "cp $png_file $new_png";
$new_dat = createNewMapDatFile($output_dir, $dat_file, \@type2_names, \@type2_idens, $map_id);
push @temp_files, $new_png, $new_dat;
#print "new_dat = $new_dat\n";

#== call Brome with the data files:
if ($option eq "r") {
    $num_images = @titles /2;
    if ($num_images == 1) {
        $out_file = $output_dir . $map_id . "_out_1.png";
        if (@type1_array > 0) {
            $brome_cmd .= "--t1s " . $output_dir . "SAMPLE_1.type1 "
                . "--t1r " . $output_dir . "REFER_1.type1 "
                . "--t1o r ";
        }
        if (@type2_array > 0) {
            $brome_cmd .= "--t2s " . $output_dir . "SAMPLE_1.type2 "
                . "--t2r " . $output_dir . "REFER_1.type2 "
                . "--t2o r ";
        }
        #$brome_cmd .= "--rc " . $brome_home . "/etc/brome.conf "
        $brome_cmd .= "--rc " . $rc_name ." "
            . "-i $output_dir$map_id -o  $out_file -f PNG";
    } else {
        #= this has be use the -s option and script file
        $script_str = "";

        for ($i=1; $i<=$num_images; $i++) {
            $out_file = $output_dir . $map_id . "_out_" . $i . ".png";
            if (@type1_array > 0) {
                $script_str .= "--t1s " . $output_dir . "SAMPLE_" . $i . ".type1\n--t1r " . $output_dir . "REFER_" . $i . ".type1\n--t1o r\n";
            }
            if (@type2_array > 0) {
                $script_str .= "--t2s " . $output_dir . "SAMPLE_" . $i . ".type2\n--t2r " . $output_dir . "REFER_" . $i . ".type2\n--t2o r\n";
            }
            #$script_str .= "--rc " . $brome_home . "/etc/brome.conf\n"
            $brome_cmd .= "--rc " . $rc_name ."\n"
                . "-i $output_dir$map_id\n-o $out_file\n-f PNG\n///\n";
        }
        $script_file = $output_dir . "script.txt";
        open (SCRIPT, ">$script_file") or die "can't open script file $script_file\n";
        print SCRIPT $script_str;
        close SCRIPT;
        push @temp_files, $script_file;
        $brome_cmd .= "-s $script_file";
    }
    #print "$brome_cmd\n";
    $ok = system "$brome_cmd";
} elsif ($option eq "a") {
    if (@titles == 1) {
        $out_file = $output_dir . $map_id . "_out_1.png";
        $title = $titles[0];
        if (@type1_array > 0) {
            $brome_cmd .= "--t1s " . $output_dir . $title . ".type1 "
                . "--t1o a ";
        }
        if (@type2_array > 0) {
            $brome_cmd .= "--t2s " . $output_dir . $title . ".type2 "
                . "--t2o a ";
        }
        #$brome_cmd .= "--rc " . $brome_home . "/etc/brome.conf "
        $brome_cmd .= "--rc " . $rc_name . " "
            . "-i $output_dir$map_id -o  $out_file -f PNG";
        #print "brome_cmd = $brome_cmd\n";
        $ok = system "$brome_cmd";
    } else {
        $script_str = "";
        for ($i=1; $i<=@titles; $i++) {
            $out_file = $output_dir . $map_id . "_out_" . $i . ".png";
            $title = $titles[$i-1];
            if (@type1_array > 0) {
                $script_str .= "--t1s " . $output_dir . $title . ".type1\n--t1o a\n";
            }
            if (@type2_array > 0) {
                $script_str .= "--t2s " . $output_dir . $title . ".type2\n--t2o a\n";
            }
            #$script_str .= "--rc " . $brome_home . "/etc/brome.conf\n"
            $script_str .= "--rc " . $rc_name . "\n"
                . "-i $output_dir$map_id\n-o $out_file\n-f PNG\n///\n";
        }
        $script_file = $output_dir . "script.txt";
        open (SCRIPT, ">$script_file") or die "can't open script file $script_file\n";
        print SCRIPT $script_str;
        close SCRIPT;
        push @temp_files, $script_file;
        $brome_cmd .= "-s $script_file";
        print "$brome_cmd\n";
        $ok = system "$brome_cmd";
    }
}
        
print "ok = $ok\n";

#== delete all temp_files:
#foreach $file (@temp_files) {
#    unlink $file or warn "can not unlink $file\n";
    #print "$file will be deleted\n";
#}

exit($ok);


###########################################################################
# subroutine: createArrayRef(@v):
# Create a reference for a known array.
#
# parameters in: @v --- the initial array
# parameter out: $ref_array --- reference of the array.
#
###########################################################################
sub createArrayRef {
    my (@v) = @_;
    my ($ref_array, @array);

    $ref_array = [];
    @array = @$ref_array;
    push @array, @v;
    return \@array;
}

###########################################################################
# subroutine: createValueArray($v):
# This subroutine create a anynomouse array. 
#
# parameters in: $v --- the first value for the anynomous array
# parameter out: $ref_array --- the reference of the anynomous array
#
###########################################################################
sub createValueArray {
    my ($v) = @_;
    my ($ref_array, @array);

    $ref_array = [];
    @array = @$ref_array;
    push @array, $v;
    return \@array;
}

###########################################################################
# subroutine: updateArrayWithValue($rv1, $v2)
#
# parameters in: $rv1 --- the original arry reference
#                $v2 --- the new value
# parameter out: $ref_array --- reference of the new array 
#
###########################################################################
sub updateArrayWithValue {
    my ($rv1, $v2) = @_;
    my (@value1, $ref_array, @array);

    $ref_array = [];
    @array = @$ref_array;
    @value1 = @$rv1;
    push @array, @value1;
    push @array, $v2;
    return \@array;
}

###########################################################################
# subroutine:
#  createNewMapDatFile($old_dat, $r_type2_names, $r_type2_idens, $map_id):
# Create a new map_id.dat for DOME's Brome.
#
# parameters in: $old_dat --- the file name of the old .dat file,
#                $r_type2_names -- the reference of the @type2_names,
#                $r_type2_idens -- the reference of the @type2_idnetifiers,
#                $map_id -- the map_id,
# parameter out: $new_dat_file --- the name of recreated .dat file.
#
###########################################################################

sub createNewMapDatFile {
    my ($out_dir, $old_dat, $r_type2_names, $r_type2_idens, $map_id) = @_;
    my (@lines, $type2_names, @type2_idens, $r_names, %names, $i, $name, $id,
        $r_ids, $copy_of_ids, $str, $line, $shape, $x, $y, $w, $h, $id, $type,
        $tomap, $x1, $y1, $new_w, $new_h, $new_line, $total_width, @idens, 
        $iden, $num_items, $num_col);

    open (OLD, "<$old_dat") or die "can't open original .dat file $old_dat\n";
    @lines = <OLD>;
    close OLD;

    #== sort out the names and identifiers. count the number of identifiers for
    #== each name for reactangle devide.
    @type2_names = @$r_type2_names;
    @type2_idens = @$r_type2_idens;
    #print "type2_names = @type2_names\n";
    #print "type2_idens = @type2_idens\n";
    $r_names = {};
    %names = %$r_names;
    for($i=0; $i<@type2_names; $i++) {
        $name = $type2_names[$i];
        $id = $type2_idens[$i];
        if (!exists $names{$name}) {
            $r_ids = createValueArray($id);
            $names{$name} = $r_ids;
        } else {
            $r_ids = $names{$name};
            $r_ids = updateArrayWithValue($r_ids, $id);
            $names{$name} = $r_ids;
        }
    }

    #foreach $name (keys %names) {
    #    $r_ids = $names{$name};
    #    $copy_of_ids = $r_ids;
    #    @idens = @$copy_of_ids;
    #    print "name = $name, idens = @idens num_of_id = " . @idens . "\n";
    #}

    #== recreate file string:
    $str = "";
    foreach $line (@lines) {
        if (!($line =~ /\tTYPEII\t/)) {
            $str .= $line;
        } else {
            $line =~ s/\n|\r//g;
            ($shape, $x, $y, $w, $h, $id, $type, $tomap) = split /\t/, $line;
            if (!exists $names{$id}) {
                $str .= $line . "\n";
            } else {
                $r_ids = $names{$id};
                $copy_of_ids = $r_ids;
                @idens = @$copy_of_ids;
                if (@idens == 1) {
                    $iden = $idens[0];
                    $new_line = "$shape\t$x\t$y\t$w\t$h\t$iden\t$type\t$tomap\n";
                    $str .= $new_line;
                } elsif (@idens > 1 && @idens < 6){
                    $new_w = ceil($w / @idens);
                    $total_width = $new_w * @idens;
                    if ($total_width > $w + 1) {
                        $new_w--;
                        $total_width = $new_w * @idens;
                    }
                    if ($total_width >= $w) {
                        for($i=0; $i<@idens; $i++) {
                            $iden = $idens[$i];
                            $x1 = $x + $i * $new_w;
                            $total_width += $new_w;
                            $new_line = "$shape\t$x1\t$y\t$new_w\t$h\t$iden\t$type\t$tomap\n";
                            $str .= $new_line;
                        }
                    } else {
                        $total_width = 0;
                        for($i=0; $i<@idens-1; $i++) {
                            $iden = $idens[$i];
                            $x1 = $x + $i * $new_w;
                            $total_width += $new_w;
                            $new_line = "$shape\t$x1\t$y\t$new_w\t$h\t$iden\t$type\t$tomap\n";
                            $str .= $new_line;
                        }
                        $iden = $idens[@idens-1];
                        $x1 = $x + (@idens-1) * $new_w;
                        $new_w = $w - $total_width;
                        $new_line = "$shape\t$x1\t$y\t$new_w\t$h\t$iden\t$type\t$tomap\n";
                        $str .= $new_line;
                    }
                } elsif (@idens >= 6 ) {
                    $num_items = @idens;
                    if ($num_items > 10) {
                        $num_items = 10;
                    }
                    $num_col = ceil($num_items / 2);
                    $new_w = ceil($w / $num_col);
                    $new_h = ceil($h / 2);
                    if ($new_w * $num_col > ($w+1)) {
                        $new_w--;
                    }
                    if ( $num_items % 2 == 0) {
                        #= there are even number of idens. both row are full:
                        #- firsr row:
                        $total_width = 0;
                        for($i=0; $i<$num_col-1; $i++) {
                            $iden = $idens[$i];
                            $x1 = $x + $i * $new_w;
                            $total_width += $new_w;
                            $new_line = "$shape\t$x1\t$y\t$new_w\t$new_h\t$iden\t$type\t$tomap\n";
                            $str .= $new_line;
                        }
                        $i = $num_col - 1;
                        $iden = $idens[$i];
                        $x1 = $x + $i * $new_w;
                        $new_w = $w - $total_width;
                        $new_line = "$shape\t$x1\t$y\t$new_w\t$new_h\t$iden\t$type\t$tomap\n";
                        $str .= $new_line;
                        
                        #- second row:
                        $total_width = 0;
                        $y1 = $y + $new_h;
                        for($i=0; $i<$num_col-1; $i++) {
                            $iden = $idens[$num_col+$i];
                            $x1 = $x + $i * $new_w;
                            $total_width += $new_w;
                            $new_line = "$shape\t$x1\t$y1\t$new_w\t$new_h\t$iden\t$type\t$tomap\n";
                            $str .= $new_line;
                        }
                        $i = $num_col - 1;
                        $iden = $idens[$num_col+$i];
                        $x1 = $x + $i * $new_w;
                        $new_w = $w - $total_width;
                        $new_line = "$shape\t$x1\t$y1\t$new_w\t$new_h\t$iden\t$type\t$tomap\n";
                        $str .= $new_line;
                    } else {
                        #= there are odd number of idens. first row is full but
                        #= second row has one less iden.
                        #- firsr row:
                        $total_width = 0;
                        for($i=0; $i<$num_col-1; $i++) {
                            $iden = $idens[$i];
                            $x1 = $x + $i * $new_w;
                            $total_width += $new_w;
                            $new_line = "$shape\t$x1\t$y\t$new_w\t$new_h\t$iden\t$type\t$tomap\n";
                            $str .= $new_line;
                        }
                        $i = $num_col - 1;
                        $iden = $idens[$i];
                        $x1 = $x + $i * $new_w;
                        $new_w = $w - $total_width;
                        $new_line = "$shape\t$x1\t$y\t$new_w\t$new_h\t$iden\t$type\t$tomap\n";
                        $str .= $new_line;
                        
                        #- second row:
                        $total_width = 0;
                        $y1 = $y + $new_h;
                        for($i=0; $i<$num_col-2; $i++) {
                            $iden = $idens[$num_col+$i];
                            $x1 = $x + $i * $new_w;
                            $total_width += $new_w;
                            $new_line = "$shape\t$x1\t$y1\t$new_w\t$new_h\t$iden\t$type\t$tomap\n";
                            $str .= $new_line;
                        }
                        $i = $num_col - 2;
                        $iden = $idens[$num_col+$i];
                        $x1 = $x + $i * $new_w;
                        $new_w = $w - $total_width;
                        $new_line = "$shape\t$x1\t$y1\t$new_w\t$new_h\t$iden\t$type\t$tomap\n";
                        $str .= $new_line;
                    }
                } else {
                    #== Other conditions
                    next;
                }
            }
        }
    }
    
    $new_dat = $out_dir . $map_id . ".dat";
    open (OUT, ">$new_dat") or die "can't open new .dat file $new_dat\n";
    print OUT $str;
    close OUT;

    return $new_dat;
}

###########################################################################
# subroutine: createNewTypeDatFile
#             ($output_dir, $r_titles, $r_type_idens, $r_type_arrays, $code):
# Create a new source data files for Brome. Different code will have different
# data. 
#
# parameters in: $output_dir -- the prefix for each file,
#                $r_titles -- the reference of the @titles,
#                $r_type_idens -- the reference of the @type_idnetifiers,
#                $r_type_array,-- the original data values of this type,
#                $code -- the type of normalizetions required
# parameter out: $r_new_type_files --- the names of new data files.
#
###########################################################################
sub createNewTypeDatFile {
    my ($output_dir, $r_titles, $r_type_idens, $r_type_arrays, $code, $type) = @_;

    my (@type_idens, @titles, @array, $error_code, $r_new_files, @new_files,
        $i, $j, $title, $file_name, $identifier, $ref_value, $value,
        @comp_array, $sum, $v, $r_mean_array, @mean_array, $mean,
        $r_stddev_array, @stddev_array, $std_dev
        );

    @type_idens = @$r_type_idens;
    @titles = @$r_titles;
    @array = @$r_type_arrays;
    $error_code = 0;

    $r_new_files = [];
    @new_files = @$r_new_files;
    if($code == 1 || @titles == 1) {
        #= this is for no normalization. print the actual values to out files:
        foreach ($i=0; $i<@titles; $i++) {
            $title = $titles[$i];
            $file_name = $output_dir . $title . $type;
            open (OUT, ">$file_name") or die "Can't open output file $file_name\n";
            print OUT "Name\tValue\n";
            for($j=0; $j<@type_idens; $j++) {
                $identifier = $type_idens[$j];
                $ref_value = $array[$j];
                $value = $ref_value->[$i];
                print OUT "$identifier\t$value\n";
            }
            close OUT;
            push @new_files, $file_name;
        }
    } elsif ($code == 2) {
        #= this is for normalized by mean. i.e. I have to calculate the means
        #= for each component first then divide each value in that row by it.
        $r_mean_array = [];
        @mean_array = @$r_mean_array;
        for($j=0; $j<@type_idens; $j++) {
            $ref_value = $array[$j];
            @comp_array = @$ref_value;
            $sum = 0;
            foreach $v (@comp_array) {
                $sum += $v;
            }
            $mean_array[$j] = $sum/@comp_array;
        }
        foreach ($i=0; $i<@titles; $i++) {
            $title = $titles[$i];
            $file_name = $output_dir . $title . $type;
            open (OUT, ">$file_name") or die "Can't open output file $file_name\n";
            print OUT "Name\tValue\n";
            for($j=0; $j<@type_idens; $j++) {
                $identifier = $type_idens[$j];
                $ref_value = $array[$j];
                if ($mean_array[$j] == 0) {
                    $error_code = 1;
                    next;
                }
                $value = $ref_value->[$i] / $mean_array[$j];
                print OUT "$identifier\t$value\n";
            }
            close OUT;
            push @new_files, $file_name;
        }
    } elsif ($code == 3) {
        #= this is normalized by stardard_deviation.
        $r_mean_array = [];
        @mean_array = @$r_mean_array;
        $r_stddev_array = [];
        @stddev_array = @$r_stddev_array;
        for($j=0; $j<@type_idens; $j++) {
            $ref_value = $array[$j];
            ($mean, $std_dev) = cal_mean_stddev($ref_value);
            $mean_array[$j] = $mean;
            $stddev_array[$j] = $std_dev;
        }
        foreach ($i=0; $i<@titles; $i++) {
            $title = $titles[$i];
            $file_name = $output_dir . $title . $type;
            open (OUT, ">$file_name") or die "Can't open output file $file_name\n";
            print OUT "Name\tValue\n";
            for($j=0; $j<@type_idens; $j++) {
                $identifier = $type_idens[$j];
                $ref_value = $array[$j];
                if ($std_dev == 0){
                    $error_code = 2;
                    next;
                }
                $value = ($ref_value->[$i]-$mean_array[$j])/$stddev_array[$j];
                print OUT "$identifier\t$value\n";
            }
            close OUT;
            push @new_files, $file_name;
        }
    }
    
    return \@new_files, $error_code;
}

###########################################################################
# subroutine: cal_mean_stddev($ref_array):
# This subroutine creates the mean and stardard_deviation of an array. 
#
# parameters in: $ref_array --- the reference of the array;
# parameter out: $mean --- the mean of the array,
#                $std_dev -- the standard_deviation of the array.
#
###########################################################################
sub cal_mean_stddev {
    my ($ref_array) = @_;
    my (@array, $sum, $v, $mean, $std_dev);

    @array = @$ref_array;
    $sum = 0;
    foreach $v (@array) {
        $sum += $v;
    }
    $mean = $sum/@array;

    $sum = 0;
    foreach $v (@array) {
        $sum += ($v - $mean)*($v - $mean);
    }
    $std_dev = sqrt($sum/(@array-1));

    return $mean, $std_dev;
}
