#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use Math::Trig;
use POSIX;
use strict "vars";
use GD;

#===============================================================================
# This script is copied from script callBromeForDOME.pl. It reads the data files
# from DOME front-end, copy the .png file from BromeHome to outdir, read .dat 
# file from BromeHome and recreate .dat file in outdir and call perl_Brome.pl
# with the corresponding data files.
# 
# Author: X. Jing Li
# Date: Dec. 4, 2006
# Reversions: 
#  $normalize_code == 1: actual data only
#  $normalize_code == 2: normalized by mean only;
#    v(i,j) = v(i,j)/Vj where Vj = average(v(1,j), v(2,j), ..., v(n,j));
#    j = 1, 2, ..., m;
#    n = number of columns, m = number of rows;
#  $normalize_code == 3: normalized by standard diviation only;
#    v(i,j) = (v(i,j)-Vmj)/Vj where 
#    Vmj = average(v(1,j), v(2,j), ..., v(n,j));
#    Vj = std_dev(v(1,j), v(2,j), ..., v(n,j));
#    j = 1, 2, ..., m;
#    n = number of columns, m = number of rows;
#===============================================================================

my ($input_file, $output_dir, $map_id, $rc_name, $normalize_code, $anim, $delay,
    $database, $username, $password, $bnet_DB, $specieDB, $dbh, $sth, $rc, $sql,
    $specie_name, $specie_id, $shape, $coord, $search_name, $i, 
    $brome_home, $temp, @lines, $line, @items, @titles, $ok, %printed,
    @array, @names, @idens, @types, $type, $name, $identifier, $ref_value, $ref_new_files, $title, $error_code, $orig_dir, $png_file, $dat_file, $new_png, $new_dat, $num_images, $brome_cmd, $r_temp_files, @temp_files, $file_type, $index, @new_data_files, $output_file, $file, @gif_files, $animatedGif, $image, $frame, $anim_file);

($input_file, $output_dir, $map_id, $rc_name, $normalize_code, $anim, $delay) = @ARGV;
#print "input parameters: input_file = $input_file, output_dir = $output_dir, map_id = $map_id, rc_name = $rc_name, normalize_code = $normalize_code, anim = $anim, delay = $delay\n";

#== open temp file for data checking:
#open (TM, ">./tmp-plots/callPerlBrome.tmp"); 

#== 1. get the dir name where brome image files are:
open (PAR, "paramSettings.php") || die "Can not open parameter file.$!";
while ($line = <PAR>) {
    if ($line =~ /^\s+\$\_SESSION\[\'BROME_HOME\'\]/) {
        chomp $line;
        ($temp, $brome_home) = split /\s\=\s/, $line;
        $brome_home =~ s/\'|\;|\s$//g;
#    } elsif ($line =~ /^\s+\$\_SESSION\[\'SPECIES_NAME\'\]/) {
#        chomp $line;
#        ($temp, $specie_name) = split /\s\=\s/, $line;
#        $specie_name =~ s/\'|\;//g;
#        #print "specie_name = $specie_name\n";
    }
}
close PAR;

#== 2. connect the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#$sql = "select record_id from allnames where kw_table_id = 15 and upper(aname) = upper(\'$specie_name\')";
#$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
#$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#$specie_id = $sth->fetchrow();
#$specie_id = 1;

#== 3. open the input file and reformat it to Brome data files:
open (IN, "<$input_file") or die "can not open input file $input_file\n";
@lines = <IN>;
close IN;

#= 3.1 read the first line and decide how many files for each type:
$line = shift @lines;
$line =~ s/\n|\r//g;        #delete the new line char
@items = split /\t/, $line;
shift @items;               # delete TYPE
shift @items;               # delete NAME
shift @items;               # delete Identifier
@titles = @items;

#= seperate the data, save data into array of arrays for each type, 
#= names and identifiers are saved in arrays which have the same order as the 
#= array of arrays.
foreach $line (@lines) {
    chomp $line;
    #print $line;
    $line =~ s/\n|\r//g;
    @items = split /\t/, $line;
    $type = shift @items;
    $name = shift @items;
    $identifier = shift @items;
    #$name = shift @items;
    push @names, $name;
    push @types, $type;
    push @idens, $identifier;

    $ref_value = createArrayRef(@items);
    push @array, $ref_value;
}
#print "names = @names\nidens = @idens\n";

#= 3.2 print data into different files:
$r_temp_files = [];
@temp_files = @$r_temp_files;
if (@names > 0) {
    #= type_1 is not empty, print it to file(s):
    $file_type = ".txt";
    ($ref_new_files, $error_code) = createNewDataFile($output_dir, \@titles, \@types, \@idens, \@array, $normalize_code, $file_type);
    push @temp_files, @$ref_new_files;
}

#== 4. create the image_info.dat file by query BNET:
if (@names > 0) {
    $dat_file = $output_dir . $map_id . ".dat";
    open (OUT, ">$dat_file") || die "Can not open image_info_file $dat_file.$!";
}

for($i = 0; $i<@names; $i++) {
    $name = $names[$i];
    $type = $types[$i];
    if (exists $printed{$name}) {
        next;
    }
    $sql = "select distinct shape, coordinates from map_component where map_id = (select distinct map_id from map where map_number = \'$map_id\') and component_name = \'$name\' and shape is not null and coordinates is not null";
    #print "$sql\n";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    while (($shape, $coord) = $sth->fetchrow()) {
        if ($shape ne "" || $coord ne "") {
            print OUT "$type\t$name\t$shape\t$coord\n";
        }
    }
    $printed{$name} = 1;
}
close OUT;

#== 5. copy map_id.png from BromeHome to output_dir and recreate map_id.dat:
#$png_file = $brome_home . $map_id . ".png";
#$new_png = $output_dir . $map_id . ".png";
$png_file = $brome_home . $map_id;
$new_png = $output_dir . $map_id;
#print "png_file = $png_file, new_png = $new_png\n";

$ok = system "cp $png_file $new_png";
$new_dat = createNewMapDatFileForPerlBrome($output_dir, $dat_file, \@names, \@idens, $map_id);
push @temp_files, $new_png, $new_dat;

#== call perl_brome with the data files:
$index = 0;
@new_data_files = @$ref_new_files;

foreach $title (@titles) {
    $output_file = $output_dir . "out_" . ($index+1) . "_" . $map_id;
    $input_file = $new_data_files[$index];

    $brome_cmd = "perl perl_brome.pl $input_file $output_file $rc_name $png_file $new_dat";
    #print "brome_cmd = $brome_cmd\n";
    $ok = system "$brome_cmd";
    if ($ok != 0) {
        exit $ok;
    }
    $index++;
    push @gif_files, $output_file;
}

if ($anim == 1) {
    $delay = $delay*100;
    $file = $gif_files[0];
    $image = GD::Image->new($file) || die "Can not start the animated gif image. $!";
    $animatedGif = $image->gifanimbegin(1, 0);
    $animatedGif .= $image->gifanimadd(0, 0, 0, $delay);
    unlink $file or warn "can not delete file $file. $!\n";
    for($i=1;$i<@gif_files; $i++) {
        $file = $gif_files[$i];
        $frame = GD::Image->new($file);
        $animatedGif .= $frame->gifanimadd(0, 0, 0, $delay);
        #-- delete the png file:
        unlink $file or warn "can not delete file $file. $!\n";
    }
    $animatedGif .= $image->gifanimend;
    $anim_file = $output_dir;
    $anim_file =~ s/\_$//g;
    $anim_file .= ".gif";
    open (OUT, ">$anim_file") ||die "Can not open file $anim_file. $!";
    print OUT $animatedGif;
    close OUT;
}

#== delete all temp_files:
foreach $file (@temp_files) {
    unlink $file or warn "can not unlink $file.$!\n";
}

$sth->finish;
$dbh->disconnect;
exit ($ok);


###########################################################################
# subroutine: createArrayRef(@v):
# Create a reference for a known array.
#
# parameters in: @v --- the initial array
# parameter out: $ref_array --- reference of the array.
#
###########################################################################
sub createArrayRef {
    my (@v) = @_;
    my ($ref_array, @array);

    $ref_array = [];
    @array = @$ref_array;
    push @array, @v;
    return \@array;
}

###########################################################################
# subroutine: createValueArray($v):
# This subroutine create a anynomouse array. 
#
# parameters in: $v --- the first value for the anynomous array
# parameter out: $ref_array --- the reference of the anynomous array
#
###########################################################################
sub createValueArray {
    my ($v) = @_;
    my ($ref_array, @array);

    $ref_array = [];
    @array = @$ref_array;
    push @array, $v;
    return \@array;
}

###########################################################################
# subroutine: updateArrayWithValue($rv1, $v2)
#
# parameters in: $rv1 --- the original arry reference
#                $v2 --- the new value
# parameter out: $ref_array --- reference of the new array 
#
###########################################################################
sub updateArrayWithValue {
    my ($rv1, $v2) = @_;
    my (@value1, $ref_array, @array);

    $ref_array = [];
    @array = @$ref_array;
    @value1 = @$rv1;
    push @array, @value1;
    push @array, $v2;
    return \@array;
}

###########################################################################
# subroutine:
#  createNewMapDatFileForPerlBrome($output_dir, $old_dat, $r_names, $map_id):
# Create a new map_id.dat for DOME's Perl_Brome.
#
# parameters in: $old_dat --- the file name of the old .dat file,
#                $r_names -- the reference of the @names,
#                $map_id -- the map_id,
# parameter out: $new_dat_file --- the name of recreated .dat file.
#
###########################################################################
sub createNewMapDatFileForPerlBrome {
    my ($out_dir, $old_dat, $r_names, $r_idens, $map_id) = @_;
    my (@lines, $names, @idens, $iden, %names, $i, $name, $id, %printed_names,
        $r_ids, $copy_of_ids, $str, $line, $shape, $coord, $new_coord, $x, $y, $w, $h, $id, $type,
        $tomap, $x1, $y1, $new_w, $new_h, $new_line, $total_width, $num_items, $num_col);
    
    #== sort out the names and identifiers. count the number of identifiers for
    #== each name for reactangle devide.
    @names = @$r_names;
    @idens = @$r_idens;
    for($i=0; $i<@names; $i++) {
        $name = $names[$i];
        $id = $idens[$i];
        if (!exists $names{$name}) {
            $r_ids = createValueArray($id);
            $names{$name} = $r_ids;
        } else {
            $r_ids = $names{$name};
            $r_ids = updateArrayWithValue($r_ids, $id);
            $names{$name} = $r_ids;
        }
    }

    #foreach $name (keys %names) {
    #    $r_ids = $names{$name};
    #    $copy_of_ids = $r_ids;
    #    @idens = @$copy_of_ids;
    #    print "name = $name, idens = @idens num_of_id = " . @idens . "\n";
    #}

    open (OLD, "<$old_dat") or die "can't open original .dat file $old_dat\n";
    @lines = <OLD>;
    close OLD;

    #== recreate file string:
    $str = "";
    foreach $line (@lines) {
        if ($line !~ /^GENE\t/) {
            $str .= $line;
        } else {   #-- these are genes. They may need to be divided.
            $line =~ s/\n|\r|
//g;
            ($type, $name, $shape, $coord) = split /\t/, $line;
;
            if (!exists $names{$name}) {
                $str .= $line . "\n";
            } else {
                $r_ids = $names{$name};
                $copy_of_ids = $r_ids;
                @idens = @$copy_of_ids;
                if (@idens == 1) {
                    $iden = $idens[0];
                    $new_line = "$type\t$iden\t$shape\t$coord\n";
                    $str .= $new_line;
                } elsif (@idens > 1 && @idens < 6){
                    ($x, $y, $w, $h) = split /\_/, $coord;
                    $new_w = ceil($w / @idens);
                    $total_width = $new_w * @idens;
                    if ($total_width > $w + 1) {
                        $new_w--;
                        $total_width = $new_w * @idens;
                    }
                    if ($total_width >= $w) {
                        for($i=0; $i<@idens; $i++) {
                            $iden = $idens[$i];
                            $x1 = $x + $i * $new_w;
                            $total_width += $new_w;
                            $new_coord = $x1."_".$y."_".$new_w."_".$h;
                            $new_line = "$type\t$iden\t$shape\t$new_coord\n";
                            $str .= $new_line;
                        }
                    } else {
                        $total_width = 0;
                        for($i=0; $i<@idens-1; $i++) {
                            $iden = $idens[$i];
                            $x1 = $x + $i * $new_w;
                            $total_width += $new_w;
                            $new_coord = $x1."_".$y."_".$new_w."_".$h;
                            $new_line = "$type\t$iden\t$shape\t$new_coord\n";
                            $str .= $new_line;
                        }
                        $iden = $idens[@idens-1];
                        $x1 = $x + (@idens-1) * $new_w;
                        $new_w = $w - $total_width;
                        $new_coord = $x1."_".$y."_".$new_w."_".$h;
                        $new_line = "$type\t$iden\t$shape\t$new_coord\n";
                        $str .= $new_line;
                    }
                } elsif (@idens >= 6 ) {
                    ($x, $y, $w, $h) = split /\_/, $coord;
                    $num_items = @idens;
                    if ($num_items > 10) {
                        $num_items = 10;
                    }
                    $num_col = ceil($num_items / 2);
                    $new_w = ceil($w / $num_col);
                    $new_h = ceil($h / 2);
                    if ($new_w * $num_col > ($w+1)) {
                        $new_w--;
                    }
                    if ( $num_items % 2 == 0) {
                        #= there are even number of idens. both row are full:
                        #- first row:
                        $total_width = 0;
                        for($i=0; $i<$num_col-1; $i++) {
                            $iden = $idens[$i];
                            $x1 = $x + $i * $new_w;
                            $total_width += $new_w;
                            $new_coord = $x1."_".$y."_".$new_w."_".$new_h;
                            $new_line = "$type\t$iden\t$shape\t$new_coord\n";
                            $str .= $new_line;
                        }
                        $i = $num_col - 1;
                        $iden = $idens[$i];
                        $x1 = $x + $i * $new_w;
                        $new_w = $w - $total_width;
                        $new_coord = $x1."_".$y."_".$new_w."_".$new_h;
                        $new_line = "$type\t$iden\t$shape\t$new_coord\n";
                        $str .= $new_line;
                        
                        #- second row:
                        $total_width = 0;
                        $y1 = $y + $new_h;
                        for($i=0; $i<$num_col-1; $i++) {
                            $iden = $idens[$num_col+$i];
                            $x1 = $x + $i * $new_w;
                            $total_width += $new_w;
                            $new_coord = $x1."_".$y1."_".$new_w."_".$new_h;
                            $new_line = "$type\t$iden\t$shape\t$new_coord\n";
                            $str .= $new_line;
                        }
                        $i = $num_col - 1;
                        $iden = $idens[$num_col+$i];
                        $x1 = $x + $i * $new_w;
                        $new_w = $w - $total_width;
                        $new_coord = $x1."_".$y1."_".$new_w."_".$new_h;
                        $new_line = "$type\t$iden\t$shape\t$new_coord\n";
                        $str .= $new_line;
                    } else {
                        #= there are odd number of idens. first row is full but
                        #= second row has one less iden.
                        #- firsr row:
                        $total_width = 0;
                        for($i=0; $i<$num_col-1; $i++) {
                            $iden = $idens[$i];
                            $x1 = $x + $i * $new_w;
                            $total_width += $new_w;
                            $new_coord = $x1."_".$y."_".$new_w."_".$new_h;
                            $new_line = "$type\t$iden\t$shape\t$new_coord\n";
                            $str .= $new_line;
                        }
                        $i = $num_col - 1;
                        $iden = $idens[$i];
                        $x1 = $x + $i * $new_w;
                        $new_w = $w - $total_width;
                        $new_coord = $x1."_".$y."_".$new_w."_".$new_h;
                        $new_line = "$type\t$iden\t$shape\t$new_coord\n";
                        $str .= $new_line;
                        
                        #- second row:
                        $total_width = 0;
                        $y1 = $y + $new_h;
                        for($i=0; $i<$num_col-2; $i++) {
                            $iden = $idens[$num_col+$i];
                            $x1 = $x + $i * $new_w;
                            $total_width += $new_w;
                            $new_coord = $x1."_".$y1."_".$new_w."_".$new_h;
                            $new_line = "$type\t$iden\t$shape\t$new_coord\n";
                            $str .= $new_line;
                        }
                        $i = $num_col - 2;
                        $iden = $idens[$num_col+$i];
                        $x1 = $x + $i * $new_w;
                        $new_w = $w - $total_width;
                        $new_coord = $x1."_".$y1."_".$new_w."_".$new_h;
                        $new_line = "$type\t$iden\t$shape\t$new_coord\n";
                        $str .= $new_line;
                    }
                } else {
                    #== Other conditions
                    next;
                }
            }
        }
    }
    
    $new_dat = $out_dir . $map_id . ".dat";
    open (OUT, ">$new_dat") or die "can't open new .dat file $new_dat\n";
    print OUT $str;
    close OUT;

    return $new_dat;
}

###########################################################################
# subroutine: createNewDataFile
#             ($output_dir, $r_titles, $r_idens, $r_arrays, $code):
# Create a new source data files for Brome. Different code will have different
# data. 
#
# parameters in: $output_dir -- the prefix for each file,
#                $r_titles -- the reference of the @titles,
#                $r_idens -- the reference of the @idnetifiers,
#                $r_array,-- the original data values of this type,
#                $code -- the type of normalizetions required
# parameter out: $r_new_files --- the names of new data files.
#
###########################################################################
sub createNewDataFile {
    my ($output_dir, $r_titles, $r_types, $r_idens, $r_arrays, $code, $file_type) = @_;

    my (@idens, @types, @titles, @array, $error_code, $r_new_files, @new_files,
        $i, $j, $title, $file_name, $identifier, $type, $ref_value, $value,
        @comp_array, $sum, $v, $r_mean_array, @mean_array, $mean,
        $r_stddev_array, @stddev_array, $std_dev
        );

    @idens = @$r_idens;
    @types = @$r_types,
    @titles = @$r_titles;
    @array = @$r_arrays;
    $error_code = 0;

    $r_new_files = [];
    @new_files = @$r_new_files;
    if($code == 1 || @titles == 1) {
        #= this is for no normalization. print the actual values to out files:
        foreach ($i=0; $i<@titles; $i++) {
            $title = $titles[$i];
            $title =~ s/\ /\_/g;
            $file_name = $output_dir . "BROMETITLE" . $title. $file_type;
            open (OUT, ">$file_name") or die "Can't open output file \"$file_name\"\n";
            print OUT "Type\tName\tValue\n";
            for($j=0; $j<@idens; $j++) {
                $identifier = $idens[$j];
                $type = $types[$j];
                $ref_value = $array[$j];
                $value = $ref_value->[$i];
                print OUT "$type\t$identifier\t$value\n";
            }
            close OUT;
            push @new_files, $file_name;
        }
    } elsif ($code == 2) {
        #= this is for normalized by mean. i.e. I have to calculate the means
        #= for each component first then divide each value in that row by it.
        $r_mean_array = [];
        @mean_array = @$r_mean_array;
        for($j=0; $j<@idens; $j++) {
            $ref_value = $array[$j];
            @comp_array = @$ref_value;
            $sum = 0;
            foreach $v (@comp_array) {
                $sum += $v;
            }
            $mean_array[$j] = $sum/@comp_array;
        }
        foreach ($i=0; $i<@titles; $i++) {
            $title = $titles[$i];
            $title =~ s/\ /\_/g;
            $file_name = $output_dir . $title . $file_type;
            open (OUT, ">$file_name") or die "Can't open output file $file_name\n";
            print OUT "Type\tName\tValue\n";
            for($j=0; $j<@idens; $j++) {
                $identifier = $idens[$j];
                $type = $types[$j];
                $ref_value = $array[$j];
                if ($mean_array[$j] == 0) {
                    $error_code = 1;
                    next;
                }
                $value = $ref_value->[$i] / $mean_array[$j];
                print OUT "$type\t$identifier\t$value\n";
            }
            close OUT;
            push @new_files, $file_name;
        }
    } elsif ($code == 3) {
        #= this is normalized by stardard_deviation.
        $r_mean_array = [];
        @mean_array = @$r_mean_array;
        $r_stddev_array = [];
        @stddev_array = @$r_stddev_array;
        for($j=0; $j<@idens; $j++) {
            $ref_value = $array[$j];
            ($mean, $std_dev) = cal_mean_stddev($ref_value);
            $mean_array[$j] = $mean;
            $stddev_array[$j] = $std_dev;
        }
        foreach ($i=0; $i<@titles; $i++) {
            $title = $titles[$i];
            $title =~ s/\ /\_/g;
            $file_name = $output_dir . $title . $file_type;
            open (OUT, ">$file_name") or die "Can't open output file $file_name\n";
            print OUT "Type\tName\tValue\n";
            for($j=0; $j<@idens; $j++) {
                $identifier = $idens[$j];
                $type = $types[$j];
                $ref_value = $array[$j];
                if ($std_dev == 0){
                    $error_code = 2;
                    next;
                }
                $value = ($ref_value->[$i]-$mean_array[$j])/$stddev_array[$j];
                print OUT "$type\t$identifier\t$value\n";
            }
            close OUT;
            push @new_files, $file_name;
        }
    }
    
    return \@new_files, $error_code;
}

###########################################################################
# subroutine: cal_mean_stddev($ref_array):
# This subroutine creates the mean and stardard_deviation of an array. 
#
# parameters in: $ref_array --- the reference of the array;
# parameter out: $mean --- the mean of the array,
#                $std_dev -- the standard_deviation of the array.
#
###########################################################################
sub cal_mean_stddev {
    my ($ref_array) = @_;
    my (@array, $sum, $v, $mean, $std_dev);

    @array = @$ref_array;
    $sum = 0;
    foreach $v (@array) {
        $sum += $v;
    }
    $mean = $sum/@array;

    $sum = 0;
    foreach $v (@array) {
        $sum += ($v - $mean)*($v - $mean);
    }
    $std_dev = sqrt($sum/(@array-1));

    return $mean, $std_dev;
}
