<?php
session_start();

include_once("DBConnect.php");
include_once("login_header.php");
include_once("workFlow.php");

modifyPHPIniSettings(false);

/**
 * Its a hash containing the Table IDs of all the Tables.
 * The hash is indexed by the table name all in CAPS.
 * Following is an example to retrieve the value for the ID of
 * COMPOUND table. $_SESSION['TableID']['COMPOUND']
 * @global hash $_SESSION['TableID']
 */
$_SESSION['TableID'] = fillTableIDs();
fillTableInfo();

/** 
* $_SESSION['CurrentSpecieID'] stores the specie_id for a specific specie. This Front-End 
* is made to work for a specific specie and this shld be set
* to the ID of the specie for the experiment from the specie
* table in B-NET.
* @global number $_SESSION['CurrentSpecieID']
*/

if (!$_SESSION['CurrentSpecieID'])
{	$_SESSION['CurrentSpecieID'] = setSpecieID(convertToSearchName($_SESSION['SPECIES_NAME']));	}

if ($_SESSION['totReplicates'] <= 0)
{
	$_SESSION['totReplicates'] = fillTotalReplicates();
}

function modifyPHPIniSettings($printErrorMsgs = false)
{
	global $errMsgs;
	if (!(strtoupper($_SESSION['role']) == 'EDITOR' OR strtoupper($_SESSION['role']) == 'MANAGER'))
	{	$printErrorMsgs = false;	}
	if (!ini_set('max_execution_time', 600) AND $printErrorMsgs)
	{	$errMsgs[] = "Warning: Failed to set php ini 'max_execution_time'. Long queries may fail to execute.";	}
	if (!ini_set('memory_limit', '1024M') AND $printErrorMsgs)	//Does not work in 'Safe Mode'.
	{	$errMsgs[] = "Warning: Failed to set php ini 'memory_limit'. Results of long queries may fail.";	}
	/*if (!ini_set('session.cookie_lifetime', 0) AND $printErrorMsgs)
	{	$errMsgs[] = "Warning: Failed to set php ini 'session.cookie_lifetime'. Operations requiring long time to execute on the server may fail.";	}*/
	if (!ini_set('session.gc_maxlifetime', 86400) AND $printErrorMsgs)	//86400 secs = 24 Hrs
	{	$errMsgs[] = "Warning: Failed to set php ini 'session.gc_maxlifetime'. Operations requiring long time to execute on the server may fail.";	}
	/*if (!ini_set('upload_max_filesize', '20M') AND $printErrorMsgs)	//This will not work. php.ini HAS to be modified.
	{	$errMsgs[] = "Warning: Failed to set php ini 'upload_max_filesize'. Uploading of large files may fail.";	}*/
	
	if ($errMsgs)
	{	$_SESSION['errMsgs'] = array_merge($_SESSION['errMsgs'], $errMsgs);	}
}

/**
 * Thus function retrieves the TableIDs for all tables and sets up the
 * {@link $_SESSION['TableID']} hash.
 */
function fillTableIDs()
{
  $strSQL = "SELECT KW_TABLE_ID, UPPER(KW_TABLE) AS KW_TABLE FROM REF_KW_TABLE";

  $rstRefTable = $_SESSION['dbh']->query($strSQL);
  if (DB::isError($rstRefTable))
  {
	  print $strSQL;
	  printError("Error getting Table IDs from 'ref_kw_table'. Quitting.");
	  die ($rstRefTable->getMessage());
  }

  while( $row = $rstRefTable->fetchRow() )
  {
	  
    $key =  trim("{$row[1]}");
    $TableID[$key] = "{$row[0]}";
  }
  return $TableID;
}


function setSpecieID($specieName)
{
	
	$specieNameUpper = strtoupper($specieName);
	$strSQL = "SELECT s.specie_id FROM species s, allnames an 
		WHERE kw_table_id = '{$_SESSION['TableID']['SPECIES']}'
		AND an.record_id = s.specie_id
		AND UPPER(an.search_name) = '$specieNameUpper'";

	$rst = $_SESSION['dbh']->query($strSQL);
	if (DB::isError($rst))
	{
	  printError("Error searching for Specie '$specieName' in database.");
	  die ($rst->getMessage());
	}
	if ($rst->numrows() == 0)
	{
		printError("specie_id for '$specieName' not found in 'allnames' or 'species' or both tables. Please enter relevant records in database for '$specieName' otherwise operations involving species will NOT function properly.");
		return 0;	
	}
	else
	{
  		$row = $rst->fetchRow();
    	return $row[0];
	}
	return 0;
}



function fillTotalReplicates()
{
	$sql = "SELECT MAX(count) FROM 
			(SELECT COUNT(*) FROM sampling_replicate GROUP BY sampling_point_id) c";
	
	$rst = $_SESSION['dbh']->query($sql);
	if (DB::isError($rst))
	{
		printError( "Error getting the number of Total Replicates. Operations involving total number of replicates may not function properly." . $rst->getMessage());
	}
	if ($rst->numRows() > 0)
	{
		$row = $rst->fetchRow();
		return $row[0];
	}
	else
	{
		//printError( "'sampling_replicate' table does not have any rows pointing to any 'sampling_point_id'. Operations involving total number of replicates may not function properly.");	
		return 0;
	}
}

function filterByUserInput($prevPageName)
{
  $prevPageNameUpper = strtoupper($prevPageName);
  $strSQLPrev = $_SESSION[$prevPageNameUpper]['QUERY'];
  $firstCondition = 1;
  $totalTableCount = count($_SESSION[$prevPageNameUpper]['TABLE_DATA']);
    
  if (strlen($strSQLPrev) > 0 && $totalTableCount > 0)
  {
    $strSQLPrev .= " AND ";
    for ($i = 0; $i < $totalTableCount; $i++)
    {
      $TableNameUpper = strtoupper (trim($_SESSION[$prevPageNameUpper]['TABLE_DATA'][$i]['TABLE_NAME']));

      $strSQLPrev .= " (UPPER({$_SESSION[$prevPageNameUpper]['TABLE_DATA'][$i]['TABLE_NAME_FIELD']}) = '{$TableNameUpper}' AND "
        . "{$_SESSION[$prevPageNameUpper]['TABLE_DATA'][$i]['RECORD_ID_FIELD']}"
        . " in (";
      $IDs = $_SESSION[$prevPageNameUpper][$TableNameUpper];
      $firstCondition = 1;
      while (list ($key, $val) = each ($IDs))
      {
        if ($val)
        {
          if ($firstCondition)
          { 
            $firstCondition = 0;
            $strSQLPrev .= $key;
          }
          else
          {
            $strSQLPrev .= ", " . $key;
          }
          //$strSQLPrev .= " (UPPER({$_SESSION[$prevPageNameUpper]['TABLE_DATA']"
          //   . "[$i]['TABLE_NAME_FIELD']}) = '{$TableNameUpper}' AND "
          //   . "{$_SESSION[$prevPageNameUpper]['TABLE_DATA'][$i]['RECORD_ID_FIELD']}"
          //   . " = '$key')";
        }
      }
      $strSQLPrev .= " )";
    }
    $strSQLPrev .= " )";
  }

  //print "PREVSQL: $strSQLPrev";
  return $strSQLPrev;
}

function setStepNumber()
{
  if ($_REQUEST['stepno'])
  {
    $_SESSION['STEP_NUMBER'] = $_REQUEST['stepno'];
  }
  else
  {
    if (!isset($_SESSION['STEP_NUMBER']))
    {
      $_SESSION['STEP_NUMBER'] = 1;
    }
  }
}

function incrementStep($originalStepStr = "", $stepNoToAppend = 1)
{
	if ($originalStepStr == "")
	{	$originalStepStr = $_SESSION['STEP_NUMBER'];	}
	$_SESSION['STEP_NUMBER'] = "$originalStepStr:$stepNoToAppend";
}
function decrementStep($originalStepStr = "")
{
	if ($originalStepStr == "")
	{	$originalStepStr = $_SESSION['STEP_NUMBER'];	}
	$steps = split(':', $originalStepStr);
	$steps_count = count($steps) - 1;
	for ($i = 0; $i < $steps_count; $i++)
	{
		if ($step)
		{	$step .= ":$steps[$i]";	}
		else
		{	$step = "$steps[$i]";	}
	}
	$_SESSION['STEP_NUMBER'] = $step;
}

function printControlBar($stepNumber, $printNext = 1, $printSwitchTrack = 0, $printSaveSelections = 0)
{
	
  print "\n<div class=\"controlBar\">";
  print "\n<input type=\"hidden\" name=\"stepno\" value=\"{$stepNumber}\" />";
    
  $NumericalStepNumber = count (split(':', $stepNumber));
  if ($NumericalStepNumber > 1)
  {
    print "\n<input type=\"submit\" value=\"< Previous\" name=\"submit\" />";
  }
  if ($NumericalStepNumber > 1)
  {
    print "\n<input type=\"submit\" value=\"Main Menu\" name=\"submit\" />";
    if ($printSwitchTrack)
    {
	    if ($_SESSION['IS_REPLICATE_TRACK'])
	    {	print "\n<input type=\"submit\" value=\"Switch to Summary Track\" name=\"submit\" />";	}
	    else
	    {	print "\n<input type=\"submit\" value=\"Switch to Replicate Track\" name=\"submit\" />";	}
    }
  }
  if ($printSaveSelections)
  {	print "\n<input type=\"submit\" value=\"Save Selections\" name=\"submit\" />";	}
  if ($printNext AND $NumericalStepNumber < $_SESSION['TOTAL_STEPS'])
  {
    print "\n<input type=\"submit\" value=\"Next >\" name=\"submit\" />";
  }
  print "\n</div>";
}

/**Function that manages the clicks to 'Previous', 'Next' and 'Main Menu'
*  buttons.
* @param number $callProcessForm If set to non-zero value will call the function
* named 'processForm($currPageName)' when 'Next' is clicked. Function 'processForm'
* must return a non-zero value in order for this function to redirect to the 
* next step in line.
* @param number $callClearForm If set to non-zero value will call the function
* named 'clearForm($currPageName)' when 'Previous' is clicked.
*
*/

function doControlBarRedirection($callProcessForm = 0, $callClearForm = 0)
{
	global $currPageName, $currPageFile;
	$currPageNameUpper = strtoupper($currPageName);
	
	$self = $_SERVER['PHP_SELF'];
		
	if (strtoupper($_REQUEST['submit']) == "CANCEL")
	{	$_SESSION[$currPageNameUpper]['force_add'] = 0;	}
	elseif (strtoupper($_REQUEST['submit']) == "NEXT >" OR strtoupper($_REQUEST['submit']) == "CONTINUE")
	{	
		if ($callProcessForm)
		{	
			if (strtoupper($_REQUEST['submit']) == "CONTINUE")
			{	$inputValid = processForm($currPageName, $_SESSION[$currPageNameUpper]['force_add']);	}
			else
			{	$inputValid = processForm($currPageName);	}
		}
		
		if (($callProcessForm == 0) || ($callProcessForm == 1 && $inputValid))
		{
			if ('Table Format' == $_REQUEST['analysis'])
	    	{	incrementStep($_REQUEST['stepno'], 1);	}
	    	elseif ('Tab Delimited File' == $_REQUEST['analysis'])
	    	{	incrementStep($_REQUEST['stepno'], 2);	}
			elseif ('Plot Data' == $_REQUEST['analysis'])
	    	{	incrementStep($_REQUEST['stepno'], 3);	}
	    	elseif ('K-Means Clustering' == $_REQUEST['analysis'])
	    	{	incrementStep($_REQUEST['stepno'], 4);	}
	    	elseif (preg_match('/^Map - /', $_REQUEST['analysis']))
	    	{	incrementStep($_REQUEST['stepno'], 5);	}
	    	elseif ('PCA' == $_REQUEST['analysis'])
	    	{	incrementStep($_REQUEST['stepno'], 6);	}
	    	elseif ('FA' == $_REQUEST['analysis'])
	    	{	incrementStep($_REQUEST['stepno'], 7);	}
	    	elseif ('NMF' == $_REQUEST['analysis'])
	    	{	incrementStep($_REQUEST['stepno'], 8);	}
	    	elseif ('Biplot' == $_REQUEST['analysis'])
	    	{	incrementStep($_REQUEST['stepno'], 9);	}
		else		    
		    {	incrementStep($_REQUEST['stepno'], 1); 	}
      		 
			setVariables($_SESSION['STEP_NUMBER']);
			
	 	}
	 	//print "{$_SESSION['STEP_NUMBER']}$pageGoingTo:$currPageNameUpper:" . substr($_SERVER['PHP_SELF'], strrpos($_SERVER['PHP_SELF'], "/") + 1);
	  	$phpSelf = substr($_SERVER['PHP_SELF'], strrpos($_SERVER['PHP_SELF'], "/") + 1);
	 	if ('searchGene.php' == $phpSelf OR $currPageFile != $phpSelf)
	  	{ //redirect
	      	print "<meta http-equiv=\"refresh\" content=\"0; URL={$currPageFile}?stepno={$_SESSION['STEP_NUMBER']}\">";
			exit();
	  	}
	}
	elseif (strtoupper($_REQUEST['submit']) == "< PREVIOUS")
	{
    	if ($callClearForm)
    	{	clearForm($currPageName);	}
		unset($_SESSION[$currPageNameUpper]);
    	decrementStep($_REQUEST['stepno']);
    	setVariables($_SESSION['STEP_NUMBER']);
    	
    	$phpSelf = substr($_SERVER['PHP_SELF'], strrpos($_SERVER['PHP_SELF'], "/") + 1);
    	if ('searchGene.php' == $phpSelf OR $currPageFile != $phpSelf)
    	{ //redirect
    		print "<meta http-equiv=\"refresh\" content=\"0; URL={$currPageFile}?stepno={$_SESSION['STEP_NUMBER']}\">";
     		exit();
    	}
  	}
	elseif (strtoupper($_REQUEST['submit']) == "MAIN MENU")
  	{
    	resetAllSessionVariables();
    	//redirect
    	print "<meta http-equiv=\"refresh\" content=\"0; URL=intro.php\">";
    	exit();
  	}
  	elseif (!(false === strpos($_REQUEST['submit'], "Switch to")))
  	{
	  	if ($_SESSION['IS_REPLICATE_TRACK'])
    	{	
	    	$_SESSION['TECHNIQUE'] = $_SESSION['TECHNIQUE_REPLICATE']; unset($_SESSION['TECHNIQUE_REPLICATE']);
	    	$_SESSION['SEARCHGENE'] = $_SESSION['SEARCHGENE_REPLICATE']; unset($_SESSION['SEARCHGENE_REPLICATE']);
    	}
    	else
    	{
	    	$_SESSION['TECHNIQUE_REPLICATE'] = $_SESSION['TECHNIQUE']; unset($_SESSION['TECHNIQUE']);
	    	$_SESSION['SEARCHGENE_REPLICATE'] = $_SESSION['SEARCHGENE']; unset($_SESSION['SEARCHGENE']);
	    	
    	}
    	//Set corresponding step number in other track.
    	$steps = split(':', $_SESSION['STEP_NUMBER']);
    	$steps_count = count($steps) - 1;
		
		if (1 == $steps[1])
		{	$steps[1] = 2;	}
		elseif (2 == $steps[1])
		{	$steps[1] = 1;	}
		
		for ($i = 0; $i <= $steps_count; $i++)
		{
			if ($step)
			{	$step .= ":$steps[$i]";	}
			else
			{	$step = "$steps[$i]";	}
		}
		
		$_SESSION['STEP_NUMBER'] = $step;
		setVariables($_SESSION['STEP_NUMBER']);
		
		$phpSelf = substr($_SERVER['PHP_SELF'], strrpos($_SERVER['PHP_SELF'], "/") + 1);
	 	if ('searchGene.php' == $phpSelf OR $currPageFile != $phpSelf)
	  	{ //redirect
	      	print "<meta http-equiv=\"refresh\" content=\"0; URL={$currPageFile}?stepno={$_SESSION['STEP_NUMBER']}\">";
			exit();
	  	}
  	}
  	elseif ('Save Selections' == $_REQUEST['submit'])
  	{
  		$selectionArray['EXPERIMENT_SET'] = $_SESSION['EXPERIMENT_SET']; 
	  	$selectionArray['EXPERIMENT'] = $_SESSION['EXPERIMENT']; 
	  	$selectionArray['SAMPLING_POINT'] = $_SESSION['SAMPLING_POINT']; 
	  	
	  	if ($_SESSION['IS_REPLICATE_TRACK'])
    	{	
	    	$selectionArray['TECHNIQUE_REPLICATE'] = $_SESSION['TECHNIQUE_REPLICATE']; 
	    	$selectionArray['SEARCHGENE_REPLICATE'] = $_SESSION['SEARCHGENE_REPLICATE'];  	
	    }
    	else
    	{  	
	    	$selectionArray['TECHNIQUE'] = $_SESSION['TECHNIQUE']; 
	    	$selectionArray['SEARCHGENE'] = $_SESSION['SEARCHGENE'];	
	    }
    	$selectionArray['STEP_FILE_NAME'] = $_SESSION['STEP_FILE_NAME'];
	    $selectionArray['STEP_NUMBER'] = $_SESSION['STEP_NUMBER'];
    	$selectionArray['IS_REPLICATE_TRACK'] = $_SESSION['IS_REPLICATE_TRACK'];
    	header('Content-Disposition: attachment; filename="savedSelections.php"');
    	echo '<?php $selectionArray = '; var_export($selectionArray);
    	echo "\n?>";
    	exit();
  	}
}

function resetAllSessionVariables()
{
  global $currPageName;
  while($_SESSION['STEP_NUMBER'])
  {
	setVariables($_SESSION['STEP_NUMBER']);
    $currPageNameUpper = strtoupper($currPageName);
    unset($_SESSION[$currPageNameUpper]);
    decrementStep();
  }
  setVariables($_SESSION['STEP_NUMBER']);
}

function microtime_float()
{
   list($usec, $sec) = explode(" ", microtime());
   return ((float)$usec + (float)$sec);
}

function getReactionString($reaction_id, $dbh = "", $with_links = 0)
{
	if (!$dbh)
	{	$dbh = &$_SESSION['dbh'];	}
    $blnFirstSubstrateFound = 0;
    $blnFirstProductFound = 0;

    //Getting all event components for the given Event ID.
    $strSQLEventComponent = "SELECT EVENT_ID, KW_TABLE_ID, RECORD_ID, " .
        "IS_SUBSTRATE, STOICHIOMETRY FROM EVENT_COMPONENT " .
        "WHERE EVENT_ID = '$reaction_id'";
    //Now joining the Primary Names of the Event Components.
    $strSQLAllNamesReactants = "SELECT evntCmpnt.EVENT_ID, " .
        "evntCmpnt.KW_TABLE_ID, evntCmpnt.RECORD_ID, " .
        "evntCmpnt.IS_SUBSTRATE, evntCmpnt.STOICHIOMETRY, an.ANAME " .
        "FROM ALLNAMES an, ($strSQLEventComponent) evntCmpnt " .
        "WHERE an.KW_TABLE_ID = evntCmpnt.KW_TABLE_ID " .
        "AND an.RECORD_ID = evntCmpnt.RECORD_ID " .
        "AND an.IS_PRIMARY = 'Y' " .
        "ORDER BY UPPER(an.ANAME)";
    //print $strSQLAllNamesReactants;

    $rstAllNamesReactants = $dbh->query($strSQLAllNamesReactants);
    if (DB::isError($rstAllNamesReactants)) {printError("Could Not build reaction string for ID '$reaction_id'"); return ""; }

    while( $rowReactant = $rstAllNamesReactants->fetchRow() )
    {


        if ($with_links)
        {
            $ReactantURL = getURLToObject($rowReactant[1], $rowReactant[2]);
                                            //evntCmpnt.KW_TABLE_ID     evntCmpnt.RECORD_ID
            $ReactantName = "<a href=\"$ReactantURL\">{$rowReactant[5]}</a>";
                                    //an.ANAME Primary name of Event Component
        }
        else
        {   $ReactantName = $rowReactant[5];   }


        $Stoichiometry = $rowReactant[4];    //STOICHIOMETRY
        if ( strtoupper("{$rowReactant[3]}") == "Y" )  //if component is Substrate
        {
            if ($blnFirstSubstrateFound)
            {
                $strSubstratesString .= "  +  ";
                if ( $Stoichiometry > 1 )
                {   $strSubstratesString .= "$Stoichiometry ";    }

                $strSubstratesString .=  $ReactantName;  //Append Primary Name of component
            }
            else    //First Substrate Not found
            {
                if ($Stoichiometry > 1 )
                {   $strSubstratesString .= "$Stoichiometry ";    }
                $strSubstratesString .=  $ReactantName;  //Append Primary Name of component

                $blnFirstSubstrateFound = 1;
            }
        }
        else    //Reactant is a Product
        {
            if ($blnFirstProductFound)
            {
                $strProductsString .= "  +  ";
                if ( $Stoichiometry > 1 ) //if STOICHIOMETRY is greater than 1.
                {   $strProductsString .= "$Stoichiometry ";    }

                $strProductsString .=  $ReactantName;  //Append Primary Name of component
            }
            else    //First Substrate Not found
            {
                if ($Stoichiometry > 1 ) //if STOICHIOMETRY is greater than 1.
                {   $strProductsString .= "$Stoichiometry ";    }
                $strProductsString .=  $ReactantName;  //Append Primary Name of component

                $blnFirstProductFound = 1;
            }
        }
    }
    $ReactionString = "$strSubstratesString  <-->  $strProductsString";
    return $ReactionString;
}


function writeFile(&$data, $outputPath = "", $filename = "")
{
	if (!$outputPath)
	{	$outputPath = $_SESSION['BROWSER_TMP_RELATIVE'];	}
	if (!$filename)
	{	$filename = session_id() . ".dat";	}
  $dataFile = $outputPath . $filename;
  if (!$dataFileHandle = fopen($dataFile, 'w'))
  {
         echo "Cannot open file ($dataFile)";
         return 0;
    }
    // Write $data to our opened file.
  if (!fwrite($dataFileHandle, $data))
  {
    echo "Cannot write to file ($dataFile)";
    return 0;
  }
  fclose($dataFileHandle);
  return 1;
}

/**Prints the usual selections for Metadata.
* @param string $identifier If more than one set of controls 
* on the same form, an identifier can be used. The controls will
* be grouped in the array which has a name contained in the string $identifier.
* @param mixedArray $valuesArray The default values for the controls.
* @param boolean $printBiologicalReplicate Whether to also print 
* the Biological Replicate Control. Defaults to true.
* @param boolean $printTechnicalReplicate Whether to also print 
* the Technical Replicate Control. Defaults to false.
* @param booleanArray $requiredArray Prints a '*' in front of the labels of 
* the controls to indicate that an input is necessary for the form to 
* progress.
* @param stringArray $labelsArray Used to override the default names of 
* the labels next to the controls.
* @return none
*/

function printMetaDataSelections($identifier = "", $valuesArray, $printBiologicalReplicate = true, $printTechnicalReplicate = false, $printTissueType = true, $requiredArray = "", $labelsArray = "")
{
	if (!isset($labelsArray['experiment_set_id']))
	{	$labelsArray['experiment_set_id'] = 'Experiment Set';	}
	if (!isset($labelsArray['exp_name']))
	{	$labelsArray['exp_name'] = 'Experiment';	}
	if (!isset($labelsArray['sampling_point_label_id']))
	{	$labelsArray['sampling_point_label_id'] = 'Sampling Point';	}
	if (!isset($labelsArray['tissue_type_label_id']))
	{	$labelsArray['tissue_type_label_id'] = 'Tissue Type';	}
	if (!isset($labelsArray['biological_replicate']))
	{	$labelsArray['biological_replicate'] = 'Biological Replicate';	}
	if (!isset($labelsArray['technical_replicate']))
	{	$labelsArray['technical_replicate'] = 'Technical Replicate';	}
	
	//Print Experiment Sets. 
	$sql = "SELECT exp_set_id, exp_set_name  FROM experiment_set ORDER BY UPPER(exp_set_name)";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Experiment Sets. " . $sth->getMessage());	return;	}
	print "\n<p><label>{$labelsArray['experiment_set_id']}";
	if ($requiredArray['experiment_set_id'])
	{	print " *";	}
	print ": </label>";
	print "<select name=\"";
	if ($identifier)
	{	print "{$identifier}[";	}
	print "experiment_set_id";
	if ($identifier)
	{	print "]";	}
	print "\">";
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($valuesArray['experiment_set_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
	
	//Print Experiments. 
	$sql = "SELECT DISTINCT \"exp_name\", UPPER(\"exp_name\") FROM \"experiment\" 
		ORDER BY UPPER(\"exp_name\")";
    $sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Experiments. " . $sth->getMessage());	return;	}
	
	print "\n<p><label>{$labelsArray['exp_name']}";
	if ($requiredArray['exp_name'])
	{	print " *";	}
	print ": </label>";
	print "<select name=\"";
	if ($identifier)
	{	print "{$identifier}[";	}
	print "exp_name";
	if ($identifier)
	{	print "]";	}
	print "\">";
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($valuesArray['exp_name'] == $row[0])
		{	print " selected";	}
		print ">$row[0]</option>";
	}
	print "\n</select></p>";
	
	//Print Sampling Points. 
	$sql = "SELECT label_id, name FROM label 
		WHERE kw_table_id = '{$_SESSION['TableID']['SAMPLING_POINT']}'
		ORDER by \"order\"";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Sampling Points. " . $sth->getMessage());	return;	}
	
	print "\n<p><label>{$labelsArray['sampling_point_label_id']}";
	if ($requiredArray['sampling_point_label_id'])
	{	print " *";	}
	print ": </label>";
	
	print "<select name=\"";
	if ($identifier)
	{	print "{$identifier}[";	}
	print "sampling_point_label_id";
	if ($identifier)
	{	print "]";	}
	print "\">";
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($valuesArray['sampling_point_label_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
	
	if ($printTissueType)
	{
		//Print Tissue Type. 
		$sql = "SELECT \"label_id\", \"name\" FROM \"label\" 
				WHERE \"kw_table_id\" = '{$_SESSION['TableID']['SAMPLE']}' ORDER BY UPPER(\"name\")";
		$sth = $_SESSION['dbh']->query($sql);
		if ( DB::isError($sth) ) 
		{	printError("Error retrieving Tissue Types. " . $sth->getMessage());	return;	}
			
		print "\n<p><label>{$labelsArray['tissue_type_label_id']}";
		if ($requiredArray['tissue_type_label_id'])
		{	print " *";	}
		print ": </label>";
		print "<select name=\"";
		if ($identifier)
		{	print "{$identifier}[";	}
		print "tissue_type_label_id";
		if ($identifier)
		{	print "]";	}
		print "\">";
		print "\n\t<option value=\"0\">--Select One--</option>";
		while ($row = $sth->fetchRow()) 
		{
			print "\n\t<option value=\"$row[0]\"";
			if ($valuesArray['tissue_type_label_id'] == $row[0])
			{	print " selected";	}
			print ">$row[1]</option>";
		}
		print "\n</select></p>";
	}
		
	if ($printBiologicalReplicate)
	{
		//Print Biological Replicates.
		print "\n<p><label>{$labelsArray['biological_replicate']}";
		
		if ($requiredArray['biological_replicate'])
		{	print " *";	}
		print ": </label>";
		print "<select name=\"";
		if ($identifier)
		{	print "{$identifier}[";	}
		print "biological_replicate";
		if ($identifier)
		{	print "]";	}
		print "\">";
		print "\n\t<option value=\"0\">--Select One--</option>";
		for ($i = 1; $i <= $_SESSION['totReplicates']; $i++)
		{
			print "\n\t<option value=\"$i\"";
			if ($valuesArray['biological_replicate'] == $i)
			{	print " selected";	}
			print ">$i</option>";
		}
		print "\n</select></p>";
	}
	
	if ($printTechnicalReplicate)
	{
		//Print Technical Replicates.
		print "\n<p><label>{$labelsArray['technical_replicate']}";
		
		if ($requiredArray['technical_replicate'])
		{	print " *";	}
		print ": </label>";
		print "<input name=\"";
		if ($identifier)
		{	print "{$identifier}[";	}
		print "technical_replicate";
		if ($identifier)
		{	print "]";	}
		print "\"";
		if (isset($valuesArray['technical_replicate']))
		{	$defaultValue = $valuesArray['technical_replicate'];	}
		else
		{	$defaultValue = 1;	}
		
		print " value=\"$defaultValue\" size=\"2\" />";
		print "</p>";
	}
}

function printExtractMethod($technique_ids = "")
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	if ($technique_ids)
	{
		$sql = "SELECT \"extract_id\", \"name\" FROM \"extract\" e, \"technique\" t
			WHERE e.\"technique_id\" = t.\"technique_id\"
			AND t.\"technique_id\" IN ($technique_ids)
			ORDER BY UPPER(e.\"name\")";
	}
	else
	{	
		$sql = "SELECT \"extract_id\", \"name\" FROM \"extract\" 
			ORDER BY UPPER(\"name\")";
	}
	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving extract names. " . $sth->getMessage());	return;	}
	
	print "\n<p><label>Extraction Method: </label>";
	print "<select name=\"extract_id\">";
	
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['extract_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
}

function printChannel($channelNo, $fieldsetAdditionalString = "", $rawFileLabel = 'Tiff File', $printTechnicalReplicate = false)
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	$identifier = "channel$channelNo";
	$valuesArray = &$_SESSION[$currPageNameUpper][$identifier];
	print "<fieldset><legend> &nbsp;&nbsp;Channel $channelNo";
	if ($fieldsetAdditionalString)
	{	print " - $fieldsetAdditionalString";	}
	print "&nbsp;&nbsp; </legend>";	
	

	printMetaDataSelections($identifier, $valuesArray, true, $printTechnicalReplicate);
	if (strlen($rawFileLabel))
	{
		print '<input type="hidden" name="MAX_FILE_SIZE" value="100000000" />';
		print "<p><label>$rawFileLabel: </label>";
		print "<input name=\"channel{$channelNo}_tiff_file\" size=\"48\" type=\"file\" value=\"Select File using browse\"/>";
		print "</p>";
	}
	
	print "</fieldset>";
}

function getSampleLabelsFor2Channels(&$sample_label)
{
	for ($i = 1; $i <= 2; $i++)
	{
		$sql = "SELECT \"sample_labeling_id\", \"label_used\" FROM 
			\"sample_labeling\" WHERE \"sample_labeling_id\" = '$i'";
		$sth = $_SESSION['dbh']->query($sql);
		if ( DB::isError($sth) ) 
		{	printError("Error retrieving Sample Labeling. " . $sth->getMessage());	return false;	}
		if ($sth->numRows() == 0)
		{	return false;	}
		$row = $sth->fetchRow();
		$sample_label[$i] = $row[1];
	}
	return true;
}

function getPageSelections(&$selectionsHash, $protocolsAllTechniques = false)
{
	global $techniqueKey, $searchGeneKey, $fields;

	//==Hard Coding Here.
	$pages = array('Experiment Sets'=>'EXPERIMENT_SET', 'Experiments'=>'EXPERIMENT', 'Sampling Points'=>'SAMPLING_POINT', 'Techniques'=>$techniqueKey, 'Protocols'=>'PROTOCOL_TYPE');
	foreach ($pages as $key=>$page)
	{
		if ('TECHNIQUE_REPLICATE' == $page)
		{	$table = 'TECHNIQUE';	}	
		else
		{	$table = $page;	}
		
		if (isset($_SESSION[$page][$table]))
		{	$recordIDArray = $_SESSION[$page][$table];	}
		else
		{	$recordIDArray = array();	}
		
		$firstOver = false;
		$record_ids = '';
		foreach ($recordIDArray as $record_id=>$value)
		{
			if ($value)
			{
				if (!$firstOver)
				{	
					$record_ids = $record_id;
					$firstOver = true;	
				}
				else
				{	$record_ids .= ", $record_id";	}
			}
		}
				
		if(strlen($record_ids) == 0)
		{
			$sql = "";
		}
		elseif ('EXPERIMENT_SET' == $page)
		{
			$sql = "SELECT \"exp_set_name\" FROM \"experiment_set\" 
				WHERE \"exp_set_id\" IN ($record_ids) 
				ORDER BY UPPER(\"exp_set_name\")";
		}
		elseif ('EXPERIMENT' == $page)
		{	
			$sql = "SELECT DISTINCT \"exp_name\", UPPER(\"exp_name\") FROM \"experiment\" 
				WHERE \"experiment_id\" IN ($record_ids) 
				ORDER BY UPPER(\"exp_name\")";
		}
		elseif ('SAMPLING_POINT' == $page)
		{
			$sql = "SELECT DISTINCT l.\"name\", l.\"order\" FROM \"sampling_point\" sp, \"label\" l 
				WHERE sp.\"sampling_point_id\" IN ($record_ids)
				AND sp.\"label_id\" = l.\"label_id\"
				ORDER BY l.\"order\"";
		}
		elseif ($techniqueKey == $page)
		{
			$sql = "SELECT \"technique_name\" FROM \"technique\" 
				WHERE \"technique_id\" IN ($record_ids) 
				ORDER BY UPPER(\"technique_name\")";
		}
		if ('PROTOCOL_TYPE' == $page)
		{
			if (false == $protocolsAllTechniques)
			{	
				if (isset($_SESSION[$techniqueKey][$techniqueKey]))
				{	$recordIDArray = $_SESSION[$techniqueKey][$techniqueKey];	}
				else
				{	$recordIDArray = array();	}
				$firstOver = false;
				$t_record_ids = '';
				foreach ($recordIDArray as $record_id=>$value)
				{
					if ($value)
					{
						if (!$firstOver)
						{	
							$t_record_ids = $record_id;
							$firstOver = true;	
						}
						else
						{	$t_record_ids .= ", $record_id";	}
					}
				}
			}
			
			$protocol_type_ids = $_SESSION[$searchGeneKey]['protocol_type_id'][$_SESSION['TableID']['COMPOUND']];
			if (count($protocol_type_ids) > 0)
			{	$protocol_type_id_str = implode( ", ", $protocol_type_ids);	}
			
			$protocol_type_ids = $_SESSION[$searchGeneKey]['protocol_type_id'][$_SESSION['TableID']['REPORTER']];
			if (count($protocol_type_ids) > 0)
			{	
				if (strlen($protocol_type_id_str))
				{	$protocol_type_id_str .= ", ";	}
				$protocol_type_id_str .= implode( ", ", $protocol_type_ids);
			}
			
			$protocol_type_ids = $_SESSION[$searchGeneKey]['protocol_type_id'][$_SESSION['TableID']['REFERENCE_SPOT']];
			if (count($protocol_type_ids) > 0)
			{	
				if (strlen($protocol_type_id_str))
				{	$protocol_type_id_str .= ", ";	}
				$protocol_type_id_str .= implode( ", ", $protocol_type_ids);
			}
			
			if (strlen($protocol_type_id_str))
			{	
				$sql = "SELECT \"protocol_type_name\" FROM \"protocol_type\" 
					WHERE \"protocol_type_id\" IN ($protocol_type_id_str)";
				if (false == $protocolsAllTechniques AND strlen($t_record_ids))
				{	$sql .= " AND \"technique_id\" IN ($t_record_ids)";	}
					
				$sql .= " ORDER BY UPPER(\"protocol_type_name\")";
			}
			else
			{	$sql = "";	}

		}
		if ($sql)
		{
			$sth = $_SESSION['dbh']->query($sql);
			if ( DB::isError($sth) ) 
			{	printError("Error retrieving Selections from page '$page'. " . $sth->getMessage());	$errOccured = true;	continue;	}
	
			$firstOver = false;
			while ($row = $sth->fetchRow())
			{
				if (!$firstOver)
				{	
					$strSelection = $row[0];
					$firstOver = true;	
				}
				else
				{	$strSelection .= "; $row[0]";	}
				
			}
			$selectionsHash[$key] = $strSelection;
		}
	}
	
	if (isset($_SESSION[$searchGeneKey]['FIELDS']))
	{	
		$firstOver = false;
		foreach($_SESSION[$searchGeneKey]['FIELDS'] as $fieldKey)
		{
			if (!$firstOver)
			{	
				$strSelection = $fields[$fieldKey];
				$firstOver = true;	
			}
			else
			{	$strSelection .= ", $fields[$fieldKey]";	}
			
		}
		$selectionsHash['Fields'] = $strSelection;	
	}
	
	if ('--none--' != $_SESSION[$searchGeneKey]['transformation'])
	{	$selectionsHash['Transformation'] = $_SESSION[$searchGeneKey]['transformation'];	}
		
	return !$errOccured;
}

function printPageSelections($selectionsHash = "", $double = true, $protocolsAllTechniques = false)
{
	if (!$selectionsHash)
	{	getPageSelections($selectionsHash, $protocolsAllTechniques);	}
	if (count($selectionsHash) == 0)
	{	return;	}
	
	$printHTML = "\n<table width=\"100%\"><tr><td width=\"95%\">";
	$printHTML .= "<label>Your Selections";
	if (0 == $_SESSION['IS_REPLICATE_TRACK'])
	{	$printHTML .= " in <b>Summary  track</b>";	}
	elseif (1 == $_SESSION['IS_REPLICATE_TRACK'])
	{	$printHTML .= " in <b>Replicate  track</b>";	}
	elseif (2 == $_SESSION['IS_REPLICATE_TRACK'])
	{	$printHTML .= " for <b>Raw Data</b> Files";	}
	
	$printHTML .= ":</label><table class=\"page-selections\">";
	
	$cl = 'light-blue'; $printTR = true;
	foreach ($selectionsHash as $page=>$selections)
	{
		if ($selections)
		{
			if ($printTR)
			{	$printHTML .= "\n<tr class=\"$cl\">";	}
			$printHTML .= "\n<th><label>$page</label></th><td>$selections</td>";
			if ($double AND $printTR)
			{	$printTR = false;	continue;	}
			else
			{	$printTR = true;	}

			$printHTML .= "</tr>";	
			if ('dark-blue' == $cl)
			{	$cl = 'light-blue';	}
			else
			{	$cl = 'dark-blue';	}
		}
		
	}
	$printHTML .= "</table></td><td align=\"right\" valign=\"top\">" . printHelpLink();
	$printHTML .= "</td></table>";
	return $printHTML;
}

    
   
/**
* This function takes a recordset and displays
* the results nicely in a table.
* @param Recordset $tableRecordset The recordset resulting from executing
* the SQL query string.
* @param string $tableName The entity type using which the table
* and its headers will be named (in style sheet).
* @param StringArray $headerArray  An array containing the field names
* for this table
* @param number $startCol The index of the column in the resultset
* at which to start displaying.
* @param number $endCol The index of the column in the resultset
* at which to end displaying (optional - if not specified the function will 
* take the index of the last column in the recordset).
* @param bool $SerialNumbers Whether to display serial numbers before
* each row.
*/
function printTable($tableRecordset, $tableName = "result", $headerArray = 0,
                                      $startCol = 0, $endCol = 0, $SerialNumbers = 0, $printHeaders = 1)
{
    if ($endCol == 0)
    {
		$endCol = $tableRecordset->numCols();
    }
    if ($tableName == "")
    {
        $tableName = "result";
    }
    if ($endCol < $startCol)
    {
       return;
    }

    print "\n<div class=\"$tableName\">\n<table class=\"$tableName-table\">\n";
    if ($headerArray && $printHeaders)
    {
        print " <tr class=\"$tableName-table-title\">";
        if($SerialNumbers)
        {   print "<th>Sr. No.</th>  ";  }
        for ($i = $startCol; $i < $endCol; $i++)
        {
            print "<th>$headerArray[$i]</th>  ";
            if (strtoupper($headerArray[$i]) == "LOCATOR" && strtoupper($headerArray[$i + 1]) == "PUBMED")
            {   $i++;   //Bypass Pubmed Accession (always after locator).
            }
            
        }

        if ($tableName == "evidences")
        { print "<th>GO Accession</th>  "; }

        print "</tr>\n";
    }

    $alternate = "2"; // number of alternating rows
    $firstDone = false;
    while( $row = $tableRecordset->fetchRow() )
    {
        if ($alternate == "1")
        {
            $class = "light-row";
            $alternate = "2";
        }
        else
        {
            $class = "dark-row";
            $alternate = "1";
        }
        print " <tr class=\"$class\">";
        $srno++;
        if($SerialNumbers)
        {   
            print "<td>";
        	if ('compound-reactions' == $tableName)
        	{
            	$EventURL = getURLToObject($_SESSION['TableID']['EVENT'], $row[1]);
            	print "<a href=\"$EventURL\">$srno</a>";
        	}
        	else
        	{	print $srno;	}
        	print "</td>";    
        }

        for ($i = $startCol; $i < $endCol; $i++)
        {
            print "<td>";
            if (strtoupper($headerArray[$i]) == "SPECIES")
            {
                print "<span class=\"species-name\">";
                $SpecieLink = getURLToObject($_SESSION['TableID']['SPECIES'], $row[1]);
                                    //$row[1]: Expected the RECORD_ID for SPECIE in $row[1].
                print "<a href=$SpecieLink>$row[$i]</a>";
                print "</span>";
            }
            else if (strtoupper($headerArray[$i]) == "LOCATOR")
            {
                if (preg_match('#^(http://|https://|ftp://)#i', $row[$i]))
                {   
                    print "<a href=$row[$i]>$row[$i]</a>";
                    
                }
                else if (strtoupper($headerArray[$i + 1]) == "PUBMED")
                {
                    if( '' != $row[$i + 1] )
                    {
                       print "<a href=\"http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Retrieve&db=PubMed&list_uids=";
                       print "{$row[$i + 1]}&dopt=Abstract\" target=\"_blank\">$row[$i]</a>";
                    }
                    else
                    {   print "$row[$i]";    }
                }
                if (strtoupper($headerArray[$i + 1]) == "PUBMED")
                {	
	                $i++;   //Bypass the column after LOCATOR which is always r.pubmed_accession	
                }
            }
            else if (strtoupper($headerArray[$i]) == "PROTEIN NAME")
            {
                $link = getURLToObject($_SESSION['TableID']['PROTEIN'], $row[0]);
                                    //$row[0]: Expected the RECORD_ID for PROTEIN in $row[0].
                print "<a href=$link>$row[$i]</a>";
            }
            else if (strtolower($tableName) == "compound-reactions")
            {
	            if (strtoupper($headerArray[$i]) == "REACTION")
	            {
	            	print getReactionString($row[$i], '', 1);
		        }
	            else if (strtoupper($headerArray[$i]) == "EC")
                {
                	if ($row[0] > 0)
                	{
		                $link = getURLToObject($_SESSION['TableID']['EC'], $row[0]);
	                                        //$row[0]: Expected the EC_ID in $row[0].
	                    print "<a href=$link>$row[$i]</a>";
                    }
	            }
            }
            else if (strtolower($tableName) == "ec-evidences" && strtoupper($headerArray[$i]) == "REACTION")
            {
            	print $GLOBALS['ReactionsData'][$row[$i]]['SrNo'];
	        }
	        else if (strtolower($tableName) == "protein-states" && strtoupper($headerArray[$i]) == "LOCATIONS")
            {
            	$sql = "SELECT l.\"location_id\", l.\"location_name\" FROM \"protein_state_location\" psl, \"location\" l
            		WHERE psl.\"protein_state_id\" = $row[0] 
            		AND psl.\"location_id\" = l.\"location_id\"
            		ORDER BY UPPER(l.\"location_name\")";
            		
            	$rst = $_SESSION['dbh']->query($sql);
    			if (DB::isError($rst)) {printError("Cannot retrieve locations for protein_state_id '$row[0]'. [" . $rst->getMessage() . "]"); }
				while ($rec = $rst->fetchRow())
				{
					if (!$firstDone)
					{	print $rec[1]; $firstDone = true;	}
					else
					{	print ", $rec[1]";	}
				}
	        }
            else
            {   print "$row[$i]";    }
            print "</td>  ";
        }
        if ($tableName == "evidences")
        {
            $GOAccessionString = getGOAccessions($row[0]); //EVIDENCE_ID
            print "<td>$GOAccessionString</td>  ";
        }
        print "</tr>\n";
      }
      print "</table></div>\n";
 }

function convertToSearchName($aname)
{
	$aname = str_replace("\"", "", $aname);
	$aname = preg_replace('/ /', '', $aname);
	$aname = preg_replace('/-/', '', $aname);
	return $aname;
}


/**
* Prints the HTML string with External Database links for an entity.
* Note: It ignores the link for "CAS".
* @param number $TableID The Table ID for the entity type whose 
* External Links is to be displayed.
* @param number $RecordID The Record ID for the entity whose 
* External Links is to be displayed.
*/
function getExternalLinks($TableID, $RecordID)
{
    if ($TableID <= 0 || $RecordID <= 0)
        return;

    $strSQLEL = "SELECT ed.database_name, ed.url, el.external_id FROM external_database ed, external_links el WHERE el.external_database_id = ed.external_database_id AND el.kw_table_id = '$TableID' AND el.record_id = '$RecordID'";
    //print $strSQLEL;
    $rstEL = $_SESSION['dbh']->query($strSQLEL);
    if (DB::isError($rstEL)) {  printError("Error retrieving External Links."); return;}


    
    $firstLinkFound = 0;
    while ($rec = $rstEL->fetchRow())
    {
        if (strtoupper($rec[0]) == "CAS")    //Dont show CAS Links
        {   continue;   }
        $strLink = preg_replace('/<>/', $rec[2], $rec[1]);
        //print $strLink;
        if (!$firstLinkFound)
        {	$firstLinkFound = 1;	}
        else
        {   $extLinksStr .= " | "; }
        $extLinksStr .= "<a href=\"$strLink\" target=\"_blank\">$rec[0]</a>";
    }
    return $extLinksStr;
}

/**
 * Prints the Error message formatted with the 'error' stylesheet class.
 * @param string $msg The error message.
 */
function printError($msg)
{
  print "<div class=\"error\">\n<p>$msg</p>\n</div>\n";
}
/**
 * Prints an informational message formatted with the 'message' stylesheet class.
 * @param string $msg The error message.
 */
function printMessage($msg)
{
  print "<div class=\"message\">\n<p>$msg</p>\n</div>\n";
}

?>
