#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script will copy data from quantification to ma_norm with and without 
# background subtraction. 
#
# author: X. Jing Li
# date: August 30, 2005
# reversion: 
#===============================================================================

my ($prefix, $srid, $ptid, $extract_id, $tissue_type_label_id) = @ARGV;
my ($database, $username, $password, $bnet_DB, $specieDB, $dbh, $sth, $rc,$sql,
    $sql1, $sql2, $mainsql, $count, $msg, $exp_name, $exp_set_name, $replicate,
    $lname, $extract_name,
    $sample_extract_id, $iaid, $array_id, $fid, $rid, %fid_rids, %multi_rids,
    $method, @options, $opt, $value, %values, %values_l, $multiple, $intensity,
    $intensity_l);

#== start time:
#print "ARGV = @ARGV\n";
print "Starting time = ". TimeStamp() . "\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die "Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== check the existance of this arrays:
$sql = "select count(*) from \"ma_norm\" where protocol_type_id = $ptid AND sampling_replicate_id = $srid";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$count = $sth->fetchrow();
if ($count > 0) {
    $msg = getErrorDetailForMaNorm($dbh, $srid, $ptid, $extract_id);
    print "$msg\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    print "Ending time = ". TimeStamp() . "\n";
    exit 1;
}

#== get the sample_extract_id from sample_extract:
$sql = "select SE.sample_extract_id, IA.image_analysis_id, H.array_id from \"sample\" S, sample_extract SE, labeled_extract LE, hybridization H, raw_data RD, image_analysis IA where S.sampling_replicate_id = $srid and S.label_id = $tissue_type_label_id and S.sample_id = SE.sample_id and SE.extract_id = $extract_id and LE.sample_extract_id = SE.sample_extract_id and H.labeled_extract2_id = LE.labeled_extract_id and RD.hybridization_id = H.hybridization_id and IA.raw_data_id = RD.raw_data_id";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
if ($sth->rows() == 0) {
    ($exp_name, $exp_set_name, $replicate, $lname, $extract_name) = getErrorDetail($dbh, $srid, $extract_id);
    print "There is no original data from extract \"$extract_name\" for experiment \"$exp_name\" of experiment_set \"$exp_set_name\" at time \"$lname\" of replicate number \"$replicate\". Check if you had uploaded the original data file to database. \n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    print "Ending time = ". TimeStamp() . "\n";
    exit 1;
} else {
    ($sample_extract_id, $iaid, $array_id) = $sth->fetchrow();
}

#== get feature_id and reporter_ids for this array:
$sql = "select feature_id, reporter_id from feature where array_batch_id = (select array_batch_id from \"array\" where array_id = $array_id)";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($fid, $rid) = $sth->fetchrow()) {
    $fid_rids{$fid} = $rid;
    if (!exists $multi_rids{$rid}) {
        $multi_rids{$rid} = 1;
    } else {
        $multi_rids{$rid} ++;
    }
}

#== get the method for this protocol_type_id:
$sql = "select method_options from protocol_type where protocol_type_id = $ptid";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$value = $sth->fetchrow();
if ($value =~ /SCRIPT/) {
    ($value, $method) = split /\#\#/, $value;
    $method =~ s/OPT=//g;
    if ($method =~ /@@/) {
        @options = split /@@/, $method;
        $method = shift @options;
    }
}

#== get all data from quantification for this array:
$sql = "select feature_id, $method from quantification where image_analysis_id = $iaid";
if (@options > 0) {
    foreach $opt (@options) {
        $sql .= " and $opt";
    }
}
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($fid, $value) = $sth->fetchrow()) {
    if ($value > 0) {
        $values{$fid} = $value;
        $values_l{$fid} = log($value)/log(2);
    } elsif ($value == 0) {
        $values{$fid} = $value;
    } 
}

#== insert into ma_norm:
$mainsql = "";
foreach $fid (sort {$a<=>$b} keys %values) {
    $rid = $fid_rids{$fid};
    $multiple = $multi_rids{$rid};
    $intensity = $values{$fid};
    $intensity_l = $values_l{$fid};
    $sql1 = "insert into \"ma_norm\" (ma_norm_id, sampling_replicate_id, sample_extract_id, protocol_type_id, reporter_id, feature_id, intensity";
    $sql2 = " values (nextval('ma_norm_id_seq'), $srid, $sample_extract_id, $ptid, $rid, $fid, $intensity";
    if ($multiple > 1) {
        $sql1 .= ", multiple_reporter";
        $sql2 .= ", 'true'";
    } else {
        $sql1 .= ", multiple_reporter";
        $sql2 .= ", 'false'";
    }
    if ($intensity_l ne "") {
        $sql1 .= ", intensity_l";
        $sql2 .= ", $intensity_l";
    }
    $sql1 .= ")";
    $sql2 .= ");\n";
    $sql = $sql1 . $sql2;
    #print "$sql";
    $mainsql .= $sql;
}

if ($mainsql ne "") {
    $sth = $dbh->prepare($mainsql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    #print "rc = $rc\n";
}
if ($rc == 1)  {
    $dbh->commit;
    $sth->finish;
    $dbh->disconnect;
    print "Data normalization is SUCCESS\n";
    print "Ending time = " . TimeStamp() . ".\n";
    exit 0;
} else {
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    print "Data normalization FAILED\n";
    print "Ending time = " . TimeStamp() . ".\n";
    exit 1;
}

###########################################################################
# subroutine: getErrorDetailForMaNorm($dbh, $srid, $ptid, $extract_id)
#
# parameters in: $dbh --- the database handle,
#                $srid --- the sampling_replicate_id,
#                $ptid --- the statistical protocol_type_id,
#                $extract_id ---
# parameter out: $msg --- the message for this set of parameters.
#
# X. Jing Li, 8/18/2005
###########################################################################
sub getErrorDetailForMaNorm {
    my ($dbh, $srid, $ptid, $extract_id) = @_;
    my ($sth, $rc, $sql, $msg, $exp_name, $exp_set_name, $replicate, $lname, $pt_name);

    $sql = "select E.exp_name, ES.exp_set_name, SR.replicate, L.\"name\" from experiment E, experiment_set ES, \"sample\" S, sample_extract SE, label L, sampling_point SP, sampling_replicate SR where SE.extract_id = $extract_id and SE.sample_id = S.sample_id and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_replicate_id = $srid and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($exp_name, $exp_set_name, $replicate, $lname) = $sth->fetchrow();
    
    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pt_name = $sth->fetchrow();
    $msg = "The protocol_type \"$pt_name\" for this sample has been calulated before. The experiment_set name is \"$exp_set_name\", the experiment name is \"$exp_name\", the time point is \"$lname\", the biological replicate is \"$replicate\". ";
    $sth->finish;
    return $msg;
}

###########################################################################
# subroutine: getErrorDetail($dbh, $srid, $extract_id)
#
# parameters in: $dbh --- the database handle,
#                $srid --- the sampling_replicate_id,
#                $extract_id ---
# parameter out: $exp_name -- the name of the experiment'
#                $exp_set_name -- experiment_set name,
#                $replicate -- the number of the biological replicate,
#                $lname -- the time point label name,
#                $extract_name -- the full name of the extraction.
#
# X. Jing Li, 9/6/2005
###########################################################################
sub getErrorDetail {
    my ($dbh, $srid, $extract_id) = @_;
    my ($sth, $rc, $sql, $exp_name, $exp_set_name, $replicate, $lname, $extract_name);
    $sql = "select E.exp_name, ES.exp_set_name, SR.replicate, L.\"name\" from experiment E, experiment_set ES, label L, sampling_point SP, sampling_replicate SR where SR.sampling_replicate_id = $srid and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($exp_name, $exp_set_name, $replicate, $lname) = $sth->fetchrow();
    
    $sql = "select \"name\" from extract where extract_id = $extract_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $extract_name = $sth->fetchrow();
    
    $sth->finish;
    return ($exp_name, $exp_set_name, $replicate, $lname, $extract_name);
}
