<?php
/**
* This script is a Front-End for comparing data
* from two sampling points.
* Date: 18 May 2005
* @author Aejaaz Kamal <aekamal@vt.edu>
*/


include_once("common.php");

$currPageName = 'CUSTOM_COMPARE_DATA';
?>

<?php
$title = "Custom Compare Data between two Samples";
include("header.php");
if (!(strtoupper($_SESSION['role']) == 'MANAGER' OR strtoupper($_SESSION['role']) == 'EDITOR'))
{	printError("You are not authorized to view this page. Your role level should be 'editor' or 'manager' to view this page.");	exit();}
?>

<?php

if ($_REQUEST['submit'] == 'Compare And Store')
{	processForm();	}

if ($errMsgs)
{  
	foreach ($errMsgs as $msg)
	{	printError($msg);	}
}

if ($_SESSION['Message'])
{  
	foreach ($_SESSION['Message'] as $msg)
	{	printMessage($msg);	}
	unset($_SESSION['Message']);
}

fillForm();
include_once("footer.php");
?>

<?php
//Functions
function fillForm()
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	$self = $_SERVER[PHP_SELF];
	print "<form name=\"init_form\" method=\"post\" action=\"$self\" enctype=\"multipart/form-data\">\n";

	print '<p class="help-tips">This page compares the expression value between two samples. It calculates ratio, log-ratio, and difference in expression between the expression values for two samples. It also calculates t-test and stores these information into the database for further quering. Please provide information about which two data points you want to compare. Please select a method and data only for that method will be used. You will have to choose methods one at a time if you want to compare for more than one method.</p>';
	//This MAX_SIZE is an advisory to the browser. (So that the user does not find
	// that his file is rejected by the php on the server after uploading it).
	print '<input type="hidden" name="MAX_FILE_SIZE" value="30000000" />';
	print "<p><label>Data File: </label>";
	print '<input name="uploadFile" size="48" type="file" value="Select File using browse"/>';
	print "</p>";
	
	print "<table><tr><td>";	//start table
	//Print Sample 1 Selections
	printSample(1);
	print "</td><td>";
	
	//Print Sample 2 Selections
	printSample(2);
	print "</td></tr></table>"; //end table
	
	//Print Normalization Method
	$sql = "SELECT \"protocol_type_id\", \"protocol_type_name\" FROM 
		\"protocol_type\" WHERE UPPER(\"method\") = 'STATISTICAL' 
		ORDER BY UPPER(\"protocol_type_name\")";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving protocol_types. " . $sth->getMessage());	return;	}
	print "<p><label>Normalization Method: </label>";
	print "<select name=\"protocol_type_id\">";
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['protocol_type_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
	
	//Print Statistical Test
	$sql = "SELECT DISTINCT p.\"protocol_name\", UPPER(p.\"protocol_name\") FROM 
		 \"protocol\" p, \"protocol_type\" pt 
		 WHERE p.\"protocol_type_id\" = pt.\"protocol_type_id\"
		 AND UPPER(pt.\"method\") = 'STATISTICAL' 
		 AND UPPER(p.\"protocol_name\") != 'MEAN'
		ORDER BY UPPER(p.\"protocol_name\")";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Statistical Tests. " . $sth->getMessage());	return;	}
	print "<p><label>Statistical Test: </label>";
	print "<select name=\"protocol_name\">";
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['protocol_name'] == $row[0])
		{	print " selected";	}
		print ">$row[0]</option>";
	}
	print "\n</select></p>";
	
	printExtractMethod();
	
	//Is Log Checkbox
	print "\n<p><input type=\"checkbox\" name=\"is_log\"";
	if ($_SESSION[$currPageNameUpper]['is_log'])
	{	print "\"checked\"";	}
	print ">Data is logged</input>";
	print "</p>";
	print "<input type=\"submit\" name=\"submit\" value=\"Compare And Store\" />";

	print "\n</form>";

}

function printSample($sampleNo)
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	$identifier = "sample$sampleNo";
	
	print "\n<fieldset><legend> &nbsp;&nbsp;Sample $sampleNo &nbsp;&nbsp; </legend>";
	printMetaDataSelections($identifier, $_SESSION[$currPageNameUpper][$identifier], false);
	print "\n</fieldset>";
}



/**============================
* Request Functions
*============================*/
function processForm()
{
	global $errMsgs, $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	$_SESSION[$currPageNameUpper]['sample1'] = $_REQUEST['sample1'];
	$_SESSION[$currPageNameUpper]['sample2'] = $_REQUEST['sample2'];
	$_SESSION[$currPageNameUpper]['protocol_type_id'] = $_REQUEST['protocol_type_id'];
	$_SESSION[$currPageNameUpper]['protocol_name'] = $_REQUEST['protocol_name'];
	$_SESSION[$currPageNameUpper]['extract_id'] = $_REQUEST['extract_id'];
	$_SESSION[$currPageNameUpper]['is_log'] = $_REQUEST['extract_id'];
	
	if ($_FILES['uploadFile']['name'] == "")
	{	$errMsgs[] = "Please select the Data Fil to upload.";	}
	$noOfSamples = 2;
	for ($i = 1; $i <= $noOfSamples; $i++)
	{
		$identifier = "sample$i";
		
		//Validating Sample $i
		if (!$_SESSION[$currPageNameUpper][$identifier]['experiment_set_id'])
		{	$errMsgs[] = "Please Select an Experiment Set for Sample $i.";	}
		if (!$_SESSION[$currPageNameUpper][$identifier]['exp_name'])
		{	$errMsgs[] = "Please Select an Experiment for Sample $i.";	}
		if (!$_SESSION[$currPageNameUpper][$identifier]['sampling_point_label_id'])
		{	$errMsgs[] = "Please Select a Sampling Point for Sample $i";	}
		if (!$_SESSION[$currPageNameUpper][$identifier]['tissue_type_label_id'])
		{	$errMsgs[] = "Please Select a Tissue Type for Sample $i";	}
	}
	
	//Validating Rest
	
	if (!$_SESSION[$currPageNameUpper]['protocol_type_id'])
	{	$errMsgs[] = "Please Select Normalization Method.";	}
	if (!$_SESSION[$currPageNameUpper]['protocol_name'])
	{	$errMsgs[] = "Please Select Statistical Test.";	}
	if (!$_SESSION[$currPageNameUpper]['extract_id'])
	{	$errMsgs[] = "Please Select an Extraction Method.";	}
	
	if ($errMsgs)
	{	return 0;	}
	
	$uploaddir = $_SESSION['BROWSER_TMP_FULL'];
	$uploadfile = $uploaddir . $_FILES['uploadFile']['name'];
	if (move_uploaded_file($_FILES['uploadFile']['tmp_name'], $uploadfile)) 
	{
	   //File is valid, and was successfully uploaded
	   $_SESSION[$currPageNameUpper]['uploadFile'] = $uploadfile;
	   $_SESSION[$currPageNameUpper]['uploadFileName'] = $_FILES['uploadFile']['name'];
	   system("chmod 644 $uploadfile");
	} 
	else 
	{
		$errMsgs[] = "Could not move the uploaded file to the Destination on the server";
		return 0;
	}	
	
	if ($errMsgs)
	{	return 0;	}
	
	
	//Finding/Validating sampling_point_ids
	for ($i = 1; $i <= $noOfSamples; $i++)
	{
		$identifier = "sample$i";
	
		$sql = 	"SELECT sp.\"sampling_point_id\" FROM 
			\"sampling_point\" sp, \"experiment\" e
			WHERE e.\"exp_set_id\" = '{$_SESSION[$currPageNameUpper][$identifier]['experiment_set_id']}'
			AND e.\"exp_name\" = '{$_SESSION[$currPageNameUpper][$identifier]['exp_name']}'
			AND sp.\"experiment_id\" = e.\"experiment_id\"
			AND sp.\"label_id\" = '{$_SESSION[$currPageNameUpper][$identifier]['sampling_point_label_id']}'";
			
		//print $sql;
		$sth = $_SESSION['dbh']->query($sql);
		if ( DB::isError($sth) ) 
		{	$errMsgs[] = "Error retrieving sampling_point_id for sample $i. " . $sth->getMessage();	return 0;	}
		
		if ($sth->numrows() > 0)
		{
			$row = $sth->fetchRow();
			$_SESSION[$currPageNameUpper][$identifier]['sampling_point_id'] = $row[0];
		}
		else
		{	$errMsgs[] = "Sampling Point not found in the database for sample $i.";	}

	}
	
	
	//Finding/Validating protocol_id
	$sql = "SELECT p.\"protocol_id\" FROM \"protocol\" p, \"protocol_type\" pt
		WHERE p.\"protocol_type_id\" = pt.\"protocol_type_id\"
		AND pt.\"protocol_type_id\" = {$_SESSION[$currPageNameUpper]['protocol_type_id']}
		AND UPPER(p.\"protocol_name\") = '" . strtoupper($_SESSION[$currPageNameUpper]['protocol_name']) . "'";
		

	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	$errMsgs[] = "Error retrieving protocol_id. " . $sth->getMessage();	return 0;	}
	
	if ($sth->numrows() > 0)
	{
		$row = $sth->fetchRow();
		$_SESSION[$currPageNameUpper]['protocol_id'] = $row[0];
	}
	else
	{	$errMsgs[] = "Statistical Test not found in the database for the selected Normalization Method.";	}

	
	//Find extract_id and technique_type
	$sql = 	"SELECT t.\"technique_type\" FROM 
		\"extract\" e, \"protocol_type\" pt, \"technique\" t
		WHERE pt.\"protocol_type_id\" = '{$_SESSION[$currPageNameUpper]['protocol_type_id']}'
		AND e.\"extract_id\" = '{$_SESSION[$currPageNameUpper]['extract_id']}'
		AND pt.\"technique_id\" = e.\"technique_id\"
		AND t.\"technique_id\" = e.\"technique_id\"";
			
	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	$errMsgs[] = "Error retrieving technique type. " . $sth->getMessage();	return 0;	}
	
	if ($sth->numrows() > 0)
	{
		$row = $sth->fetchRow();
		$_SESSION[$currPageNameUpper]['technique_type'] = $row[0];
	}
	else
	{	$errMsgs[] = "Extraction Method not found in the database for the protocol selected.";	}
	
	if ($errMsgs)
	{	return 0;	}
	
	
	$perlScriptFile = "submitComparisonFromUser.pl";
	$data_file = $_SESSION[$currPageNameUpper]['uploadFile'];
	//parameters to the perl script
	$sampling_point_id1 = $_SESSION[$currPageNameUpper]['sample1']['sampling_point_id'];
	$tissue_type_label_id1 = $_SESSION[$currPageNameUpper]['sample1']['tissue_type_label_id'];
	$sampling_point_id2 = $_SESSION[$currPageNameUpper]['sample2']['sampling_point_id'];
	$tissue_type_label_id2 = $_SESSION[$currPageNameUpper]['sample2']['tissue_type_label_id'];
	$extract_id = $_SESSION[$currPageNameUpper]['extract_id'];
	$protocol_id = $_SESSION[$currPageNameUpper]['protocol_id'];
	if ($_SESSION[$currPageNameUpper]['is_log'])
	{	$is_log = 1;	}
	else
	{	$is_log = 0;	}
	
	$com = "{$_SESSION['PERL_DIR']}perl $perlScriptFile \"$data_file\" $sampling_point_id1 $tissue_type_label_id1 $sampling_point_id2 $tissue_type_label_id2 $extract_id $protocol_id $is_log 2>&1;";
	$retString = exec($com, $retLines, $retValue);
	
	if ($retLines)
  	{
		foreach ($retLines as $retLine)
		{	$msg .= "$retLine <br />";	}
  	}
	if ($retValue == 0)	//success
  	{
		if (!$msg)
	  	{	$msg = "system command to '$perlScriptFile' returned a successful value.";	}
	  	
	  	$_SESSION['Message'][] = $msg;
		//redirect
		print "<meta http-equiv=\"refresh\" content=\"0; URL=intro.php?stepno=1\">";
		exit();
	}
	else
	{	
	  	if (!$msg)
	  	{	$msg = "system command to '$perlScriptFile' failed. <br />Command was \"$com\"\nReturn Value = $retValue";	}
	  	$errMsgs[] = $msg;
	}
	
  	if ($errMsgs)
	{	return 0;	}
	return 1;
}

?>
