<?php
/**
* This script is a Front-End for normalizing affymetrix data
* from a user file and adding it to the DOME database.
* Date: 9 Jun 2006
* @author Aejaaz Kamal <aekamal@vt.edu>
*/


include_once("common.php");

$currPageName = 'CUSTOM_NORMALIZED_AFFY';
$currPageNameUpper = strtoupper($currPageName);
?>

<?php
$title = "Upload Normalized Affymetrix Data";
include("header.php");
if (!(strtoupper($_SESSION['role']) == 'MANAGER' OR strtoupper($_SESSION['role']) == 'EDITOR'))
{	printError("You are not authorized to view this page. Your role level should be 'editor' or 'manager' to view this page.");	exit();}
?>

<?php


if ($_REQUEST['submit'] == 'Upload')
{	processForm();	}

if ($errMsgs)
{  
	foreach ($errMsgs as $msg)
	{	printError($msg);	}
}

if ($_SESSION['Message'])
{  
	foreach ($_SESSION['Message'] as $msg)
	{	printMessage($msg);	}
	unset($_SESSION['Message']);
}
fillForm();
include_once("footer.php");
?>

<?php
//Functions
function fillForm()
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	$self = $_SERVER[PHP_SELF];
	print "<form name=\"init_form\" method=\"post\" action=\"$self\" enctype=\"multipart/form-data\">\n";

	print '<p class="help-tips">Please provide the data and information about which data sample you want to normalize and choose a protocol to be used for normalization. The data would be stored in the database after normalization to assist in further quering.</p>';
	
	//Get technique_id
	$sql = "SELECT \"technique_id\", \"technique_name\" FROM \"technique\" 
	WHERE UPPER(\"technique_name\") = 'AFFYMETRIX'";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	$errMsgs[] = "Error retrieving Techniques. " . $sth->getMessage();	return;	}
	$row = $sth->fetchRow();
	$_SESSION[$currPageNameUpper]['technique_id'] = $row[0];
	
	//This MAX_SIZE is an advisory to the browser. (So that the user does not find
	// that his file is rejected by the php on the server after uploading it).
	print '<input type="hidden" name="MAX_FILE_SIZE" value="30000000" />';
	print "<p><label>Data File: </label>";
	print '<input name="uploadFile" size="48" type="file" value="Select File using browse"/>';
	print "</p>";
	
	printMetaDataSelections("", $_SESSION[$currPageNameUpper], true, true);
	
	//Print Normalization Method
	$sql = "SELECT \"protocol_type_id\", \"protocol_type_name\" FROM \"protocol_type\" 
			WHERE UPPER(\"method\") = 'STATISTICAL' 
			AND technique_id = '{$_SESSION[$currPageNameUpper]['technique_id']}'
		ORDER BY UPPER(\"protocol_type_name\")";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Protocol Type. " . $sth->getMessage());	return;	}
	print "\n<p><label>Normalization Method: </label>";
	print "<select name=\"protocol_type_id\">";
	
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['protocol_type_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
	
	printExtractMethod($_SESSION[$currPageNameUpper]['technique_id']);
	
	print "<input type=\"submit\" name=\"submit\" value=\"Upload\" />";

	print "\n</form>";

}


/**============================
* Request Functions
*============================*/
function processForm()
{
	global $errMsgs, $currPageName;
	$currPageNameUpper = strtoupper($currPageName);

	$_SESSION[$currPageNameUpper]['experiment_set_id'] = $_REQUEST['experiment_set_id'];
	$_SESSION[$currPageNameUpper]['exp_name'] = $_REQUEST['exp_name'];
	$_SESSION[$currPageNameUpper]['sampling_point_label_id'] = $_REQUEST['sampling_point_label_id'];
	$_SESSION[$currPageNameUpper]['tissue_type_label_id'] = $_REQUEST['tissue_type_label_id'];
	$_SESSION[$currPageNameUpper]['biological_replicate'] = $_REQUEST['biological_replicate'];
	$_SESSION[$currPageNameUpper]['technical_replicate'] = $_REQUEST['technical_replicate'];
	$_SESSION[$currPageNameUpper]['protocol_type_id'] = $_REQUEST['protocol_type_id'];
	$_SESSION[$currPageNameUpper]['extract_id'] = $_REQUEST['extract_id'];
		
	
	if ($_FILES['uploadFile']['name'] == "")
	{	$errMsgs[] = "Please select the Data File to upload.";	}
		
	if (!$_SESSION[$currPageNameUpper]['experiment_set_id'])
	{	$errMsgs[] = "Please Select an Experiment Set.";	}
	
	if (!$_SESSION[$currPageNameUpper]['exp_name'])
	{	$errMsgs[] = "Please Select an Experiment.";	}
	
	if (!$_SESSION[$currPageNameUpper]['sampling_point_label_id'])
	{	$errMsgs[] = "Please Select a Sampling Point";	}
	
	if (!$_SESSION[$currPageNameUpper]['tissue_type_label_id'])
	{	$errMsgs[] = "Please Select a Tissue Type";	}
	
	if (!$_SESSION[$currPageNameUpper]['biological_replicate'])
	{	$errMsgs[] = "Please Select the biological replicate number.";	}
	
	if (!$_SESSION[$currPageNameUpper]['technical_replicate'])
	{	$errMsgs[] = "Please Select the technical replicate number.";	}
	
	if (!$_SESSION[$currPageNameUpper]['protocol_type_id'])
	{	$errMsgs[] = "Please Select a Normalization Method.";	}
	
	if (!$_SESSION[$currPageNameUpper]['extract_id'])
	{	$errMsgs[] = "Please Select an Extraction Method.";	}
	
	if ($errMsgs)
	{	return 0;	}
	
	$uploaddir = $_SESSION['BROWSER_TMP_FULL'];
	$uploadfile = $uploaddir . $_FILES['uploadFile']['name'];
	if (move_uploaded_file($_FILES['uploadFile']['tmp_name'], $uploadfile)) 
	{
	   //File is valid, and was successfully uploaded
	   $_SESSION[$currPageNameUpper]['uploadFile'] = $uploadfile;
	   $_SESSION[$currPageNameUpper]['uploadFileName'] = $_FILES['uploadFile']['name'];
	   system("chmod 644 $uploadfile");
	} 
	else 
	{
		$errMsgs[] = "Could not move the uploaded file to the Destination on the server";
		return 0;
	}
	
	
	if ($errMsgs)
	{	return 0;	}
	
	//Now find out sampling_replicate_id from the given info.
	$sql = "SELECT sr.\"sampling_replicate_id\" FROM \"sampling_replicate\" sr
					, \"sampling_point\" sp
					, \"experiment\" e
				WHERE e.\"exp_set_id\" = '{$_SESSION[$currPageNameUpper]['experiment_set_id']}'
				AND e.\"exp_name\" = '{$_SESSION[$currPageNameUpper]['exp_name']}'
				AND sp.\"experiment_id\" = e.\"experiment_id\"
				AND sp.\"label_id\" = '{$_SESSION[$currPageNameUpper]['sampling_point_label_id']}'
				AND sr.\"sampling_point_id\" = sp.sampling_point_id
				AND sr.\"replicate\" = '{$_SESSION[$currPageNameUpper]['biological_replicate']}'";
	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	
		$errMsgs[] = "Error retrieving Sampling Replicate. " . $sth->getMessage();	
		return 0;
	}
	
	if ($sth->numrows() > 0)
	{
		$row = $sth->fetchRow();
		$_SESSION[$currPageNameUpper]['sampling_replicate_id'] = $row[0];
		//print "Set sampling_replicate_id = {$_SESSION[$currPageNameUpper]['sampling_replicate_id']} ($n)";
	}
	else
	{	$errMsgs[] = "Sampling Replicate not found in the database for the information provided";	}
	
	if ($errMsgs)
	{	return 0;	}
	

	$perlScriptFile = "submitUserNormalizedAffy.pl";
	$data_file = $_SESSION[$currPageNameUpper]['uploadFile'];
	$sampling_replicate_id = $_SESSION[$currPageNameUpper]['sampling_replicate_id'];
	$technical_replicate = $_SESSION[$currPageNameUpper]['technical_replicate'];
	$tissue_type_label_id = $_SESSION[$currPageNameUpper]['tissue_type_label_id'];
	$extract_id = $_SESSION[$currPageNameUpper]['extract_id'];
	$protocol_type_id = $_SESSION[$currPageNameUpper]['protocol_type_id'];
	

	$com = "{$_SESSION['PERL_DIR']}perl $perlScriptFile \"$data_file\" $sampling_replicate_id $technical_replicate $tissue_type_label_id $extract_id $protocol_type_id 2>&1;";
	//print $com;
	$retString = exec($com, $retLines, $retValue);
	
	if ($retLines)
  	{
		foreach ($retLines as $retLine)
		{	$msg .= "$retLine <br />";	}
  	}
	if ($retValue == 0)	//success
  	{	
		if (!$msg)
	  	{	$msg = "system command to '$perlScriptFile' returned a successful value.";	}
	  	
	  	$_SESSION['Message'][] = $msg;
		//redirect
		print "<meta http-equiv=\"refresh\" content=\"0; URL=intro.php?stepno=1\">";
		exit();
	}
	else
	{	
	  	if (!$msg)
	  	{	$msg = "system command to '$perlScriptFile' failed. <br />Command was \"$com\"\nReturn Value = $retValue";	}
	  	$errMsgs[] = $msg;
	}
	
  	if ($errMsgs)
	{	return 0;	}

	return 1;
}

?>
