<?php
/**
* This script is a Front-End for deleting compared data
* from two sampling points.
* Date: 10 Jul 2006
* @author Aejaaz Kamal <aekamal@vt.edu>
*/


include_once("common.php");

$currPageName = 'DELETE_COMPARED';
$currPageNameUpper = strtoupper($currPageName);
?>

<?php
$title = "Delete Comparison Data between two Samples";
include("header.php");
if (strtoupper($_SESSION['role']) != 'MANAGER')
{	printError("You are not authorized to view this page. Your role level should be 'manager' to view this page.");	exit();}
?>

<?php

if ('Delete' == $_REQUEST['submit'] OR 'Continue' == $_REQUEST['submit'] )
{	processForm($_SESSION[$currPageNameUpper]['force_delete']);	}
elseif ($_REQUEST['submit'] == 'Cancel')
{	$_SESSION[$currPageNameUpper]['force_delete'] = 0;	}
else
{	$_SESSION[$currPageNameUpper]['force_delete'] = 0;	}

if ($errMsgs)
{  
	foreach ($errMsgs as $msg)
	{	printError($msg);	}
}
if ($_SESSION['Message'])
{  
	foreach ($_SESSION['Message'] as $msg)
	{	printMessage($msg);	}
	unset($_SESSION['Message']);
}
	
fillForm($_SESSION[$currPageNameUpper]['force_delete']);
include_once("footer.php");
?>

<?php
//Functions
function fillForm($force_delete = 0)
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	$self = $_SERVER[PHP_SELF];
	print "<form name=\"init_form\" method=\"post\" action=\"$self\">\n";
	if ($force_delete)
	{
		print "\n<div class=\"controlBar\">";
		print "\n<input type=\"submit\" name=\"submit\" value=\"Cancel\" /> ";
		print "<input type=\"submit\" name=\"submit\" value=\"Continue\" />";
		print "\n</div>";
	}
	else
	{
		print '<p class="help-tips"></p>';
		print "<table><tr><td>";	//start table
		//Print Sample 1 Selections
		printSample(1);
		print "</td><td>";
		
		//Print Sample 2 Selections
		printSample(2);
		print "</td></tr></table>"; //end table
		
		//Print Normalization Method
		$sql = "SELECT \"protocol_type_id\", \"protocol_type_name\" FROM 
			\"protocol_type\" WHERE UPPER(\"method\") = 'STATISTICAL' 
			ORDER BY UPPER(\"protocol_type_name\")";
		$sth = $_SESSION['dbh']->query($sql);
		if ( DB::isError($sth) ) 
		{	printError("Error retrieving Normalization Method. " . $sth->getMessage());	return;	}
		print "<p><label>Normalization Method: </label>";
		print "<select name=\"protocol_type_id\">";
		print "\n\t<option value=\"0\">--Select One--</option>";
		while ($row = $sth->fetchRow()) 
		{
			print "\n\t<option value=\"$row[0]\"";
			if ($_SESSION[$currPageNameUpper]['protocol_type_id'] == $row[0])
			{	print " selected";	}
			print ">$row[1]</option>";
		}
		print "\n</select></p>";
		
		//Print Statistical Test
		$sql = "SELECT DISTINCT p.\"protocol_name\", UPPER(p.\"protocol_name\") FROM 
			 \"protocol\" p, \"protocol_type\" pt 
			 WHERE p.\"protocol_type_id\" = pt.\"protocol_type_id\"
			 AND UPPER(pt.\"method\") = 'STATISTICAL' 
			 AND UPPER(p.\"protocol_name\") != 'MEAN'
			ORDER BY UPPER(p.\"protocol_name\")";
		$sth = $_SESSION['dbh']->query($sql);
		if ( DB::isError($sth) ) 
		{	printError("Error retrieving protocols. " . $sth->getMessage());	return;	}
		print "<p><label>Statistical Test: </label>";
		print "<select name=\"protocol_name\">";
		print "\n\t<option value=\"0\">--Select One--</option>";
		while ($row = $sth->fetchRow()) 
		{
			print "\n\t<option value=\"$row[0]\"";
			if ($_SESSION[$currPageNameUpper]['protocol_name'] == $row[0])
			{	print " selected";	}
			print ">$row[0]</option>";
		}
		print "\n</select></p>";
		
		printExtractMethod();
		
		print "<input type=\"submit\" name=\"submit\" value=\"Delete\" />";
		print "\n<p><a href=\"deleteDataMenu.php\"><- Back to Delete Data Menu</a></p>";
	}
	print "\n</form>";

}

function printSample($sampleNo)
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	$identifier = "sample$sampleNo";
	
	print "\n<fieldset><legend> &nbsp;&nbsp;Sample $sampleNo &nbsp;&nbsp; </legend>";
	printMetaDataSelections($identifier, $_SESSION[$currPageNameUpper][$identifier], false);
	print "\n</fieldset>";
}



/**============================
* Request Functions
*============================*/
function processForm($force_delete = 0)
{
	global $errMsgs, $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	if (!$force_delete)
	{
		$_SESSION[$currPageNameUpper]['sample1'] = $_REQUEST['sample1'];
		$_SESSION[$currPageNameUpper]['sample2'] = $_REQUEST['sample2'];
		$_SESSION[$currPageNameUpper]['protocol_type_id'] = $_REQUEST['protocol_type_id'];
		$_SESSION[$currPageNameUpper]['protocol_name'] = $_REQUEST['protocol_name'];
		$_SESSION[$currPageNameUpper]['extract_id'] = $_REQUEST['extract_id'];
		
		
		$noOfSamples = 2;
		for ($i = 1; $i <= $noOfSamples; $i++)
		{
			$identifier = "sample$i";
			
			//Validating Sample $i
			if (!$_SESSION[$currPageNameUpper][$identifier]['experiment_set_id'])
			{	$errMsgs[] = "Please Select an Experiment Set for Sample $i.";	}
			if (!$_SESSION[$currPageNameUpper][$identifier]['exp_name'])
			{	$errMsgs[] = "Please Select an Experiment for Sample $i.";	}
			if (!$_SESSION[$currPageNameUpper][$identifier]['sampling_point_label_id'])
			{	$errMsgs[] = "Please Select a Sampling Point for Sample $i";	}
			if (!$_SESSION[$currPageNameUpper][$identifier]['tissue_type_label_id'])
			{	$errMsgs[] = "Please Select a Tissue Type for Sample $i";	}
		}
		
		//Validating Rest
		if (!$_SESSION[$currPageNameUpper]['protocol_type_id'])
		{	$errMsgs[] = "Please Select Normalization Method.";	}
		if (!$_SESSION[$currPageNameUpper]['protocol_name'])
		{	$errMsgs[] = "Please Select Statistical Test.";	}
		if (!$_SESSION[$currPageNameUpper]['extract_id'])
		{	$errMsgs[] = "Please Select an Extraction Method.";	}
			
		if ($errMsgs)
		{	return 0;	}
		
		
		//Finding/Validating sampling_point_ids
		for ($i = 1; $i <= $noOfSamples; $i++)
		{
			$identifier = "sample$i";
		
			$sql = 	"SELECT sp.\"sampling_point_id\" FROM 
				\"sampling_point\" sp, \"experiment\" e
				WHERE e.\"exp_set_id\" = '{$_SESSION[$currPageNameUpper][$identifier]['experiment_set_id']}'
				AND e.\"exp_name\" = '{$_SESSION[$currPageNameUpper][$identifier]['exp_name']}'
				AND sp.\"experiment_id\" = e.\"experiment_id\"
				AND sp.\"label_id\" = '{$_SESSION[$currPageNameUpper][$identifier]['sampling_point_label_id']}'";
				
			//print $sql;
			$sth = $_SESSION['dbh']->query($sql);
			if ( DB::isError($sth) ) 
			{	$errMsgs[] = "Error retrieving sampling_point_id for sample $i. " . $sth->getMessage();	return 0;	}
			
			if ($sth->numrows() > 0)
			{
				$row = $sth->fetchRow();
				$_SESSION[$currPageNameUpper][$identifier]['sampling_point_id'] = $row[0];
			}
			else
			{	$errMsgs[] = "Sampling Point not found in the database for sample $i.";	}
	
		}
		
		
		//Finding/Validating protocol_id
		$sql = "SELECT p.\"protocol_id\" FROM \"protocol\" p, \"protocol_type\" pt
			WHERE p.\"protocol_type_id\" = pt.\"protocol_type_id\"
			AND pt.\"protocol_type_id\" = {$_SESSION[$currPageNameUpper]['protocol_type_id']}
			AND UPPER(p.\"protocol_name\") = '" . strtoupper($_SESSION[$currPageNameUpper]['protocol_name']) . "'";
			
	
		//print $sql;
		$sth = $_SESSION['dbh']->query($sql);
		if ( DB::isError($sth) ) 
		{	$errMsgs[] = "Error retrieving protocol_id. " . $sth->getMessage();	return 0;	}
		
		if ($sth->numrows() > 0)
		{
			$row = $sth->fetchRow();
			$_SESSION[$currPageNameUpper]['protocol_id'] = $row[0];
		}
		else
		{	$errMsgs[] = "Statistical Test not found in the database for the selected Normalization Method.";	}
	
	}
	
	
	
	$perlScriptFile = "deleteDataFromSpComparison.pl";
	
	//parameters to the perl script
	$sampling_point_id1 = $_SESSION[$currPageNameUpper]['sample1']['sampling_point_id'];
	$sampling_point_id2 = $_SESSION[$currPageNameUpper]['sample2']['sampling_point_id'];
	$protocol_type_id = $_SESSION[$currPageNameUpper]['protocol_type_id'];
	$extract_id = $_SESSION[$currPageNameUpper]['extract_id'];
	$tissue_type_label_id1 = $_SESSION[$currPageNameUpper]['sample1']['tissue_type_label_id'];
	$tissue_type_label_id2 = $_SESSION[$currPageNameUpper]['sample2']['tissue_type_label_id'];
	$force_delete = $_SESSION[$currPageNameUpper]['force_delete'];
	
	$com = "{$_SESSION['PERL_DIR']}perl $perlScriptFile $sampling_point_id1 $tissue_type_label_id1 $sampling_point_id2 $tissue_type_label_id2 $extract_id $protocol_type_id $force_delete 2>&1;";
	$retString = exec($com, $retLines, $retValue);
	
	if ($retLines)
  	{
		foreach ($retLines as $retLine)
		{	$msg .= "$retLine <br />";	}
  	}
	if ($retValue == 0)	//success
  	{
	  	$_SESSION[$currPageNameUpper]['force_delete'] = 0;
		if (!$msg)
	  	{	$msg = "system command to '$perlScriptFile' returned a successful value.";	}
	  	
	  	$_SESSION['Message'][] = $msg;
	}
	elseif ($retValue == 99)	//data exists
	{
		$_SESSION[$currPageNameUpper]['force_delete'] = 1;
		$_SESSION['Message'][] = $msg;
	}
	else
	{	
		$_SESSION[$currPageNameUpper]['force_delete'] = 0;
	  	if (!$msg)
	  	{	$msg = "system command to '$perlScriptFile' failed. <br />Command was \"$com\"\nReturn Value = $retValue";	}
	  	$errMsgs[] = $msg;
	}
	
  	if ($errMsgs)
	{	return 0;	}
	return 1;
}

?>
