#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script deletes the Affy-metrix data and all its related processed data.
#
# sampling_replicate_id
# tissue_type_id
# technical_replicate
# extract_id
# hybridization_protocol_id
# scanning_protocol_id
# force_delete
#
# Author: X. Jing Li
# Date: June 21, 2006
# Reversions: 
#===============================================================================

my ($database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc, 
    $sql1, $sth1, $rc1, 
    $srid, $ttlid, $tech_repl, $extract_id, $hybrid_prot_id, $scan_prot_id, $force_delete, $msg, $data_ok, $file_msg, $ptid, $pt_name, $count, $norm_msg, $sum_msg, $comp_msg, $iaid, $cel_fllid, $rdid, $hid, $array_id, $leid, @items);

#== ARGV
($srid, $ttlid, $tech_repl, $extract_id, $hybrid_prot_id, $scan_prot_id, $force_delete) = @ARGV;

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

if ($force_delete == 0) {
    #== array information:
    ($data_ok, $file_msg) = getAffyArrayInfo($dbh, $srid, $ttlid, $tech_repl, $extract_id);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $file_msg;
        exit 1;
    }

    #== check the processed data detail:
    #-- 1. normalized data:
    ($data_ok, $norm_msg) = getArrayNormalizedInfo($dbh, $srid, $ttlid, $tech_repl, $extract_id);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $file_msg . $norm_msg . "Are you sure you want to delete the data from DOME?\n";
        exit 99;
    }

    #-- 2. sp_summary:
    ($data_ok, $sum_msg) = getSummaryInfo($dbh, $srid, $ttlid, $tech_repl, $extract_id);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $file_msg . $norm_msg . $sum_msg . "If you delete the array data, you will deleted all the normalized data as well. Are you sure you want to delete the data from DOME?\n";
        exit 99;
    }

    #-- 3. sp_comparison:
    ($data_ok, $comp_msg) = getComparisonInfo($dbh, $srid, $ttlid, $tech_repl, $extract_id);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $file_msg . $norm_msg . $sum_msg . $comp_msg . "If you delete the array data, you will deleted all the normalized and summarized data as well. Are you sure you want to delete the data from DOME?\n";
        exit 99;
    }
    $msg .= $file_msg . $norm_msg . $sum_msg . $comp_msg . "If you delete the array data, you will deleted all the processed data as well. Are you sure you want to delete the data from DOME?\n";
    print $msg;
    $sth->finish;
    $dbh->rollback;
    $dbh->disconnect;
    exit 99;
} elsif ($force_delete == 1) {
    #-- 1. delete data from sp_comparison:
    $sql = "delete from sp_comparison where extract_id = $extract_id and process1_id in (select process1_id from process1 where (sampling_point1_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp1_tissue_type_label_id = $ttlid) or (sampling_point2_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp2_tissue_type_label_id = $ttlid))";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc =~ /^\d/ && $rc !~ /\W/) {
        if ($rc != 0) {
            $msg = "$rc records deleted from comparison data.<br><br>\n";
        }
    } else {
        $msg = "There are some mistakes occured when trying to delete the comparison data. NO data deleted in database.<br><br>\n";
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }

    #-- 2. delete from sp_summary:
    $sql = "delete from sp_summary where extract_id = $extract_id and label_id = $ttlid and sampling_point_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid)";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc =~ /^\d/ && $rc !~ /\W/) {
        if ($rc != 0) {
            $msg = "$rc records deleted from summarized data.<br><br>\n" . $msg;
        }
    } else {
        $msg = "There are some mistakes occured when trying to delete the summarized data. NO data deleted in database.<br><br>\n" . $msg;
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }

    #-- 3. delete from ma_norm:
    $sql = "delete from ma_norm where sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and technical_replicate = $tech_repl and sample_id in (select sample_id from \"sample\" where label_id = $ttlid and sampling_replicate_id = $srid))";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc =~ /^\d/ && $rc !~ /\W/) {
        if ($rc != 0) {
            $msg = "$rc records deleted from normalized data.<br><br>\n" . $msg;
        }
    } else {
        $msg = "There are some mistakes occured when trying to delete the normalized Affymetrix array data. NO data deleted in database.<br><br>\n" . $msg;
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }

    #-- 4. delete from sample, sample_extract, labeled_extract, hybridization, 
    #-- raw_data, image_analysis, array, file, file_location_link:
    $sql = "select IA.image_analysis_id, IA.file_location_link_id, RD.raw_data_id, H.hybridization_id, H.array_id, H.labeled_extract1_id from image_analysis IA, raw_data RD, hybridization H, labeled_extract LE, sample_extract SE, sample S where IA.raw_data_id = RD.raw_data_id and RD.scanning_protocol_id = $scan_prot_id and RD.hybridization_id = H.hybridization_id and H.hybridization_protocol_id = $hybrid_prot_id and H.labeled_extract1_id = LE.labeled_extract_id and LE.sample_extract_id = SE.sample_extract_id and SE.extract_id = $extract_id and SE.technical_replicate = $tech_repl and SE.sample_id = S.sample_id and S.label_id = $ttlid and S.sampling_replicate_id = $srid";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($iaid, $cel_fllid, $rdid, $hid, $array_id, $leid) = $sth->fetchrow();
#    print "iaid = $iaid, cel_fllid cel_fllid, rdid = $rdid, hybrid_id = $hid, array_id = $array_id, leid1 = $leid\n";

    #-- 4.1 delete sample_id:
    $sql = "delete from sample where sample_id = (select sample_id from sample_extract where sample_extract_id = (select sample_extract_id from labeled_extract where labeled_extract_id = $leid))";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc == 1) {
        $data_ok = 1;
    }
#    print "sample: rc = $rc\n";

    #-- 4.2 delete sample_extract_ids:
    $sql = "delete from sample_extract where sample_extract_id = (select sample_extract_id from labeled_extract where labeled_extract_id = $leid)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc == 1) {
        $data_ok++;
    }
#    print "sample_extract: rc = $rc\n//\n";
    
    #-- 4.3 delete labeled_Extract_ids:
    $sql = "delete from labeled_extract where labeled_extract_id = $leid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc == 1) {
        $data_ok++;
    }
#    print "labeled_extract: rc = $rc\n//\n";

    #-- 4.4 delete array_id:
    $sql = "delete from \"array\" where array_id = $array_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc == 1) {
        $data_ok++;
    }
#    print "array: rc = $rc\n//\n";

    #-- 4.6 delete from iamge_analysis:
    $sql = "delete from image_analysis where image_analysis_id = $iaid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc == 1) {
        $data_ok++;
    }
#    print "image_analysis: rc = $rc\n//\n";

    #-- 4.7 delete from raw_data:
    $sql = "delete from raw_data where raw_data_id = $rdid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc == 1) {
        $data_ok++;
    }
#    print "raw_data: rc = $rc\n//\n";

    #-- 4.8 delete from hybridization:
    $sql = "delete from hybridization where hybridization_id = $hid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc == 1) {
        $data_ok++;
    }
#    print "hybridization: rc = $rc\n//\n";

    #-- 4.9 delete from file:
    $sql = "delete from file where file_id = (select file_id from file_location_link where file_location_link_id = $cel_fllid)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc == 1) {
        $data_ok++;
    }
#    print "file: rc = $rc\n";
    $sql = "delete from file_location_link where file_location_link_id = $cel_fllid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc == 1) {
        $data_ok++;
    }
#    print "file_location_link: rc = $rc\n//\n";

    if ($data_ok == 9) {
        $msg = "1 record for Affymetrix is deleted from DOME.<br><br>\n" . $msg;
        print $msg;
        $sth->finish;
#        $dbh->rollback;
        $dbh->commit;
        $dbh->disconnect;
        exit;
    } else {
        $msg = "There are some mistakes occured when trying to delete the Affymetrix data from DOME. NO data changed in database.<br><br>\n";
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }
}

###########################################################################
# subroutine: getAffyArrayInfo($dbh, $srid, $ttlid, $tech_repl, $extract_id)
#
# parameters in: $dbh --- the database handle,
#                $srid --- the sampling_replicate_id,
#                $ttlidid -- tissue_type_label_id;
# parameter out: $data_ok -- 0 for there is data and 1 for no data
#                $msg --- the detailed message for the array;
#
# X. Jing Li, 6/21/2006
###########################################################################
sub getAffyArrayInfo {
    my ($dbh, $srid, $ttlid, $tech_repl, $extract_id) = @_;
    my ($sql, $sth, $rc, $es_name, $exp_name, $lname, $ext_name, $bio_repl, $ttlid_name, $array_batch_name, $array_name, $cel_fllid, $cel_file_name, $time, $person, $data_ok, $msg, $iaid, $count);

    $data_ok = 0;

    #== get sample detail:
    $sql = "select distinct ES.exp_set_name, E.exp_name, L.name, EXT.\"name\", SR.replicate from experiment_set ES, experiment E, label L, sampling_point SP, sampling_replicate SR, \"sample\" S, sample_extract SE, labeled_extract LE, hybridization H, extract EXT where H.labeled_extract1_id = LE.labeled_extract_id and LE.sample_extract_id = SE.sample_extract_id and SE.extract_id = EXT.extract_id and EXT.extract_id = $extract_id and S.sample_id = SE.sample_id and S.label_id = $ttlid and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_replicate_id = $srid and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($es_name, $exp_name, $lname, $ext_name, $bio_repl) = $sth->fetchrow(); 
    if ($sth->rows() == 0) {
        $sql = "select ES.exp_set_name, E.exp_name, L.name, SR.replicate from experiment_set ES, experiment E, label L, sampling_point SP, sampling_replicate SR where SR.sampling_replicate_id = $srid and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        ($es_name, $exp_name, $lname, $bio_repl) = $sth->fetchrow();

        $sql = "select \"name\" from extract where extract_id = $extract_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        $ext_name = $sth->fetchrow();

        $msg = "<B>Experimental Data:</B><BR>There is no Affymetrix microarray for experiment_set = \"$es_name\", experiment = \"$exp_name\", time_point = \"$lname\", biological_replicate = \"$bio_repl\",extraction name = \"$ext_name\".\n";
        $sth->finish;
        $data_ok = 1;
        return $data_ok, $msg;
    }

    #-- get tissue_type_label_names:
    $sth = $dbh-> prepare("select \"name\" from label where label_id = $ttlid") or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $ttlid_name = $sth->fetchrow();

    #-- get file_ids:
    $sql = "select AB.array_design_name, A.array_name, IA.file_location_link_id, IA.image_analysis_id from \"array\" A, array_batch AB, raw_data RD, image_analysis IA, hybridization H where H.labeled_extract1_id in (select labeled_extract_id from labeled_extract where sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and sample_id in (select sample_id from \"sample\" where sampling_replicate_id = $srid and label_id = $ttlid))) and H.hybridization_id = RD.hybridization_id and RD.raw_data_id = IA.raw_data_id and H.array_id = A.array_id and AB.array_batch_id = A.array_batch_id";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($array_batch_name, $array_name, $cel_fllid, $iaid) = $sth->fetchrow();
    ($cel_file_name, $time, $person) = getFileInfoFromFLLid($dbh, $cel_fllid);

    $msg = "<h3>Summary about the array:</h3>\n<b>Array Name:</b> $array_name<br>\n<b>Data File Name:</b>$cel_file_name<br>\n<b>Array Design Name:</b> $array_batch_name<br><br>\n";
    $msg .= "<B>Metadata about the array:</B><br><TABLE BORDER=1>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Exp_set Name</b></TD><TD>$es_name</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Experiment Name</b></TD><TD>$exp_name</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Time Point</b></TD><TD>$lname</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Tissue Type</b></TD><TD>$ttlid_name</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Biological Replicate</b></TD><TD>$bio_repl</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Technical Replicate</b></TD><TD>$tech_repl</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Uploading Time</b></TD><TD>$time</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Researcher Name</b></TD><TD>$person</TD></TR>\n";
    $msg .= "</TABLE><BR>";

    $sth->finish;
    return $data_ok, $msg;
}
    
###########################################################################
# subroutine: getFileInfoFromFLLid($dbh, $file_location_link_id)
#
# parameters in: $dbh --- the database handle,
#                $file_location_link_id --
# parameter out: $file_name --- the file name
#                $time_uploaded -- the time the file has been uploaded
#                $person -- researcher's name
#
# X. Jing Li, 6/14/2006
###########################################################################
sub getFileInfoFromFLLid {
    my ($dbh, $fllid) = @_;
    my ($sth, $rc, $file_name, $time_uploaded, $person_name);

    $sql = "select F.file_name, F.time_uploaded, P.person_name from file F, file_location_link FLL, person P where FLL.file_location_link_id = $fllid and FLL.file_id = F.file_id and F.person_id = P.person_id";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() > 0) {
        ($file_name, $time_uploaded, $person_name) = $sth->fetchrow();
    } else {
        $sql = "select F.file_name from file F, file_location_link FLL where FLL.file_location_link_id = $fllid and FLL.file_id = F.file_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        $file_name = $sth->fetchrow();
        $time_uploaded = "-";
        $person_name = "-";
    }

    $sth->finish;
    return $file_name, $time_uploaded, $person_name;
}
   
###########################################################################
# subroutine: getArrayNormalizedInfo
#           ($dbh, $srid1, $ttlid1, $srid2, $ttlid2, $tech_repl, $extract_id)
#
# parameters in: $dbh --- the database handle,
#                $srid_id --- the sampling_replicate_id for sample,
#                $ttlid --- sample tissue_type_lable_id
#                $tech_repl --- technical_replicate of this sample
#                $extract_id --- extract_id
# parameter out: $msg --- the protocol_name in html table row format;
#
# X. Jing Li, 6/21/2006
###########################################################################
sub getArrayNormalizedInfo {
    my ($dbh, $srid, $ttlid, $tech_repl, $extract_id) = @_;
    my ($sql, $sth, $rc, $sql1, $sth1, $rc1, $ptid, $pt_name, $data_ok, $msg);

    $data_ok = 0;

    $sql = "select protocol_type_id, protocol_type_name from protocol_type where protocol_type_id in (select distinct protocol_type_id from ma_norm where sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and technical_replicate = $tech_repl and sample_id in (select sample_id from \"sample\" where label_id = $ttlid and sampling_replicate_id = $srid)))";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() == 0) {
        $msg = "There is no normalized data for this array.\n";
        $data_ok = 1;
    } elsif ($sth->rows() == 1 ) {
        ($ptid, $pt_name) = $sth->fetchrow();
        $sql1 = "select count(*) from ma_norm where protocol_type_id = $ptid and sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and technical_replicate = $tech_repl and sample_id in (select sample_id from \"sample\" where label_id = $ttlid and sampling_replicate_id = $srid))";
        $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
        $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
        $count = $sth1->fetchrow();
        $msg = "<B>Normalized Data for this array:</B> <BR>There are $count records in DOME which are normalised by \"$pt_name\". If you delete the data, you will deleted all the processed data as well. Are you sure you want to delete the data from DOME?\n";
    } else {
        $msg = "<B>Normalized Data for this array:</B> <TABLE BORDER=1><TR><TH>Process Protocol Name</TH><TH>Number of Records</TH></TR>\n";
        while (($ptid, $pt_name) = $sth->fetchrow() ) {
            $sql1 = "select count(*) from ma_norm where protocol_type_id = $ptid and sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and technical_replicate = $tech_repl and sample_id in (select sample_id from \"sample\" where label_id = $ttlid and sampling_replicate_id = $srid))";
            $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
            $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
            $count = $sth1->fetchrow();
            $msg .= "<TR ALIGN=CENTER><TD>$pt_name</TD><TD>$count</TD></TR>\n";
        }
        $msg .= "</TABLE><BR>";
        $sth1->finish;
    }
    $sth->finish;
    return $data_ok, $msg;
}

###########################################################################
# subroutine: getSummaryInfo
#           ($dbh, $srid, $ttlid, $extract_id)
#
# parameters in: $dbh --- the database handle,
#                $srid_id --- the sampling_replicate_id for sample,
#                $ttlid --- sample tissue_type_lable_id
#                $extract_id --- extract_id
# parameter out: $msg --- the protocol_name in html table row format;
#
# X. Jing Li, 6/19/2006
###########################################################################
sub getSummaryInfo {
    my ($dbh, $srid, $ttlid, $extract_id) = @_;
    my ($sql, $sth, $rc, $sql1, $sth1, $rc1, $ptid, $pt_name, $count, $data_ok, $msg);

    $data_ok = 0;

    $sql = "select protocol_type_id, protocol_type_name from protocol_type where protocol_type_id in (select protocol_type_id from protocol where protocol_id in ( select distinct protocol_id from sp_summary where extract_id = $extract_id and label_id = $ttlid and sampling_point_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid)))";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() == 0) {
        $msg = "There is no summarized data for this array.\n";
        $data_ok = 1;
    } elsif ($sth->rows() == 1) {
        ($ptid, $pt_name) = $sth->fetchrow();
        $sql1 = "select count(*) from sp_summary where extract_id = $extract_id and label_id = $ttlid and sampling_point_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid)";
        $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
        $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
        $count = $sth1->fetchrow();
        $sum_msg = "There are $count records in sp_summsry\n";
    } elsif ($sth->rows() > 1) {
        $msg = "<B>Summarized data for this array:</B><br><TABLE BORDER=1><TR><TH>Process Protocol Name</TH><TH>Number of Records</TH></TR>\n";
        while (($ptid, $pt_name) = $sth->fetchrow() ) {
            $sql1 = "select count(*) from sp_summary where extract_id = $extract_id and label_id = $ttlid and sampling_point_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid)";
            $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
            $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
            $count = $sth1->fetchrow();    
            $msg .= "<TR ALIGN=CENTER><TD>$pt_name</TD><TD>$count</TD></TR>\n";
        }
        $msg .= "</TABLE><BR>\n";
        $sth1->finish;
    }
    $sth->finish;
    return $data_ok, $msg;
}

###########################################################################
# subroutine: getComparisonInfo
#           ($dbh, $srid, $ttlid, $extract_id)
#
# parameters in: $dbh --- the database handle,
#                $srid_id --- the sampling_replicate_id for sample,
#                $ttlid --- sample tissue_type_lable_id
#                $extract_id --- extract_id
# parameter out: $msg --- the protocol_name in html table row format;
#
# X. Jing Li, 6/19/2006
###########################################################################
sub getComparisonInfo {
    my ($dbh, $srid, $ttlid, $extract_id) = @_;
    my ($sql, $sth, $rc, $sql1, $sth1, $rc1, $sql2, $sth2, $rc2, $ptid, $pt_name, $count, $data_ok, $msg, $es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2, $process1_id);

    $data_ok = 0;

    $sql = "select protocol_type_id, protocol_type_name from protocol_type where protocol_type_id in (select protocol_type_id from protocol where protocol_id in ( select distinct protocol_id from sp_comparison where extract_id = $extract_id and process1_id in (select process1_id from process1 where (sampling_point1_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp1_tissue_type_label_id = $ttlid) or (sampling_point2_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp2_tissue_type_label_id = $ttlid))))";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() == 0) {
        $data_ok = 1;
        $msg = "There is no comparison data for this array.\n";
    } elsif ($sth->rows() >= 1) {   # there is at least 1 protocol_type_ids
        $msg = "<B>Comparison(s) related to this array:</B><br><TABLE BORDER=1><TR><TH ROWSPAN=2>Process Protocol Name</TH><TH COLSPAN=3>Sample 1</TH><TH COLSPAN=3>Sample 2</TH><TH ROWSPAN=2>Number of Records</TH></TR><TR><TH>Exp_set_name</TH><TH>Experiment</TH><TH>Time_Point</TH><TH>Exp_set_name</TH><TH>Experiment</TH><TH>Time_Point</TH></TR>\n";
        while (($ptid, $pt_name) = $sth->fetchrow()) {
            #-- get the distinct process1_id. If only one then write a msg in line
            #-- format. If more one process1_id are found create a table.
            $sql1 = "select distinct process1_id from sp_comparison where process1_id in (select process1_id from process1 where sampling_point1_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp1_tissue_type_label_id = $ttlid or sampling_point2_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp2_tissue_type_label_id = $ttlid)";
#            print"$sql1\n";
            $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
            $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
            while ($process1_id = $sth1->fetchrow()) {
                #-- get the sampling_point1_id and sampling_point2_id info:
                $sql2 = "select ES1.exp_set_name, E1.exp_name, L1.\"name\", ES2.exp_set_name, E2.exp_name, L2.\"name\" from experiment_set ES1, experiment_set ES2, experiment E1, experiment E2, label L1, label L2, sampling_point SP1, sampling_point SP2, process1 P where ES1.exp_set_id = E1.exp_set_id and SP1.experiment_id = E1.experiment_id and SP1.label_id = L1.label_id and SP1.sampling_point_id = P.sampling_point1_id and ES2.exp_set_id = E2.exp_set_id and E2.experiment_id = SP2.experiment_id and SP2.label_id = L2.label_id and SP2.sampling_point_id = P.sampling_point2_id and P.process1_id = $process1_id";
#                print"pt_name = $pt_name, process1_id = $process1_id\nsql2 = $sql2\n";
                $sth2 = $dbh-> prepare($sql2) or die "Can't prepare: $DBI::errstr.";
                $rc2 = $sth2->execute or die "Can't execute: $DBI::errstr.";
                ($es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2) = $sth2->fetchrow();
                #-- get the count of this protocol_type_id:
                $sql2 = "select count(*) from sp_comparison where extract_id = $extract_id and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and process1_id = $process1_id";
                $sth2 = $dbh-> prepare($sql2) or die "Can't prepare: $DBI::errstr.";
                $rc2 = $sth2->execute or die "Can't execute: $DBI::errstr.";
                $count = $sth2->fetchrow();
                if ($count > 0) {
                    $msg .= "<TR ALIGN=CENTER><TD>$pt_name</TD><TD>$es_name1</TD><TD>$exp_name1</TD><TD>$lname1</TD><TD>$es_name2</TD><TD>$exp_name2</TD><TD>$lname2</TD><TD>$count</TD></TR>\n";
                }
            }
        }
        $msg .= "</TABLE><br>\n";
    }
    $sth2->finish;
    $sth1->finish;
    $sth->finish;
    return $data_ok, $msg;
}

