#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script deletes the metabolite data and all its related processed data.
#
# sampling_replicate_id
# tissue_type_id
# technical_replicate
# extract_id
# force_delete
#
# Author: X. Jing Li
# Date: June 22, 2006
# Reversions: 
#===============================================================================

my ($database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc, 
    $sql1, $sth1, $rc1, 
    $srid, $ttlid, $tech_repl, $extract_id, $hybrid_prot_id, $scan_prot_id, $force_delete, $msg, $data_ok, $file_msg, $ptid, $pt_name, $count, $norm_msg, $sum_msg, $comp_msg, $mmid, $proc_fllid, $raw_fllid, @items);

#== ARGV
($srid, $ttlid, $tech_repl, $extract_id, $force_delete) = @ARGV;
#print "@ARGV\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

if ($force_delete == 0) {
    #== array information:
    ($data_ok, $file_msg) = getMetaboliteInfo($dbh, $srid, $ttlid, $tech_repl, $extract_id);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $file_msg;
        exit 1;
    }

    #== check the processed data detail:
    #-- 1. normalized data:
    ($data_ok, $norm_msg) = getMetaboliteNormalizedInfo($dbh, $srid, $ttlid, $tech_repl, $extract_id);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $file_msg . $norm_msg . "Are you sure you want to delete the metabolite data from DOME?";
        exit 99;
    }

    #-- 2. sp_summary:
    ($data_ok, $sum_msg) = getSummaryInfo($dbh, $srid, $ttlid, $extract_id);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $file_msg . $norm_msg . $sum_msg . "Are you sure you want to delete the data from DOME?";
        exit 99;
    }

    #-- 3. sp_comparison:
    ($data_ok, $comp_msg) = getComparisonInfo($dbh, $srid, $ttlid, $extract_id);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $file_msg . $norm_msg . $sum_msg . $comp_msg . "If you delete the data for this sample, you will deleted all the summarized data as well. Are you sure you want to delete the data from DOME?";
        exit 99;
    }
    $msg .= $file_msg . $norm_msg . $sum_msg . $comp_msg . "If you delete the data for this sample, you will deleted all the summarized and comparison data as well. Are you sure you want to delete the data from DOME?";
    print $msg;
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    exit 99;
} elsif ($force_delete == 1) {
    #-- 1. delete data from sp_comparison:
    $sql = "delete from sp_comparison where extract_id = $extract_id and process1_id in (select process1_id from process1 where (sampling_point1_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp1_tissue_type_label_id = $ttlid) or (sampling_point2_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp2_tissue_type_label_id = $ttlid))";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc =~ /^\d/ && $rc !~ /\W/) {
        if ($rc != 0) {
            $msg = "$rc records deleted from comparison data.<br>";
        }
    } else {
        $msg = "There are some mistakes occured when trying to delete the comparison data. NO data deleted in database.<br>";
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }
#    print "delete from sp_comparison: \n$sql\nrc = $rc\n//\n";

    #-- 2. delete from sp_summary:
    $sql = "delete from sp_summary where extract_id = $extract_id and label_id = $ttlid and sampling_point_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid)";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc =~ /^\d/ && $rc !~ /\W/) {
        if ($rc != 0) {
            $msg = "$rc records deleted from summarized data.<br>" . $msg;
        }
    } else {
        $msg = "There are some mistakes occured when trying to delete the summarized data. NO data deleted in database.<br>";
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }
#    print "delete from sp_summary: \n$sql\nrc = $rc\n//\n";

    #-- 4. delete from sample, sample_extract, metabolite_measurement,
    #-- metabolite_info, metabolite_derivative_exp_set, metabolite_processed2
    $sql = "select metabolite_measurement_id, raw_file_location_link_id, proc_file_location_link_id from metabolite_measurement where sample_extract_id = (select sample_extract_id from sample_extract where extract_id = $extract_id and technical_replicate = $tech_repl and sample_id in (select sample_id from \"sample\" where label_id = $ttlid and sampling_replicate_id = $srid))";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($mmid, $raw_fllid, $proc_fllid) = $sth->fetchrow();
#    print "iaid = $iaid, cel_fllid cel_fllid, rdid = $rdid, hybrid_id = $hid, array_id = $array_id, leid1 = $leid\n";

    #-- 4.1 delete sample_id:
#    $sql = "select * from sample where sample_id = (select sample_id from sample_extract where sample_extract_id = (select sample_extract_id from labeled_extract where labeled_extract_id = $leid))";
#    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
#    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#    @items = $sth->fetchrow();
#    print "sample: @items\n//\n";
    $sql = "delete from sample where sample_id = (select sample_id from sample_extract where sample_extract_id = (select sample_extract_id from metabolite_measurement where metabolite_measurement_id = $mmid))";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#    print "sample: \n$sql\nrc = $rc\n//\n";

    #-- 4.2 delete sample_extract_ids:
    #$sql = "select * from sample_extract where sample_extract_id = (select sample_extract_id from labeled_extract where labeled_extract_id = $leid)";
    #$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    #$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    #@items = $sth->fetchrow();
    #print "sample_extract: @items\n//\n";
    $sql = "delete from sample_extract where sample_extract_id = (select sample_extract_id from  metabolite_measurement where metabolite_measurement_id = $mmid)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#    print "sample_extract: \n$sql\nrc = $rc\n//\n";

    #-- 4.3 delete from metabolite_processed2
    $sql = "delete from metabolite_processed2 where metabolite_measurement_id = $mmid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc =~ /^\d/ && $rc !~ /\W/) {
        if ($rc != 0) {
            $msg = "$rc records deleted from metabolite data.<br>" . $msg;
        }
    } else {
        $msg = "There are some mistakes occured when trying to delete the metabolite data from DOME. NO data deleted in database.<br>";
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }
#    print "metabolite_processed2: \n$sql\nrc = $rc\n//\n";
 
    #-- 4.4 delete from metabolite_derivative_exp_set:
    $sql = "delete from metabolite_derivative_exp_set where metabolite_info_id in (select metabolite_info_id from metabolite_info where metabolite_measurement_id = $mmid)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#    print "metabolite_derivative_exp_set: \n$sql\nrc = $rc\n//\n";

    #-- 4.5 delete from metabolite_info:
    $sql = "delete from metabolite_info where metabolite_measurement_id = $mmid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#    print "metabolite_info: \n$sql\nrc = $rc\n//\n";

    #-- 4.6 delete from metabolite_measurement:
    #$sql = "delete from metabolite_measurement where sample_extract_id = (select sample_extract_id from sample_extract where extract_id = $extract_id and technical_replicate = $tech_repl and sample_id in (select sample_id from \"sample\" where label_id = $ttlid and sampling_replicate_id = $srid))";
    $sql = "delete from metabolite_measurement where metabolite_measurement_id = $mmid";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#    @items = $sth->fetchrow();
#    print "metabolite_measurement: \n$sql\nrc = $rc\n//\n";
 
    #-- 4.7 delete from file:
    $sql = "delete from file where file_id = (select file_id from file_location_link where file_location_link_id = $proc_fllid)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#    print "file: \n$sql\nrc = $rc\n";
    $sql = "delete from file_location_link where file_location_link_id = $proc_fllid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#    print "file_location_link: \n$sql\nrc = $rc\n//\n";
    if ($raw_fllid ne "") {
        $sql = "delete from file where file_id = (select file_id from file_location_link where file_location_link_id = $raw_fllid)";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#        print "file: \n$sql\nrc = $rc\n";
        $sql = "delete from file_location_link where file_location_link_id = $raw_fllid";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#        print "file_location_link: \n$sql\nrc = $rc\n//\n";
    }

    print $msg;
    $sth->finish;
    $dbh->commit;
    #$dbh->rollback;
    $dbh->disconnect;
    exit;
}

###########################################################################
# subroutine: getMetaboliteInfo($dbh, $srid, $ttlid, $tech_repl, $extract_id)
#
# parameters in: $dbh --- the database handle,
#                $srid --- the sampling_replicate_id,
#                $ttlid -- tissue_type_label_id;
#                $tech_repl -- technical replicate
#                $extract_id -- extract_id
# parameter out: $data_ok -- 0 for there is data and 1 for no data
#                $msg --- the detailed message for the array;
#
# X. Jing Li, 6/22/2006
###########################################################################
sub getMetaboliteInfo {
    my ($dbh, $srid, $ttlid, $tech_repl, $extract_id) = @_;
    my ($sql, $sth, $rc, $es_name, $exp_name, $lname, $ext_name, $bio_repl, $mmid, $ttlid_name, $raw_fllid, $proc_fllid, $proc_file_name, $time, $person, $raw_file_name, $count, $data_ok, $msg, $count);

    $data_ok = 0;

    #== get sample detail:
    $sql = "select distinct ES.exp_set_name, E.exp_name, L.name, EXT.\"name\", SR.replicate, MM.metabolite_measurement_id from experiment_set ES, experiment E, label L, sampling_point SP, sampling_replicate SR, \"sample\" S, sample_extract SE, extract EXT, metabolite_measurement MM where MM.sample_extract_id = SE.sample_extract_id and SE.extract_id = EXT.extract_id and EXT.extract_id = $extract_id and SE.technical_replicate = $tech_repl and S.sample_id = SE.sample_id and S.label_id = $ttlid and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_replicate_id = $srid and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($es_name, $exp_name, $lname, $ext_name, $bio_repl, $mmid) = $sth->fetchrow(); 
    if ($sth->rows == 0) {
        $sql = "select ES.exp_set_name, E.exp_name, L.name, SR.replicate from experiment_set ES, experiment E, label L, sampling_point SP, sampling_replicate SR where SR.sampling_replicate_id = $srid and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        ($es_name, $exp_name, $lname, $bio_repl) = $sth->fetchrow();

        $sql = "select \"name\" from extract where extract_id = $extract_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        $ext_name = $sth->fetchrow();
        $msg = "There is no metabolite data for experiment_set = \"$es_name\", experiment = \"$exp_name\", time_point = \"$lname\", biological_replicate = \"$bio_repl\", technical_replicate = \"$tech_repl\", extraction = \"$ext_name\".<br>";
        $sth->finish;
        $data_ok = 1;
        return $data_ok, $msg;
    }

    #-- get tissue_type_label_names:
    $sth = $dbh-> prepare("select \"name\" from label where label_id = $ttlid") or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $ttlid_name = $sth->fetchrow();

    #-- get file_ids:
    $sql = "select raw_file_location_link_id, proc_file_location_link_id from metabolite_measurement where metabolite_measurement_id = $mmid";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($raw_fllid, $proc_fllid) = $sth->fetchrow();
#    print "raw_fllid = $raw_fllid, proc_fllid = $proc_fllid\n";
    ($proc_file_name, $time, $person) = getFileInfoFromFLLid($dbh, $proc_fllid);
    if ($raw_fllid ne "") {
        ($raw_file_name, $time, $person) = getFileInfoFromFLLid($dbh, $raw_fllid);
    }

    $msg = "<h3>Summary about the Sample:</h3>";
    $msg .= "Metadata about the sample:<CENTER><TABLE BORDER=1>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Exp_set Name</b></TD><TD>$es_name</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Experiment Name</b></TD><TD>$exp_name</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Time Point</b></TD><TD>$lname</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Tissue Type</b></TD><TD>$ttlid_name</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Biological Replicate</b></TD><TD>$bio_repl</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Technical Replicate</b></TD><TD>$tech_repl</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Processed File Name</b></TD><TD>$proc_file_name</TD></TR>";
    if ($raw_file_name ne "") {
        $msg .= "<TR ALIGN=CENTER><TD><b>Raw File Name</b></TD><TD>$raw_file_name</TD></TR>";
    }
    $msg .= "<TR ALIGN=CENTER><TD><b>Uploading Time</b></TD><TD>$time</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Researcher Name</b></TD><TD>$person</TD></TR>";
    $msg .= "</TABLE></CENTER><BR>";
    #print $msg . "\n";
    $sth->finish;
    return $data_ok, $msg;
}
    
###########################################################################
# subroutine: getFileInfoFromFLLid($dbh, $file_location_link_id)
#
# parameters in: $dbh --- the database handle,
#                $file_location_link_id --
# parameter out: $file_name --- the file name
#                $time_uploaded -- the time the file has been uploaded
#                $person -- researcher's name
#
# X. Jing Li, 6/14/2006
###########################################################################
sub getFileInfoFromFLLid {
    my ($dbh, $fllid) = @_;
    my ($sth, $rc, $file_name, $time_uploaded, $person_name);

    $sql = "select F.file_name, F.time_uploaded, P.person_name from file F, file_location_link FLL, person P where FLL.file_location_link_id = $fllid and FLL.file_id = F.file_id and F.person_id = P.person_id";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() > 0) {
        ($file_name, $time_uploaded, $person_name) = $sth->fetchrow();
    } else {
        $sql = "select F.file_name from file F, file_location_link FLL where FLL.file_location_link_id = $fllid and FLL.file_id = F.file_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        $file_name = $sth->fetchrow();
        $time_uploaded = "-";
        $person_name = "-";
    }

    $sth->finish;
    return $file_name, $time_uploaded, $person_name;
}
   
###########################################################################
# subroutine: getMetaboliteNormalizedInfo
#           ($dbh, $srid1, $ttlid1, $srid2, $ttlid2, $tech_repl, $extract_id)
#
# parameters in: $dbh --- the database handle,
#                $srid_id --- the sampling_replicate_id for sample,
#                $ttlid --- sample tissue_type_lable_id
#                $tech_repl --- technical_replicate of this sample
#                $extract_id --- extract_id
# parameter out: $msg --- the protocol_name in html table row format;
#
# X. Jing Li, 6/19/2006
###########################################################################
sub getMetaboliteNormalizedInfo {
    my ($dbh, $srid, $ttlid, $tech_repl, $extract_id) = @_;
    my ($sql, $sth, $rc, $sql1, $sth1, $rc1, $ptid, $pt_name, $data_ok, $msg);

    $data_ok = 0;

    $sql = "select protocol_type_id, protocol_type_name from protocol_type where protocol_type_id in (select distinct protocol_type_id from metabolite_processed2 where metabolite_measurement_id in ( select metabolite_measurement_id from metabolite_measurement where sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and technical_replicate = $tech_repl and sample_id in (select sample_id from \"sample\" where label_id = $ttlid and sampling_replicate_id = $srid))))";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() == 0) {
        $msg = "There is no normalized data for this sample.";
        $data_ok = 1;
    } else {
        $msg = "Normalized Data for this sample: <CENTER><TABLE BORDER=1><TR><TH>Process Protocol Name</TH><TH>Number of Records</TH></TR>";
        while (($ptid, $pt_name) = $sth->fetchrow() ) {
            $sql1 = "select count(*) from metabolite_processed2 where protocol_type_id = $ptid and metabolite_measurement_id in (select metabolite_measurement_id from metabolite_measurement where sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and technical_replicate = $tech_repl and sample_id in (select sample_id from \"sample\" where label_id = $ttlid and sampling_replicate_id = $srid)))";
            $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
            $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
            $count = $sth1->fetchrow();
            $msg .= "<TR ALIGN=CENTER><TD>$pt_name</TD><TD>$count</TD></TR>";
        }
        $msg .= "</TABLE><CENTER><BR>";
        $sth1->finish;
    }
    $sth->finish;
    return $data_ok, $msg;
}

###########################################################################
# subroutine: getSummaryInfo
#           ($dbh, $srid, $ttlid, $tech_repl, $extract_id)
#
# parameters in: $dbh --- the database handle,
#                $srid_id --- the sampling_replicate_id for sample,
#                $ttlid --- sample tissue_type_lable_id
#                $extract_id --- extract_id
# parameter out: $msg --- the protocol_name in html table row format;
#
# X. Jing Li, 6/19/2006
###########################################################################
sub getSummaryInfo {
    my ($dbh, $srid, $ttlid, $extract_id) = @_;
    my ($sql, $sth, $rc, $sql1, $sth1, $rc1, $ptid, $pt_name, $data_ok, $msg);

    $data_ok = 0;

    $sql = "select protocol_type_id, protocol_type_name from protocol_type where protocol_type_id in (select protocol_type_id from protocol where protocol_id in ( select distinct protocol_id from sp_summary where extract_id = $extract_id and label_id = $ttlid and sampling_point_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid)))";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() == 0) {
        $msg = "There is no summarized data for this sample.";
        $data_ok = 1;
    } elsif ($sth->rows() >= 1) {
        $msg = "Summarized data for this sample:<CENTER><TABLE BORDER=1><TR><TH>Process Protocol Name</TH><TH>Number of Records</TH></TR>";
        while (($ptid, $pt_name) = $sth->fetchrow() ) {
            $sql1 = "select count(*) from sp_summary where extract_id = $extract_id and label_id = $ttlid and sampling_point_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid)";
            $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
            $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
            $count = $sth1->fetchrow();    
            $msg .= "<TR ALIGN=CENTER><TD>$pt_name</TD><TD>$count</TD></TR>";
        }
        $msg .= "</TABLE></CENTER><BR>";
        $sth1->finish;
    }
    $sth->finish;
    return $data_ok, $msg;
}

###########################################################################
# subroutine: getComparisonInfo
#           ($dbh, $srid, $ttlid, $extract_id)
#
# parameters in: $dbh --- the database handle,
#                $srid_id --- the sampling_replicate_id for sample,
#                $ttlid --- sample tissue_type_lable_id
#                $extract_id --- extract_id
# parameter out: $msg --- the protocol_name in html table row format;
#
# X. Jing Li, 6/19/2006
###########################################################################
sub getComparisonInfo {
    my ($dbh, $srid, $ttlid, $extract_id) = @_;
    my ($sql, $sth, $rc, $sql1, $sth1, $rc1, $sql2, $sth2, $rc2, $ptid, $pt_name, $data_ok, $msg,
        $es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2, $process1_id);

    $data_ok = 0;

    $sql = "select protocol_type_id, protocol_type_name from protocol_type where protocol_type_id in (select protocol_type_id from protocol where protocol_id in ( select distinct protocol_id from sp_comparison where extract_id = $extract_id and process1_id in (select process1_id from process1 where (sampling_point1_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp1_tissue_type_label_id = $ttlid) or (sampling_point2_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp2_tissue_type_label_id = $ttlid))))";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() == 0) {
        $data_ok = 1;
        $msg = "There is no comparison data for this array.";
    } elsif ($sth->rows() >= 1) {   # there is at least 1 protocol_type_ids
        $msg = "Comparison(s) related to this array:<CENTER><TABLE BORDER=1><TR><TH ROWSPAN=2>Process Protocol Name</TH><TH COLSPAN=3>Sample 1</TH><TH COLSPAN=3>Sample 2</TH><TH ROWSPAN=2>Number of Records</TH></TR><TR><TH>Exp_set_name</TH><TH>Experiment</TH><TH>Time_Point</TH><TH>Exp_set_name</TH><TH>Experiment</TH><TH>Time_Point</TH></TR>";
        while (($ptid, $pt_name) = $sth->fetchrow()) {
#            print "ptid = $ptid, pt_name = $pt_name\n";
            #-- get the distinct process1_id. If only one then write a msg in line
            #-- format. If more one process1_id are found create a table.
            $sql1 = "select distinct process1_id from sp_comparison where process1_id in (select process1_id from process1 where sampling_point1_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp1_tissue_type_label_id = $ttlid or sampling_point2_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp2_tissue_type_label_id = $ttlid)";
#            print"$sql1\n";
            $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
            $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
            while ($process1_id = $sth1->fetchrow()) {
                #-- get the sampling_point1_id and sampling_point2_id info:
                $sql2 = "select ES1.exp_set_name, E1.exp_name, L1.\"name\", ES2.exp_set_name, E2.exp_name, L2.\"name\" from experiment_set ES1, experiment_set ES2, experiment E1, experiment E2, label L1, label L2, sampling_point SP1, sampling_point SP2, process1 P where ES1.exp_set_id = E1.exp_set_id and SP1.experiment_id = E1.experiment_id and SP1.label_id = L1.label_id and SP1.sampling_point_id = P.sampling_point1_id and ES2.exp_set_id = E2.exp_set_id and E2.experiment_id = SP2.experiment_id and SP2.label_id = L2.label_id and SP2.sampling_point_id = P.sampling_point2_id and P.process1_id = $process1_id";
#                print"pt_name = $pt_name, process1_id = $process1_id\nsql2 = $sql2\n";
                $sth2 = $dbh-> prepare($sql2) or die "Can't prepare: $DBI::errstr.";
                $rc2 = $sth2->execute or die "Can't execute: $DBI::errstr.";
                ($es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2) = $sth2->fetchrow();
                #-- get the count of this protocol_type_id:
                $sql2 = "select count(*) from sp_comparison where extract_id = $extract_id and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and process1_id = $process1_id";
                $sth2 = $dbh-> prepare($sql2) or die "Can't prepare: $DBI::errstr.";
                $rc2 = $sth2->execute or die "Can't execute: $DBI::errstr.";
                $count = $sth2->fetchrow();
                if ($count > 0) {
                    $msg .= "<TR ALIGN=CENTER><TD>$pt_name</TD><TD>$es_name1</TD><TD>$exp_name1</TD><TD>$lname1</TD><TD>$es_name2</TD><TD>$exp_name2</TD><TD>$lname2</TD><TD>$count</TD></TR>";
                }
            }
        }
        $msg .= "</TABLE></CENTER>,BR>";
    }
    $sth2->finish;
    $sth1->finish;
    $sth->finish;
    return $data_ok, $msg;
}

