#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script deletes the cDNA and Affy normalized data and all its related 
# processed data.
#
# sampling_replicate_id
# tissue_type_id
# technical_replicate
# protocol_type_id
# extract_id
#
# Author: X. Jing Li
# Date: June 29, 2006
# Reversions: 
#===============================================================================

my ($database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc, $sql1, $sth1, $rc1, 
    $srid, $ttlid, $tech_repl, $extract_id, $hybrid_prot_id, $scan_prot_id, $ptid, $force_delete, $msg, $array_msg, $norm_msg, $sum_msg, $comp_msg, $data_ok);

#== ARGV
($srid, $ttlid, $tech_repl, $extract_id, $ptid, $force_delete) = @ARGV;

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

if ($force_delete == 0) {
    #== check the processed data detail:
    ($data_ok, $array_msg) = getAffyArrayInfoWithPtid($dbh, $srid, $ttlid, $tech_repl, $extract_id, $ptid);

    #-- 1. normalized data:
    ($data_ok, $norm_msg) = getArrayNormalizedInfoWithPtid($dbh, $srid, $ttlid, $tech_repl, $extract_id, $ptid);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $array_msg . $norm_msg;
        exit 1;
    }

    #-- 2. sp_summary:
    ($data_ok, $sum_msg) = getSummaryInfoWithPtid($dbh, $srid, $ttlid, $extract_id, $ptid);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $array_msg . $norm_msg . $sum_msg . "Are you sure you want to delete the normalized data for this array from DOME?";
        exit 99;
    }

    #-- 3. sp_comparison:
    ($data_ok, $comp_msg) = getComparisonInfoWithPtid($dbh, $srid, $ttlid, $extract_id, $ptid);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $array_msg . $norm_msg . $sum_msg . $comp_msg . "If you delete the normalized data for this array, you will deleted all the summarized data for it as well. Are you sure you want to delete the data from DOME?";
        exit 99;
    }
    print $array_msg . $norm_msg . $sum_msg . $comp_msg . "If you delete the normalized data for this array, you will deleted all the summarizzed and comparison data as well. Are you sure you want to delete the data from DOME?";
    $sth->finish;
    $dbh->rollback;
    $dbh->disconnect;
    exit 99;
} elsif ($force_delete == 1) {
    #-- 1. delete data from sp_comparison:
    $sql = "delete from sp_comparison where process1_id in (select process1_id from process1 where (sampling_point1_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp1_tissue_type_label_id = $ttlid) or (sampling_point2_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp2_tissue_type_label_id = $ttlid)) and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and extract_id = $extract_id";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc =~ /^\d/ && $rc !~ /\W/) {
        if ($rc != 0) {
            $msg = "$rc records deleted from comparison data.<br><br>";
        }
    } else {
        $msg = "There are some mistakes occured when trying to delete the comparison data for Affymetrix data from DOME. NO data changed in databaase.<br><br>";
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }        

    #-- 2. delete from sp_summary:
    $sql = "delete from sp_summary where sampling_point_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and extract_id = $extract_id and label_id = $ttlid";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc =~ /^\d/ && $rc !~ /\W/) {
        if ($rc != 0) {
            $msg = "$rc records deleted from summarized data.<br><br>" . $msg;
        }
    } else {
        $msg = "There are some mistakes occured when trying to delete the summarizeddata for normalized Affymetrix data from DOME. NO data deleted in database.<br><br>";
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }
#    print "delete from sp_summary: rc = $rc\n";
    #print "sp_summary count: " . $sth->fetchrow() . "\n//\n";

    #-- 3. delete from ma_norm:
    $sql = "delete from ma_norm where sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and technical_replicate = $tech_repl and sample_id in (select sample_id from \"sample\" where label_id = $ttlid and sampling_replicate_id = $srid)) and protocol_type_id = $ptid";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc =~ /^\d/ && $rc !~ /\W/) {
        if ($rc != 0) {
            $msg = "$rc records deleted from normalized data.<br><br>" . $msg;
        }
    } else {
        $msg = "There are some mistakes occured when trying to delete the normalized Affymetrix data from DOME. NO data deleted in database.<br><br>";
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }
    print $msg;
    $sth->finish;
    #$dbh->rollback;
    $dbh->commit;
    $dbh->disconnect;
    exit 0;
}

###########################################################################
# subroutine:
#  getAffyArrayInfoWithPtid($dbh, $srid, $ttlid, $tech_repl, $extract_id)
#
# parameters in: $dbh --- the database handle,
#                $srid -- sampling_replicate_id
#                $ttlid -- tissue_type_label_id
#                $tech_repl -- technical replicate
#                $extract -- extract id
#                $ptid --- the protocol_type_id,
# parameter out: $data_ok -- 0 for successful and 1 for failed
#                $msg --- the detail info for the array;
#
# X. Jing Li, 6/29/2006
###########################################################################
sub getAffyArrayInfoWithPtid {
    my ($dbh, $srid, $ttlid, $tech_repl, $extract_id, $ptid) = @_;
    my ($sql, $sth, $rc, $es_name, $exp_name, $lname, $ext_name, $bio_repl, $ttlid_name, $array_batch_name, $array_name, $cel_fllid, $cel_file_name, $time, $person, $person, $data_ok, $msg, $iaid, $count);

    $data_ok = 0;

    #== get sample detail:
    $sql = "select distinct ES.exp_set_name, E.exp_name, L.name, EXT.\"name\", SR.replicate from experiment_set ES, experiment E, label L, sampling_point SP, sampling_replicate SR, \"sample\" S, sample_extract SE, labeled_extract LE, hybridization H, extract EXT where H.labeled_extract1_id = LE.labeled_extract_id and LE.sample_extract_id = SE.sample_extract_id and SE.extract_id = EXT.extract_id and EXT.extract_id = $extract_id and S.sample_id = SE.sample_id and S.label_id = $ttlid and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_replicate_id = $srid and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($es_name, $exp_name, $lname, $ext_name, $bio_repl) = $sth->fetchrow(); 
    if ($sth->rows() == 0) {
        $sql = "select ES.exp_set_name, E.exp_name, L.name, SR.replicate from experiment_set ES, experiment E, label L, sampling_point SP, sampling_replicate SR where SR.sampling_replicate_id = $srid and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        ($es_name, $exp_name, $lname, $bio_repl) = $sth->fetchrow();

        $sql = "select \"name\" from extract where extract_id = $extract_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        $ext_name = $sth->fetchrow();

        $msg = "There is no Affymetrix microarray for experiment_set = \"$es_name\", experiment = \"$exp_name\", time_point = \"$lname\", biological_replicate = \"$bio_repl\",extraction name = \"$ext_name\".";
        $sth->finish;
        $data_ok = 1;
        return $data_ok, $msg;
    }

    #-- get tissue_type_label_names:
    $sth = $dbh-> prepare("select \"name\" from label where label_id = $ttlid") or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $ttlid_name = $sth->fetchrow();

    #-- get file_ids:
    $sql = "select AB.array_design_name, A.array_name, IA.file_location_link_id, IA.image_analysis_id from \"array\" A, array_batch AB, raw_data RD, image_analysis IA, hybridization H where H.labeled_extract1_id in (select labeled_extract_id from labeled_extract where sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and sample_id in (select sample_id from \"sample\" where sampling_replicate_id = $srid and label_id = $ttlid))) and H.hybridization_id = RD.hybridization_id and RD.raw_data_id = IA.raw_data_id and H.array_id = A.array_id and AB.array_batch_id = A.array_batch_id";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($array_batch_name, $array_name, $cel_fllid, $iaid) = $sth->fetchrow();
    ($cel_file_name, $time, $person) = getFileInfoFromFLLid($dbh, $cel_fllid);

    $msg = "<h3>Summary about the array:</h3><b>Array Name:</b> $array_name<br><b>Data File Name:</b>$cel_file_name<br><b>Array Design Name:</b> $array_batch_name<br><br>";
    $msg .= "<B>Metadata about the array:</B><br><CENTER><TABLE BORDER=1>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Exp_set Name</b></TD><TD>$es_name</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Experiment Name</b></TD><TD>$exp_name</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Time Point</b></TD><TD>$lname</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Tissue Type</b></TD><TD>$ttlid_name</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Biological Replicate</b></TD><TD>$bio_repl</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Technical Replicate</b></TD><TD>$tech_repl</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Uploading Time</b></TD><TD>$time</TD></TR>";
    $msg .= "<TR ALIGN=CENTER><TD><b>Researcher Name</b></TD><TD>$person</TD></TR>";
    $msg .= "</TABLE></CENTER><BR>";

    $sth->finish;
    return $data_ok, $msg;
}
    
###########################################################################
# subroutine: getFileInfoFromFLLid($dbh, $file_location_link_id)
#
# parameters in: $dbh --- the database handle,
#                $file_location_link_id --
# parameter out: $file_name --- the file name
#                $time_uploaded -- the time the file has been uploaded
#                $person -- researcher's name
#
# X. Jing Li, 6/14/2006
###########################################################################
sub getFileInfoFromFLLid {
    my ($dbh, $fllid) = @_;
    my ($sth, $rc, $file_name, $time_uploaded, $person_name);

    $sql = "select F.file_name, F.time_uploaded, P.person_name from file F, file_location_link FLL, person P where FLL.file_location_link_id = $fllid and FLL.file_id = F.file_id and F.person_id = P.person_id";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() > 0) {
        ($file_name, $time_uploaded, $person_name) = $sth->fetchrow();
    } else {
        $sql = "select F.file_name from file F, file_location_link FLL where FLL.file_location_link_id = $fllid and FLL.file_id = F.file_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        $file_name = $sth->fetchrow();
        $time_uploaded = "-";
        $person_name = "-";
    }

    $sth->finish;
    return $file_name, $time_uploaded, $person_name;
}
   
###########################################################################
# subroutine: getArrayNormalizedInfoWithPtid
#           ($dbh, $srid, $ttlid, $tech_repl, $extract_id, $ptid)
#
# parameters in: $dbh --- the database handle,
#                $srid_id --- the sampling_replicate_id for sample,
#                $ttlid --- sample tissue_type_lable_id
#                $tech_repl --- technical_replicate of this sample
#                $extract_id --- extract_id
#                $ptid -- protocol_type_id
# parameter out: $data_ok -- 0 for success and 1 for failed
#                $msg --- the protocol_name in html table row format;
#
# X. Jing Li, 6/29/2006
###########################################################################
sub getArrayNormalizedInfoWithPtid {
    my ($dbh, $srid, $ttlid, $tech_repl, $extract_id, $ptid) = @_;
    my ($sql, $sth, $rc, $pt_name, $count, $data_ok, $msg);

    $data_ok = 0;
    $msg = "<b>Normalized Data:</b><br>";

    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pt_name = $sth->fetchrow();

    $sql = "select count(*) from ma_norm where protocol_type_id = $ptid and sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and technical_replicate = $tech_repl and sample_id in (select sample_id from \"sample\" where label_id = $ttlid and sampling_replicate_id = $srid))";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $count = $sth->fetchrow();
    if ($count > 0) {
        $msg .= "There are $count records in DOME which are normalised by \"$pt_name\".<br>";
    } else {
        $data_ok = 1;
        $msg .= "There are no normalized data for this array using protocol \"$pt_name\".<br>";
    }
    $msg .= "<BR>";
    $sth->finish;
    return $data_ok, $msg;
}

###########################################################################
# subroutine: getSummaryInfoWithPtid
#           ($dbh, $srid, $ttlid, $extract_id, $ptid)
#
# parameters in: $dbh --- the database handle,
#                $srid_id --- the sampling_replicate_id for sample,
#                $ttlid --- sample tissue_type_lable_id
#                $extract_id --- extract_id
#                $ptid -- protocol_type_id
# parameter out: $msg --- the protocol_name in html table row format;
#                $data_ok -- 0 for success and 1 for failed
#
# X. Jing Li, 6/29/2006
###########################################################################
sub getSummaryInfoWithPtid {
    my ($dbh, $srid, $ttlid, $extract_id, $ptid) = @_;
    my ($sql, $sth, $rc, $pt_name, $count, $data_ok, $msg);

    $data_ok = 0;
    $msg = "<b>Summarized Data:</b><br>";
    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pt_name = $sth->fetchrow();

    $sql = "select count(*) from sp_summary where sampling_point_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and extract_id = $extract_id and label_id = $ttlid";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $count = $sth->fetchrow();
    if ($count > 0) {
        $msg .= "There are $count records in DOME using protocol \"$pt_name\".<br>";
    } else {
        $data_ok = 1;
        $msg .= "There is no summerized data for this array using protocol \"$pt_name\".<br>";
    }
    $msg .= "<BR>";
    $sth->finish;
    return $data_ok, $msg;
}

###########################################################################
# subroutine: getComparisonInfoWithPtid
#           ($dbh, $srid, $ttlid, $extract_id, $ptid)
#
# parameters in: $dbh -- the database handle,
#                $srid_id -- the sampling_replicate_id for sample,
#                $ttlid -- sample tissue_type_lable_id
#                $extract_id -- extract_id
#                $pt-d -- protocol_type_id
# parameter out: $data_ok -- 0 for success and 1 for failed
#                $msg -- the protocol_name in html table row format;
#
# X. Jing Li, 6/29/2006
###########################################################################
sub getComparisonInfoWithPtid {
    my ($dbh, $srid, $ttlid, $extract_id, $ptid) = @_;
    my ($sql, $sth, $rc, $sql1, $sth1, $rc1, $pt_name, $count, $data_ok, $msg,
        $es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2, $process1_id);

    $data_ok = 0;
    $msg = "<b>Comparison Data:</B><br>";

    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pt_name = $sth->fetchrow();

    #-- get the distinct process1_id. If only one then write a msg in line
    #-- format. If more one process1_id are found create a table.
    $sql = "select distinct process1_id from sp_comparison where process1_id in (select process1_id from process1 where sampling_point1_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp1_tissue_type_label_id = $ttlid or sampling_point2_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and sp2_tissue_type_label_id = $ttlid)";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() > 0) {
        $msg .= "<CENTER><TABLE BORDER=1><TR><TH COLSPAN=3>Sample 1</TH><TH COLSPAN=3>Sample 2</TH><TH ROWSPAN=2>Number of Records</TH></TR><TR><TH>Exp_set_name</TH><TH>Experiment</TH><TH>Time_Point</TH><TH>Exp_set_name</TH><TH>Experiment</TH><TH>Time_Point</TH></TR>";
        while ($process1_id = $sth->fetchrow()) {
        #-- get the sampling_point1_id and sampling_point2_id info:
            $sql1 = "select ES1.exp_set_name, E1.exp_name, L1.\"name\", ES2.exp_set_name, E2.exp_name, L2.\"name\" from experiment_set ES1, experiment_set ES2, experiment E1, experiment E2, label L1, label L2, sampling_point SP1, sampling_point SP2, process1 P where ES1.exp_set_id = E1.exp_set_id and SP1.experiment_id = E1.experiment_id and SP1.label_id = L1.label_id and SP1.sampling_point_id = P.sampling_point1_id and ES2.exp_set_id = E2.exp_set_id and E2.experiment_id = SP2.experiment_id and SP2.label_id = L2.label_id and SP2.sampling_point_id = P.sampling_point2_id and P.process1_id = $process1_id";
#        print"pt_name = $pt_name, process1_id = $process1_id\nsql1 = $sql1\n";
            $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
            $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
            ($es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2) = $sth1->fetchrow();
#        print "$es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2\n";
            #-- get the count of this protocol_type_id:
            $sql1 = "select count(*) from sp_comparison where extract_id = $extract_id and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and process1_id = $process1_id";
#            print "$sql1\n";
            $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
            $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
            $count = $sth1->fetchrow();
#            print "count = $count\n";
            if ($count > 0) {
                $msg .= "<TR ALIGN=CENTER><TD>$es_name1</TD><TD>$exp_name1</TD><TD>$lname1</TD><TD>$es_name2</TD><TD>$exp_name2</TD><TD>$lname2</TD><TD>$count</TD></TR>";
            }
        }
        $msg .= "</TABLE></CENTER><br>";
    } else {
        $msg .= "There is no comparison data for protocol \"$pt_name\".<br>";
        $data_ok = 1;
    }
    $sth1->finish;
    $sth->finish;
    return $data_ok, $msg;
}

