#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script deletes the cDNA and Affy normalized data and all its related 
# processed data.
#
# sampling_point1_id -- sampling_point_id for numeriter
# tissue_type_id_1
# sampling_point2_id -- sampling_point_id for demominator
# tissue_type_id_2
# extract_id
# protocol_type_id
#
# Author: X. Jing Li
# Date: June 30, 2006
# Reversions: 
#===============================================================================

my ($database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc,
    $spid1, $ttlid1, $spid2, $ttlid2, $extract_id, $ptid, $force_delete, $msg,  $sample_msg, $comp_msg, $data_ok);

#== ARGV
($spid1, $ttlid1, $spid2, $ttlid2, $extract_id, $ptid, $force_delete) = @ARGV;

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

if ($force_delete == 0) {
    #== check the processed data detail:
    ($data_ok, $sample_msg) = getMetadata($dbh, $ttlid1, $ttlid2, $extract_id, $ptid);

    #-- 2. sp_comparison:
    ($data_ok, $comp_msg) = getComparisonInfoWithPtid($dbh, $spid1, $ttlid1, $spid2, $ttlid2, $extract_id, $ptid);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        $msg = $sample_msg . $comp_msg;
        print $msg;
        exit 0;
    }
    print $sample_msg . $comp_msg . "Are you sure you want to delete the comparison data for these samples?<br>\n";
    $sth->finish;
    $dbh->rollback;
    $dbh->disconnect;
    exit 99;
} elsif ($force_delete == 1) {
    #-- 1. delete data from sp_comparison:
    $sql = "delete from sp_comparison where process1_id in (select process1_id from process1 where sampling_point1_id = $spid1 and sp1_tissue_type_label_id = $ttlid1 and sampling_point2_id = $spid2 and sp2_tissue_type_label_id = $ttlid2) and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and extract_id = $extract_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc =~ /^\d/ && $rc !~ /\W/) {
        if ($rc != 0) {
            $msg = "$rc records deleted from comparison data.<br>\n";
        }
    } else {
        $msg = "There are some mistakes occured when trying to delete the comparison data. NO data deleted in database.<br><br>\n";
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }
}
print $msg;
$sth->finish;
#$dbh->rollback;
$dbh->commit;
$dbh->disconnect;
exit 0;

###########################################################################
# subroutine:
#  getMetadata($dbh, $ttlid1, $ttlid2, $extract_id, $ptid)
#
# parameters in: $dbh --- the database handle,
#                $ttlid1 -- tissue_type_label_id for numeriter
#                $ttlid2 -- tissue_type_label_id for denominater
#                $extract -- extract id
#                $ptid --- the protocol_type_id,
# parameter out: $data_ok -- 0 for successful and 1 for failed
#                $msg --- the detail info for the array;
#
# X. Jing Li, 6/30/2006
###########################################################################
sub getMetadata {
    my ($dbh, $ttlid1, $ttlid2, $extract_id, $ptid) = @_;
    my ($sql, $sth, $rc, $ttlid_name_1, $ttlid_name_2, $ext_name, $technique_name, $pt_name, $data_ok, $msg);

    $data_ok = 0;

    $sql = "select EXT.\"name\", T.technique_name from extract EXT, technique T where EXT.extract_id = $extract_id and EXT.technique_id = T.technique_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($ext_name, $technique_name) = $sth->fetchrow();

    #-- get tissue_type_label_names:
    $sth = $dbh-> prepare("select \"name\" from label where label_id = $ttlid1") or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $ttlid_name_1 = $sth->fetchrow();

    $sth = $dbh-> prepare("select \"name\" from label where label_id = $ttlid2") or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $ttlid_name_2 = $sth->fetchrow();

    #-- get protocol_type_name:
    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pt_name = $sth->fetchrow();

    $msg = "<h3>Summary about the Samples:</h3>\n";
    $msg .= "<B>Matadata:</B><br><TABLE BORDER=1>\n";
    $msg .= "<TR ALIGN=CENTER><TH>Data Type</TH><TH>Name</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Sample 1 Tissue Type</b></TD><TD>$ttlid_name_1</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Sample 2 Tissue Type</b></TD><TD>$ttlid_name_2</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Technique</b></TD><TD>$technique_name</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Extraction</b></TD><TD>$ext_name</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Process Protocol</b></TD><TD>$pt_name</TD></TR>\n";
    $msg .= "</TABLE><BR>";

    $sth->finish;
    return $data_ok, $msg;
}
    
###########################################################################
# subroutine: getComparisonInfoWithPtid
#           ($dbh, $spid, $ttlid, $extract_id, $ptid)
#
# parameters in: $dbh -- the database handle,
#                $spid -- the sampling_point_id for sample,
#                $ttlid -- sample tissue_type_lable_id
#                $extract_id -- extract_id
#                $pt-d -- protocol_type_id
# parameter out: $data_ok -- 0 for success and 1 for failed
#                $msg -- the protocol_name in html table row format;
#
# X. Jing Li, 6/29/2006
###########################################################################
sub getComparisonInfoWithPtid {
    my ($dbh, $spid1, $ttlid1, $spid2, $ttlid2, $extract_id, $ptid) = @_;
    my ($sql, $sth, $rc, $sql1, $sth1, $rc1, $pt_name, $count, $data_ok, $msg,
        $es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2, $process1_id);

    $data_ok = 0;
    $msg = "<b>Comparison Data:</B><br>";

    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pt_name = $sth->fetchrow();

    #-- get the distinct process1_id. If only one then write a msg in line
    #-- format. If more one process1_id are found create a table.
    $sql = "select distinct process1_id from sp_comparison where extract_id = $extract_id and process1_id in (select process1_id from process1 where sampling_point1_id = $spid1 and sp1_tissue_type_label_id = $ttlid1 and sampling_point2_id = $spid2 and sp2_tissue_type_label_id = $ttlid2)";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() > 0) {
        $msg .= "<TABLE BORDER=1><TR><TH COLSPAN=3>Sample 1</TH><TH COLSPAN=3>Sample 2</TH><TH ROWSPAN=2>Number of Records</TH></TR><TR><TH>Exp_set_name</TH><TH>Experiment</TH><TH>Time_Point</TH><TH>Exp_set_name</TH><TH>Experiment</TH><TH>Time_Point</TH></TR>\n";

        while ($process1_id = $sth->fetchrow()) {
            #-- get the sampling_point1_id and sampling_point2_id info:
            $sql1 = "select ES1.exp_set_name, E1.exp_name, L1.\"name\", ES2.exp_set_name, E2.exp_name, L2.\"name\" from experiment_set ES1, experiment_set ES2, experiment E1, experiment E2, label L1, label L2, sampling_point SP1, sampling_point SP2, process1 P where ES1.exp_set_id = E1.exp_set_id and SP1.experiment_id = E1.experiment_id and SP1.label_id = L1.label_id and SP1.sampling_point_id = P.sampling_point1_id and ES2.exp_set_id = E2.exp_set_id and E2.experiment_id = SP2.experiment_id and SP2.label_id = L2.label_id and SP2.sampling_point_id = P.sampling_point2_id and P.process1_id = $process1_id";
            $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
            $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
            ($es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2) = $sth1->fetchrow();
            #print "$es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2\n";
            #-- get the count of this protocol_type_id:
            $sql1 = "select count(*) from sp_comparison where extract_id = $extract_id and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and process1_id = $process1_id";
            $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
            $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
            $count = $sth1->fetchrow();
            #print "count = $count\n";
            if ($count > 0) {
                $msg .= "<TR ALIGN=CENTER><TD>$es_name1</TD><TD>$exp_name1</TD><TD>$lname1</TD><TD>$es_name2</TD><TD>$exp_name2</TD><TD>$lname2</TD><TD>$count</TD></TR>\n";
                $sth1->finish;
            }
        }
        $msg .= "</TABLE><br>\n";
    } else {
        $msg .= "There is no comparison data for protocol \"$pt_name\" from these samples.<br>\n";
        $msg .= "<TABLE BORDER=1><TR><TH COLSPAN=3>Sample 1</TH><TH COLSPAN=3>Sample 2</TH></TR><TR><TH>Exp_set_name</TH><TH>Experiment</TH><TH>Time_Point</TH><TH>Exp_set_name</TH><TH>Experiment</TH><TH>Time_Point</TH></TR>\n";

        $sql1 = "select ES1.exp_set_name, E1.exp_name, L1.\"name\", ES2.exp_set_name, E2.exp_name, L2.\"name\" from experiment_set ES1, experiment_set ES2, experiment E1, experiment E2, label L1, label L2, sampling_point SP1, sampling_point SP2 where ES1.exp_set_id = E1.exp_set_id and SP1.experiment_id = E1.experiment_id and SP1.label_id = L1.label_id and SP1.sampling_point_id = $spid1 and ES2.exp_set_id = E2.exp_set_id and E2.experiment_id = SP2.experiment_id and SP2.label_id = L2.label_id and SP2.sampling_point_id = $spid2 ";
        $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
        $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
        ($es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2) = $sth1->fetchrow();
        $msg .= "<TR ALIGN=CENTER><TD>$es_name1</TD><TD>$exp_name1</TD><TD>$lname1</TD><TD>$es_name2</TD><TD>$exp_name2</TD><TD>$lname2</TD></TR>\n";
        $msg .= "</TABLE>\n";
        $sth1->finish;
         $data_ok = 1;
    }
    $sth->finish;
    return $data_ok, $msg;
}

