#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script deletes the cDNA and Affy normalized data and all its related 
# processed data.
#
# sampling_replicate_id
# tissue_type_id
# technical_replicate
# protocol_type_id
# extract_id
#
# Author: X. Jing Li
# Date: June 29, 2006
# Reversions: 
#===============================================================================

my ($database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc, $sql1, $sth1, $rc1, 
    $spid, $ttlid, $extract_id, $ptid, $force_delete, $msg, $sample_msg,  $sum_msg, $comp_msg, $data_ok);

#== ARGV
($spid, $ttlid, $extract_id, $ptid, $force_delete) = @ARGV;

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

if ($force_delete == 0) {
    #== check the processed data detail:
    ($data_ok, $sample_msg) = getSampleInfoWithPtid($dbh, $spid, $ttlid, $extract_id, $ptid);

    #-- 1. sp_summary:
    ($data_ok, $sum_msg) = getSummaryInfoWithPtid($dbh, $spid, $ttlid, $extract_id, $ptid);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $sample_msg . $sum_msg;
        exit 1;
    }

    #-- 2. sp_comparison:
    ($data_ok, $comp_msg) = getComparisonInfoWithPtid($dbh, $spid, $ttlid, $extract_id, $ptid);
    if ($data_ok) {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print $sample_msg . $sum_msg . $comp_msg . "Are you sure you want to delete the summarized data for this sample from DOME?\n";
        exit 99;
    }
    print $sample_msg . $sum_msg . $comp_msg . "If you delete the summarized data for this sample, you will deleted all the comparison data as well. Are you sure you want to delete the data from DOME?\n";
    $sth->finish;
    $dbh->rollback;
    $dbh->disconnect;
    exit 99;
} elsif ($force_delete == 1) {
    #-- 1. delete data from sp_comparison:
    $sql = "delete from sp_comparison where process1_id in (select process1_id from process1 where (sampling_point1_id = $spid and sp1_tissue_type_label_id = $ttlid) or (sampling_point2_id = $spid and sp2_tissue_type_label_id = $ttlid)) and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and extract_id = $extract_id";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc =~ /^\d/ && $rc !~ /\W/) {
        if ($rc != 0) {
            $msg = "$rc records deleted from comparison data.<br><br>\n";
        }
    } else {
        $msg = "There are some mistakes occured when trying to delete the comparison data. NO data deleted in database.<br><br>\n";
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }
#    print "delete from sp_comparison: rc = $rc\n";
    #print "sp_comparison count: " . $sth->fetchrow() . "\n//\n";

    #-- 2. delete from sp_summary:
    $sql = "delete from sp_summary where sampling_point_id = $spid and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and extract_id = $extract_id and label_id = $ttlid";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc =~ /^\d/ && $rc !~ /\W/) {
        if ($rc != 0) {
            $msg = "$rc records deleted from summarized data.<br><br>\n" . $msg;
        }
    } else {
        $msg = "There are some mistakes occured when trying to delete the summarized data from DOME. NO data deleted in database.<br><br>\n";
        print $msg;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 1;
    }
#    print "delete from sp_summary: rc = $rc\n";
    #print "sp_summary count: " . $sth->fetchrow() . "\n//\n";
}

print $msg;
$sth->finish;
#$dbh->rollback;
$dbh->commit;
$dbh->disconnect;
exit 0;

###########################################################################
# subroutine:
#  getSampleInfoWithPtid($dbh, $srid, $ttlid, $extract_id, $ptid)
#
# parameters in: $dbh --- the database handle,
#                $srid -- sampling_replicate_id
#                $ttlid -- tissue_type_label_id
#                $extract -- extract id
#                $ptid --- the protocol_type_id,
# parameter out: $data_ok -- 0 for successful and 1 for failed
#                $msg --- the detail info for the array;
#
# X. Jing Li, 6/29/2006
###########################################################################
sub getSampleInfoWithPtid {
    my ($dbh, $spid, $ttlid, $extract_id, $ptid) = @_;
    my ($sql, $sth, $rc, $es_name, $exp_name, $lname, $ext_name, $technique_name, $ttlid_name, $pt_name, $data_ok, $msg);

    $data_ok = 0;

    $sql = "select EXT.\"name\", T.technique_name from extract EXT, technique T where EXT.extract_id = $extract_id and EXT.technique_id = T.technique_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($ext_name, $technique_name) = $sth->fetchrow();

    #== get sample detail:
    $sql = "select ES.exp_set_name, E.exp_name, L.name from experiment_set ES, experiment E, label L, sampling_point SP where SP.sampling_point_id = $spid and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($es_name, $exp_name, $lname) = $sth->fetchrow();

    #-- get tissue_type_label_names:
    $sth = $dbh-> prepare("select \"name\" from label where label_id = $ttlid") or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $ttlid_name = $sth->fetchrow();

    #-- get protocol_type_name:
    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pt_name = $sth->fetchrow();

    $msg = "<h3>Summary about the Sample:</h3>\n";
    $msg .= "<B>Metadata about the sample:</B><br><TABLE BORDER=1>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Exp_set Name</b></TD><TD>$es_name</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Experiment Name</b></TD><TD>$exp_name</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Time Point</b></TD><TD>$lname</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Tissue Type</b></TD><TD>$ttlid_name</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Technique</b></TD><TD>$technique_name</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Extraction</b></TD><TD>$ext_name</TD></TR>\n";
    $msg .= "<TR ALIGN=CENTER><TD><b>Process Protocol</b></TD><TD>$pt_name</TD></TR>\n";
    $msg .= "</TABLE><BR>";

    $sth->finish;
    return $data_ok, $msg;
}
    

###########################################################################
# subroutine: getSummaryInfoWithPtid
#           ($dbh, $spid, $ttlid, $extract_id, $ptid)
#
# parameters in: $dbh --- the database handle,
#                $spid --- the sampling_point_id for sample,
#                $ttlid --- sample tissue_type_lable_id
#                $extract_id --- extract_id
#                $ptid -- protocol_type_id
# parameter out: $msg --- the protocol_name in html table row format;
#                $data_ok -- 0 for success and 1 for failed
#
# X. Jing Li, 6/29/2006
###########################################################################
sub getSummaryInfoWithPtid {
    my ($dbh, $spid, $ttlid, $extract_id, $ptid) = @_;
    my ($sql, $sth, $rc, $pt_name, $count, $data_ok, $msg);

    $data_ok = 0;
    $msg = "<b>Summarized Data:</b><br>";
    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pt_name = $sth->fetchrow();

    $sql = "select count(*) from sp_summary where sampling_point_id = $spid and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and extract_id = $extract_id and label_id = $ttlid";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $count = $sth->fetchrow();
    if ($count > 0) {
        $msg .= "There are $count records in DOME using protocol \"$pt_name\".<br>\n";
    } else {
        $data_ok = 1;
        $msg .= "There is no summerized data for this sample using protocol \"$pt_name\".<br>\n";
    }
    $msg .= "<BR>";
    $sth->finish;
    return $data_ok, $msg;
}

###########################################################################
# subroutine: getComparisonInfoWithPtid
#           ($dbh, $spid, $ttlid, $extract_id, $ptid)
#
# parameters in: $dbh -- the database handle,
#                $spid -- the sampling_point_id for sample,
#                $ttlid -- sample tissue_type_lable_id
#                $extract_id -- extract_id
#                $pt-d -- protocol_type_id
# parameter out: $data_ok -- 0 for success and 1 for failed
#                $msg -- the protocol_name in html table row format;
#
# X. Jing Li, 6/29/2006
###########################################################################
sub getComparisonInfoWithPtid {
    my ($dbh, $spid, $ttlid, $extract_id, $ptid) = @_;
    my ($sql, $sth, $rc, $sql1, $sth1, $rc1, $pt_name, $count, $data_ok, $msg,
        $es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2, $process1_id);

    $data_ok = 0;
    $msg = "<b>Comparison Data:</B><br>";

    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pt_name = $sth->fetchrow();

    #-- get the distinct process1_id. If only one then write a msg in line
    #-- format. If more one process1_id are found create a table.
    $sql = "select distinct process1_id from sp_comparison where process1_id in (select process1_id from process1 where sampling_point1_id = $spid and sp1_tissue_type_label_id = $ttlid or sampling_point2_id = $spid and sp2_tissue_type_label_id = $ttlid)";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() > 0) {
        $msg .= "<TABLE BORDER=1><TR><TH COLSPAN=3>Sample 1</TH><TH COLSPAN=3>Sample 2</TH><TH ROWSPAN=2>Number of Records</TH></TR><TR><TH>Exp_set_name</TH><TH>Experiment</TH><TH>Time_Point</TH><TH>Exp_set_name</TH><TH>Experiment</TH><TH>Time_Point</TH></TR>\n";
        while ($process1_id = $sth->fetchrow()) {
        #-- get the sampling_point1_id and sampling_point2_id info:
            $sql1 = "select ES1.exp_set_name, E1.exp_name, L1.\"name\", ES2.exp_set_name, E2.exp_name, L2.\"name\" from experiment_set ES1, experiment_set ES2, experiment E1, experiment E2, label L1, label L2, sampling_point SP1, sampling_point SP2, process1 P where ES1.exp_set_id = E1.exp_set_id and SP1.experiment_id = E1.experiment_id and SP1.label_id = L1.label_id and SP1.sampling_point_id = P.sampling_point1_id and ES2.exp_set_id = E2.exp_set_id and E2.experiment_id = SP2.experiment_id and SP2.label_id = L2.label_id and SP2.sampling_point_id = P.sampling_point2_id and P.process1_id = $process1_id";
#        print"pt_name = $pt_name, process1_id = $process1_id\nsql1 = $sql1\n";
            $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
            $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
            ($es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2) = $sth1->fetchrow();
#        print "$es_name1, $exp_name1, $lname1, $es_name2, $exp_name2, $lname2\n";
            #-- get the count of this protocol_type_id:
            $sql1 = "select count(*) from sp_comparison where extract_id = $extract_id and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and process1_id = $process1_id";
#            print "$sql1\n";
            $sth1 = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
            $rc1 = $sth1->execute or die "Can't execute: $DBI::errstr.";
            $count = $sth1->fetchrow();
#            print "count = $count\n";
            if ($count > 0) {
                $msg .= "<TR ALIGN=CENTER><TD>$es_name1</TD><TD>$exp_name1</TD><TD>$lname1</TD><TD>$es_name2</TD><TD>$exp_name2</TD><TD>$lname2</TD><TD>$count</TD></TR>\n";
            }
        }
        $msg .= "</TABLE><br>\n";
    } else {
        $msg .= "There is no comparison data for protocol \"$pt_name\".<br>\n";
        $data_ok = 1;
    }
    $sth1->finish;
    $sth->finish;
    return $data_ok, $msg;
}

