#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script will delete all gene information for certain specie. It will travel through tables "gene", "evidence", "allnames", "external_links" under the condition that there is no experiment data depending on the gene_ids which will be deleted. If there is any data in table "composite_sequence", "reporter", "ma_norm", "sp_summary", "sp_comparison", the script will not finish unless the data in these tables is deleted first.
# command line parameters:
# specie_name -- the specie name which the gene information should be deleted,
# logfile -- the full name of the log file which the records will be deleted will be saved in cse rollback is needed.
#
# Author: X. Jing Li
# Date: March 15, 2006
# Reversions: 
#============================================================================== 

my ($database, $username, $password, $bnet_DB, $specieDB, $sql, $mainsql, $dbh, $sth, $rc,
    $specie_name, $logfile, $search_specie_name, $specie_id, $kwid, $msg, $cs_count, $reporter_count, $quantif_count, $ma_norm_count, $summary_count, $comparison_count, $gene_id, %gene_ids, $link_id, $ex_dbid, $ex_id, %link_ids, %link_exids, %link_exdbs, $evidence_id, $ref_id, $evid_code_id, %evidence_ids, %evidence_ref_ids, %evidence_code_ids, $name_id, $name, $isPrimary, $search_name, %name_ids, %names, %primarys, %search_names, $internal_field_name);

($specie_name, $logfile) = @ARGV;
open (LOG, ">$logfile") or die "Can not open log file $logfile. $!\n";
print "Starting time = " . TimeStamp() . "\n";
print LOG "Starting time = " . TimeStamp() . "\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== 1. get the spiece_id and kw_table_id for GENE:
$search_specie_name = uc $specie_name;
$search_specie_name =~ s/\s|\-|\,|\"|\'//g;
#print "search_specie_name = $search_specie_name\n";
$sql = "select \"record_id\" from \"allnames\" where upper(search_name) = \'$search_specie_name\' and \"kw_table_id\" = (select kw_table_id from \"ref_kw_table\" where \"kw_table\" = \'SPECIES\')";
#print "$sql\n";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$specie_id = $sth->fetchrow();
if ($specie_id eq "") {
    $msg = "There is no specie_id for specie \"$specie_name\". \n";
    print $msg;
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}
$sql = "select kw_table_id from ref_kw_table where kw_table = 'GENE'";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$kwid = $sth->fetchrow();


#== 2. check the data in DOME for dependency:
#-- 2.1. count the record number in "composite_sequence";
$sql = "select count(*) from \"composite_sequence\" where \"gene_id\" in (select gene_id from \"gene\" where \"specie_id\" = \'$specie_id\')";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$cs_count = $sth->fetchrow();
if ($cs_count > 0) {
    $msg .= "There are $cs_count records in \"composite_sequence\" table.\n";
}

#-- 2.2. count the record number in "reporter";
$sql = "select count(*) from \"reporter\" where \"composite_sequence_id\" in (select composite_sequence_id from \"composite_sequence\" where \"gene_id\" in (select gene_id from \"gene\" where \"specie_id\" = \'$specie_id\'))";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$reporter_count = $sth->fetchrow();
if ($reporter_count > 0) {
    $msg .= "There are $reporter_count records in \"reporter\" table.\n";
}

#-- 2.3. count the record number in "quantification";
$sql = "select count(*) from \"quantification\" where feature_id in (select feature_id from feature where \"reporter_id\" in (select \"reporter_id\" from \"reporter\" where \"composite_sequence_id\" in (select composite_sequence_id from \"composite_sequence\" where \"gene_id\" in (select gene_id from \"gene\" where \"specie_id\" = \'$specie_id\'))))";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$quantif_count = $sth->fetchrow();
if ($quantif_count > 0) {
    $msg .= "There are $quantif_count records in \"quantification\" table.\n";
}

#-- 2.4. count the record number in "ma_norm";
$sql = "select count(*) from \"ma_norm\" where \"reporter_id\" in (select \"reporter_id\" from \"reporter\" where \"composite_sequence_id\" in (select composite_sequence_id from \"composite_sequence\" where \"gene_id\" in (select gene_id from \"gene\" where \"specie_id\" = \'$specie_id\')))";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$ma_norm_count = $sth->fetchrow();
if ($ma_norm_count > 0) {
    $msg .= "There are $ma_norm_count records in \"ma_norm\" table.\n";
}

#-- 2.5. count the record number in "sp_summary";
$sql = "select count(*) from \"sp_summary\" where \"kw_table_id\" = 33 and \"record_id\" in (select \"reporter_id\" from \"reporter\" where \"composite_sequence_id\" in (select composite_sequence_id from \"composite_sequence\" where \"gene_id\" in (select gene_id from \"gene\" where \"specie_id\" = \'$specie_id\')))";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$summary_count = $sth->fetchrow();
if ($summary_count > 0) {
    $msg .= "There are $summary_count records in \"sp_summary\" table.\n";
}

#-- 2.6. count the record number in "sp_comparison";
$sql = "select count(*) from \"sp_comparison\" where \"kw_table_id\" = 33 and \"record_id\" in (select \"reporter_id\" from \"reporter\" where \"composite_sequence_id\" in (select composite_sequence_id from \"composite_sequence\" where \"gene_id\" in (select gene_id from \"gene\" where \"specie_id\" = \'$specie_id\')))";
#print "$sql\n";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$comparison_count = $sth->fetchrow();
if ($comparison_count > 0) {
    $msg .= "There are $comparison_count records in \"sp_comparison\" table.\n";
}

if ($msg ne "") {
    $msg .= "You must delete all data from the above table(s) before you delete gene information about specie \"$specie_name\" from \"$bnet_DB\". \n";
    print $msg;
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}

#== 3. get all gene info for this specie and save ids in hashes
#-- 3.1 get gene_ids
$sql = "select \"gene_id\" from \"gene\" where \"specie_id\" = \'$specie_id\'";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while ($gene_id = $sth->fetchrow()) {
    $gene_ids{$gene_id} = 1;
}

#-- 3.2 get external_links_ids:
$sql = "select \"external_links_id\", \"record_id\", \"external_database_id\", \"external_id\" from \"external_links\" where \"kw_table_id\" = $kwid and \"record_id\" in (select \"gene_id\" from \"gene\" where \"specie_id\" = \'$specie_id\')";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($link_id, $gene_id, $ex_dbid, $ex_id) = $sth->fetchrow()) {
    $link_ids{$link_id} = $gene_id;
    $link_exids{$link_id} = $ex_id;
    $link_exdbs{$link_id} = $ex_dbid;
}

#-- 3.3 get evidence_ids:
$sql = "select \"evidence_id\",\"reference_id\", \"evidence_code_id\", \"record_id\" from \"evidence\" where \"kw_table_id\" = $kwid and \"record_id\" in (select \"gene_id\" from \"gene\" where \"specie_id\" = \'$specie_id\')";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($evidence_id, $ref_id, $evid_code_id, $gene_id) = $sth->fetchrow()) {
    $evidence_ids{$evidence_id} = $gene_id;
    $evidence_ref_ids{$evidence_id} = $ref_id;
    $evidence_code_ids{$evidence_id} = $evid_code_id;
}

#-- 3.4 get name_ids from allnames:
$sql = "select \"name_id\", \"record_id\", \"aname\", \"is_primary\", \"search_name\" from \"allnames\" where \"kw_table_id\" = $kwid and \"record_id\" in (select \"gene_id\" from \"gene\" where \"specie_id\" = \'$specie_id\')";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($name_id, $gene_id, $name, $isPrimary, $search_name) = $sth->fetchrow()) {
    $name_ids{$name_id} = $gene_id;
    $names{$name_id} = $name;
    $primarys{$name_id} = $isPrimary;
    $search_names{$name_id} = $search_name;
}

#== 4. print all info into log file:
#-- 4.1 print allanmes info"
if (keys %names > 0) {
    print LOG "ALLNAMES ------------------------\n";
    print LOG "name_id\tkw_table_id\trecord_id\taname\tis_primary\tsearch_name\n";
    foreach $name_id (sort {$a <=> $b} keys %name_ids) {
        print LOG "$name_id\t$kwid\t" . $name_ids{$name_id} . "\t" . $names{$name_id} . "\t" . $primarys{$name_id} . "\t" . $search_names{$name_id} . "\n";
    };
}
#-- 4.2 print external_links info:
if (keys %link_ids > 0) {
    $internal_field_name = "gene_id";
    print LOG "EXTERNAL_LINKS ------------------------\n";
    print LOG "external_links_id\tkw_table_id\trecord_id\tinternal_field_name\texternal_database_id\texteernal_id\n";
    foreach $link_id (sort {$a <=> $b} keys %link_ids) {
        print LOG "$link_id\t$kwid\t" . $link_ids{$link_id} . "\t" . $internal_field_name . "\t" . $link_exdbs{$link_id} . "\t" . $link_exids{$link_id} . "\n";
    }
}

#-- 4.3 print the evidence info:
if (keys %evidence_ids > 0) {
    print LOG "EVIDENCE ------------------------\n";
    print LOG "evidence_id_id\treference_id\tevidence_code_id\tkw_table_id\trecord_id\n";
    foreach $evidence_id (sort {$a <=> $b} keys %evidence_ids) {
        print LOG "$evidence_id\t" . $evidence_ref_ids{$evidence_id} . "\t" . $evidence_code_ids{$evidence_id} . "\t$kwid\t" . $evidence_ids{$evidence_id} . "\n";
    }
}

#== 5. start to delete the records in DOME
#-- 5.1 delete from extrernal_links:
if (keys %link_ids > 0) {
    $sql = "delete from \"external_links\" where kw_table_id = $kwid and record_id in (select gene_id from gene where specie_id = \'$specie_id\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    print LOG "delete from external_links: rc = $rc\n";
    print "delete from external_links: rc = $rc\n";
}

#-- 5.2 delete from evidence:
if (keys %evidence_ids > 0) {
    $sql = "delete from \"evidence\" where kw_table_id = $kwid and record_id in (select gene_id from gene where specie_id = \'$specie_id\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    print LOG "delete from evidence: rc = $rc\n";
    print "delete from evidence: rc = $rc\n";
}

#-- 5.3 delete all synonames from allnames.
$mainsql = "";
foreach $name_id (keys %name_ids) {
    $isPrimary = $primarys{$name_id};
    if ($isPrimary ne "Y") {
        $mainsql .= "delete from allnames where name_id = $name_id;\n";
    }
} 
$sth = $dbh-> prepare($mainsql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
print LOG "delete from allnames: rc = $rc\n";
print "delete from allnames: rc = $rc\n";

#-- 5.4 delete from gene::
$sql = "delete from gene where specie_id = \'$specie_id\'";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
print LOG "delete from gene: rc = $rc\n";
print "delete from gene: rc = $rc\n";

#-- 5.4 delete all primary names from allnames.
$mainsql = "";
foreach $name_id (keys %name_ids) {
    $isPrimary = $primarys{$name_id};
    if ($isPrimary eq "Y") {
        $mainsql .= "delete from allnames where name_id = $name_id;\n";
    }
} 
$sth = $dbh-> prepare($mainsql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
print LOG "delete from allnames: rc = $rc\n";
print "delete from allnames: rc = $rc\n";

#$dbh->rollback;
$dbh->commit;
$sth->finish;
$dbh->disconnect;

print "Ending time = " . TimeStamp() . "\n";
print LOG "Ending time = " . TimeStamp() . "\n";
close LOG;

exit;
