CREATE OR REPLACE FUNCTION "do_update"() RETURNS TRIGGER AS 'DECLARE
	-- Declare a variable to hold the kw_table_id.
	table_id INTEGER;
	-- Declare a variable to hold the record_id.
	rec_id INTEGER;
	
	-- Declare a variable to hold the operation type (I for INSERT, U for UPDATE or D for DELETE).
	operation_letter CHAR;
	
	-- Declare a variable to hold the primary_name.
	identifier VARCHAR(5000);
	
	-- A general placeholder for a row.
	ROW RECORD;
	
	-- Declare a variable for the separation character for identifier.
	separator VARCHAR(1) := '':'';
	
	-- Declare a variable to hold the SQL
	SQL TEXT;

BEGIN
 	
	SELECT INTO table_id kw_table_id FROM ref_kw_table WHERE LOWER(kw_table) = TG_RELNAME;
	
	-- If there was no matching ID number, raise an exception.
	IF NOT FOUND THEN
	RAISE EXCEPTION ''Trigger do_update: Table ID for table ''''%'''' not present in table ''''ref_kw_table''''. Please put an entry in ''''ref_kw_table''''.'', TG_RELNAME;
	END IF;
	

	IF TG_OP = ''INSERT'' THEN
	 operation_letter := ''I'';
	ELSIF TG_OP = ''UPDATE'' THEN
	operation_letter := ''U'';
	ELSIF TG_OP = ''DELETE'' THEN
	operation_letter := ''D'';
	END IF;
 
 
	IF TG_OP = ''INSERT'' OR TG_OP = ''UPDATE'' THEN
		 IF TG_RELNAME = ''activity'' THEN
		    rec_id := NEW.activity_id;
		 ELSIF TG_RELNAME = ''allnames'' THEN
		    rec_id := NEW.name_id;
		 ELSIF TG_RELNAME = ''complex'' THEN
		    rec_id := NEW.complex_id;
		 ELSIF TG_RELNAME = ''complex_member'' THEN
		    rec_id := NEW.complex_member_id;
		 ELSIF TG_RELNAME = ''compound'' THEN
		    rec_id := NEW.cpd_id;
		 ELSIF TG_RELNAME = ''cpd_derivative'' THEN
		    rec_id := NEW.cpd_derivative_id;
		 ELSIF TG_RELNAME = ''ec'' THEN
		    rec_id := NEW.ec_id;
		 ELSIF TG_RELNAME = ''event'' THEN
		    rec_id := NEW.event_id;
		 ELSIF TG_RELNAME = ''event_component'' THEN
		    rec_id := NEW.event_component_id;
		 ELSIF TG_RELNAME = ''event_ec'' THEN
		    rec_id := NEW.event_ec_id;       
		 ELSIF TG_RELNAME = ''event_type'' THEN
		    rec_id := NEW.event_type_id;       
		 ELSIF TG_RELNAME = ''evidence'' THEN
		    rec_id := NEW.evidence_id;        
		 ELSIF TG_RELNAME = ''evidence_code'' THEN
		    rec_id := NEW.evidence_code_id;          
		 ELSIF TG_RELNAME = ''evidence_go_accession'' THEN
		    rec_id := NEW.evidence_go_acc_id;       
		 ELSIF TG_RELNAME = ''external_database'' THEN
		    rec_id := NEW.external_database_id;       
		 ELSIF TG_RELNAME = ''external_links'' THEN
		    rec_id := NEW.external_links_id;      
		 ELSIF TG_RELNAME = ''gene'' THEN
		    rec_id := NEW.gene_id;
		 ELSIF TG_RELNAME = ''gene_tf'' THEN
		    rec_id := NEW.gene_tf_id;
		 ELSIF TG_RELNAME = ''gene_tf_info'' THEN
		    rec_id := NEW.gene_tf_info_id;
		 ELSIF TG_RELNAME = ''go_accession'' THEN
		    rec_id := NEW.go_accession_id;                   
		 ELSIF TG_RELNAME = ''location'' THEN
		    rec_id := NEW.location_id;       
		 ELSIF TG_RELNAME = ''mrna'' THEN
		    rec_id := NEW.mrna_id;       
		 ELSIF TG_RELNAME = ''pathway'' THEN
		    rec_id := NEW.pathway_id;
		 ELSIF TG_RELNAME = ''pathway_class'' THEN
		    rec_id := NEW.pathway_class_id;       
		 ELSIF TG_RELNAME = ''pathway_event'' THEN
		    rec_id := NEW.pathway_event_id;       
		 ELSIF TG_RELNAME = ''phenotype'' THEN
		    rec_id := NEW.phenotype_id;       
		 ELSIF TG_RELNAME = ''protein'' THEN
		    rec_id := NEW.protein_id;               
		 ELSIF TG_RELNAME = ''protein_state'' THEN
		    rec_id := NEW.protein_state_id;
		 ELSIF TG_RELNAME = ''protein_state_location'' THEN
		    rec_id := NEW.protein_state_location_id;
		 ELSIF TG_RELNAME = ''reference'' THEN
		    rec_id := NEW.reference_id;           
		 ELSIF TG_RELNAME = ''specie_compound'' THEN
		    rec_id := NEW.specie_cpd_id;
		 ELSIF TG_RELNAME = ''species'' THEN
		    rec_id := NEW.specie_id;
		 ELSIF TG_RELNAME = ''state'' THEN
		    rec_id := NEW.state_id;    
		 END IF;
	ELSIF TG_OP = ''DELETE'' THEN
		IF TG_RELNAME = ''activity'' THEN
		    rec_id := OLD.activity_id;
		    IF OLD.event_id IS NOT NULL THEN
		    	SELECT INTO ROW an.unikey FROM allnames an, ref_kw_table r
		    		WHERE an.record_id = OLD.event_id
					AND an.kw_table_id = r.kw_table_id
					AND UPPER(r.kw_table) = ''EVENT''
					AND UPPER(an.is_primary) = ''Y'';
				
				-- If there was no Primary Name, raise an exception.
				IF NOT FOUND THEN
					RAISE EXCEPTION ''Trigger do_update: Primary Name for event or kw_table not found for activity_id ''''%'''' in table ''''%''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
				END IF;	
				identifier := ROW.unikey;
			ELSE
				identifier := ''NULL'';
		    END IF;
		    
		    IF OLD.ec_id IS NOT NULL THEN
		    	SELECT INTO ROW an.unikey FROM allnames an, ref_kw_table r
		    		WHERE an.record_id = OLD.ec_id
					AND an.kw_table_id = r.kw_table_id
					AND UPPER(r.kw_table) = ''EC''
					AND UPPER(an.is_primary) = ''Y'';
				
				-- If there was no Primary Name, raise an exception.
				IF NOT FOUND THEN
					RAISE EXCEPTION ''Trigger do_update: Primary Name for ec or kw_table not found for activity_id ''''%'''' in table ''''%''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
				END IF;	
				identifier := identifier || separator || ROW.unikey;
			ELSE
				identifier := identifier || separator || ''NULL'';
		    END IF;
		    
		    IF OLD.protein_state_id IS NOT NULL THEN
		    	SELECT INTO ROW an.unikey, s.type FROM 
		    		allnames an, ref_kw_table r, protein_state ps, state s
		    		WHERE ps.protein_state_id = OLD.protein_state_id
		    		AND an.record_id = ps.protein_id
					AND an.kw_table_id = r.kw_table_id
					AND UPPER(r.kw_table) = ''PROTEIN''
					AND UPPER(an.is_primary) = ''Y''
					AND s.state_id = ps.state_id;
				
				-- If there was no Primary Name, raise an exception.
				IF NOT FOUND THEN
					RAISE EXCEPTION ''Trigger do_update: Primary Name for protein, state or kw_table not found for activity_id ''''%'''' in table ''''%''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
				END IF;	
				identifier := identifier || separator || separator || ROW.unikey || separator || ROW.type || separator || separator;
			ELSE
				identifier := identifier || separator || separator || ''NULL'' || separator || separator;
		    END IF;

		 ELSIF TG_RELNAME = ''allnames'' THEN
		    rec_id := OLD.name_id;
		    IF OLD.is_primary = ''N'' OR OLD.is_primary = ''A'' THEN
			    SELECT INTO ROW TRIM(an.unikey) as unique_name, UPPER(r.kw_table) as kw_table
					 FROM  allnames an, ref_kw_table r
					WHERE r.kw_table_id = OLD.kw_table_id
					AND an.record_id = OLD.record_id
					AND an.kw_table_id = OLD.kw_table_id
					AND UPPER(an.is_primary) = ''Y'';
					
				-- If there was no Primary Name, raise an exception.
				IF NOT FOUND THEN
					RAISE EXCEPTION ''Trigger do_update: Primary Name  not found for Name ID ''''%'''' in table ''''%''''. Please put an entry in ''''allnames'''' before deleting.'', rec_id, TG_RELNAME;
				END IF;
			    identifier := ROW.unique_name || separator || ROW.kw_table || separator || OLD.aname || separator || OLD.is_primary;
			ELSIF OLD.is_primary = ''Y'' THEN
				SELECT INTO ROW UPPER(r.kw_table) as kw_table
					 FROM  ref_kw_table r WHERE r.kw_table_id = OLD.kw_table_id;
					
				-- If there was no Primary Name, raise an exception.
				IF NOT FOUND THEN
					RAISE EXCEPTION ''Trigger do_update: kw_table corresponding to name_id ''''%'''' in table ''''%'''' not found. Please put an entry in ''''ref_kw_table'''' before deleting.'', rec_id, TG_RELNAME;
				END IF;
			    identifier := OLD.unikey || separator || ROW.kw_table || separator || OLD.aname || separator || OLD.is_primary;
			END IF;
			
		 ELSIF TG_RELNAME = ''complex'' THEN
		    rec_id := OLD.complex_id;
		 ELSIF TG_RELNAME = ''complex_member'' THEN
		    rec_id := OLD.complex_member_id;
		    SELECT INTO ROW TRIM(an1.unikey) as complex_pn, UPPER(r2.kw_table) as kw_table, TRIM(an2.unikey) as complex_member_pn
				FROM  allnames an1, allnames an2, ref_kw_table r1, ref_kw_table r2
				WHERE an1.record_id = OLD.complex_id
				AND UPPER(r1.kw_table) = ''COMPLEX''
				AND an1.kw_table_id = r1.kw_table_id
				AND UPPER(an1.is_primary) = ''Y''
				AND an2.record_id = OLD.record_id
				AND an2.kw_table_id = OLD.kw_table_id
				AND UPPER(an2.is_primary) = ''Y''
				AND r2.kw_table_id = OLD.kw_table_id;
				
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name not found for compound corresponding to Complex Member ID ''''%'''' in table ''''%''''. Please put an entry in ''''allnames'''' before deleting.'', rec_id, TG_RELNAME;
			END IF;
		    identifier := ROW.complex_pn || separator || ROW.kw_table || separator || ROW.complex_member_pn;
		 ELSIF TG_RELNAME = ''compound'' THEN
		    rec_id := OLD.cpd_id;
		 ELSIF TG_RELNAME = ''cpd_derivative'' THEN
		    rec_id := OLD.cpd_derivative_id;
		    	
			SELECT INTO ROW TRIM(an.unikey) as unique_name
				FROM allnames an, ref_kw_table r
				WHERE an.record_id = OLD.cpd_id
				AND an.kw_table_id = r.kw_table_id
				AND UPPER(an.is_primary) = ''Y''
				AND UPPER(r.kw_table) = ''COMPOUND'';
				
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name not found for compound_derivative ID ''''%'''' in table ''''%''''. Please put an entry in ''''%'''' before deleting.'', rec_id, TG_RELNAME, TG_RELNAME;
			END IF;
		    identifier := ROW.unique_name || separator || OLD.name;
		    
		 ELSIF TG_RELNAME = ''ec'' THEN
		    rec_id := OLD.ec_id;
		    identifier := OLD.ec_name;
		    
		 ELSIF TG_RELNAME = ''event'' THEN
		    rec_id := OLD.event_id;
		 ELSIF TG_RELNAME = ''event_component'' THEN
		    rec_id := OLD.event_component_id;
		    
		   SELECT INTO ROW an1.unikey as event_pn, UPPER(r1.kw_table) as ec_kw_table, 
		   		an2.unikey as ec_pn, l.location_name
				FROM allnames an1, allnames an2, ref_kw_table r1, ref_kw_table r2, location l
				WHERE l.location_id = OLD.location_id
				AND r1.kw_table_id = OLD.kw_table_id 
				AND an2.record_id = OLD.record_id
				AND an2.kw_table_id = OLD.kw_table_id
				AND UPPER(an2.is_primary) = ''Y''
				AND UPPER(r2.kw_table) = ''EVENT''
				AND an1.record_id = OLD.event_id
				AND an1.kw_table_id = r2.kw_table_id
				AND UPPER(an1.is_primary) = ''Y'';
		    
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name for event_component or the event itself not found for event_component_id ''''%'''' in table ''''%'''' or there is a missing entry in ''''ref_kw_table''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
			END IF;
			identifier := ROW.event_pn || separator || ROW.ec_kw_table || separator || ROW.ec_pn || separator || OLD.is_substrate;

		 ELSIF TG_RELNAME = ''event_ec'' THEN
		    rec_id := OLD.event_ec_id;
		    SELECT INTO ROW TRIM(an.unikey) as unique_name, ec.ec_name FROM allnames an, ec, ref_kw_table r
				WHERE ec.ec_id = OLD.ec_id
				AND an.record_id = OLD.event_id
				AND an.kw_table_id = r.kw_table_id
				AND UPPER(r.kw_table) = ''EVENT''
				AND UPPER(an.is_primary) = ''Y'';
				
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name for event or kw_table not found for event_ec_id ''''%'''' in table ''''%''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
			END IF;
		    identifier := ROW.unique_name || separator || ROW.ec_name;
		 ELSIF TG_RELNAME = ''event_type'' THEN
		    rec_id := OLD.event_type_id;  
		    identifier := OLD.event_type_name;
		 ELSIF TG_RELNAME = ''evidence'' THEN
		    rec_id := OLD.evidence_id;        
		    
		    SELECT INTO ROW kw_table_id, UPPER(r.kw_table) as ev_kw_table
		    	FROM ref_kw_table r WHERE r.kw_table_id = OLD.kw_table_id;
		    IF ROW.ev_kw_table = ''SPECIE_COMPOUND'' THEN
		    --Since Evidence to COMPOUND point to specie_compound
		    	SELECT INTO ROW TRIM(an.unikey) as unique_name, UPPER(r1.kw_table) as ev_kw_table, 
					ref.locator, ec.evidence_code, UPPER(r2.kw_table) as ec_kw_table, 
					ec.technique, ec.strength
					 FROM  allnames an, reference ref, 
					 evidence_code ec, ref_kw_table r1, 
					 ref_kw_table r2, ref_kw_table r3, specie_compound sc
					WHERE ref.reference_id = OLD.reference_id 
					AND ec.evidence_code_id = OLD.evidence_code_id
					AND r1.kw_table = ''COMPOUND''
					AND ec.kw_table_id = r2.kw_table_id
					AND sc.specie_cpd_id = OLD.record_id
					AND an.record_id = sc.cpd_id
					AND an.kw_table_id = r3.kw_table_id
					AND UPPER(r3.kw_table) = ''COMPOUND''
					AND UPPER(an.is_primary) = ''Y'';
		    
		    ELSE
				SELECT INTO ROW TRIM(an.unikey) as unique_name, UPPER(r1.kw_table) as ev_kw_table, 
					ref.locator, ec.evidence_code, UPPER(r2.kw_table) as ec_kw_table, 
					ec.technique, ec.strength
					 FROM  allnames an, reference ref, 
					 evidence_code ec, ref_kw_table r1, ref_kw_table r2
					WHERE ref.reference_id = OLD.reference_id 
					AND ec.evidence_code_id = OLD.evidence_code_id
					AND r1.kw_table_id = OLD.kw_table_id
					AND ec.kw_table_id = r2.kw_table_id
					AND an.record_id = OLD.record_id
					AND an.kw_table_id = OLD.kw_table_id
					AND UPPER(an.is_primary) = ''Y'';
			END IF;
				
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name or evidence_code not found for evidence_id ''''%'''' in table ''''%''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
			END IF;
			identifier := ROW.unique_name || separator || ROW.ev_kw_table || separator || ROW.locator || separator || separator || ROW.evidence_code || separator || ROW.ec_kw_table || separator || ROW.technique || separator || ROW.strength || separator || separator;
		 
		 ELSIF TG_RELNAME = ''evidence_code'' THEN
		    rec_id := OLD.evidence_code_id;
		    
		    SELECT INTO ROW UPPER(r.kw_table) as kw_table FROM ref_kw_table r
				WHERE r.kw_table_id = OLD.kw_table_id;
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: kw_table entry corresponding to evidence_code_id ''''%'''' not found in table ''''ref_kw_table''''. Please put an entry in the ref_kw_table before deleting.'', rec_id;
			END IF;
		    identifier := OLD.evidence_code || separator || ROW.kw_table || separator || OLD.technique || separator || OLD.strength;
		    
		 ELSIF TG_RELNAME = ''external_database'' THEN
		    rec_id := OLD.external_database_id;  
		    identifier := OLD.full_db_name;
		 ELSIF TG_RELNAME = ''external_links'' THEN
		    rec_id := OLD.external_links_id;      
		    SELECT INTO ROW TRIM(an.unikey) as el_pn, UPPER(r.kw_table) as kw_table, ed.full_db_name
				FROM allnames an, ref_kw_table r, external_database ed
				WHERE ed.external_database_id = OLD.external_database_id
				AND r.kw_table_id = OLD.kw_table_id 
				AND an.record_id = OLD.record_id
				AND an.kw_table_id = OLD.kw_table_id
				AND UPPER(an.is_primary) = ''Y'';
				
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name or external_database_name or kw_table not found for external_links_id ''''%'''' in table ''''%''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
			END IF;
		    identifier := ROW.el_pn || separator || ROW.kw_table || separator || ROW.full_db_name;
		 ELSIF TG_RELNAME = ''gene'' THEN
		    rec_id := OLD.gene_id;       
		 ELSIF TG_RELNAME = ''go_accession'' THEN
		    rec_id := OLD.go_accession_id;                   
		 ELSIF TG_RELNAME = ''location'' THEN
		    rec_id := OLD.location_id;       
		    identifier := OLD.location_name;
		 ELSIF TG_RELNAME = ''mrna'' THEN
		    rec_id := OLD.mrna_id;       
		    SELECT INTO ROW TRIM(an1.unikey) as gene_pn, an2.unikey as protein_pn FROM 
		    	allnames an1, allnames an2, ref_kw_table r1, ref_kw_table r2
				WHERE an1.record_id = OLD.gene_id
				AND an1.kw_table_id = r1.kw_table_id
				AND UPPER(r1.kw_table) = ''GENE''
				AND UPPER(an1.is_primary) = ''Y''
				AND an2.record_id = OLD.protein_id
				AND an2.kw_table_id = r2.kw_table_id
				AND UPPER(r2.kw_table) = ''PROTEIN''
				AND UPPER(an2.is_primary) = ''Y'';
				
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name for gene or protein, or kw_table not found for mrna_id ''''%'''' in table ''''%''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
			END IF;
		    identifier := ROW.gene_pn || separator || ROW.protein_pn;

		 ELSIF TG_RELNAME = ''pathway'' THEN
		    rec_id := OLD.pathway_id;                   
		 ELSIF TG_RELNAME = ''pathway_event'' THEN
		    rec_id := OLD.pathway_event_id;       
		 ELSIF TG_RELNAME = ''phenotype'' THEN
		    rec_id := OLD.phenotype_id;       
		    SELECT INTO ROW TRIM(an.unikey) AS gene_pn FROM 
		    	allnames an, ref_kw_table r
				WHERE an.record_id = OLD.gene_id
				AND an.kw_table_id = r.kw_table_id
				AND UPPER(r.kw_table) = ''GENE''
				AND UPPER(an.is_primary) = ''Y'';
				
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name for gene, kw_table or phenotype not found for phenotype_id ''''%'''' in table ''''%''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
			END IF;
		    identifier := ROW.gene_pn || separator || OLD.phenotype;
		 ELSIF TG_RELNAME = ''protein'' THEN
		    rec_id := OLD.protein_id;
		 ELSIF TG_RELNAME = ''protein_ec'' THEN
		    rec_id := OLD.protein_ec_id;
		    SELECT INTO ROW TRIM(an.unikey) as unique_name, ec.ec_name FROM allnames an, ec, ref_kw_table r
				WHERE ec.ec_id = OLD.ec_id
				AND an.record_id = OLD.protein_id
				AND an.kw_table_id = r.kw_table_id
				AND UPPER(r.kw_table) = ''PROTEIN''
				AND UPPER(an.is_primary) = ''Y'';
				
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name for protein or kw_table not found for protein_ec_id ''''%'''' in table ''''%''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
			END IF;
		    identifier := ROW.unique_name || separator || ROW.ec_name; 
		 ELSIF TG_RELNAME = ''protein_state'' THEN
		    rec_id := OLD.protein_state_id;
		    SELECT INTO ROW TRIM(an.unikey) as unique_name, s.type FROM allnames an, state s, ref_kw_table r
				WHERE s.state_id = OLD.state_id
				AND an.record_id = OLD.protein_id
				AND an.kw_table_id = r.kw_table_id
				AND UPPER(r.kw_table) = ''PROTEIN''
				AND UPPER(an.is_primary) = ''Y'';
				
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name for protein or kw_table not found for protein_state_id ''''%'''' in table ''''%''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
			END IF;
		    identifier := ROW.unique_name || separator || ROW.type;             
		 ELSIF TG_RELNAME = ''protein_state_location'' THEN
		    rec_id := OLD.protein_state_location_id;       
		    SELECT INTO ROW TRIM(an.unikey) AS protein_pn, s.type, l.location_name FROM 
		    	allnames an, ref_kw_table r, state s, location l, protein_state ps
				WHERE ps.protein_state_id = OLD.protein_state_id
				AND an.record_id = ps.protein_id
				AND an.kw_table_id = r.kw_table_id
				AND UPPER(r.kw_table) = ''PROTEIN''
				AND UPPER(an.is_primary) = ''Y''
				AND l.location_id = OLD.location_id
				AND s.state_id = ps.state_id;
				
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name for protein, kw_table, state.type or location_name not found for protein_state_location_id ''''%'''' in table ''''%''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
			END IF;
		    identifier := ROW.protein_pn || separator || ROW.type || separator || ROW.location_name;
		 
		 ELSIF TG_RELNAME = ''reference'' THEN
		    rec_id := OLD.reference_id;           
		    identifier := OLD.locator;
		 ELSIF TG_RELNAME = ''specie_compound'' THEN
		    rec_id := OLD.specie_cpd_id;
		    
		    SELECT INTO ROW TRIM(an1.unikey) as cpd_pn, TRIM(an2.unikey) as specie_pn
				FROM  allnames an1, allnames an2, ref_kw_table r1, ref_kw_table r2
				WHERE an1.record_id = OLD.cpd_id
				AND an1.kw_table_id = r1.kw_table_id
				AND UPPER(r1.kw_table) = ''COMPOUND''
				AND UPPER(an1.is_primary) = ''Y''
				AND an2.record_id = OLD.specie_id
				AND an2.kw_table_id = r2.kw_table_id
				AND UPPER(r2.kw_table) = ''SPECIES''
				AND UPPER(an2.is_primary) = ''Y'';
				
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name for compound or species, or kw_table not found for specie_cpd_id ''''%'''' in table ''''%''''. Please put an entry in the respective table before deleting.'', rec_id, TG_RELNAME;
			END IF;
		    identifier := ROW.cpd_pn || separator || ROW.specie_pn;
		 ELSIF TG_RELNAME = ''species'' THEN
		    rec_id := OLD.specie_id;  
		 ELSIF TG_RELNAME = ''state'' THEN
		    rec_id := OLD.state_id;           
		    identifier := OLD.type;  
		 END IF;
	END IF;
	--RAISE EXCEPTION ''RID=% OP=% RN=%'', rec_id, operation_letter, TG_RELNAME;
	IF TG_OP = ''DELETE'' THEN 
		IF TG_RELNAME = ''complex'' OR TG_RELNAME = ''compound'' OR TG_RELNAME = ''event'' OR TG_RELNAME = ''ec'' OR TG_RELNAME = ''gene'' OR TG_RELNAME = ''protein'' OR TG_RELNAME = ''species'' THEN
 	    	SELECT INTO identifier TRIM(unikey) as unique_name FROM allnames WHERE record_id = rec_id AND kw_table_id = table_id AND UPPER(is_primary) = ''Y'';
			-- If there was no Primary Name, raise an exception.
			IF NOT FOUND THEN
				RAISE EXCEPTION ''Trigger do_update: Primary Name for table ''''%'''' with ID ''''%'''' not present in table ''''allnames''''. Please put an entry in ''''allnames'''' before deleting.'', TG_RELNAME, rec_id;
			END IF;
		END IF;
		SQL := ''INSERT INTO update (update_id, kw_table_id, record_id, operation, identifier) VALUES (nextval('' || quote_literal(''update_id_seq'') || ''), '' || table_id || '', '' || rec_id || '', '' || quote_literal(operation_letter) || '', '' || quote_literal(identifier) || '')'';
	ELSE
		SQL := ''INSERT INTO update (update_id, kw_table_id, record_id, operation) VALUES (nextval('' || quote_literal(''update_id_seq'') || ''), '' || table_id || '', '' || rec_id || '', '' || quote_literal(operation_letter) || '')'';	
	END IF;
			
 
 --RAISE EXCEPTION ''%'', SQL;
 EXECUTE SQL;
 RETURN NEW;
END;
' LANGUAGE 'plpgsql';