#!/usr/local/bin/perl



use DBI;

use DOMELIB;

use Math::Trig;

use POSIX;

use strict "vars";



#===============================================================================

# This script will get data from processed1_data and calculate the ratio and

# and pvalue for both logged and un-logged intensities of microarray data.

#

# author: X. Jing Li

# date: May 20, 2005

# reversion: 

# 7/27/2006: add ratio_of_medians, difference_of_medians, ratio_of_medians_l, 

#   difference_of_medians_l into script.

#===============================================================================



#===============================================================================

my ($spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2,

    $protocol_type_id, $extract_id, $force_add,

    $database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc,

    $ok, $time, $count, $detail, $process1_id);

    

#== get parameters from ARGV:

#($spid1, $spid2, $protocol_type_id, $extract_id, $tissue_type_label_id1, $tissue_type_label_id2, $force_add) = @ARGV;

($spid1, $spid2, $protocol_type_id, $extract_id, $tissue_type_label_id1, $tissue_type_label_id2) = @ARGV;

#print "@ARGV\n";



#== connect to the database:

($database, $username, $password, $bnet_DB, $specieDB) = database_login2();

$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";

$dbh->begin_work;

$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";

#print "$sql\n";

$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#print "rc = $rc\n";



#== start 

#print "Starting time = " . TimeStamp() . "\n";



#== get the process1_id:

$sql = "select process1_id from process1 where sampling_point1_id = $spid1 AND sampling_point2_id = $spid2 AND sp1_tissue_type_label_id = $tissue_type_label_id1 AND sp2_tissue_type_label_id = $tissue_type_label_id2";

$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

$process1_id = $sth->fetchrow();



if ($process1_id ne "") {

    #== check for existance of data in comparison table:

    $sql = "select count(*) from sp_comparison where process1_id = $process1_id and protocol_id = (select protocol_id from protocol where protocol_type_id = $protocol_type_id and protocol_name = \'ttest\') and extract_id = $extract_id";

    #print "$sql\n";

    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $count = $sth->fetchrow();

    #print "count = $count\n";

    if ($count > 0) {

        $detail = getDetail($dbh, $protocol_type_id, $extract_id, $spid1, $spid2, $tissue_type_label_id1, $tissue_type_label_id2);

        print $detail;

        $sth->finish;

        $dbh->rollback;

        $dbh->disconnect;

        #$time = TimeStamp();

        #print "Ending time = $time\n";

        exit 99;

    }

}



$ok = calculateRatioPvalue($dbh, $protocol_type_id, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2, $extract_id);

if ($ok eq "SUCCESS") {

    $dbh->commit;

    print "Data insertion is Success\n";

    $sth->finish;

    $dbh->disconnect;

    #$time = TimeStamp();

    #print "Ending time = $time\n";

    exit 0;

} else {

    $dbh->rollback;

    print "Data insertion is failed.\n";

    $sth->finish;

    $dbh->disconnect;

    exit 1;

}



###########################################################################

# subroutine: calculateRatioPvalue

# ($dbh, $protocol_type_id, $sampling_point_id, $tissue_type_label_id)

#    this subroutine calculate the 

#

# parameters in: $dbh --- the database handle,

#                $spid -- sampling point id,

#                $ptid -- protocol_type_id,

#                $extract_id -- 

#                $tissue_type_label_id --- the tissue type id,

# parameter out: $ok --- the success of the program. "SUCCESS" or "FAILED"

#

# X. Jing Li, 5/17/2005

###########################################################################

sub calculateRatioPvalue {

    my ($dbh, $ptid, $spid, $ttlid1, $spid2, $ttlid2, $extract_id) = @_;

    my ($sql, $sth, $rc, $mainsql, $sql1, $sql2, 

        $kwid, $prot_id, $prot_name, $pid_mean, $pid_ttest, $process1_id,

        $ref_medians_1, $ref_Means_1, $ref_Stdevs_1, $ref_N_1, $ref_medians_l_1, $ref_Means_l_1, $ref_Stdevs_l_1,

        %Medians_1, %Means_1, %Stdevs_1, %Counts_1, %Medians_l_1, %Means_l_1, %Stdevs_l_1,

        $median1, $mean1, $stdev1, $count1, $median1_l, $mean1_l, $stdev1_l,

        $ref_medians_2, $ref_Means_2, $ref_Stdevs_2, $ref_N_2, $ref_medians_l_2, $ref_Means_l_2, $ref_Stdevs_l_2,

        %Medians_2, %Means_2, %Stdevs_2, %Counts_2, %Medians_l_2, %Means_l_2, %Stdevs_l_2,

        $median2, $mean2, $stdev2, $count2, $median2_l, $mean2_l, $stdev2_l,

        @rids, $rid, $diff, $diff_l,$pvalue, $pvalue_l, $ratio_lin, $ratio_log, $ratio_medians, $ratio_medians_l, $diff_medians, $diff_medians_l);





    #$sql = "SELECT kw_table_id FROM ref_kw_table WHERE upper(kw_table) = 'REPORTER'";

    $sql = "select distinct(kw_table_id) from sp_summary where extract_id = $extract_id";

#    print "$sql\n";

    $sth = $dbh->prepare($sql) or die "Can't prepare:$DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $kwid = $sth->fetchrow();

    if ($kwid eq "") {

        print "There is no summary data been calculated for these sample points.\n";

        $sth->finish;

        return "FAILED";

    }

    #print "kwid = $kwid\n";



    #== get protocol_ids for mean and ttest:

    $sql = "select protocol_id, protocol_name from protocol where protocol_type_id = $ptid";

    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    while (($prot_id, $prot_name) = $sth->fetchrow()) {

        if ($prot_name eq "ttest") {

            $pid_ttest = $prot_id;

        } elsif ($prot_name eq "Mean") {

            $pid_mean = $prot_id;

        }

    }



    #== check the existance of the process1_id and existance of pre-calculated 

    #== data in comparison:

    $sql = "select process1_id from process1 where sampling_point1_id = $spid1 and sampling_point2_id = $spid2 and sp1_tissue_type_label_id = $ttlid1 and sp2_tissue_type_label_id = $ttlid2";

    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $process1_id = $sth->fetchrow();

    if ($process1_id eq "") {

        $process1_id = createProcess1Id($dbh, $spid1, $ttlid1, $spid2, $ttlid2);

    }

    

    #== get mean, std_dev, N_data from processed1_data and save in hashes:

    #print "before call sub: $kwid, $pid_mean, $spid1, $ttlid1, $extract_id\n";

    ($ref_Means_1, $ref_Stdevs_1, $ref_medians_1, $ref_N_1, $ref_Means_l_1, $ref_Stdevs_l_1, $ref_medians_l_1)

        = getMeansStdevs($dbh, $pid_mean, $kwid, $spid1, $ttlid1, $extract_id);

    ($ref_Means_2, $ref_Stdevs_2, $ref_medians_2, $ref_N_2, $ref_Means_l_2, $ref_Stdevs_l_2, $ref_medians_l_2)

        = getMeansStdevs($dbh, $pid_mean, $kwid, $spid2, $ttlid2, $extract_id);

    

    %Means_1 = %$ref_Means_1;

    %Stdevs_1 = %$ref_Stdevs_1;

    %Counts_1 = %$ref_N_1;

    %Medians_1 = %$ref_medians_1;

    %Means_l_1 = %$ref_Means_l_1;

    %Stdevs_l_1 = %$ref_Stdevs_l_1;

    %Medians_l_1 = %$ref_medians_l_1;



    %Means_2 = %$ref_Means_2;

    %Stdevs_2 = %$ref_Stdevs_2;

    %Medians_2 = %$ref_medians_2;

    %Means_l_2 = %$ref_Means_l_2;

    %Stdevs_l_2 = %$ref_Stdevs_l_2;

    %Medians_l_2 = %$ref_medians_l_2;

    %Counts_2 = %$ref_N_2;



    #== check if any set is empty. If yes print error message and return failed:

    if (keys %Counts_1 == 0 && (keys %Counts_2 == 0)) {

        print "No mean and standard-deviation been calculated for samples in both Sample 1 and Sample 2.\n";

        $sth->finish;

        return "FAILED";

    } elsif (keys %Counts_1 == 0) {

        print "No means and standard-deviation been calculated for the sample in Sample 1.\n";

        $sth->finish;

        return "FAILED";

    } elsif (keys %Counts_2 == 0) {

        print "No means and standard-deviation been calculated for the sample in Sample 2.\n";

        $sth->finish;

        return "FAILED";

    }



    #== get all ids from Mean_1 and Meand_2:

    @rids = keys %Means_1;

    foreach $rid (keys %Means_2) {

        if (! exists $Means_1{$rid}) {

            push @rids, $rid;

        }

    }



    @rids = sort {$a <=> $b} @rids;

    $mainsql = "";

    foreach $rid (@rids) {

        $diff = null;

        $diff_l = null;

        $pvalue = null;

        $pvalue_l = null;

        $ratio_lin = null;

        $ratio_log = null;

        $ratio_medians = null;

        $ratio_medians_l = null;

        $diff_medians = null;

        $diff_medians_l = null;



        $mean1 = $Means_1{$rid};

        $stdev1 = $Stdevs_1{$rid};

        $median1 = $Medians_1{$rid};

        $mean1_l = $Means_l_1{$rid};

        $stdev1_l = $Stdevs_l_1{$rid};

        $median1_l = $Medians_l_1{$rid};

        $count1 = $Counts_1{$rid};



        $mean2 = $Means_2{$rid};

        $stdev2 = $Stdevs_2{$rid};

        $median2 = $Medians_2{$rid};

        $mean2_l = $Means_l_2{$rid};

        $stdev2_l = $Stdevs_l_2{$rid};

        $median2_l = $Medians_l_2{$rid};

        $count2 = $Counts_2{$rid};

        

        #== get ratio and difference:

        if ($mean1 ne "" && $mean2 ne "") {

            $diff = $mean1 - $mean2;

        }

        if ($mean1_l ne "" && $mean2_l ne "") {

            $diff_l = $mean1_l - $mean2_l;

        }

        if ($mean1 ne "" && $mean2 != 0) {

            $ratio_lin = $mean1 / $mean2;

        }

        if ($mean1_l ne "" && $mean2_l != 0) {

            $ratio_log = $mean1_l / $mean2_l;

        }

        if ($median1 ne "" && $median2 != 0) {

            $ratio_medians = $median1 / $median2;

        }

        if ($median1 ne "" && $median2 ne "") {

            $diff_medians = $median1 - $median2;

        }

        if ($median1_l ne "" and $median2_l != 0) {

            $ratio_medians_l = $median1_l / $median2_l;

        }

        if ($median1_l ne "" && $median2_l ne "") {

            $diff_medians_l = $median1_l - $median2_l;

        }



        #== get p-value

        if ($count1 > 1 && $count2 > 1) {

            $pvalue = computePvalue($mean1, $stdev1, $count1, $mean2, $stdev2, $count2);

            $pvalue_l = computePvalue($mean1_l, $stdev1_l, $count1, $mean2_l, $stdev2_l, $count2);

        }



        if ($ratio_lin != null || $ratio_log != null || $diff != null || $diff_l != null || $pvalue != null || $pvalue!= null) {

            $sql = "insert into sp_comparison ";

            $sql1 = "(sp_comparison_id, protocol_id, process1_id, extract_id, kw_table_id, record_id";

            $sql2 = "(nextval('sp_comparison_id_seq'), $pid_ttest, $process1_id, $extract_id, $kwid, $rid";

            

            if ($diff != null) {

                $sql1 .= ", \"difference_of_means\"";

                $sql2 .= ", $diff";

            }

            if ($diff_l != null) {

                $sql1 .= ", \"difference_of_means_l\"";

                $sql2 .= ", $diff_l";

            }

            if ($pvalue != null) {

                $sql1 .= ", \"p_value\"";

                $sql2 .= ", $pvalue";

            }

            if ($pvalue_l != null) {

                $sql1 .= ", \"p_value_l\"";

                $sql2 .= ", $pvalue_l";

            }

            if ($ratio_lin != null) {

                $sql1 .= ", \"ratio_of_means\"";

                $sql2 .= ", $ratio_lin";

            }

            if ($ratio_log != null) {

                $sql1 .= ", \"ratio_of_means_l\"";

                $sql2 .= ", $ratio_log";

            }

            if ($ratio_medians != null) {

                $sql1 .= ", \"ratio_of_medians\"";

                $sql2 .= ", $ratio_medians";

            }

            if ($ratio_medians_l != null) {

                $sql1 .= ", \"ratio_of_medians_l\"";

                $sql2 .= ", $ratio_medians_l";

            }

            if ($diff_medians != null) {

                $sql1 .= ", \"difference_of_medians\"";

                $sql2 .= ", $diff_medians";

            }

            if ($diff_medians_l != null) {

                $sql1 .= ", \"difference_of_medians_l\"";

                $sql2 .= ", $diff_medians_l";

            }

            $sql1 .= ")";

            $sql2 .= ")";

            $sql .= $sql1 . " VALUES " . $sql2 . ";\n";

            #print "sql = $sql\n";

            $mainsql .= $sql;

        }

    }

#    my $temp_file = "./tmp-plots/temp.sql";

#    open (TMP, ">$temp_file") or die "Can't open the temp file $temp_file.$!";

#    print TMP $mainsql;

#    close TMP;



    if ($mainsql ne "") {

        $sth = $dbh->prepare($mainsql) or die "Can't prepare:$DBI::errstr.";

        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

        $sth->finish;

        return "SUCCESS";

    } else {

        print "There is no sufficient data for calculating ratio and p_values from these parameters. No data has been inserted.\n";

        $sth->finish;

        return "FAILED";

    }

}



###########################################################################

# subroutine: getMeansStdevs

#          ($dbh, $kwid, $pid_mean, $spid, $tissue_type_label_id, $extract_id)

# this subroutine get means, sted_devs and number of items from processed1_data

# for a specifical sampling_point, protocol_type_id.

#

# parameters in: $mean1, $mean2 -- the means of two samples

#                $stdev1, $stdev2 -- the stardard deviation of 2 samples,

#                $count1, $count2 -- the number of items in each sample,

# parameter out: $pvalue --- the p-value

#

###########################################################################

sub getMeansStdevs {

    my ($dbh, $pid_mean, $kwid, $spid, $tissue_type_label_id, $extract_id) = @_;

    my ($sql, $sth, $rc, %Means, %Stdevs, %Medians, %Means_l, %Stdevs_l, %Medians_l, %Counts,

        $rid, $mean, $stdev, $median, $count, $mean_l, $stdev_l, $median_l);



    #print "$pid_mean, $kwid, $spid, $tissue_type_label_id, $extract_id\n";

    $sql = "SELECT record_id, \"mean\", std_dev, \"N_data\", \"mean_l\", std_dev_l, \"median\", \"median_l\" FROM sp_summary WHERE kw_table_id = $kwid AND sampling_point_id = $spid AND label_id = $tissue_type_label_id AND protocol_id = $pid_mean AND extract_id = $extract_id ORDER BY record_id";

#    print "$sql\n";

    $sth = $dbh->prepare($sql) or die "Can't prepare:$DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    while (($rid, $mean, $stdev, $count, $mean_l, $stdev_l, $median, $median_l) = $sth->fetchrow()) {

        $Means{$rid} = $mean;

        $Stdevs{$rid} = $stdev;

        $Medians{$rid} = $median;

        $Means_l{$rid} = $mean_l;

        $Stdevs_l{$rid} = $stdev_l;

        $Medians_l{$rid} = $median_l;

        $Counts{$rid} = $count;

    }



    $sth->finish;

    return \%Means, \%Stdevs, \%Medians, \%Counts, \%Means_l, \%Stdevs_l, \%Medians_l;

}



###########################################################################

# subroutine: computerPvalue($mean1, $stdev1, $count1, $mean2, $stdev2, $count2)

# this subroutine calculate the p-value by means, sted_devs and number of items.

#

# parameters in: $mean1, $mean2 -- the means of two samples

#                $stdev1, $stdev2 -- the stardard deviation of 2 samples,

#                $count1, $count2 -- the number of items in each sample,

# parameter out: $pvalue --- the p-value

#

###########################################################################

sub computePvalue {

    my ($mean1, $stdev1, $count1, $mean2, $stdev2, $count2) = @_;

    my ($var1, $var2, $ttest, $numeritor, $denominator, $df, $abs_ttest, 

        $pvalue);



    $var1 = $stdev1**2;

    $var2 = $stdev2**2;

    if ($var1 != 0 && $var2 != 0) {

        $ttest = ($mean1 - $mean2)/sqrt($var1/$count1 + $var2/$count2);

    }

    if ($ttest != 0) {

        $numeritor = ($var1/$count1 + $var2/$count2)**2;

        $denominator = ($var1/$count1)** 2 / ($count1-1);

        $denominator += ($var2/$count2)** 2 / ($count2-1);

        $df = $numeritor / $denominator;

        if (($df - floor($df)) >= 0.5) {

            $df = ceil($df);

        } else {

            $df = floor($df);

        }

        $abs_ttest = abs($ttest);

        $pvalue = 2 * getPvalue($abs_ttest, $df);

    }



    $sth->finish;

    return $pvalue;

}



###########################################################################

# subroutine: getPvalue($t_score, $df)

# this subroutine calculate the p-value by t-score and degree of freedom

# parameters in: $tscore --- t-test score 

#                $df --- degree of freedom

# parameter out: $pvalue --- the p-value

#

###########################################################################

sub getPvalue {

    my ($ttest, $df) = @_;

    my ($a, $w, $s, $c, $t1, $t2, $j1, $j2, $k2);

    

    $a = 0.36338023;

    $w = atan($ttest / sqrt($df));

    $s = sin($w);

    $c = cos($w);

    

    if ($df % 2 == 0) {  # degree of freedom is even

        $t1 = $s;

        if ($df == 2) {

            return (1 - 0.5 * (1 + $t1));

        } else {

            $t2 = $s;

            $j1 = -1;

            $j2 = 0;

            $k2 = ($df - 2)/2;

        }

    } else {

        $t1 = $w;

        if ($df == 1) {    # special case $df = 1

            return 1 - (0.5 * (1 + $t1 * (1 - $a)));

        } else {

            $t2 = $s * $c;

            $t1 = $t1 + $t2;

            if ($df == 3) {    # special case $df = 3

                return 1 - 0.5 * (1 + $t1 * (1 - $a));

            } else {

                $j1 = 0;

                $j2 = 1;

                $k2 = ($df - 3)/2;

            }

        }

    }

    for (my $i=1; $i<= $k2; $i++) {

        $j1 = $j1 + 2;

        $j2 = $j2 + 2;

        $t2 = $t2 * $c * $c * $j1/$j2;

        $t1 = $t1 + $t2;

    }

    return 1 - 0.5 * (1 + $t1 * ( 1 -$a * ($df % 2)));

}



###########################################################################

# subroutine: getDetail

# this subroutine get the error details for the set of parameters;

#

# parameters in: $dbh -- the database handle,

#                $ptid -- the protocol_type id,

#                $extract_id --

#                $spid1 -- sampling_point_id for the one to be compared;

#                $spid2 -- sampling_point_id for the sample to be compared with,

#                $tissue_type_label_id1 -- the tissue type for spid1,

#                $tissue_type_label_id2 -- the tissue type for spid2,

# parameter out: $detail -- the details for the two samples and the protocol 

#                   type name;

#

# X. Jing Li, 5/24/2005

#

###########################################################################

sub getDetail {

    my ($dbh, $ptid, $extract_id, $spid1, $spid2, $tissue_type_label_id1, $tissue_type_label_id2) = @_;

    my ($detail, $sql, $sth, $rc, $pt_name, $extract_name, $exp_set_name, 

        $exp_name, $label_name, $spid, $ttlid, $tt_name, %exp_details);



    #== get the details for the protocol_type_id:

    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";

    $sth = $dbh->prepare($sql) or die "Can't prepare:$DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $pt_name = $sth->fetchrow();



    #== get the details for the extract_id:

    $sql = "select \"name\" from extract where extract_id = $extract_id";

    $sth = $dbh->prepare($sql) or die "Can't prepare:$DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $extract_name = $sth->fetchrow();



    #== get detail about the sampling_point_ids:

    $sql = "select ES.exp_set_name, E.exp_name, L.\"name\", SP.sampling_point_id from experiment_set ES, experiment E, label L, sampling_point SP where SP.sampling_point_id in ($spid1, $spid2) and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";

    $sth = $dbh-> prepare($sql) or die "Can't preparet: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    while(($exp_set_name, $exp_name, $label_name, $spid) = $sth->fetchrow()) {

        $exp_details{$spid} = "experiment set \"$exp_set_name\", experiment \"$exp_name\" at time point \"$label_name\" "; 

    }



    #== get the tissue_type name:

    $sql = "select label_id, \"name\" from label where label_id in ($tissue_type_label_id1, $tissue_type_label_id2)";

    $sth = $dbh-> prepare($sql) or die "Can't preparet: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    while(($ttlid, $tt_name) = $sth->fetchrow()) {

        if ($ttlid == $tissue_type_label_id1) {

            $exp_details{$spid1} .= "for tissue type \"$tt_name\" ";

        } elsif ($ttlid == $tissue_type_label_id2) {

            $exp_details{$spid2} .= "for tissue type \"$tt_name\" ";

        }

    }

    

    #== create the error message detail:

    $detail = "The ratio and P-value for " . $exp_details{$spid1} . "VS. " . $exp_details{$spid2} . "with extraction \"$extract_name\", analytical method \"$pt_name\" has been calculated before. Unless you changed the mean and standard deviations for the samples at these time points, you will not be recalculate the ratio and p-values for these samples.\n";



    $sth->finish;

    return $detail;

}





###########################################################################

# subroutine: createProcess1Id($dbh, $spid1, $tissue_type_label_id1, $spid2, 

#         $tissue_type_label_id2):

# This subroutine create a process1_id. 

#

# parameters in: $dbh -- the database handle,

#                $spid1 -- sampling_point_id for the one to be compared;

#                $spid2 -- sampling_point_id for the sample to be compared with,

#                $tissue_type_label_id1 -- the tissue type for spid1,

#                $tissue_type_label_id2 -- the tissue type for spid2,

# parameter out: $process1_id -- the new process1_id;

#

# X. Jing Li, 5/24/2005

###########################################################################

sub createProcess1Id {

    my ($dbh, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2) = @_;

    my ($sql, $sth, $rc, $process1_id);



    $sql = "select nextval('process1_id_seq')";

    $sth = $dbh-> prepare($sql) or die "Can't preparet: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $process1_id = $sth->fetchrow();



    $sql = "insert into process1 (process1_id, sampling_point1_id, sp1_tissue_type_label_id, sampling_point2_id, sp2_tissue_type_label_id) values ($process1_id, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2)";

    #print "$sql\n";

    $sth = $dbh-> prepare($sql) or die "Can't preparet: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";



    $sth->finish;

    return $process1_id;

}

