#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use Math::Trig;
use POSIX;
use strict "vars";

#===============================================================================
# This script will calculate the mean and std_dev for microarray data.
#
# author: X. Jing Li
# date: May 17, 2005
# reversion:
# 5/26/2005: change DOME table from processed1_data and comparison to SP_summary
#   and SP_comparison.
#===============================================================================

#===============================================================================
my ($spid, $protocol_type_id, $extract_id, $tissue_type_label_id, $force_add,
    $database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc,
    $ok, $time, $count, $detail);

#== get parameters from ARGV:
($spid, $protocol_type_id, $extract_id, $tissue_type_label_id, $force_add) = @ARGV;
#print "ARGV = @ARGV\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#print "rc = $rc\n";

#== start 
$time = TimeStamp();
print "starting time = $time\n";

if ($force_add == 0) {
#== check for existance of the parameters combination in processed1_data table:
    $sql = "select count(*) from sp_summary where sampling_point_id = $spid and protocol_id = (select protocol_id from protocol where protocol_type_id = $protocol_type_id and protocol_name = \'Mean\') and label_id = $tissue_type_label_id";
#    print "$sql\n";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $count = $sth->fetchrow();
    #print "count = $count\n";
    if ($count > 0) {
        $detail = getDetail($dbh, $spid, $protocol_type_id, $extract_id, $tissue_type_label_id);
        print $detail;
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 99;
    }
} elsif ($force_add == 1) {
    #== must delete all records in processed1_data and related records in 
    #== comparison.
    $sql = "delete from sp_summary where sampling_point_id = $spid and protocol_id = (select protocol_id from protocol where protocol_type_id = $protocol_type_id and protocol_name = \'Mean\') and label_id = $tissue_type_label_id";
    #print "$sql\n";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $sql = "delete from sp_comparison where process1_id in (select process1_id from process1 where sampling_point1_id = $spid OR sampling_point2_id = $spid) and protocol_id = (select protocol_id from protocol where protocol_type_id = $protocol_type_id and protocol_name = \'ttest\')";
    #print "$sql\n";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
}

$ok = calculateMeanStdev($dbh, $protocol_type_id, $spid, $extract_id, $tissue_type_label_id);

if ($ok eq "SUCCESS") {
    $dbh->commit;
    print "Data insertion is Success\n";
    $sth->finish;
    $dbh->disconnect;
    $time = TimeStamp();
    print "Ending time = $time\n";
    exit 0;
} else {
    $dbh->rollback;
    print "Data insertion is failed\n";
    $sth->finish;
    $dbh->disconnect;
    $time = TimeStamp();
    print "Ending time = $time\n";
    exit 1;
}

###########################################################################
# subroutine: getSamplingReplicateIds($dbh, $sampling_point_id):
#    this subroutine get all sampling_replicate_ids for the sampling point id.
#
# parameters in: $dbh --- the database handle,
#                $sampling_point_id --- the sampling point id 
# parameter out: @ids --- ref for the replicate ids for the sampling_point_id.
#
###########################################################################
sub getSamplingReplicateIds {
    my ($dbh, $sampling_point_id) = @_;
    my ($sql, $sth, $rc, $ref_ids, @ids, $id);
    #print "sampling_point_id = $sampling_point_id\n";

    $ref_ids = [];
    @ids = @$ref_ids;
    $sql = "select sampling_replicate_id from sampling_replicate where ";
    $sql .= " sampling_point_id = \'$sampling_point_id\'";
    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    while ($id = $sth->fetchrow) {
        push (@ids, $id);
    }
    @ids = sort {$a <=> $b } @ids;
    $sth->finish;
    return @ids;
}

###########################################################################
# subroutine: calculateMeanStdev
# ($dbh, $protocol_id, $sampling_point_id, $tissue_type_label_id)
#    this subroutine calculate the 
#
# parameters in: $dbh --- the database handle,
#                $spid -- sampling point id,
#                $ptid -- protocol_type_id,
#                $extract_id -- 
#                $tissue_type_label_id --- the tissue type id,
# parameter out: $ok --- the success of the program. "SUCCESS" or "FAILED"
#
# X. Jing Li, 5/17/2005
###########################################################################
sub calculateMeanStdev {
    my ($dbh, $ptid, $spid, $extract_id, $tissue_type_label_id) = @_;
    my (@ids, $id, $ref_original_value, $ok,
        $ref_value_hashs, @value_hashs, $ref_hash,
        $ref_l_value_hashs, @l_value_hashs, $ref_original_value_l, $ref_l_hash);

    #== get all sampling_replicate_ids 
    @ids = getSamplingReplicateIds($dbh, $spid);
#    print "spids = @ids\n";

    $ref_value_hashs = [];
    @value_hashs = @$ref_value_hashs;
    $ref_l_value_hashs = [];
    @l_value_hashs = @$ref_l_value_hashs;
    foreach $id (@ids) {
        $ref_original_value = getOriginalValue
            ($dbh, $id, "intensity", $ptid);
        push (@value_hashs, $ref_original_value);
        $ref_original_value_l = getOriginalValue
            ($dbh, $id, "intensity_l", $ptid);
        push (@l_value_hashs, $ref_original_value_l);
    }

    #== regroup the data from array of ref_hash to a hash of array with 
    #== reporter_id as the hash keys:
    if (@value_hashs == 0 && @l_value_hashs == 0) {
        print "There is no data in DOME for this parameters set.";
        return "FAILED";
    }

    $ref_hash = regroupArrayHashToHashArray(\@value_hashs);
    $ref_l_hash = regroupArrayHashToHashArray(\@l_value_hashs);
    $ok = computeMeanStdev($dbh, $extract_id, $ptid, $spid, $tissue_type_label_id, $ref_hash, $ref_l_hash);
    return $ok;
}

###########################################################################
# subroutine: getOriginalValue($dbh, $spid, $field, $ptid):
# This subroutine gets the original data from the "ma_norm" table where
# sampling_point_id and field name are given.
#
# parameters in: $dbh --- database handler;
#                $srid --- sampling_replicate_id;
#                $field --- field name;
#                $ptid --- protocol_type_id;
# parameter out: $ref_hash --- the reference of the hash where reporter_id is
#                   keys and field value are the values.
#
###########################################################################
sub getOriginalValue {
    my ($dbh, $srid, $field, $ptid) = @_;
    my ($sql, $sth, $rc, $value, $ref_hash, %value_hash, @new_array,
        $reporter_id, @value_array, $ref_value_array, %repeated_rids, $rid);

    $ref_hash = ();
    %value_hash = %$ref_hash;

    $sql = "select reporter_id, $field from \"ma_norm\" where \"sampling_replicate_id\" = $srid and protocol_type_id = $ptid";
    #print "sql = $sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    while (($reporter_id, $value) = $sth->fetchrow) {
        #print "$reporter_id, $value\n";
        if ( $value != null) {
            #== save in the hash:
            if (!exists ($value_hash{$reporter_id})) {
                $value_hash{$reporter_id} = $value;
            } else {
                if ( !(exists $repeated_rids{$reporter_id})) {
                    $ref_value_array = createValueArray($value);
                    @new_array = ($value_hash{$reporter_id});
                    $ref_value_array = updateValueArray($ref_value_array, \@new_array);
                    $repeated_rids{$reporter_id} = $ref_value_array;
                } else {
                    $ref_value_array = $repeated_rids{$reporter_id};
                    @new_array = ($value);
                    $ref_value_array = updateValueArray($ref_value_array, \@new_array);
                    $repeated_rids{$reporter_id} = $ref_value_array;
                }
            }
        } 
#        print "rid = $reporter_id, value_array = @value_array\n";
    }
    foreach $rid (keys %repeated_rids) {
        $ref_value_array = $repeated_rids{$rid};
        #print "reprint, rid = $rid, ref_array = $ref_value_array\n";
        $value = getArrayMean($ref_value_array);
        $value_hash{$rid} = $value;
    }

#    my @rids = keys %value_hash;
#    print "ptid = $ptid, field = $field, there are  " . @rids . " rids for srid = $srid\n";

    $sth->finish;
    $ref_hash = \%value_hash;

    return $ref_hash;
}

###########################################################################
# subroutine: createValueArray($v):
# This subroutine create a anynomouse array. 
#
# parameters in: $v --- the first value for the anynomous array
# parameter out: $ref_array --- the reference of the anynomous array
#
###########################################################################
sub createValueArray {
    my ($v) = @_;
    my ($ref_array, @array);

    $ref_array = [];
    @array = @$ref_array;
    push @array, $v;
    return \@array;
}

###########################################################################
# subroutine: updateValueArray($rv1, $rv2)
#
# parameters in: $rv1 --- the original arry reference
#                $rv2 --- the new array reference
# parameter out: $ref_array --- reference of the new array 
#
###########################################################################
sub updateValueArray {
    my ($rv1, $rv2) = @_;
    my (@value1, @value2, $ref_array, @array);

    $ref_array = [];
    @array = @$ref_array;
    @value1 = @$rv1;
    @value2 = @$rv2;
    push @array, @value1;
    push @array, @value2;
    return \@array;
}
###########################################################################
# subroutine: getArrayMean($ref_array):
# This subroutine get the mean of an array. 
#
# parameters in: $ref_array --- the reference of the array
# parameter out: $mean --- the average value of the array
# X. Jing Li, 5/18/2005
###########################################################################
sub getArrayMean {
    my ($ref_array) = @_;
    my (@array, $v, $mean);

    $mean = 0;
    @array = @$ref_array;
    #print "array = @array\n";
    if (@array > 0) {
        foreach $v (@array) {
            $mean = $mean + $v;
        }
        $mean = $mean / @array;
    } else {
        $mean = 0;
    }
    return $mean;
}

###########################################################################
# subroutine: regroupArrayHashsToHashArrays($ref_array_hashs):
# re-group a array of hashs to a hash of arrays with the sub-hash keys as the 
# main hash's keys.
#
# parameters in: $ref_array --- the ref of array of hashs
# parameter out: $ref_hash --- reference of the hash of the arrays.
#
###########################################################################
sub regroupArrayHashToHashArray {
    my ($ref_array_hashs) = @_;
    my (@array_hashs, $ref_hash_arrays, %hash_arrays, $i, $ref_value, %hash, 
        $id, $value, $ref_v);

    #print "array_ref = $ref_array_hashs\n";
    @array_hashs = @$ref_array_hashs;
    #print "array items = " . @array_hashs . "\n";
    $ref_hash_arrays = {};
    %hash_arrays = %$ref_hash_arrays;
    for($i=0; $i<@array_hashs; $i++) {
        $ref_value = $array_hashs[$i];
        #print "ref_value = $ref_value\n";
        %hash = %$ref_value;
        foreach $id (keys %hash) {
            $value = $hash{$id};
            if (!exists ($hash_arrays{$id})) {
                $hash_arrays{$id} = createValueArray($value); 
            } else {
                $ref_v = $hash_arrays{$id};
                push (@$ref_v, $value);
                $hash_arrays{$id} = $ref_v;
            }
        }
    }
    return \%hash_arrays;
}

###########################################################################
# subroutine:
# computeMeanStdev($dbh,  $extract_id, $ptid, $spid, $ref_hash, $ref_l_hash)
# parameters in: $dbh -- database handle, 
#                $extract_id -- ,
#                $ptid -- protocol_type_id
#                $spid -- the sampling_point_id,
#                $ref_hash, $ref_l_hash -- the references for hash of arrays of
#                     data of intensity
#                     and intensity_l. 
# parameter out: $ok -- the success of the program.
#
###########################################################################
sub computeMeanStdev {
    my ($dbh, $extract_id, $ptid, $spid, $tissue_type_label_id, $ref_hash, $ref_l_hash) = @_;
    my ($sql, $sth, $rc, $mainsql, $sql1, $sql2, $pid_mean, $kwid, @keys,
        @l_keys, $rid, %all_rids, %hash, $ref_array, $mean, $stddev, $data_N, 
        %hash_l, $ref_l_array, $l_mean, $l_stddev, $data_N_l);

    my $temp_file = "./tmp-plots/temp.sql";
    open (LOG, ">$temp_file") or die "Can not open log file $temp_file.\n";

    #== get the protocol_id:
    $sql = "select protocol_id from protocol where protocol_type_id = $ptid and upper(protocol_name) = \'MEAN\'";
    $sth = $dbh-> prepare($sql) or die "Can't preparet: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pid_mean = $sth->fetchrow();
#    print "pid_mean = $pid_mean\n";
    if ($pid_mean eq "") {
        print "There is no protocol_id for Means of protocol_type_id \"$ptid\".\n";
        $sth->finish;
        return "FAILED";
    }

    #== get kw_table_id:
    $sql = "select kw_table_id from ref_kw_table where kw_table = 'REPORTER'";
    $sth = $dbh-> prepare($sql) or die "Can't preparet: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $kwid = $sth->fetchrow();

    %hash = %$ref_hash;
    %hash_l = %$ref_l_hash;
    @keys = keys (%hash);
    @l_keys = keys %hash_l;
    foreach $rid (@keys) {
        $all_rids{$rid} = 1;
    }
    foreach $rid (@l_keys) {
        $all_rids{$rid} = 1;
    }
    @keys = sort {$a <=> $b} keys %all_rids;
#    print "total keys = " . @keys . "\n@keys\n";

    $mainsql = "";
    foreach $rid (@keys) {
        $ref_array = $hash{$rid};
#        my @array = @$ref_l_array;
#        print LOG "rid = $rid, array = @array\n";
        ($mean, $stddev, $data_N) = getMeanStdev($ref_array);
        $ref_l_array = $hash_l{$rid};
#        print "ref_l_array = $ref_l_array\n";
        ($l_mean, $l_stddev, $data_N_l) = getMeanStdev($ref_l_array);
#        print "rid = $rid, l_mean = $l_mean, l_stdev = $l_stddev, $data_N_l\n";

        #== create the sql query:
        if ($mean ne null || $stddev ne null || $data_N > 0 || $l_mean ne null || $l_stddev ne null) {
            $sql1 = "insert into sp_summary ";
            $sql1 .= "(sp_summary_id, extract_id, sampling_point_id, record_id, kw_table_id, protocol_id, label_id";
            $sql2 = " values (nextval('sp_summary_id_seq'), $extract_id, $spid, $rid, $kwid, $pid_mean, $tissue_type_label_id";
            if ($mean ne null){
                $sql1 .= ", \"mean\"";
                $sql2 .= ", $mean";
            }
            if ($stddev ne null){
                $sql1 .= ", \"std_dev\"";
                $sql2 .= ", $stddev";
            }
            if ($data_N > 0) {
                $sql1 .= ", \"N_data\"";
                $sql2 .= ", $data_N";
            }
            if ($l_mean ne null){
                $sql1 .= ", \"mean_l\"";
                $sql2 .= ", $l_mean";
            }
            if ($l_stddev ne null){
                $sql1 .= ", \"std_dev_l\"";
                $sql2 .= ", $l_stddev";
            }

            $sql1 .= ")";
            $sql2 .= ");\n";
            $mainsql .= $sql1 . $sql2;
        }
    }

#    my $temp_file = "./tmp-plots/temp.sql";
#    open (LOG, ">$temp_file") or die "Can not open log file $temp_file.\n";
#    print LOG "$mainsql";
#    close LOG;

    if ($mainsql ne "") {
        $sth = $dbh-> prepare($mainsql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    }
    $sth->finish;

    return "SUCCESS";
}

###########################################################################
# subroutine: getMeanStdev($ref_array):
# This subroutine get the mean of an array. 
#
# parameters in: $ref_array --- the reference of the array
# parameter out: $mean --- the average value of the array
#                $std_dev -- the standard deviation of the array
#                $count -- the number of items in th array
# X. Jing Li, 5/18/2005
###########################################################################
sub getMeanStdev {
    my ($ref_array) = @_;
    my (@array, $count, $mean, $stddev, $var, $e);

    @array = @$ref_array;
    $count = @array;
    $mean = null;
    $var = null;
    $stddev = null;

    #== find the mean, variance and std_dev for the array
    if (@array > 0) {
        $mean = 0;
        foreach $e (@array){
            $mean += $e;
        }
        $mean = $mean / @array;
        foreach $e (@array) {
            $var += ($e - $mean)**2;
        }
        if (@array > 1) {
            $stddev = sqrt($var / (@array - 1));
            $var = $var / (@array - 1);
        }
    }

    return $mean, $stddev, $count;
}

###########################################################################
# subroutine: getDetail
# this subroutine get the error details for the set of parameters;
#
# parameters in: $dbh -- the database handle,
#                $spid -- sampling_point_id;
#                $ptid -- the protocol_type id,
#                $extract_id --
#                $tissue_type_label_id -- the tissue type for spid,
#
# parameter out: $detail -- the details for the two samples and the protocol 
#                   type name;
#
# X. Jing Li, 5/26/2005
#
###########################################################################
sub getDetail {
    my ($dbh, $spid, $ptid, $extract_id, $tissue_type_label_id) = @_;
    my ($sql, $sth, $rc, $detail, $exp_set_name, $exp_name, $label_name, 
        $tissue_type, $pt_name, $extract_name);

    #== get exp_set name, exp_naem, time point for spid:
    $sql = "select ES.exp_set_name, E.exp_name, L.\"name\" from experiment_set ES, experiment E, label L, sampling_point SP where SP.sampling_point_id = $spid and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't preparet: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($exp_set_name, $exp_name, $label_name) = $sth->fetchrow();
    $detail = "The Mean, Std_dev has been calculated for experiment set \"$exp_set_name\", experiment \"$exp_name\" at time \"$label_name\" for";

    $sql = "select \"name\" from label where label_id = $tissue_type_label_id";
    $sth = $dbh-> prepare($sql) or die "Can't preparet: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $tissue_type = $sth->fetchrow();
    $detail .= " tissue type \"$tissue_type\", ";

    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    $sth = $dbh-> prepare($sql) or die "Can't preparet: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pt_name = $sth->fetchrow();
    $detail .= " with normalization method \"$pt_name\" ";

    $sql = "select \"name\" from extract where extract_id = $extract_id";
    $sth = $dbh-> prepare($sql) or die "Can't preparet: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $extract_name = $sth->fetchrow();
    $detail .= "for extraction \"$extract_name\".\n";

    $detail .= "If you detele the old data for this set of parameters in DOME, all ratio and p-value data related to this data set will also be deleted as well.\n";

    $sth->finish;
    return $detail;
}
    
