#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script will read AffyMetrix annotation info and add data into tables 
# "composite_sequence", "reporter", "gene", "evidence", "allnames".
#
# Author: X. Jing Li
# Date: April 19, 2006
# Reversions: 
#============================================================================== 

my ($database, $username, $password, $bnet_DB, $specieDB, $sql, $dbh, $sth, $rc,
    $infile, @lines, $line, $probe_id, $p_annot, %probe_annotation, $kw_table_id, %hadSeenProbeAnnot, %missing_pids,, %cs_annots, %pa_annots, $cs_annot, $mainsql, $index, $loop
    );

#== get the global parameters:
($infile) = @ARGV;

print "Start time: " . TimeStamp() . "\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== get all probe_names from the gal file:
open (IN, "<$infile") || die "Can not open the annotation file $infile.\n";
@lines = <IN>;
close IN;

shift @lines;
foreach $line (@lines) {
    chomp $line;
    $line =~ s/\"//g;
    ($probe_id, $p_annot) = split /\t/, $line;
    if ($p_annot =~ /\w/) {
        $probe_annotation{$probe_id} = $p_annot;
        $missing_pids{$probe_id} = $p_annot;
    }
}

#== get kw_table_id, specie_id, taxon_ncbi_accession:
$sql = "select \"kw_table_id\" from \"ref_kw_table\" where \"kw_table\" = 'GENE'";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$kw_table_id = $sth->fetchrow();

#== get all probe_names with the gene_id by quary allnames:
$sql = "create TEMP table probe_annot (probe_annot varchar(1028), probe_id varchar(50) ) ON COMMIT DROP;\n";
foreach $probe_id (keys %probe_annotation) {
    $p_annot = $probe_annotation{$probe_id};
    $p_annot =~ s/\'/\'\'/g;
    if ($p_annot ne "" && !exists $hadSeenProbeAnnot{$p_annot}) {
        $sql .= "insert into \"probe_annot\" (probe_annot, probe_id) values (\'$p_annot\', \'$probe_id\');\n";
        $hadSeenProbeAnnot{$p_annot} = 1;
    } else {
        $hadSeenProbeAnnot{$p_annot} += 1;
    }
}
#open (TMP, ">./tmp-plots/insertAffyAnnot_temp.sql") or die "Can not open log file. $!";
#print TMP $sql;
#close TMP;
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$sql = "select count(*) from probe_annot";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#my $count = $sth->fetchrow();
#print TimeStamp() . ": there are $count records in temp table.\n";
#$sth->finish;
#$dbh->rollback;
#$dbh->disconnect;
#exit;

#== 2. get all reporter_id, composite_sequence_id, probe_name and annotations:
$sql = "select CS.probe_name, CS.annotation, PA.probe_annot from \"composite_sequence\" CS, \"probe_annot\" PA where PA.probe_id = CS.probe_name";
#print "$sql\n";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($probe_id, $cs_annot, $p_annot) = $sth->fetchrow()) {
    delete $missing_pids{$probe_id};
    $cs_annots{$probe_id} = $cs_annot;
    $pa_annots{$probe_id} = $p_annot;
}
#== drop the temp tabl:
$sql = "drop table \"probe_annot\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

my $log_file = "./tmp-plots/insertAffyAnnotationn.log";
open (LOG, ">$log_file") or warn "Can not open log file $log_file. $!";
if (keys %missing_pids > 0) {
    print LOG "New probe_ids in this file:\n" . (sort keys %missing_pids) . "\n";
}

$index = 0;
#$loop = 0;
foreach $probe_id (sort keys %probe_annotation) {
    $p_annot = $probe_annotation{$probe_id};
    $p_annot =~ s/\'/\'\'/g;
    if (exists $cs_annots{$probe_id} && $p_annot ne $cs_annots{$probe_id} && $cs_annots{$probe_id} ne "" && $p_annot ne "") {
        $index++;
        print LOG "$probe_id\tcs_annot = $cs_annot\tp_annot = $p_annot\n";
        $mainsql .= "update composite_sequence set annotation = \'$p_annot\' where probe_name = \'$probe_id\';\n";
    } elsif ($p_annot ne "") {
        $index++;
        $mainsql .= "update composite_sequence set annotation = \'$p_annot\' where probe_name = \'$probe_id\';\n";
    } elsif (exists $missing_pids{$probe_id}) {
        $index++;
        print LOG "$probe_id\tMissing\t$p_annot\n";
        $mainsql .= "insert into composite_sequence (composite_sequence_id, probe_name, annotation) values (nextval('composite_sequence_id_seq'), \'$probe_id\', \'$p_annot\');\n";
    }
    if ($index > 500) {
        $loop++;
        print LOG $mainsql;
        $sth = $dbh-> prepare($mainsql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#        print TimeStamp() . ": loop = $loop\n";
        $index = 0;
        $mainsql = "";
    }
}
print LOG $mainsql;
close LOG;

$sth = $dbh-> prepare($mainsql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

$sth->finish;
#$dbh->rollback;
$dbh->commit;
$dbh->disconnect;
print "Ending time: " . TimeStamp() . "\n";
print "AffyMetrix annotation file has been inserted into DOME successfully.\n";
exit 0;

