#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script will read AffyMetrix annotation info and add data into tables 
# "composite_sequence", "reporter", "gene", "evidence", "allnames".
#
# Author: X. Jing Li
# Date: March 14, 2006
# Reversions: 
# 4/19/2006: changed the script for no probe and gene annotations in the infile
# 8/8/2006: add code for updating probe_name and/or probe_set names
#============================================================================== 

my ($database, $username, $password, $bnet_DB, $specieDB, $sql, $dbh, $sth, $rc,
    $infile, $array_batch_id, $file_home, $person_id, $is_new_array, @lines, $line, $probe_id, $gname, %probe_names, $kw_table_id, %hadSeenProbeName, $gene_id, %gname_gene_ids, $rid, $csid, %probe_rids, %rid_csids, %probe_gids, $paramSettings_file, $temp, $specie_name, $reference_id, $specie_id, $evidence_code_id, $taxon_id, $unikey, $evidence_id, $pname_id, $gname_id, $index, $file_name, $file_type_id, $file_id, $file_location_link_id, $file_location
    );

#== get the global parameters:
#($infile, $array_batch_id) = @ARGV;
($infile, $array_batch_id, $file_home, $person_id) = @ARGV;
#print "ARGV = @ARGV\n";
print "Start time: " . TimeStamp() . "\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== check if this is a new array design or an updating one. Since when a
#== array_batch is created where is no file_location_link_id associated with it
#== so the array is a new one. If there is already a file_location_link_id
#== associated with it htat means it is an existing array desing so all we need
#== is to update the gene_ids if the probe_name is already in the table and
#== only create records for the new ones.
#$sql = "select file_location_link_id from array_batch where array_batch_id = $array_batch_id";
#$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
#$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#$file_location_link_id = $sth->fetchrow();
#if ($file_location_link_id eq "") {
#    $is_new_array = 1;
#} else {
#    $is_new_array = 0;
#}

#== get all probe_names from the gal file:
open (IN, "<$infile") || die "Can not open the annotation file $infile.\n";
@lines = <IN>;
close IN;

shift @lines;
foreach $line (@lines) {
    chomp $line;
    $line =~ s/\"|
//g;
    $line =~ s/\'/\'\'/g;
    ($probe_id, $gname) = split /\t/, $line;
    $probe_names{$probe_id} = $gname;
}
#print "There are " . (keys %probe_names) . " lines in file.\n";

#== get kw_table_id, specie_id, taxon_ncbi_accession:
$sql = "select \"kw_table_id\" from \"ref_kw_table\" where \"kw_table\" = 'GENE'";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$kw_table_id = $sth->fetchrow();

#- get referece_id:
$paramSettings_file = "./paramSettings.php";
open (PAR, "<$paramSettings_file") or die "Can not open paremeter file.";
@lines = <PAR>;
close PAR;
foreach $line (@lines) {
    chomp $line;
    if ($line =~ /\'SPECIES_NAME\'/) {
        ($temp, $specie_name) = split /=/, $line;
        $specie_name =~ s/^\s+|\'+|\;|\s+$//g;
        last;
    }
}
$sql = "select \"value\" from \"default\" where \"name\" = 'GENE_REFERENCE'";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$reference_id = $sth->fetchrow();

#- get specie_id:
$sql = "select record_id from \"allnames\" where upper(aname) = upper(\'$specie_name\') and kw_table_id = (select kw_table_id from \"ref_kw_table\" where kw_table = \'SPECIES\')";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$specie_id = $sth->fetchrow();

#- get taxon_id
$sql = "select taxon_ncbi_accession from \"species\" where specie_id = $specie_id";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$taxon_id = $sth->fetchrow();
$evidence_code_id = 14;  # IEP for "GENE" table

#== create a temp tble to hold all probe_names and gene_names for later joining
#== with allnames table. Only insert unique probe_names
$sql = "create TEMP table probe_names (probe_name varchar(500), probe_id varchar(50) ) ON COMMIT DROP;\n";
foreach $probe_id (keys %probe_names) {
    $gname = $probe_names{$probe_id};
    if ($gname ne "" && !exists $hadSeenProbeName{$gname}) {
        $sql .= "insert into \"probe_names\" (probe_name, probe_id) values (\'$gname\', \'$probe_id\');\n";
        $hadSeenProbeName{$gname} = 1;
    } elsif ($gname eq "") {
        $sql .= "insert into \"probe_names\" (probe_id) values (\'$probe_id\');\n";
    } else {
        $hadSeenProbeName{$gname} += 1;
    }
}
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== get existing gene_ids for all gene_names:
$sql = "select PN.probe_name, A.record_id from probe_names PN, allnames A where a.kw_table_id = $kw_table_id and upper(PN.probe_name) = upper(A.aname);";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($gname, $gene_id) = $sth->fetchrow()) {
    if ($gene_id ne "") {
        $gname_gene_ids{$gname} = $gene_id;
    }
}

#== 2. get all reporter_id, composite_sequence_id, probe_name and annotations:
#$sql = "select R.reporter_id, CS.composite_sequence_id, CS.gene_id, PN.probe_id from \"reporter\" R, \"composite_sequence\" CS, probe_names PN where R.composite_sequence_id = CS.composite_sequence_id and PN.probe_id = CS.probe_name order by R.reporter_id";
$sql = "select R.reporter_id, CS.composite_sequence_id, CS.gene_id, CS.probe_name from \"reporter\" R, \"composite_sequence\" CS where R.composite_sequence_id = CS.composite_sequence_id";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($rid, $csid, $gene_id, $probe_id) = $sth->fetchrow()) {
    if (!(exists $probe_rids{$probe_id})) {
        $probe_rids{$probe_id} = $rid;
        $rid_csids{$rid} = $csid;
        $probe_gids{$probe_id} = $gene_id;
    }
}
#print "There are " . (keys %probe_rids) . " old probe_ids in DB";

#== create records in gene, allanmes, composite_sequence and reporter for each 
#== probe_name is not in DOME already:
my $log_file = "./tmp-plots/insertAffyDesign.log";
open (LOG, ">$log_file") or warn "Can not open log file $log_file. $!";
foreach $probe_id (sort {$a <=> $b} keys %probe_names) {
    $index++;
    $gname = $probe_names{$probe_id};
    $gname =~ s/\'\'/\'/g;
    $probe_id =~ s/\'\'/\'/g;
    if (!exists $probe_rids{$probe_id} ) {
        #-- this is a new probe_set. If gene_name is old, just create 
        #-- composite_sequence_id and reporter_id. If gene_name is new, create 
        #-- records in gene, evidence, composite_sequence, reporter, allnames:
        if( exists $gname_gene_ids{$gname} ) {
            #-- new probe_set_id, old gene_name:
            $gene_id = $gname_gene_ids{$gname};
            $csid = createCompositeSequenceId($dbh, "", $probe_id, $gene_id);
            $rid = createReporterId($dbh, $csid);
            print LOG "$index\tnew probe, old gene\tprobe_id = $probe_id\tcsid = $csid\treporter_id = $rid\tgname = $gname, gene_id = $gene_id\n";
        } elsif ((!exists $gname_gene_ids{$gname}) && $gname ne "") {
            #-- new probe_set_id, new gene name: 
            $gene_id = createGeneId($dbh, $specie_id);
            $gname_gene_ids{$gname} = $gene_id;
            $evidence_id = createEvidenceId($dbh, $evidence_code_id, $reference_id, $kw_table_id, $gene_id);
            $unikey = $taxon_id . "_" . $gname;
            $gname_id = createAllnameId($dbh, $gname, $kw_table_id, $gene_id, "Y", $unikey);
            $csid = createCompositeSequenceId($dbh, "", $probe_id, $gene_id);
            $rid = createReporterId($dbh, $csid);
            print LOG "$index\tnew probe, new gene\tprobe_id = $probe_id\tcsid = $csid\treporter_id = $rid\tnew gname = $gname\tgene_id = $gene_id\tevidence_id = $evidence_id\tprimary_name_id = $gname_id\tunikey = $unikey\n";
        } else {
            #-- new probe_set_id, not gene_name:
            $csid = createCompositeSequenceId($dbh, "", $probe_id, "");
            $rid = createReporterId($dbh, $csid);
            print LOG "$index\tnew_probe, no gene\tprobe_id = $probe_id\tcsid = $csid\trid = $rid\n";
        }
    } else {
        #-- old probe_set. Need to see if gene_name is new. If YES create 
        #-- gene_id, evidence_id, external_link_is and update Cs.gene_id. If
        #-- NOT but the old gene_id is not the same as new gene_id, update CS:
        if ((exists $gname_gene_ids{$gname}) 
            && (exists $probe_gids{$probe_id}) 
            && ($gname_gene_ids{$gname} ne $probe_gids{$probe_id})) {
            #-- old gene_name for DB, new gene_name for probe_set_id:
            #-- update to new gene_id:
            $gene_id = $gname_gene_ids{$gname};
            $csid = updateGeneIdOrAnnotation($dbh, "", $probe_id, $gene_id);
            print LOG "$index\told probe, old gene, new coor\tprobe_id = $probe_id\tcsid = $csid\told gname = $gname, gene_id = $gene_id\n";
        } elsif ((!exists $gname_gene_ids{$gname}) && $gname ne "") {
           $gene_id = createGeneId($dbh, $specie_id);
           $gname_gene_ids{$gname} = $gene_id;
           $evidence_id = createEvidenceId($dbh, $evidence_code_id, $reference_id, $kw_table_id, $gene_id);
           $unikey = $taxon_id . "_" . $gname;
           $gname_id = createAllnameId($dbh, $gname, $kw_table_id, $gene_id, "Y", $unikey);
           $csid = updateGeneIdOrAnnotation($dbh, "", $probe_id, $gene_id);
           print LOG "$index\told probe, new gene\ttprobe_id = $probe_id\tcsid = $csid\tnew gname = $gname, gene_id = $gene_id\tevidence_id = $evidence_id\tprimary_name_id = $gname_id\tunikey = $unikey\n";
       }
    }
}

close LOG;

#== update the array design file and file_location_link_id:
my @temps = split /\//, $infile;
$file_name = pop @temps;
$file_id = getFileId($dbh, $file_name);
if ($file_id eq "") {
    $file_type_id = getFileTypeId($dbh, $infile);
    if ($file_type_id eq "") {
        $file_type_id = createFileTypeId($dbh, $file_name, "Affy array design files");
    }
    $file_location_link_id = createFileIds($dbh, $file_home, $file_name, $file_type_id, $person_id);
    $sql = "update array_batch set file_location_link_id = $file_location_link_id where array_batch_id = $array_batch_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
} else {
    $sql = "update file set person_id = $person_id, time_uploaded = (SELECT abstime(timestamp 'now')) where file_id = $file_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sql = "select FL.\"location\" from \"file_location\" FL, file_location_link FLL where FLL.file_id = $file_id and FL.file_location_id = FLL.file_location_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $file_location = $sth->fetchrow();
    $file_name = $file_location . $file_name;
    my $cmd = "cp $infile $file_name";
    #print "$cmd\n";
    system $cmd;
}

$sth->finish;
#$dbh->rollback;
$dbh->commit;
$dbh->disconnect;
print "Ending time: " . TimeStamp() . "\n";
#print "AffyMetrix Design file has been inserted into DOME successfully.\n";
exit 0;

###############################################################################
# updateGeneIdOrAnnotation($dbh, $probe_annotation, $probe_id, $gene_id)
#
# This subroutine will update an existing record in composite_sequence table 
# for its probe_annotation and/or gene_id.
#
# X. Jing Li
# 8/18/2006
###############################################################################
sub updateGeneIdOrAnnotation {
    my ($dbh, $probe_annot, $probe_id, $gene_id) = @_;
    #print "probe_annot = $probe_annot, probe_id = $probe_id, gene_id = $gene_id\n";
    my ($sth, $rc, $sql, $csid);

    if ($gene_id  eq "") {
        return;
    }
    $probe_annot =~ s/\'/\'\'/g;
    $probe_id =~ s/\'/\'\'/g;

    $sql = "select composite_sequence_id from composite_sequence where probe_name = \'$probe_id\'";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $csid = $sth->fetchrow();

    $sql = "update composite_sequence set";
    if ($probe_annot ne "" && $gene_id ne "") {
        $sql .= " \"annotation\" = \'$probe_annot\', gene_id = $gene_id";
    } elsif ($probe_annot eq "" && $gene_id ne "") {
        $sql .= " gene_id = $gene_id";
    } elsif ($probe_annot ne "" && $gene_id eq "") {
        $sql .= " \"annotation\" = \'$probe_annot\'";
    }  
    $sql .= " where probe_name = \'$probe_id\'";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    return $csid;
}

    
