#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script will read external links about metabolites, genes or proteins in
# Bnet. The file format is as following:
#
# Primary_name\tExternal Database\tExternal Id\n
# name1\tDB1\tid1\n
# ...
# The external database name must already in BNet and they are matching the Bnet
# record.
#
# Author: X. Jing Li
# Date: March 28, 2006
# Reversions: 
#============================================================================== 

my ($database, $username, $password, $bnet_DB, $specieDB, $sql, $dbh, $sth, $rc, $mainsql, 
    $infile, $kw_table_id,
    @temps, $file_name, $file_id, @lines, $line, $r_databases, %databases, $r_links, %links, $name, $db, $link, %links_dbs, $dbs_str, $dbid, %dbid_names, $msg, %hasSeenLinks, $record_id, %links_rids, $kw_table, $internal_field_name
    );

#== get the global parameters:
($infile, $kw_table_id) = @ARGV;
print "Starting time: " . TimeStamp() . "\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== check if this file has been uploaded before:
#@temps = split /\//, $infile;
#$file_name = pop @temps;;
#$file_id = getFileId($dbh, $file_name);
#if ($file_id ne "") {
#    print "This annotation file \"$file_name\" has been uploaded before. Check your file or use another array design name and file name.\n";
#    $dbh->rollback;
#    $sth->finish;
#    $dbh->disconnect;
#    unlink ($infile) or warn "Can not delete the annotaion file $infile.\n";
#    exit 1;
#}

#== check all databases and links. if a database is new to BNet then report to 
#== frontend. 
open (IN, "<$infile") || die "Can not open the annotation file $infile. $!\n";
@lines = <IN>;
close IN;

$r_databases = {};
%databases = %$r_databases;
$r_links = {};
%links = %$r_links;
shift @lines;
foreach $line (@lines) {
    chomp $line;
    ($name, $db, $link) = split /\t/, $line;
    if ($link ne "" && $db ne "") { 
        $databases{$db} = 1;
        $links{$link} = $name;
        $links_dbs{$link} = $db;
    }
}

$dbs_str = join "', '", keys %databases;
$dbs_str = "'" . $dbs_str . "'";
#print "dbs_str = $dbs_str\n";
$sql = "select external_database_id, database_name from external_database where database_name in ($dbs_str)";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($dbid, $db) = $sth-> fetchrow()) {
    $dbid_names{$db} = $dbid;
}

$msg = "";
foreach $db (keys %databases) {
    if (!exists $dbid_names{$db}) {
        $msg .= "Database \"$db\" is not in Bnet.<br>";
    }
}
if ($msg ne "") {
    $msg .= "Please add the database(s) into Bnet before add the links.<br>";
    print $msg;
    $sth->finish;
    $dbh->rollback;
    $dbh->disconnect;
    exit 1;
}

#== create a temp table for all links:
$sql = "create TEMP table temp_links (name varchar(500), link varchar(500), database_id varchar(50) ) ON COMMIT DROP;\n";
foreach $link (keys %links) {
    $name = $links{$link};
    $dbid = $dbid_names{$links_dbs{$link}};
    if (!exists $hasSeenLinks{$link} || $hasSeenLinks{$link} ne $dbid) {
        $hasSeenLinks{$link} = $dbid;
        $sql .= "insert into \"temp_links\" values (\'$name\', \'$link\', \'$dbid\');\n";
    } else {
        print "link = $link, database = " . $links_dbs{$link} . " has been seen before\n";
    }
}

#== get the record_ids:
$sql .= "select TL.link, A.record_id from temp_links TL, allnames A where kw_table_id = $kw_table_id and upper(A.aname) = upper(TL.\"name\");\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($link, $record_id) = $sth->fetchrow()) {
    $links_rids{$link} = $record_id;
}

#== get the links that already in external_links table off the hash %links_dbs:
$sql = "select EL.record_id, EL.external_database_id from temp_links TL, external_links El where TL.link = EL.external_id and TL.database_id = EL.external_database_id;\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($link, $dbid) = $sth->fetchrow()) {
    if ($links_dbs{$link} == $dbid) {
        delete $links_dbs{$link};
    }
}

#== get the internal_field_name for this kw_table_id:
$sql = "select kw_table from ref_kw_table where kw_table_id = $kw_table_id";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$kw_table = $sth->fetchrow();
if ($kw_table eq "COMPOUND") {
    $internal_field_name = "cpd_id";
} else {
    $internal_field_name = lc($kw_table) . "_id";
}

#== create the mainsql:
$mainsql = "";
foreach $link (keys %links_dbs) {
    $record_id = $links_rids{$link};
    $dbid = $dbid_names{$links_dbs{$link}};
    $mainsql .= "insert into \"external_links\" (\"external_links_id\", \"kw_table_id\", \"record_id\", \"internal_field_name\", \"external_database_id\", \"external_id\") values (nextval('external_links_id_seq'), $kw_table_id, $record_id, \'$internal_field_name\', $dbid, \'$link\');\n";
}

open (LOG, ">./tmp-plots/insertBnetExternalLinks.log") or warn "Can not openlog file. $!";
print LOG $mainsql;
close LOG;

$sth = $dbh->prepare($mainsql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
#$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#if ($rc == 1) {
#    $file_type_id = getFileTypeId($dbh, $infile);
#    if ($file_type_id eq "") {
#        $file_type_id = createFileTypeId($dbh, $infile, "external links for Bnet");
#    }
#    $file_location_link_id = createFileIds($dbh, $file_home, $infile, $file_type_id, $person_id);
#}
$dbh->commit;
$sth->finish;
$dbh->disconnect;

print "Ending time: " . TimeStamp() . "\n";

exit 0;
