#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";
use Math::Trig;
use POSIX;

#===============================================================================
# This script will read microarray batch design info file .gal and add
# the data into database "feature", "reporter" tables.
#
# Author: X. Jing Li
# Date: April 4, 2005
# Reversions: 
#============================================================================== 

my ($database, $username, $password, $bnet_DB, $specieDB, $sql, $dbh, $sth, $rc,
    $galfile, $array_batch_id, $feature_type_id, $external_db_id, $file_home, 
    $person_id, 
    @temps, $file_name, $gal_file_id, $key, %index_keys, %annotations, %pname_keys,
    $rid, $csid, $gname, $pname, @pnames, $pnames, $unikey,
    $paramSettings, @contents, $kw_table_id, $gene_id, $evidence_code_id, 
    $specie_name, $specie_id, $taxon_id,, $reference_name, $reference_id,
    $evidence_id, $external_links_id, $temp, %rid_gnames, %rids_csids, 
    %pname_rids, $plate_info, $rid_plate, %pname_gene_ids, $pname_id,$gname_id, 
    %repeated_rids, $main_sql, @lines, $line, $start, $block, $error_msg,
    $meta_r, $meta_c, $row, $col, $index, $count, $i, %hadSeenPName,
    $file_home, $gal_file_type_id, $gal_file_location_link_id );

#== get the global parameters:
($galfile, $array_batch_id, $feature_type_id, $external_db_id, $file_home, $person_id) = @ARGV;
#print "@ARGV\n";

print "Start time: " . TimeStamp() . "\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== check if this file has been uploaded before:
@temps = split /\//, $galfile;
$file_name = pop @temps;;
$gal_file_id = getFileId($dbh, $file_name);
if ($gal_file_id ne "") {
    print "This GAL file \"$file_name\" has been uploaded before. Check your file or use another array design name and file name.\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    unlink ($galfile) or warn "Can not delete the gal file $galfile.\n";
    exit 1;
}

#== get all probe_names from the gal file:
open (IN, "<$galfile") || die "Can not open the gal file $galfile.\n";
@lines = <IN>;
close IN;

$start = "0";
$index = 0;
foreach $line (@lines) {
    if ($line !~ /^Block\tRow\tColumn\tName\tID/ && $start eq "0") {
        next;
    } elsif ($line =~ /^Block\tRow\tColumn\tName\tID/) {
        $start = 1;
        next;
    }

    chomp $line;
    $line =~ s /\
//g;
    ($block, $row, $col, $gname, $pname) = split /\t/, $line;
    $key = $block . "_" . $row . "_" . $col;
    $index_keys{$index} = $key;
    $gname =~ s/\"//g;
    $pname =~ s/\"//g;
    $annotations{$key} = $gname;
    $pname_keys{$key} = $pname;
    push @pnames, $pname;
    $index++;
}

#== in case of new names in file, get all info needed for BNET:
#== get kw_table_id, specie_id, taxon_ncbi_accession:
$sql = "select \"kw_table_id\" from \"ref_kw_table\" where \"kw_table\" = 'GENE'";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$kw_table_id = $sth->fetchrow();
 
#== get all probe_names with the gene_id by quary allnames:
$sql = "create TEMP table probe_names (probe_name varchar(500) ) ON COMMIT DROP;\n";
foreach $pname (@pnames) {
    if ($pname ne "" && !exists $hadSeenPName{$pname}) {
        $sql .= "insert into \"probe_names\" values (\'$pname\');\n";
        $hadSeenPName{$pname} = 1;
    } else {
        $hadSeenPName{$pname} += 1;
    }
}

$sql .= "select PN.probe_name, A.record_id from probe_names PN, allnames A where a.kw_table_id = $kw_table_id and upper(PN.probe_name) = upper(A.aname);";
#print "$sql\n";
#print "Before create TEMP table is " . TimeStamp() . "\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($pname, $gene_id) = $sth->fetchrow()) {
    if ($gene_id ne "") {
        $pname_gene_ids{$pname} = $gene_id;
    }
}
#print "There are " . (keys %pname_gene_ids) . " TCs has gene_id\n";
#open (TEMP, ">temp.sql") or "can not open temp.sql file.\n";
#foreach $pname (keys %pname_gene_ids) {
#    print TEMP "pname = $pname, gene_id = " . $pname_gene_ids{$pname} . "\n";
#}
#print "After create TEMP table is " . TimeStamp() . "\n";

#== 2. get all reporter_id, composite_sequence_id, probe_name and annotations:
$sql = "select R.reporter_id, R.plate_info, CS.composite_sequence_id, CS.annotation, PN.probe_name, CS.gene_id from \"reporter\" R, \"composite_sequence\" CS, \"probe_names\" PN where R.composite_sequence_id = CS.composite_sequence_id and PN.probe_name = CS.probe_name order by R.reporter_id";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($rid, $plate_info, $csid, $gname, $pname, $gene_id) = $sth->fetchrow()) {
    if (!(exists $pname_rids{$pname})) {
        $pname_rids{$pname} = $rid;
        $rids_csids{$rid} = $csid;
    } else {
        $rid_plate = $pname . "::" . $rid . "::" . $plate_info . "::" . $gname;
        $repeated_rids{$rid} = $rid_plate;
    }
}

#== create records in gene, evidence, external_links, allanmes, composite_sequence
#== and reporter for each probe_name is not in DOME:
$paramSettings = "./paramSettings.php";
open (PAR, "<$paramSettings") or die "Can not open paremeter file.";
@contents = <PAR>;
close PAR;
foreach $line (@contents) {
    chomp $line;
    if ($line =~ /\'SPECIES_NAME\'/) {
        ($temp, $specie_name) = split /=/, $line;
        $specie_name =~ s/^\s+|\'+|\;|\s+$//g;
    }
#    if ($line =~ /\'GENE_REFERENCE\'/) {
#        ($temp, $reference_name) = split /=/, $line;
#        $reference_name =~ s/^\s+|\'+|\;|\s+$//g;
#    }
}
$sql = "select \"value\" from \"default\" where \"name\" = 'GENE_REFERENCE'";
#print "$sql\n";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$reference_id = $sth->fetchrow();

#print "species_name = $specie_name, reference_name = $reference_name\n";
$sql = "select record_id from \"allnames\" where aname = \'$specie_name\' and kw_table_id = (select kw_table_id from \"ref_kw_table\" where kw_table = \'SPECIES\')";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$specie_id = $sth->fetchrow();

$sql = "select taxon_ncbi_accession from \"species\" where specie_id = $specie_id";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$taxon_id = $sth->fetchrow();
$evidence_code_id = 14;  # IEP for "GENE" table

#$sql = "select reference_id from \"reference\" where locator = \'$reference_name\'";
#$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
#$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#$reference_id = $sth->fetchrow();

foreach $index (sort {$a <=> $b} keys %index_keys) {
    $key = $index_keys{$index};
    $pname = $pname_keys{$key};
    $gname = $annotations{$key};
    #print "key = $key, pname = $pname, gname = $gname\n";
    if ((!exists $pname_gene_ids{$pname}) && $pname ne "") {
        #print TEMP "index = $index, key = $key, pname = $pname, gname = $gname\n";
        $gene_id = createGeneId($dbh, $specie_id);
        $evidence_id = createEvidenceId($dbh, $evidence_code_id, $reference_id, $kw_table_id, $gene_id);
        $external_links_id = createExternalLinksId($dbh, $external_db_id, $kw_table_id, $gene_id, "gene_id", $pname);
        $unikey = $taxon_id . "_" . $pname;
        $pname_id = createAllnameId($dbh, $pname, $kw_table_id, $gene_id, "Y", $unikey);
        $gname_id = createAllnameId($dbh, $gname, $kw_table_id, $gene_id, "A", $unikey);
        $csid = createCompositeSequenceId($dbh, $gname, $pname, $gene_id);
        $rid = createReporterId($dbh, $csid);
        $rid_gnames{$rid} = $gname;
        $rids_csids{$rid} = $csid;
        $pname_rids{$pname} = $rid;
        $pname_gene_ids{$pname} = $gene_id;
    } elsif ((!exists $pname_rids{$pname}) && (exists $pname_gene_ids{$pname})) {
        $gene_id = $pname_gene_ids{$pname};
        $csid = createCompositeSequenceId($dbh, $gname, $pname, $gene_id);
        $rid = createReporterId($dbh, $csid);
        $rid_gnames{$rid} = $gname;
        $rids_csids{$rid} = $csid;
        $pname_rids{$pname} = $rid;
    }
}

#== 3. insert into database:
$main_sql = "";
foreach $index (sort {$a <=> $b} keys %index_keys) {
    $key = $index_keys{$index};
    ($block, $row, $col) = split /\_/, $key;
    $meta_r = ceil($block / 4);
    $meta_c = $block % 4;
    if ($meta_c == 0) {
        $meta_c = 4;
    }
    $pname = $pname_keys{$key};
    $gname = $annotations{$key};
    $rid = $pname_rids{$pname};
    $gene_id = $pname_gene_ids{$pname};
    if ($block ne "" && $row ne "" && $col ne "" && $rid ne "" && $gene_id ne "") {
        $main_sql .= "/* pname = $pname, rid = $rid, gene_id = $gene_id*/\ninsert into feature (feature_id, feature_type_id, metarow, metacolumn, \"row\", \"column\", array_batch_id, reporter_id, block) values (nextval('feature_id_seq'), $feature_type_id, $meta_r, $meta_c, $row, $col, $array_batch_id, $rid, $block);\n";
    } elsif ($block eq "" || $row eq "" && $col eq "" && $rid eq ""  && $gene_id eq "") {
        $error_msg = "On line \"Block:$block Row:$row Column:$col\" there is no ";
        if ($block eq "") { $error_msg .= "Block number, "; }
        if ($row eq "") { $error_msg .= "Row number, "; }
        if ($col eq "") { $error_msg .= "Column number, "; }
        if ($gname eq "") { $error_msg .= "Name, "; }
        if ($pname eq "") { $error_msg .= "ID number, "; }
        $error_msg .= "\n";
        print $error_msg;
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        #unlink ($galfile) or warn "Can not delete the gal file $galfile.\n";
        print "Ending time: " . TimeStamp() . "\n";
        exit 1;
    }
}
#open (TEMP, ">>temp.sql") or "can not open temp.sql file.\n";
#print TEMP "$main_sql";
#close TEMP;

if ($main_sql ne "") {
    $sth = $dbh-> prepare($main_sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($rc == 1) {
        #== create file_id, file_location_link_id for the gal file and update
        #== array_batch table for file_location_link_id:
        $gal_file_type_id = getFileTypeId($dbh, $galfile);
        if ($gal_file_type_id eq "") {
            $gal_file_type_id = createFileTypeId($dbh, $galfile, "2-channel microarray design GAL file");
        }
        $gal_file_location_link_id = createFileIds($dbh, $file_home, $galfile, $gal_file_type_id, $person_id);
        $sql = "update array_batch set file_location_link_id = $gal_file_location_link_id where array_batch_id = $array_batch_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        $sth->finish;
        $dbh->commit;
        $dbh->disconnect;
        #unlink ($galfile) or warn "Can not delete the gal file $galfile.\n";
        print "Ending time: " . TimeStamp() . "\n";
        print "Gal file data has been inserted into DOME successfully.\n";
        exit 0;
    } else {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        #unlink ($galfile) or warn "Can not delete the gal file $galfile.\n";
        print "Ending time: " . TimeStamp() . "\n";
        print "Failed to insert data in Gal file.\n";
        exit 1;
    }
} else {
    print "There is no data been inserted into DOME.\n";
    print "Ending time: " . TimeStamp() . "\n";
    $sth->finish;
    $dbh->rollback;
    $dbh->disconnect;
    #unlink ($galfile) or warn "Can not delete the gal file $galfile.\n";
    print "Ending time: " . TimeStamp() . "\n";
    exit 1;
}
