#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script will read proteomics reference info file and add the data into 
# database "reference_spot" and "ref_spot_protein_state" tables. When everything
# is success then create sample_id, sample_extract_id, file_id, 
# file_location_link_id and proteomics_measurement_id for this sample.
#
# Author: X. Jing Li
# Date: Oct. 26, 2005
# Reversions: 
#============================================================================== 

my ($infofile, $file_home, $srid, $tissue_type_label_id, $tech_repl, 
    $extract_id, $protocol_type_id, $person_id, $force_add, $pname_prefix,
    $database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc,
    @temps, $file_name, $file_id, @lines, $line, $i, $item, %headers, $old_spot,
    $new_spot, $plate, $well, $annotation, $glink, $score, $coverage, $matched,
    $pi, $mw, $gel_pi, $gel_mw, $support, $class, $main_sql, $sql1, $sql2,
    $result, $sample_id, $seid, $file_type_id, $file_location_link_id, $index,
    $prot_measurement_id,
    $line_index, $spot, %line_spot, %line_plate, %line_well, %line_annotation, %line_glink, %line_score, %line_coverage, %line_matched, %line_pi, %line_mw, %line_gel_pi, %line_gel_mw, %line_support, %line_class, %unique_spot, $unique_spots, @exist_spots, $exist_spot);

#== get the global parameters:
#($infofile, $file_home, $srid, $tissue_type_label_id, $tech_repl, $extract_id, $protocol_type_id, $person_id, $force_add) = @ARGV;
#print "@ARGV\n";
($infofile, $file_home, $protocol_type_id, $person_id, $force_add) = @ARGV;
print "Start time: " . TimeStamp() . "\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== check if this file has been uploaded before:
@temps = split /\//, $infofile;
$file_name = pop @temps;
#print "file_name = $file_name\n";
$file_id = getFileId($dbh, $file_name);
#print "file_id = $file_id\n";
if ($file_id ne "" && $force_add eq "0") {
    print "This file \"$file_name\" has been uploaded before. Please check your file name OR use another name for your file.\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
#    unlink ($infofile) or warn "Can not delete file $infofile.\n";
    exit 1;
} elsif ($force_add eq "1") {
    #== delete all records from "reference_spot" and "ref_spot_protein_state". 
    #== What about the data in proteomics_measurement and proteomics_data? 
}

#== get all spot_names from the file:
open (IN, "<$infofile") || die "Can not open the protein reference file $infofile.\n";
@lines = <IN>;
close IN;
#print "infile has " . @lines . " lines.\n";

$line = shift @lines;
$line =~ s/\r|\n|\
//g;
@temps = split /\t/, $line;

for($i=0; $i<@temps; $i++) {
    $item = $temps[$i];
    $item =~ s/^\s+|\s+$|\"//g;   # eliminate heading and tailing white spaces
    $item = uc $item;
#    print "i = $i, item = $item, ";
    $headers{$i} = $item;
}
#foreach my $key (keys %headers) {
#    print "key = $key, value = " . $headers{$key} . "\n";
#}
#print "\n";

$old_spot = "";
$line_index = 0;
foreach $line (@lines) {
    $line =~ s/\r|\n|\
//g;
    @temps = split /\t/, $line;
#    print "temps = @temps\n";
    $line_index++;
 
    #== reset all variables to null:
    $new_spot = "";
    $plate = "";
    $well = "";
    $annotation = "";
    $glink = "";
    $score = "";
    $coverage = "";
    $matched = "";
    $pi = "";
    $mw = "";
    $gel_pi = "";
    $gel_mw = "";
    $support = "";
    $class = "";

    for($i=0; $i<@temps; $i++) {
        $item = $headers{$i};
        if ($item eq "SPOT") {
            $new_spot = $temps[$i];
            $new_spot =~ s/^\s+|\s+$|\"//g;
            $line_spot{$line_index} = $new_spot;
        } elsif ($item eq "PLATE") {
            $plate = $temps[$i];
            $plate =~ s/^\s+|\s+$|\"//g;
            $line_plate{$line_index} = $plate;
        } elsif ($item eq "WELL") {
            $well = $temps[$i];
            $well =~ s/^\s+|\s+$|\"//g;
            $line_well{$line_index} = $well;
        } elsif ($item eq "ANNOTATION") {
            $annotation = $temps[$i];
            $annotation =~ s/^\s+|\s+$|\"//g;
            $annotation =~ s/\'/\'\'/g;
            $line_annotation{$line_index} = $annotation;
        }  elsif ($item eq "GENOME_LINK" || $item eq "GENOME LINK") {
            $glink = $temps[$i];
            $glink =~ s/^\s+|\s+$|\"//g;
            $line_glink{$line_index} = $glink;
        } elsif ($item eq "SCORE") {
            $score = $temps[$i];
            $score =~ s/^\s+|\s+$|\"//g;
            $line_score{$line_index} = $score;
        } elsif ($item eq "COVERAGE") {
            $coverage = $temps[$i];
            $coverage =~ s/^\s+|\s+$|\"//g;
            $line_coverage{$line_index} = $coverage;
        } elsif ($item eq "PEPTIDE_MATCHED" || $item eq "PEPTIDE MATCHED") {
            $matched = $temps[$i];
            $matched =~ s/^\s+|\s+$|\"//g;
            $line_matched{$line_index} = $matched;
        } elsif ($item eq "PI") {
            $pi = $temps[$i];
            $pi =~ s/^\s+|\s+$|\"//g;
            $line_pi{$line_index} = $pi;
        } elsif ($item eq "MW") {
            $mw = $temps[$i];
            $mw =~ s/^\s+|\s+$|\"//g;
            $mw =~ s/\,//g;
            $line_mw{$line_index} = $mw;
        } elsif ($item eq "GEL_PI" || $item eq "GEL PI") {
            $gel_pi = $temps[$i];
            $gel_pi =~ s/^\s+|\s+$|\"//g;
            $line_gel_pi{$line_index} = $gel_pi;
        } elsif ($item eq "GEL_MW" || $item eq "GEL MW") {
            $gel_mw = $temps[$i];
            $gel_mw =~ s/^\s+|\s+$|\"//g;
            $gel_mw =~ s/\,//g;
            $line_gel_mw{$line_index} = $gel_mw;
        } elsif ($item eq "SUPPORT") {
            $support = $temps[$i];
            $support =~ s/^\s+|\s+$|\"//g;
            $line_support{$line_index} = $support;
        } elsif ($item eq "CLASS") {
            $class = $temps[$i];
            $class =~ s/^\s+|\s+$|\"//g;
            $line_class{$line_index} = $class;
        }
    }
    $unique_spot{$new_spot} = 1;
}

#== check if any reference_spots are already in DOME:
$unique_spots = join /, /, (keys %unique_spot);
$sql = "select reference_spot_id from reference_spot where reference_spot_id in ($unique_spots)";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while ($spot = $sth->fetchrow()) {
    push @exist_spots, $spot;
}
$exist_spot = join /, /, @exist_spots;
if ($exist_spot ne "") {
    print "The following reference spot \"$exist_spot\" are already in DOME.";
    $sth->finish;
    $dbh->rollback;
    $dbh->disconnect;
    exit 99;
}

#    if ($annotation =~ /unknown/i && ($gel_pi ne "" && $gel_mw ne "")) {
#        $annotation = createProteinNameByMWPI($pname_prefix, $gel_mw, $gel_pi);
#    }
#    print "new_spot = $new_spot, plate = $plate, well = $well, annotation = $annotation, glink = $glink, score = $score, coverage = $coverage, matched = $matched, pi = $pi, mw = $mw, gel_pi = $gel_pi, gel_mw = $gel_mw, support = $support, class = $class\n";

#== insert or update the "reference_spot" table:
$old_spot = "";
foreach $index (sort {$a <=> $b} keys %line_spot) {
    $new_spot = $line_spot{$index};
    $plate = $line_plate{$index};
    $well = $line_well{$index};
    $annotation = $line_annotation{$index};
    $glink = $line_glink{$index};
    $score = $line_score{$index};
    $coverage = $line_coverage{$index};
    $matched = $line_matched{$index};
    $pi = $line_pi{$index};
    $mw = $line_mw{$index};
    $gel_pi = $line_gel_pi{$index};
    $gel_mw = $line_gel_mw{$index};
    $support = $line_support{$index};
    $class = $line_class{$index};
    if ($new_spot ne $old_spot) {
        $sql1 = "insert into \"reference_spot\" (\"reference_spot_id\"";
        $sql2 = "values ($new_spot";
        if ($plate ne "") {
            $sql1 .= ", \"plate_number\"";
            $sql2 .= ", \'$plate\'";
        }
        if ($well ne "") {
            $sql1 .= ", \"well_number\"";
            $sql2 .= ", \'$well\'";
        }
        $sql1 .= ") ";
        $sql2 .= ");\n";
        $main_sql .= $sql1 . $sql2;
    }
    if ($gel_pi ne "" && $gel_mw ne "") {
        $main_sql .= "update \"reference_spot\" set \"gel_pI\" = $gel_pi, \"gel_MW\" = $gel_mw where \"reference_spot_id\" = $new_spot;\n";   
    }

    #== insert into "ref_spot_protein_state" table:
    $sql1 = "insert into \"ref_spot_protein_state\" (\"ref_spot_protein_state_id\", \"reference_spot_id\", \"annotation\"";
    $sql2 = " values (nextval('ref_spot_protein_state_id_seq'), $new_spot, \'$annotation\'";
    if ($glink ne "") {
        $sql1 .= ", \"genome_link\"";
        $sql2 .= ", \'$glink\'";
    }
    if ($support ne "") {
        $sql1 .= ", \"support\"";
        $sql2 .= ", \'$support\'";
    }
    if ($score ne "") {
        $sql1 .= ", \"score\"";
        $sql2 .= ", $score";
    }
    if ($coverage ne "") {
        $sql1 .= ", \"coverage\"";
        $sql2 .= ", $coverage";
    }
    if ($matched ne "") {
        $sql1 .= ", \"peptide_matched\"";
        $sql2 .= ", $matched";
    }
    if ($class ne "") {
        $sql1 .= ", \"class\"";
        $sql2 .= ", \'$class\'";
    }
    if ($pi ne "") {
        $sql1 .= ", \"pI\"";
        $sql2 .= ", $pi";
    }
    if ($mw ne "") {
        $sql1 .= ", \"MW\"";
        $sql2 .= ", $mw";
    }
    $sql1 .= ")";
    $sql2 .= ");\n";
    $main_sql .= $sql1 . $sql2;

    $old_spot = $new_spot;
}
#print $main_sql;

#open (TEMP, ">./tmp_plots/temp.sql") or die "cannot open temp sql file\n";;
#print TEMP  $main_sql;
#close TEMP;

if ($main_sql ne "") {
    $sth = $dbh-> prepare($main_sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    # create sample_id
    #$sample_id = createSampleId($dbh, $srid, $tissue_type_label_id);
    #create sample_extract_id:
    #$seid = createSampleExtractId($dbh, $sample_id, $extract_id, $tech_repl);
    # create file_location_link_id;
    $file_type_id = getFileTypeId($dbh, $infofile);
    if ($file_type_id eq "") {
        $file_type_id = createFileTypeId($dbh, $infofile, "Proteomics reference gel info file");
    }
    $file_location_link_id = createFileIds($dbh, $file_home, $infofile, $file_type_id, $person_id);
    # create proteomics_measurement_id:
    $prot_measurement_id = createProteomicsMeasurementId($dbh, "", $protocol_type_id, "", "", $file_location_link_id);

    if ($prot_measurement_id ne "") {
        $sth->finish;
        $dbh->commit;
        $dbh->disconnect;
        print "File uploading is SUCCESSFUL.\n<br>";
        #print "Ending time: " . TimeStamp() . "\n";
        exit 0;
    } else {
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        print "File uploading is FAILED.<br>";
        #print "Ending time: " . TimeStamp() . "\n";
        exit 1;
    }
}
$sth->finish;
$dbh->rollback;
$dbh->disconnect;
print "No records has been inserted.File uploading is FAILED.\n";
print "Ending time: " . TimeStamp() . "\n";
exit 1;

###########################################################################
# subroutine:
#  createProteomicsMeasurementId($dbh, $seid, $protocol_type_id, $proc_file_location_link_id,
# $raw_file_location_link_id, $ref_file_location_link_id_id)
#
# parameters in: $dbh --- the database handle,
#                $file_namename --- the file name
# parameter out: $pmid --- the proteomics_measurement_id.
#
# X. Jing Li, 11/10/2005
###########################################################################
sub createProteomicsMeasurementId {
    my ($dbh, $seid, $protocol_type_id, $proc_file_location_link_id, $raw_file_location_link_id, $ref_file_location_link_id) = @_;
    my ($sth, $rc, $sql1, $sql2, $pmid);

    $sql1 = "select nextval('proteomics_measurement_id_seq')";
    $sth = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pmid = $sth->fetchrow();

    $sql1 = "insert into \"proteomics_measurement\" (proteomics_measurement_id, protocol_type_id";
    $sql2 = " values ($pmid, $protocol_type_id";
    if ($seid ne "") {
        $sql1 .= ", \"sample_extract_id\"";
        $sql2 .= ", $seid";
    }
    if ($proc_file_location_link_id ne "") {
        $sql1 .= ", \"proc_file_location_link_id\"";
        $sql2 .= ", $proc_file_location_link_id";
    }
    if ($raw_file_location_link_id ne "") {
        $sql1 .= ", \"raw_file_location_link_id\"";
        $sql2 .= ", $raw_file_location_link_id";
    }
    if ($ref_file_location_link_id ne "") {
        $sql1 .= ", \"ref_file_location_link_id\"";
        $sql2 .= ", $ref_file_location_link_id";
    }
    $sql1 .= ")";
    $sql2 .= ")";
    $sql1 .= $sql2;
    #print "createPMID: $sql1\n";
    $sth = $dbh-> prepare($sql1) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $sth->finish;
    return $pmid;
}
