#!/usr/local/bin/perl

use POSIX;
use R;
use RReferences;
use strict "vars";

#===============================================================================
# This script will get the data from PHP browser output with format as:
# blank\tcol1\tcol2\t...
# row_names\tvalue1\tvalue2\t...
# ...
# and input arguments $analysis_type and its corresponding arguments.
#
# author: X. Jing Li
# date: May 25, 2004
# Reversion:
#  6/3/2004: The original script will not work for web server calling from PHP
#     for plotting so we have to let GLUPlot to do the job. Instead using R to
#     plot the centers now it print out the data and GLUPLot will plot the 
#     centers.
# 11/22/2004: checking the script
# 1/5/2005: add plotting for kmeans back since virture terminal is running.
# 3/21/2006: changed the script for both GLUPlot and R plotting.
#===============================================================================

my ($infile, $outdir, $basename, $max_iter, $cluster, $png_in_row, $png_row_num,
    $png_height, @contents, $line, $r_rownames, %rownames, $name, $r_values, %values, 
    $i, $r_terms, $rstr, $kmeans_png, $kmeans_center, $kmeans_cluster, @centers,
    $ref_center, @col_names, $clu, $line, @terms, $term, $r_clusters, %clusters,
    $index, @keys, $k, $filename, $filetype, $outfile, @files, $file,
    $v, $r_cluster_value, @cluster_value, $max, $min, @size, $center_info_file, $center_file, $cluster_file, $dim, $j, $new_name, %newnames);

if (@ARGV == 5) {
    ($infile, $outdir, $basename, $max_iter, $cluster) = @ARGV;
    $png_in_row = 5;
} elsif (@ARGV == 6) {
    ($infile, $outdir, $basename, $max_iter, $cluster, $png_in_row) = @ARGV;
}else {
    die "Input arguments are not correct!\n";
}

#== calculate the size of the png file and row numbers of the png (4 columns)
$png_row_num = floor($cluster/$png_in_row) + 1;
$png_height = (640 / $png_in_row + 30) * $png_row_num;
#print "png row = $png_row_num, height = $png_height\n";

#== open the r_in_file and save the row_names and values in hash:
open (IN, "<$infile") or die "Can not open input file $infile. $!\n";
@contents = <IN>;
close IN;
$infile = $infile . ".copy";
open (OUT, ">$infile") or die "Can not open input file $infile:$!";

$r_rownames = {};
%rownames = %$r_rownames;
$r_values = {};
%values = %$r_values;

#-- get the first line:
$line = shift @contents;
print OUT $line;
chomp $line;
@terms = split(/\t/, $line);
for ($i=0; $i<@terms; $i++) {
    push @col_names, $terms[$i];
}

foreach ($i=0; $i<@contents; $i++) {
    $line = $contents[$i];
    chomp $line;
    $r_terms = split_line("\t", $line);
    $name = shift @$r_terms;
    $rownames{$i} = $name;
    $values{$i} = $r_terms;
    $newnames{$name} = "item" . $i;
    print OUT "item$i\t" . (join "\t", @$r_terms) . "\n";
}
close OUT;
#foreach $i (sort {$a <=> $b} keys %rownames) {
#    print "i = $i\trowname = " . $rownames{$i} . " value = " . $values{$i} . "\n";
#}

#== do the cluster analysis by R:
&R::initR("--no-save");
&R::library("RSPerl");
#print "Loaded RSPerl library\n";

#$kmeans_png = $outdir . $basename . "centers.png";
$kmeans_center = $outdir . $basename . "kmeans_temp_centers.txt";
$kmeans_cluster = $outdir . $basename . "kmeans_temp_clusters.txt";
$kmeans_png = $outdir . $basename . "kmeans_temp.png";
$rstr = 'x<-read.table("'.$infile.'", header=TRUE, sep="\t", row.names= 1);'
    . 'kmeansout<-kmeans(x, ' . $cluster. ', ' . $max_iter . ');'
    . 'center<-kmeansout$center;'
    . 'cluster<-kmeansout$cluster;'
    . 'write.table(center, file="'.$kmeans_center.'", sep="\t", col.names=NA);'
    . 'write.table(cluster, file="'.$kmeans_cluster.'", sep="\t", col.names=NA);'
    . 'max<-max(center);'
    . 'min<-min(center);'
    . 'fun_max<<-function() {ceiling(max);};'
    . 'fun_min<<-function() {floor(min);};'
    . 'fun_size<<-function() {kmeansout$size;};'
    . 'fun_dim<<-function() {dim(center)[2]};';
#    . 'bitmap("' . $kmeans_png . '", type="' . $png_type . '", width=640, height=' . $png_height . ', res=72);'
#    . 'bitmap(file="' . $kmeans_png . '");'
#    . 'png("' .$kmeans_png . '", width=640, height=' . $png_height . ', bg="white");'
#    . 'par(mfrow=c(' . $png_row_num . ', ' . $png_in_row . '), pch=".");'
#    . 'attach(kmeansout);'
#    . 'for(i in 1:' . $cluster . ') {'
#    . 'plot(kmeansout$center[i, ], ylim=c(min, max), xlab=paste("#",i,", n=", kmeansout$size[i], sep=""), ylab = "");'
#    . 'lines(kmeansout$center[i, ]);};'
#    . 'detach(kmeansout);'
#    . 'par(mfrow=c(1, 1), pch = 1);'
#    . 'dev.off();';

#print "$rstr\n";
&R::eval($rstr);
$max = &R::call("fun_max");
$min = &R::call("fun_min");
@size = &R::call("fun_size");
$dim = &R::call("fun_dim");
#print "max = $max, min = $min, dim = $dim\nsize = @size\n";

$center_info_file = $outdir . $basename . "_center_info.dat";
open (OUT, ">$center_info_file")
    || die "Can not open output cluster info file $center_info_file\n";
print OUT "max\t$max\nmin\t$min\n";
for($i=1; $i<=@size; $i++) {
    print OUT "$i\t". $size[$i-1] . "\n";
}
close OUT;

#== parse the center_str for each cluster:
open (IN, "<$kmeans_center");
@contents = <IN>;
close IN;

@centers = ();
for ($i = 0; $i<$cluster; $i++) {
    $ref_center = createArrayRef();
    $centers[$i] = $ref_center;
}
$line = shift @contents;  # skip the first line
#chomp $line;
#@terms = split /\t/, $line;
#for($i=1; $i<@terms; $i++) {
#    $term = $terms[$i];
#    $term =~ s/X|\"//g;
#    push @col_names, $term;
#}

foreach $line (@contents) {
    chomp $line;
    $line =~ s/\"//g;
    @terms = split /\t/, $line;
    $clu = shift @terms;
    #print "cluster = $clu\n";
    $ref_center = $centers[$clu-1];
    for ($i=0; $i<@terms; $i++) {
        push @$ref_center, $terms[$i];
    }
}
$center_file = $outdir . $basename . "_center.dat";
open (OUT, ">$center_file")
    || die "Can not open output center file $center_file\n";
#-- print the header line:
print OUT "#";
for ($i=1; $i<=$cluster; $i++) {
    print OUT "\t$i";
}
print OUT "\n";     # end of the header line.
for ($i=0; $i<$dim; $i++) {
    print OUT ($i+1);
    for ($j=0; $j<$cluster; $j++) {
        $ref_center = $centers[$j];
        @terms = @$ref_center;
        #print "\t" . $array[$i];
        print OUT "\t" . $terms[$i];
    }
    print OUT "\n";
}
close OUT;   # close $center_file

#foreach my $c (@col_names) {
#    print "$c ";
#}
#for ($i=0; $i<$cluster; $i++) {
#    $ref_center = $centers[$i];
#    print "cluster" . ($i+1);
#    foreach $v (@$ref_center) {
#        print "\t$v";
#    }
#    print "\n";
#}


#== parse the cluster_str to prepare the print out:
open (IN, "<$kmeans_cluster");
@contents = <IN>;
close IN;

$r_clusters = {};
%clusters = %$r_clusters;
foreach $line (@contents) {
    chomp $line;
    $line =~ s/\"//g;
    ($index, $clu) = split /\t/, $line;
    $clusters{$index} = $clu;
}

#== print the result into the a file for each cluster:
#-- delete all previous files with the same names
$filename = $basename . "_cluster";
$filetype = ".dat";
#= clear all old files for this session:
$outfile = $outdir . $filename;
opendir (DIR, $outdir) or die "Can not open directory $outdir. $!";
@files = readdir(DIR);
foreach $file (@files) {
    if ($file =~ /$filename/) {
#        print "file = $file\n";
        unlink $outdir . $file or warn "can not delete $file.$!";;
    }
}
closedir DIR;

#= start print
#- 1. print all clusters into one file:
$cluster_file = $outdir . $basename . "_cluster.dat";
open (OUT, ">$cluster_file") 
    || die "Can not open output cluster file $cluster_file. $!\n";
#-- print column names:
print OUT "CLU";
foreach  $v (@col_names) {
    print OUT "\t$v";
}
print OUT "\n";

for ($i=1; $i<=$cluster; $i++) {
    foreach $index (sort {$a <=> $b} keys %rownames) {
        $k = $rownames{$index};
        $new_name = $newnames{$k};
        if ( $clusters{$new_name} eq $i) {
            print OUT "$i\t$k";
            $r_cluster_value = $values{$index};
            @cluster_value = @$r_cluster_value;
            foreach $v (@cluster_value) {
                print OUT "\t$v";
            }
            print OUT "\n";
        }
    }
}
close OUT;   # close $cluster_file

@keys = sort (keys %clusters);
#- 2. print each cluster into one file
for ($i=0; $i<$cluster; $i++) {
    $outfile = $outdir . $filename . "_" .($i+1) . $filetype;
#    print "outfile = $outfile\n";
    open (OUT, ">$outfile") or warn "Can not open the cluster file $outfile. $!";
#    foreach my $c (@col_names) {
#        print OUT "\t$c";
#    }
    print OUT "Center:";
    $ref_center = $centers[$i-1];
    for(my $j=0; $j<@$ref_center; $j++) {
        $v = @$ref_center[$j];
        print OUT "\t$v";
#        print "\tv=$v";
    }
    print OUT "\nCluster:\n";
    
    foreach $index (sort {$a <=> $b} keys %rownames) {
        $k = $rownames{$index};
        $new_name = $newnames{$k};
        #print "k = $k, new_name = $new_name, i = $i, clusters{".$new_name."} = ".$clusters{$new_name}."\n";
        if ($clusters{$new_name} eq ($i+1)) {
            print OUT $k;
            $r_cluster_value = $values{$index};
            @cluster_value = @$r_cluster_value;
            foreach $v (@cluster_value) {
                print OUT "\t$v";
            }
            print OUT "\n";
        }
    }
    close OUT;
}
#unlink ($kmeans_center, $kmeans_cluster) or warn " can not delete files.$!";
exit;

sub split_line {
    my ($sep, $line) = @_;
    my @terms = split "$sep", $line;
    return \@terms;
}

sub createArrayRef {
    my $array_ref = [];
    return $array_ref;
}

sub createArrayReference {
    my (@terms) = @_;
    my @array;
    push @array, @terms;
    return \@array;
}
