#!/usr/local/bin/perl

use Statistics::R ;
use strict "vars";

#===============================================================================
# This script will get the data from PHP browser output as input_file with 
# format as:
# blank\tcol1\tcol2\t...
# row_names\tvalue1\tvalue2\t...
# ...
# input arguments 
# (input_file, output_dir, basename, max_iteration, cluster_num)
# output data files in directory $outdir:
# $basename_center.dat --- the center cordination of all clusters for GLUPlot
# $basename_center_info.dat --- the max and min values for plotting, item 
#                           numbers for each culster;
# $basename_cluster.dat --- input data reordered by cluster number
#
# author: X. Jing Li
# date: June 3, 2004
#===============================================================================

my ($infile, $outdir, $basename, $max_iter, $cluster, @contents, $line,
    $temp_center_file, $temp_cluster_file, @centers, $ref_center, $clu,
    $r_rownames, %rownames, $name, $r_genename, %genename, $r_values, %values,
    @col_names, $i, $r_terms, $R, $rstr, $cluster_str, $dim, $temp, 
    $center_file, $center_str, @outs, @terms, $j,
    $center_info_file, $center_max, $center_min, $cluster_size, $s, $v,
    $cluster_file, $r_clusters, %clusters, @clu, $index, @tcs, $tc, 
    $r_cluster_value, @cluster_value, %newnames, $new_name);

if (@ARGV == 5) {
    ($infile, $outdir, $basename, $max_iter, $cluster) = @ARGV;
#    print "ARGV = @ARGV\n";
} else {
    print "Input argument is not correct!\n";
    exit;
}
#$infile = "/export/home/xingjing/data/dome/analysis/R/dataForR_anova.txt";
#$outdir = "/export/home/xingjing/data/dome/analysis/R/";
#$basename = "r_kmeans";
#$max_iter = 20000;
#$cluster = 10;

#== open the in_file and save the row_names, genename and values in hashes:
open (IN, "<$infile") or die "Can not open input file $infile:$!";
@contents = <IN>;
close IN;

$infile = $infile . ".copy";
open (OUT, ">$infile") or die "Can not open input file $infile:$!";
$r_rownames = {};
%rownames = %$r_rownames;
$r_genename = {};
%genename= %$r_genename;
$r_values = {};
%values = %$r_values;

#-- get the first line:
#@col_names = ();
$line = shift @contents;
print OUT $line;
chomp $line;
@terms = split(/\t/, $line);
for ($i=1; $i<@terms; $i++) {
    push @col_names, $terms[$i];
}
#-- then the rest of line:
foreach ($i=0; $i<@contents; $i++) {
    $line = $contents[$i];
    chomp $line;
    $r_terms = split_line("\t", $line);
    $name = shift @$r_terms;
    $rownames{$i} = $name;
    $values{$i} = $r_terms;
    $newnames{$name} = "item" . $i;
    print OUT "item$i\t" . (join "\t", @$r_terms) . "\n";
}
close OUT;

#== do the cluster analysis by R:
my $temp_center_file = $outdir . $basename . "_temp_center.dat";
$temp_cluster_file = $outdir . $basename . "_temp_cluster.dat";
$R = Statistics::R->new();
$R->startR;

$rstr = 'x<-read.table("'.$infile.'", header=TRUE, sep="\t", row.names= 1);'
    . 'kmeansout<-kmeans(x, ' . $cluster . ', ' . $max_iter . ');'
    . 'center<-kmeansout$center;'
    . 'cluster<-kmeansout$cluster;'
    . 'write.table(center, file="'.$temp_center_file.'", sep="\t", col.names=NA);'
    . 'write.table(cluster, file="'.$temp_cluster_file.'", sep="\t", col.names=NA);';

#print "rstr = $rstr\n";
$R->send(qq`$rstr`);
$R->send(qq`print(cluster)`);
$cluster_str = $R->read;
$R->send(qq`dim<-dim(center)[2]; print(dim);`);
$dim = $R->read;
($temp, $dim) = split(/\]\ +/, $dim);

#== get cluster size and put into center_info:
$center_info_file = $outdir . $basename . "_center_info.dat";
open (OUT, ">$center_info_file")
    || die "Can not open output cluster info file $center_info_file. $!\n";
$R->send(qq`print(ceiling(max(center)))`);
$center_max = $R->read;
($temp, $center_max) = split(/\]\ /, $center_max);
print OUT "max\t$center_max\n";
$R->send(qq`print(floor(min(center)))`);
$center_min = $R->read;
($temp, $center_min) = split(/\]\ /, $center_min);
print OUT "min\t$center_min\n";
$R->send(qq`print(kmeansout\$size)`);
$cluster_size = $R->read;
$R->stopR();

#print "cluster_size = $cluster_size\n";
$cluster_size =~ s/^\s+//g;
$cluster_size =~ s/\[\d+\]|\n//g;
@terms = split(/\ +/, $cluster_size);
for($i=1; $i<@terms; $i++) {
    #$s = $terms[$i];
    print OUT "$i\t$terms[$i]\n";
}
close OUT;   # close file $center_info_file

#== get center data and print to center data file:
open (IN, "<$temp_center_file") 
    or die "Can not open temp_center file $temp_center_file. $!\n";
@contents = <IN>;
close IN;
#unlink $temp_center_file or warn "Can not delete temp_center_file $temp_center_file\n";

@centers = ();
for ($i = 0; $i<$cluster; $i++) {
    $ref_center = createArrayRef();
    $centers[$i] = $ref_center;
}
for ($i=1; $i<@contents; $i++) {
    $line = $contents[$i];
    $line =~ s/\"|\n//g;
    @terms = split /\t/, $line;
    $clu = $terms[0];
    #print "i = $i\nline = $line\ncluster = $clu, center_term = @terms\n";
    $ref_center = $centers[$clu-1];
    for ($j=1; $j<@terms; $j++) {
        push @$ref_center, $terms[$j];
    }
}

$center_file = $outdir . $basename . "_center.dat";
open (OUT, ">$center_file")
    || die "Can not open output center file $center_file\n";
#-- print the header line:
print OUT "#";
for ($i=1; $i<=$cluster; $i++) {
    print OUT "\t$i";
}
print OUT "\n";     # end of the header line.
for ($i=0; $i<$dim; $i++) {
    print OUT ($i+1);
    for ($j=0; $j<$cluster; $j++) {
        $ref_center = $centers[$j];
        @terms = @$ref_center;
        #print "\t" . $array[$i];
        print OUT "\t" . $terms[$i];
    }
    print OUT "\n";
#    print "\n";
}
close OUT;   # close $center_file

#for($i=1; $i<=$dim; $i++) {
#    print OUT "$i\t";
#    $rstr = 'y<-center[, ' . $i . ']; print(y, max.levels=NULL, width=5000);';
#    $R->send(qq`$rstr`);
#    $center_str = $R->read;
#    print "$center_str\n";
#    @outs = split(/\n/, $center_str);
#    foreach $line (@outs) {
#        chomp $line;
#        $line =~ s/^\ +//g;
#        if ($line =~ /\./) {
#            @terms = split(/\ +/, $line);
#            foreach $v (@terms) {
#                print OUT "$v\t";
#            }
#        }
#    }
#    print OUT "\n";
#}
#close OUT;

#== parse the cluster_str to prepare the print out:
#@outs = split /\n/, $cluster_str;
#$r_clusters = {};
#%clusters = %$r_clusters;
#foreach $line (@outs) {
#    $line =~ s/^\s*//g;
#    @clu = split /\ +/, $line;
#    $index = $clu[0];
#    $index =~ s/\[|\]//g;
#    for ($i=1; $i<=@clu; $i++) {
#        $clusters{$index} = $clu[$i];
#        $index++;
#    }
#}

#== parse the cluster_str to prepare the print out:
open (IN, "<$temp_cluster_file");
@contents = <IN>;
close IN;
#unlink $temp_cluster_file or warn "Can not delete temp_cluster_file $temp_cluster_file\n";
$r_clusters = {};
%clusters = %$r_clusters;
shift @contents;
foreach $line (@contents) {
    $line =~ s/\"|\n//g;
    ($tc, $clu) = split /\t/, $line;
    $clusters{$tc} = $clu;
}
#== print the result into the a file for each cluster:
@tcs = sort {$a<=>$b} (keys %clusters);
$cluster_file = $outdir . $basename . "_cluster.dat";
open (OUT, ">$cluster_file") 
    || die "Can not open output cluster file $cluster_file\n";
#-- print column names:
print OUT "CLU\tIdentifier";
foreach  $v (@col_names) {
    print OUT "\t$v";
}
print OUT "\n";

for ($i=1; $i<=$cluster; $i++) {
    foreach $index (sort {$a <=> $b} keys %rownames) {
        $tc = $rownames{$index};
        $new_name = $newnames{$tc};
        #print "tc = $tc, new_name = $new_name, i = $i, clusters{".$new_name."} = ".$clusters{$new_name}."\n";
        if ( $clusters{$new_name} eq $i) {
            print OUT "$i\t$tc";
            $r_cluster_value = $values{$index};
            @cluster_value = @$r_cluster_value;
            foreach $v (@cluster_value) {
                print OUT "\t$v";
            }
            print OUT "\n";
        }
    }
}
close OUT;   # close $cluster_file

#print "Cluster Centers:\n$center_str\n";

exit;

sub split_line {
    my ($sep, $line) = @_;
    my @terms = split "$sep", $line;
    return \@terms;
}

sub createArrayRef {
    my $array_ref = [];
    return $array_ref;
}

