<?php
/**
* This script is a Front-End for normalizing data already added 
* in the DOME database for different techniques.
* Date: 3 May 2005
* @author Aejaaz Kamal <aekamal@vt.edu>
*/


include_once("common.php");
$currPageName = 'NORMALIZE_2CHANNEL';
$currPageNameUpper = strtoupper($currPageName);
?>

<?php
if (isset($_REQUEST['technique_id']))
{	$_SESSION[$currPageNameUpper]['technique_id'] = $_REQUEST['technique_id'];	}

$title = "Normalize 2-Channel Microarray Data";
include("header.php");
if (!(strtoupper($_SESSION['role']) == 'MANAGER' OR strtoupper($_SESSION['role']) == 'EDITOR'))
{	printError("You are not authorized to view this page. Your role level should be 'editor' or 'manager' to view this page.");	exit();}
?>

<?php

if ($_REQUEST['submit'] == 'Normalize' AND $_SESSION[$currPageNameUpper]['force_add'] == 0)
{	processForm(0);	}
elseif ($_REQUEST['submit'] == 'Continue')
{	processForm(1);	}
elseif ($_REQUEST['submit'] == 'Cancel')
{	$_SESSION[$currPageNameUpper]['force_add'] = 0;	}
else
{	$_SESSION[$currPageNameUpper]['force_add'] = 0;	}
if ($errMsgs)
{  
	foreach ($errMsgs as $msg)
	{	printError($msg);	}
}
if ($_SESSION['Message'])
{  
	foreach ($_SESSION['Message'] as $msg)
	{	printMessage($msg);	}
	unset($_SESSION['Message']);
}
fillForm($_SESSION[$currPageNameUpper]['force_add']);
include_once("footer.php");
?>

<?php
//Functions
function fillForm($force_add = 0)
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	$self = $_SERVER[PHP_SELF];
	
		
	print "<form name=\"init_form\" method=\"post\" action=\"$self\">\n";
	if ($force_add)
	{
		print "\n<div class=\"controlBar\">";
		print "\n<input type=\"submit\" name=\"submit\" value=\"Cancel\" /> ";
		print "<input type=\"submit\" name=\"submit\" value=\"Continue\" />";
		print "\n</div>";
	}
	else
	{
		if (!getSampleLabelsFor2Channels(&$sample_label))
		{
			printError("No Sample Label found for Channel $i. Please add Sample Label Metadata.");	
			printControlBar($_SESSION['STEP_NUMBER']);
			print "\n</form>";
			return;	
		}
		
		print '<p class="help-tips">Please provide information about which data sample you want to normalize and choose a protocol to be used for normalization. The data would be stored in the database after normalization to assist in further quering.</p>';
		
		//Get technique_id
		$sql = "SELECT \"technique_id\", \"technique_name\" FROM \"technique\" 
		WHERE UPPER(\"technique_name\") = 'MICROARRAY'";
		$sth = $_SESSION['dbh']->query($sql);
		if ( DB::isError($sth) ) 
		{	$errMsgs[] = "Error retrieving Techniques. " . $sth->getMessage();	return;	}
		$row = $sth->fetchRow();
		$_SESSION[$currPageNameUpper]['technique_id'] = $row[0];
		
		print "<table width=\"100%\"><tr>";	//start table
			
		for ($i = 1; $i <= 2; $i++)
		{
			$identifier = "channel$i";
			print "<td>";
			
			print "<fieldset><legend> &nbsp;&nbsp;Channel $i - {$sample_label[$i]}";
			print "&nbsp;&nbsp; </legend>";	
			printMetaDataSelections($identifier, $_SESSION[$currPageNameUpper][$identifier]);
			print "</fieldset></td>";
		}
		print "</tr></table>"; //end table
		
		
		//Print Normalization Method
		$sql = "SELECT \"protocol_type_id\", \"protocol_type_name\" FROM \"protocol_type\" 
				WHERE UPPER(\"method\") = 'STATISTICAL' 
				AND technique_id = '{$_SESSION[$currPageNameUpper]['technique_id']}'
				ORDER BY UPPER(\"protocol_type_name\")";

		$sth = $_SESSION['dbh']->query($sql);
		if ( DB::isError($sth) ) 
		{	printError("Error retrieving Normalization Method. " . $sth->getMessage());	return;	}
		print "\n<p><label>Normalization Method: </label>";
		print "<select name=\"protocol_type_id\">";
		
		print "\n\t<option value=\"0\">--Select One--</option>";
		while ($row = $sth->fetchRow()) 
		{
			print "\n\t<option value=\"$row[0]\"";
			if ($_SESSION[$currPageNameUpper]['protocol_type_id'] == $row[0])
			{	print " selected";	}
			print ">$row[1]</option>";
		}
		print "\n</select></p>";
		
		//Print Extract.
		$sql = "SELECT \"extract_id\", \"name\" FROM \"extract\" WHERE \"technique_id\" = '{$_SESSION[$currPageNameUpper]['technique_id']}' ORDER BY UPPER(\"name\")";
		$sth = $_SESSION['dbh']->query($sql);
		if ( DB::isError($sth) ) 
		{	printError("Error retrieving Extracts. " . $sth->getMessage());	return;	}
		print "<p><label>Extraction Method: </label>";
		print "<select name=\"extract_id\">";
		print "\n\t<option value=\"0\">--Select One--</option>";
		while ($row = $sth->fetchRow()) 
		{
			print "\n\t<option value=\"$row[0]\"";
			if ($_SESSION[$currPageNameUpper]['extract_id'] == $row[0])
			{	print " selected";	}
			print ">$row[1]</option>";
		}
		print "\n</select></p>";
		
		print "<input type=\"submit\" name=\"submit\" value=\"Normalize\" />";
	}
	print "\n</form>";

}

/**============================
* Request Functions
*============================*/
function processForm($force_add = 0)
{
	global $errMsgs, $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	if (!$force_add)
	{
		$_SESSION[$currPageNameUpper]['channel1'] = $_REQUEST['channel1'];
		$_SESSION[$currPageNameUpper]['channel2'] = $_REQUEST['channel2'];
		$_SESSION[$currPageNameUpper]['protocol_type_id'] = $_REQUEST['protocol_type_id'];
		$_SESSION[$currPageNameUpper]['extract_id'] = $_REQUEST['extract_id'];
		
		
		$noOfChannels = 2;
		for ($i = 1; $i <= $noOfChannels; $i++)
		{
			$identifier = "channel$i";
			if (!$_SESSION[$currPageNameUpper][$identifier]['experiment_set_id'])
			{	$errMsgs[] = "Please Select an Experiment Set for channel $i.";	}
			
			if (!$_SESSION[$currPageNameUpper][$identifier]['exp_name'])
			{	$errMsgs[] = "Please Select an Experiment for channel $i.";	}
			
			if (!$_SESSION[$currPageNameUpper][$identifier]['sampling_point_label_id'])
			{	$errMsgs[] = "Please Select a Sampling Point for channel $i.";	}
			
			if (!$_SESSION[$currPageNameUpper][$identifier]['tissue_type_label_id'])
			{	$errMsgs[] = "Please Select a Tissue Type for channel $i.";	}
			
			if (!$_SESSION[$currPageNameUpper][$identifier]['biological_replicate'])
			{	$errMsgs[] = "Please Select the biological replicate number for channel $i.";	}
		}
		
		if (!$_SESSION[$currPageNameUpper]['protocol_type_id'])
		{	$errMsgs[] = "Please Select a Normalization Method.";	}
		
		if (!$_SESSION[$currPageNameUpper]['extract_id'])
		{	$errMsgs[] = "Please Select extraction method.";	}
		
		if ($errMsgs)
		{	return 0;	}
			
		//Now find out sampling_replicate_id from the given info.
		for ($i = 1; $i <= $noOfChannels; $i++)
		{
			$identifier = "channel$i";
			
			$sql = "SELECT sr.\"sampling_replicate_id\" FROM \"sampling_replicate\" sr
					, \"sampling_point\" sp
					, \"experiment\" e
				WHERE e.\"exp_set_id\" = '{$_SESSION[$currPageNameUpper][$identifier]['experiment_set_id']}'
				AND e.\"exp_name\" = '{$_SESSION[$currPageNameUpper][$identifier]['exp_name']}'
				AND sp.\"experiment_id\" = e.\"experiment_id\"
				AND sp.\"label_id\" = '{$_SESSION[$currPageNameUpper][$identifier]['sampling_point_label_id']}'
				AND sr.\"sampling_point_id\" = sp.sampling_point_id
				AND sr.\"replicate\" = '{$_SESSION[$currPageNameUpper][$identifier]['biological_replicate']}'";
			//print $sql;
			$sth = $_SESSION['dbh']->query($sql);
			if ( DB::isError($sth) ) 
			{	
				$errMsgs[] = "Error retrieving Sampling Replicate for channel $i. " . $sth->getMessage();	
				return 0;
			}
		
			if ($sth->numrows() > 0)
			{
				$row = $sth->fetchRow();
				$_SESSION[$currPageNameUpper][$identifier]['sampling_replicate_id'] = $row[0];
				//print "Set sampling_replicate_id = {$_SESSION[$currPageNameUpper]['sampling_replicate_id']} ($n)";
			}
			else
			{	$errMsgs[] = "Sampling Replicate not found in the database for the information provided for channel $i";	}
		}
		//Validating Normalization Method
		$sql = "SELECT \"protocol_type_id\" FROM \"protocol_type\" 
			WHERE \"technique_id\" = '{$_SESSION[$currPageNameUpper]['technique_id']}'
			AND \"protocol_type_id\" = '{$_SESSION[$currPageNameUpper]['protocol_type_id']}'";
			
		$sth = $_SESSION['dbh']->query($sql);
		if ( DB::isError($sth) ) 
		{	
			$errMsgs[] = "Error checking for Normalization Method. " . $sth->getMessage();	
			return 0;
		}
		if ($sth->numrows() == 0)
		{	$errMsgs[] = "Normalization Method Not Found for selected technique.";	}
		
		
		if ($errMsgs)
		{	return 0;	}
	}
	
	$perlScriptFile = "normalizeCDnas.pl";
	$prefix = $_SESSION['BROWSER_TMP_FULL'] . session_id();
	$sampling_replicate_id1 = $_SESSION[$currPageNameUpper]['channel1']['sampling_replicate_id'];
	$tissue_type_label_id1 = $_SESSION[$currPageNameUpper]['channel1']['tissue_type_label_id'];
	$sampling_replicate_id2 = $_SESSION[$currPageNameUpper]['channel2']['sampling_replicate_id'];
	$tissue_type_label_id2 = $_SESSION[$currPageNameUpper]['channel2']['tissue_type_label_id'];
	$protocol_type_id = $_SESSION[$currPageNameUpper]['protocol_type_id'];
	$extract_id = $_SESSION[$currPageNameUpper]['extract_id'];
	//$force_add = $_SESSION[$currPageNameUpper]['force_add'];
	
	$com = "export PATH; PATH=\$PATH:{$_SESSION['R_DIR']}; export R_HOME; R_HOME={$_SESSION['R_HOME']}; export LD_LIBRARY_PATH; LD_LIBRARY_PATH={$_SESSION['RSPERL_DIR']}libs:{$_SESSION['R_HOME']}/lib; export PERL5LIB; PERL5LIB={$_SESSION['PERL5LIB']}; {$_SESSION['PERL_DIR']}perl $perlScriptFile \"$prefix\" $sampling_replicate_id1 $tissue_type_label_id1 $sampling_replicate_id2 $tissue_type_label_id2 $protocol_type_id $extract_id  2>&1;";
	$retString = exec($com, $retLines, $retValue);
	
	if ($retLines)
  	{
		foreach ($retLines as $retLine)
		{	$msg .= "$retLine <br />";	}
  	}
	if ($retValue == 0)	//success
  	{	
	  	$_SESSION[$currPageNameUpper]['force_add'] = 0;
		if (!$msg)
	  	{	$msg = "System command to '$perlScriptFile' returned a successful value.";	}
	  	
	  	$_SESSION['Message'][] = $msg;
		//redirect
		print "<meta http-equiv=\"refresh\" content=\"2; URL=intro.php?stepno=1\">";
		exit();
	}
	elseif ($retValue == 99)	//data exists
	{
		$_SESSION[$currPageNameUpper]['force_add'] = 1;
		$_SESSION['Message'][] = $msg;
	}
	else
	{	
		$_SESSION[$currPageNameUpper]['force_add'] = 0;
	  	if (!$msg)
	  	{	$msg = "System command to '$perlScriptFile' failed. <br />Command was \"$com\"\nReturn: $retString RetValue = $retValue";	}
	  	$errMsgs[] = $msg;
	}
	
  	if ($errMsgs)
	{	return 0;	}
	return 1;
}

?>
