#!/usr/local/bin/perl

use DBI;
use DOMELIB;
#use Statistics::R;
use R;
use RReferences;
use strict "vars";

#===============================================================================
# This script will be called from Front-end and call R.ma library to read data
# from cel file and normalizing the data then insert data into "ma_norm" table.
#
# author: X. Jing Li
# date: August 18, 2005
# Reversion:
# 8/3/2006: eliminate force_add since all deletion will be handled by delte tract.
#===============================================================================

#my ($prefix, $srids, $ptid, $extract_id, $tissue_type_label_id, $force_add) = @ARGV;
my ($prefix, $srids, $ptid, $extract_id, $tissue_type_label_id, $force_add) = @ARGV;
#print "ARGV = @ARGV\n";
my ($database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc,
    $script, $method, @srids, @in_DB_srids, $srids_in_DB, $msg, $filename, $file_location, $array_id, $seid, $srid, $cel_file, %cel_file_seids, @celfiles, $all_cel_files, $output, $rstr, @lines, $line, $i, @terms, $r_probe_names, %probe_names, $pname, @values, $value, $value_2p, %cel_file_column_index, $r_pname_rids, %pname_rids, $rows, $rid, $csid, $sql1, $sql2, $mainsql, $r_value, $insert_num, $loop_num, %srid_detail, %seid_srids, %srid_seids, $seid_srids, @missing_srids, $R, $r_array_hashes, @array_hashes, $ref_hash, @rids, @index, %value_hash, $r_unseen_pnames, @unseen_pnames
    );
    
@srids = split ", ", $srids;
#print "srids = @srids\n";

#== start time:
print "Starting time = " . TimeStamp() . "\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die "Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== get the normalizing method from "protocol_type.method_options"
$sql = "select \"method_options\" from \"protocol_type\" where protocol_type_id = $ptid";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$msg = $sth->fetchrow();
($script, $method) = split /\#\#/, $msg;
#print "script = $script, method = $method\n";
$method =~ s/^OPT=//g;
$method =~ s/^method=//g;
$method =~ s/\"//g;

#== check the existance of these arrays:
$sql = "select distinct sampling_replicate_id from \"ma_norm\" where protocol_type_id = $ptid AND sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and sample_id in (select sample_id from sample where label_id = $tissue_type_label_id and sampling_replicate_id in ($srids)))";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while ($srid = $sth->fetchrow()) {
    push @in_DB_srids, $srid;
}

if (@in_DB_srids > 0) {
    $srids_in_DB = join ", ", @in_DB_srids;
    $msg = getSridsDetail($dbh, $srids_in_DB, $ptid, $extract_id, $tissue_type_label_id);
    print "$msg\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    print "Ending time = " . TimeStamp() . "\n";
    exit 1;
}

#== get the names for the CEL files:
$sql = "select distinct F.file_name, FL.location, H.array_id, SE.sample_extract_id, S.sampling_replicate_id from \"file\" F, \"file_location\" FL, \"file_location_link\" FLL, image_analysis IA, raw_data RD, hybridization H, labeled_extract LE, sample_extract SE, sample S where FLL.file_id = F.file_id AND FLL.file_location_id = FL.file_location_id AND FLL.file_location_link_id = IA.file_location_link_id AND IA.raw_data_id = RD.raw_data_id AND RD.hybridization_id = H.hybridization_id AND (H.labeled_extract1_id = LE.labeled_extract_id OR H.labeled_extract2_id = LE.labeled_extract_id) AND LE.sample_extract_id = SE.sample_extract_id AND SE.extract_id = $extract_id AND SE.sample_id = S.sample_id AND S.sampling_replicate_id in ($srids)";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($filename, $file_location, $array_id, $seid, $srid) = $sth->fetchrow()) {
    #print "filename = $filename, array_id = $array_id, seid = $seid, srid = $srid\n";
    $cel_file = $file_location . $filename;
    $cel_file =~ s/\(/\\\(/g;
    $cel_file =~ s/\)/\\\)/g;
    #print "array_id = $array_id, sample_extract_id = $seid, cel file = $cel_file\n";
    $cel_file_seids{$filename} = $seid;
    $seid_srids{$seid} = $srid;
    $srid_seids{$srid} = $seid;
    push @celfiles, $cel_file;
}
#print "cel_files = @celfiles\n";

#== check for sids without the CEL file:
foreach $srid (@srids) {
    if (!exists $srid_seids{$srid}) {
        $srid_detail{$srid} = getSridDetail($dbh, $srid, $tissue_type_label_id);
    }
}
@missing_srids = sort {$a <=> $b} keys %srid_detail;
if (@missing_srids == 1 && $force_add == 0) {
    $srid = shift @missing_srids;
    $msg = "The following sample had no CEL file found in DOME.<br>"
        . "<center><table border><TR><TH>Exp Set Name</TH><TH>Experiment Name</TH><TH>Time Point</TH><TH>Replicate</TH><TH>Tissue Type</TH></TR>"
        . $srid_detail{$srid} . "</table></center><br>"
        . "If you want to upload the CEL file, press \"Cancel\" then go to \"Add data to datebase\", OR  press \"Continue\" to normalize.<br>";
    print "$msg\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    print "Ending time = " . TimeStamp() . "\n";
    exit 99;
} elsif (@missing_srids > 1 && $force_add == 0) {
    $msg = "The following samples have no CEL files found in DOME.<br><center><table border><TR><TH>Exp Set Name</TH><TH>Experiment Name</TH><TH>Time Point</TH><TH>Replicate</TH><TH>Tissue Type</TH></TR>";
    foreach $srid (@missing_srids) {
        $msg .= $srid_detail{$srid};
    }
    $msg .= "</table></center><br>";
    $msg .= "If you want to upload the CEL files, press \"Cancel\" then go to \"Add data to datebase\", OR press \"Continue\" to normalize.<br>";
    print "$msg\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    print "Ending time = " . TimeStamp() . "\n";
    exit 99;
}

#== do the normalization by RSPerl:
#&R::initR("--silent");
&R::initR("--no-save", "--silent", "--slave");
&R::library("RSPerl");
#print "Loaded RSPerl library\n";

#== do the normalization by StatisticsR:
#$R = Statistics::R->new();
#$R->startR;

$output = $prefix . "output.txt";
$all_cel_files = join '", "', @celfiles;
$all_cel_files = '"' . $all_cel_files . '"';

my $sinkfile = $prefix . "_sinkfile.txt"; 
$rstr = 'sink(file = "' . $sinkfile . '");'
    . 'options(echo = FALSE);'
    . 'library(affy);'
    . 'files <- c(' . $all_cel_files . ');'
    . 'data <- ReadAffy(filenames = files);'
    . 'data.out <- '. $method . '(data);'
    . 'write.exprs(data.out, file="' . $output . '");';
#print $rstr; 

#== by RSPerl:
&R::eval($rstr);

#== by StatisticsR:
#$R->send(qq`$rstr`);
#$R->stopR();

#== read in all data from output file. save it in hashes and arrays for later 
#== insertion. 
#== First need to create records in composite_sequence and reporter for 
#== composite_sequence_id and reporter_id. Then insert data into "ma_norm".
#-- 1. read all info from output file:
open (IN, "<$output") or die "Can not open output file $output\n";
@lines = <IN>;
close IN;

#-- 2. get the colume header and save the correspondence to hash:
#- key = $i -- column number,
#- value = $terms[$i] -- cel_file_name
$line = shift @lines;
chomp $line;
@terms = split(/\t/, $line);
for($i=0; $i<@terms; $i++) {
    $cel_file_column_index{$i} = $terms[$i];
}

#-- 3. read in the data:
$r_probe_names = {};
%probe_names = %$r_probe_names;
foreach $line(@lines) {
    chomp $line;
    $line =~ s/\r|\n//g;
    @values = split /\t/, $line;
    $pname = shift @values;
    $probe_names{$pname} = createNewArray(@values);
}
#foreach $pname (sort keys %probe_names) {
#    $r_value = $probe_names{$pname};
#    @values = @$r_value;
#    print "pname = $pname, r_value = $r_value, value = @values\n";
#}

#-- 4 get all rids with the tc from databese and save if hash:
$r_pname_rids = {};
%pname_rids = %$r_pname_rids;
$sql = "select R.reporter_id, CS.probe_name from reporter R, composite_sequence CS where R.composite_sequence_id = CS.composite_sequence_id";

$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$rows = $sth->rows();
if ($rows > 0) {
    while (($rid, $pname) = $sth->fetchrow()) {
        $pname_rids{$pname} = $rid;
    }
} 
#-- 5. create the composite_sequence_ids and reporter_ids if a new probe_name is
#-- new:
$r_unseen_pnames = ();
@unseen_pnames = @$r_unseen_pnames;
foreach $pname ( sort {$a <=> $b} keys %probe_names ) {
    if (!(exists $pname_rids{$pname}) ) {
        #$csid = createCompositeSequenceId($dbh, $pname);
        #$rid = createReporterId($dbh, $csid);
        $pname_rids{$pname} = $rid;
#        print "probe_name = $pname, csid = $csid, rid = $rid\n";
        push @unseen_pnames, $pname;
    }
}

#-- 6 if there are unseen probe_ids report and exit:
if (@unseen_pnames > 0 && @unseen_pnames < 100) {
    $msg = " There are unknown probe ids in CEL files. The following probe_id(s) need more information.<br>";
    $pname = join ", ", @unseen_pnames;
    $msg .= $pname . "<br>";
    print $msg;
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    print " Ending time = " . TimeStamp() . "\n";
    exit 1;
} elsif (@unseen_pnames > 0 && @unseen_pnames > 100) {
    $msg = " There are " . @unseen_pnames . " unknown probe ids in CEL files.<br>";
    print $msg;
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    print " Ending time = " . TimeStamp() . "\n";
    exit 1;
} 

#-- 6. insert data into "ma_norm"
#my $temp_file = "./tmp-plots/temp.sql";
#open (LOG, ">$temp_file") or die "Can not open temp file $temp_file\n";

$r_array_hashes = [];
@array_hashes = @$r_array_hashes;
@index = keys %cel_file_column_index;
#print "\%cel_file_column_index index = @index\n";

for($i=0; $i<@index; $i++) {
    $array_hashes[$i] = createHashRef();
}
foreach $pname (sort {$a <=> $b} keys %probe_names ) {
    $rid = $pname_rids{$pname};
    $r_value = $probe_names{$pname};
    @values = @$r_value;
    for ($i=0; $i<@index; $i++) {
        $value = $values[$i];
        $ref_hash = $array_hashes[$i];
        $ref_hash->{$rid} = $value;
        $array_hashes[$i] = $ref_hash;
    }
}

foreach $i (@index) {
#    print "i = $i, time = " . TimeStamp() . "\n";
    $mainsql = "";
    $ref_hash = $array_hashes[$i];
    %value_hash = %$ref_hash;
    #$srid = $srids[$i];
    $seid = $cel_file_seids{$cel_file_column_index{$i}};
    $srid = $seid_srids{$seid};
    @rids = sort {$a <=> $b} keys %value_hash;
    #print LOG "i = $i, srid = $srid, seid = $seid, ref_hash = $ref_hash\n";
    foreach $rid (@rids) {
        $value = $ref_hash->{$rid};
        if ($value ne null) {
            $value_2p = 2 ** $value;
        } else {
            $value_2p = null;
        }
        $sql1 = "insert into \"ma_norm\" (ma_norm_id, sampling_replicate_id, reporter_id, \"intensity_l\", sample_extract_id, protocol_type_id, multiple_reporter";
        $sql2 = "values (nextval('ma_norm_id_seq'), $srid, $rid, $value, $seid, $ptid, \'N\'";
        if ($value_2p ne null) {
            $sql1 .= ", intensity";
            $sql2 .= ", $value_2p";
        }
        $sql1 .= ") ";
        $sql2 .= ");\n";
        $mainsql .= $sql1 . $sql2; 
    }
    #print LOG "$mainsql";
    $sth = $dbh-> prepare($mainsql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
}

#close LOG;
 $dbh->commit;
$sth->finish;
$dbh->disconnect;
print "Ending time = " . TimeStamp() . "\n";
exit 0;

###########################################################################
# subroutine: createNewArray(@v):
# Similar to above function with initial element as a array.
#
# parameters in: @v --- the initial array
# parameter out: $ref_array --- reference of the array.
#
###########################################################################
sub createNewArray {
    my (@v) = @_;
    my ($ref_array, @array);

    $ref_array = [];
    @array = @$ref_array;
    push @array, @v;
    return \@array;
}
###########################################################################
# subroutine: createHashRef():
# Similar to above function with refernece of a new hash.
#
# parameters in: 
# parameter out: $ref_hash --- reference of the hash.
#
###########################################################################
sub createHashRef {
    my $ref_hash = ();
    return $ref_hash;
}

###########################################################################
# subroutine: createCompositeSequenceId($dbh, $tc):
# create a composite_sequence_id for tc.
#
# parameters in: $dbh -- database handle
#                $probe_name -- the probe name
# parameter out: $csid -- the new composite_sequence_id.
# X. Jing Li: 11/10/2004
###########################################################################
sub createCompositeSequenceId {
    my ($dbh, $probe_name) = @_;
    my ($sth, $rc, $sql, $csid);

    $sth = $dbh->prepare("select nextval('composite_sequence_id_seq')") 
        or die "Can not prepare: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $csid = $sth->fetchrow;

    $sql = "insert into composite_sequence (composite_sequence_id, \"probe_name\")";
    $sql .= " values ($csid, \'$probe_name\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $csid;
}

###########################################################################
# subroutine: createReporterId($dbh, $csid):
# create a reporter_id for composite_sequence_id.
#
# parameters in: $dbh -- database handle
#                $csid -- the composite_sequence_id
# parameter out: $rid -- the new reporter_id.
# X. Jing Li: 11/10/2004
###########################################################################
sub createReporterId {
    my ($dbh, $csid) = @_;
    my ($sth, $rc, $sql, $rid);

    $sth = $dbh->prepare("select nextval('reporter_id_seq')") 
         or die "Can not prepare select nextval: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $rid = $sth->fetchrow;

    $sql = "insert into reporter (reporter_id, composite_sequence_id) values ($rid, $csid)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $rid;
}
 
###########################################################################
# subroutine: getSridDetail($dbh, $srid, $tissue_type_label_id)
# this subroutine get the exp_set_name, experiment_name, time_point, replicate
# for a specific sampling_replicate_id and create a html talbe row for it.
#
# parameters in: $dbh -- database handle
#                $srid -- the sampling_point_id
#                $tissue_type_label_id --
# parameter out: $html_table_row -- the msg in html talge row format.
# X. Jing Li: 12/21/2005
###########################################################################
sub getSridDetail {
    my ($dbh, $srid, $tissue_type_label_id) = @_;
    my ($sql, $sth, $rc, $es_name, $exp_name, $l_name, $replicate, $tissue_type,
        $html_table_row);

    $sql = "select ES.exp_set_name, E.exp_name, L.\"name\", SR.replicate from experiment_set ES, experiment E, sampling_point SP, sampling_replicate SR, label L where E.exp_set_id = ES.exp_set_id AND SP.experiment_id = E.experiment_id AND SP.label_id = L.label_id AND SP.sampling_point_id = SR.sampling_point_id AND SR.sampling_replicate_id = $srid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    ($es_name, $exp_name, $l_name, $replicate) = $sth->fetchrow();
    $sql = "select \"name\" from label where label_id = $tissue_type_label_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $tissue_type = $sth->fetchrow();

    $html_table_row = "<TR><TD><font color=\"#000000\">$es_name</font></TD><TD><font color=\"#000000\">$exp_name</font></TD><TD><font color=\"#000000\">$l_name</font></TD><TD><font color=\"#000000\">$replicate</font></TD><TD><font color=\"#000000\">$tissue_type</font></TD></TR>";
    $sth->finish;
    return $html_table_row;
}

###########################################################################
# subroutine: 
# getSridsDetail($dbh, $srids, $ptid, $extract_id, $tissue_type_label_id)
# this subroutine get the exp_set_name, experiment_name, time_point, replicate
# for a specific sampling_replicate_id and create a html talbe row for it.
#
# parameters in: $dbh -- database handle
#                $srid -- the sampling_point_id
#                $tissue_type_label_id --
# parameter out: $html_table_row -- the msg in html talge row format.
# X. Jing Li: 12/21/2005
###########################################################################
sub getSridsDetail {
    my ($dbh, $srids, $ptid, $extract_id, $tissue_type_label_id) = @_;
    my ($sql, $sth, $rc, $es_name, $exp_name, $l_name, $replicate, $tissue_type,
        $extract_name, $protocol_type_name, $msg);

    $sql = "select \"name\" from extract where extract_id = $extract_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $extract_name = $sth->fetchrow();
    $sql = "select \"name\" from label where label_id = $tissue_type_label_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $tissue_type = $sth->fetchrow();
    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $protocol_type_name = $sth->fetchrow();
    $msg = "For extraction \"$extract_name\", the following samples are already been normalized in DOME for tissue type \"$tissue_type\" and protocol \"$protocol_type_name\".<br><center><table border class=\"result-table\"><TR><th>Exp Set Name</th><th>Experiment Name</th><th>Time Point</th><th>Replicate</th></tr>";

    $sql = "select ES.exp_set_name, E.exp_name, L.\"name\", SR.replicate from experiment_set ES, experiment E, sampling_point SP, sampling_replicate SR, label L where E.exp_set_id = ES.exp_set_id AND SP.experiment_id = E.experiment_id AND SP.label_id = L.label_id AND SR.sampling_point_id = SP.sampling_point_id AND SR.sampling_replicate_id in ($srids)";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    while (($es_name, $exp_name, $l_name, $replicate) = $sth->fetchrow()) {
        $msg .= "<TR><TD><font color=\"#000000\">$es_name</font></TD><TD><font color=\"#000000\">$exp_name</font></TD><TD><font color=\"#000000\">$l_name</font></TD><TD><font color=\"#000000\">$replicate</font></TD></TR>";
    }
    $msg .= "</TABLE></center><br>You must delete the data from DOME before re-normalize the data set.<br>";

    $sth->finish;
    return $msg;
}
