#!/usr/local/bin/perl

use DBI;
use DOMELIB;
#use Statistics::R;
use R;
use RReferences;
use strict "vars";

#===============================================================================
# This script will be called from Front-end and call R.affy library to read data
# from cel file and normalizing the data then insert data into "ma_norm" table.
#
# author: X. Jing Li
# date: Feb. 2, 2006
# Reversion:
#===============================================================================

#my ($prefix, $srids, $ptid, $extract_id, $tissue_type_label_id, $force_add) = @ARGV;
my ($prefix, $srids, $ptid, $extract_id, $tissue_type_label_id, $force_add) = @ARGV;
#print "ARGV = @ARGV\n";
my ($database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc, $script, $method, @srids, %in_DB_srids, $srids_in_DB, @new_srids, $msg, $filename, $file_location, $array_id, $seid, $srid, $cel_file, %cel_file_seids, @celfiles, $cel_file_exist, @terms, $array_num, $output, $rstr, @lines, $line, $r_pname_intensity, %pname_intensity, $r_pnames, @pnames, $pname, $intensity, $intensity_l,$r_pname_rids, %pname_rids, $rows, $rid, $csid, $sql1, $sql2, $mainsql, %srid_detail, %seid_srids, $seid_srids, @missing_srids, $R, $r_unseen_pnames, @unseen_pnames);
    
@srids = split ", ", $srids;
#print "srids = @srids\n";


#== start time:
print "Starting time = " . TimeStamp() . "\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die "Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== get the normalizing method from "protocol_type.method_options"
$sql = "select \"method_options\" from \"protocol_type\" where protocol_type_id = $ptid";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$msg = $sth->fetchrow();
($script, $method) = split /\#\#/, $msg;
#print "script = $script, method = $method\n";
$method =~ s/^OPT=//g;
$method =~ s/^method=//g;
$method =~ s/\"//g;

#== check the existance of these arrays:
$sql = "select distinct sampling_replicate_id from \"ma_norm\" where protocol_type_id = $ptid AND sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and sample_id in (select sample_id from sample where label_id = $tissue_type_label_id and sampling_replicate_id in ($srids)))";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while ($srid = $sth->fetchrow()) {
    $in_DB_srids{$srid} = 1;
}
if ((keys %in_DB_srids) > 0) {
    $srids_in_DB = join ", ", (keys %in_DB_srids);
    $msg = getSridsDetail($dbh, $srids_in_DB, $ptid, $extract_id, $tissue_type_label_id);
    print "$msg\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    print "Ending time = " . TimeStamp() . "\n";
    exit 99;
} 

#== get the names for the CEL files:
$sql = "select distinct F.file_name, FL.location, H.array_id, SE.sample_extract_id, S.sampling_replicate_id from \"file\" F, \"file_location\" FL, \"file_location_link\" FLL, image_analysis IA, raw_data RD, hybridization H, labeled_extract LE, sample_extract SE, sample S where FLL.file_id = F.file_id AND FLL.file_location_id = FL.file_location_id AND FLL.file_location_link_id = IA.file_location_link_id AND IA.raw_data_id = RD.raw_data_id AND RD.hybridization_id = H.hybridization_id AND (H.labeled_extract1_id = LE.labeled_extract_id OR H.labeled_extract2_id = LE.labeled_extract_id) AND LE.sample_extract_id = SE.sample_extract_id AND SE.extract_id = $extract_id AND SE.sample_id = S.sample_id AND S.sampling_replicate_id in ($srids)";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($filename, $file_location, $array_id, $seid, $srid) = $sth->fetchrow()) {
    $cel_file = $file_location . $filename;
    $cel_file =~ s/\(/\\\(/g;
    $cel_file =~ s/\)/\\\)/g;
    #print "array_id = $array_id, sample_extract_id = $seid, cel file = $cel_file\n";
    $cel_file_seids{$filename} = $seid;
    $seid_srids{$seid} = $srid;
    push @celfiles, $cel_file;
}
#print "cel_files = @celfiles\n";

#== check for sids without the CEL file:
foreach $srid (@srids) {
    $cel_file_exist = 0;
    foreach $seid (keys %seid_srids) {
        if ($srid == $seid_srids{$seid}) { # found cel_file in DB
            $cel_file_exist = 1;
            last;
        }
    }
    if ($cel_file_exist == 0) {
        $srid_detail{$srid} = getSridDetail($dbh, $srid, $tissue_type_label_id);
    }
}
@missing_srids = sort {$a <=> $b} keys %srid_detail;
if (@missing_srids == 1 && $force_add == 0) {
    $srid = shift @missing_srids;
    $msg = "The following sample has no CEL file found in DOME.<br>"
        . "<center><table border><TR><TH>Exp Set Name</TH><TH>Experiment Name</TH><TH>Time Point</TH><TH>Replicate</TH><TH>Tissue Type</TH></TR>"
        . $srid_detail{$srid} . "</table></center><br>"
        . "If you want to upload the CEL file, press \"Cancel\" then go to \"Add data to datebase\", OR  press \"Continue\" to normalize.<br>";
    print "$msg\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    print "Ending time = " . TimeStamp() . "\n";
    exit 99;
} elsif (@missing_srids > 1 && $force_add == 0) {
    $msg = "The following samples have no CEL files found in DOME.<br><center><table border><TR><TH>Exp Set Name</TH><TH>Experiment Name</TH><TH>Time Point</TH><TH>Replicate</TH><TH>Tissue Type</TH></TR>";
    foreach $srid (@missing_srids) {
        $msg .= $srid_detail{$srid};
    }
    $msg .= "</table></center><br>";
    $msg .= "If you want to upload the CEL files, press \"Cancel\" then go to \"Add data to datebase\", OR press \"Continue\" to normalize.<br>";
    print "$msg\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    print "Ending time = " . TimeStamp() . "\n";
    exit 99;
}

#== do the normalization by RSPerl:
&R::initR("--no-save", "--silent", "--slave");
&R::library("RSPerl");

#== do the normalization by StatisticsR:
#$R = Statistics::R->new();
#$R->startR;

#== load the library "affy":
my $sinkfile = $prefix . "_sinkfile.txt"; 
$rstr = 'sink(file = "' . $sinkfile . '");'
    .'options(echo = FALSE);' 
    .'library(affy);';
#$R->send(qq`$rstr`);   # StatisticsR
&R::eval($rstr);       # RSPerl

#== for Mas5, we will normalize one array at a time:
$array_num = 0;
foreach $cel_file (@celfiles) {
    $array_num++;
    @terms = split /\//, $cel_file;
    $filename = pop @terms;
    $seid = $cel_file_seids{$filename};
    $srid = $seid_srids{$seid};
    $output = $prefix . $seid . "_output.txt";

#    print "R start time = ".TimeStamp(). ", filename = $filename, seid = $seid, srid = $srid, output = $output\n";
    $rstr = 'file <- c("' . $cel_file . '");'
        . 'data <- ReadAffy(filenames = file);'
        . 'data.out <- '. $method . '(data);'
        . 'write.exprs(data.out, file="' . $output . '");';
#    print "$rstr\n";
    #$R->send(qq`$rstr`);   # StatisticsR
    &R::eval($rstr);       # RSPerl
#    print "R end time = ".TimeStamp().", seid = $seid, cel_file = $filename\n";

    open (IN, "<$output") or die "Can not open output file $output\n";
    @lines = <IN>;
    close IN;
#    print "delete output file $output\n";
#    unlink $output or warn "can not unlink $output\n";
    shift @lines;    # get rid of the first line since it is not needed

    $r_pname_intensity = {};
    %pname_intensity = %$r_pname_intensity;
    $r_pnames = [];
    @pnames = @$r_pnames;

    #-- 3. read in the data:
    foreach $line(@lines) {
        chomp $line;
        $line =~ s/\r|\n//g;
        ($pname, $intensity) = split /\t/, $line;
        push @pnames, $pname;
        $pname_intensity{$pname} = $intensity;
    }

    #-- 4 get all rids with the tc from databese and save if hash:
    $r_pname_rids = {};
    %pname_rids = %$r_pname_rids;
    $sql = "select R.reporter_id, CS.probe_name from reporter R, composite_sequence CS where R.composite_sequence_id = CS.composite_sequence_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $rows = $sth->rows();
    if ($rows > 0) {
        while (($rid, $pname) = $sth->fetchrow()) {
            $pname_rids{$pname} = $rid;
        }
    }
    
    #-- 5. create the composite_sequence_ids and reporter_ids if a new 
    #-- probe_name is new:
    $r_unseen_pnames = ();
    @unseen_pnames = @$r_unseen_pnames;
    foreach $pname (@pnames ) {
        if (!(exists $pname_rids{$pname}) ) {
            #$csid = createCompositeSequenceId($dbh, $pname);
            #$rid = createReporterId($dbh, $csid);
            #$pname_rids{$pname} = $rid;
            #print "probe_name = $pname, csid = $csid, rid = $rid\n";
            push @unseen_pnames, $pname;
        }
    }
    if (@unseen_pnames > 0 && @unseen_pnames < 100) {
        $msg = " There are unknown probe ids in CEL file ";
        @terms = split /\//, $cel_file;
        $cel_file = pop @terms;
        $msg .= "$cel_file. The following probe_id(s) need more information.<br>";
        $pname = join ", ", @unseen_pnames;
        $msg .= $pname . "<br>";
        print $msg;
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        print " Ending time = " . TimeStamp() . "\n";
        exit 1;
    } elsif (@unseen_pnames > 0 && @unseen_pnames > 100) {
        $msg = " There are " . @unseen_pnames . " unknown probe ids in CEL file ";
        @terms = split /\//, $cel_file;
        $cel_file = pop @terms;
        $msg .= "$cel_file.<br>";
        print $msg;
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        print " Ending time = " . TimeStamp() . "\n";
        exit 1;
    } 
    
    #-- 6. insert data into "ma_norm"
    my $temp_file = $prefix . $seid . "_temp.sql";
    open (LOG, ">$temp_file") or die "Can not open temp file $temp_file\n";
    
    $mainsql = "";
    foreach $pname (@pnames) {
        $rid = $pname_rids{$pname};
        $intensity = $pname_intensity{$pname};
        if ($intensity > 0) {
            $intensity_l = log($intensity) / log(2);
        }
        $sql1 = "insert into \"ma_norm\" (ma_norm_id, sampling_replicate_id, reporter_id, \"intensity\", sample_extract_id, protocol_type_id, multiple_reporter";
        $sql2 = "values (nextval('ma_norm_id_seq'), $srid, $rid, $intensity, $seid, $ptid, \'N\'";
        if ($intensity_l ne null) {
            $sql1 .= ", intensity_l";
            $sql2 .= ", $intensity_l";
        }
        $sql1 .= ") ";
        $sql2 .= ");\n";
        $mainsql .= $sql1 . $sql2; 
    }
    print LOG "$mainsql";
    close LOG;
    
#        print "insertion start time = ".TimeStamp(). ", filename = $filename\n";
    $sth = $dbh-> prepare($mainsql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#        print "insertion end time = ".TimeStamp(). ", filename = $filename\n";
#        unlink $temp_file or warn "Can not delete temp file $temp_file.\n";

}

#$R->stopR();  # quit R from StatisticsR

$dbh->commit;
$sth->finish;
$dbh->disconnect;
print "Ending time = " . TimeStamp() . "\n";
exit 0;

###########################################################################
# subroutine: createCompositeSequenceId($dbh, $tc):
# create a composite_sequence_id for tc.
#
# parameters in: $dbh -- database handle
#                $probe_name -- the probe name
# parameter out: $csid -- the new composite_sequence_id.
# X. Jing Li: 11/10/2004
###########################################################################
sub createCompositeSequenceId {
    my ($dbh, $probe_name) = @_;
    my ($sth, $rc, $sql, $csid);

    $sth = $dbh->prepare("select nextval('composite_sequence_id_seq')") 
        or die "Can not prepare: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $csid = $sth->fetchrow;

    $sql = "insert into composite_sequence (composite_sequence_id, \"probe_name\")";
    $sql .= " values ($csid, \'$probe_name\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $csid;
}

###########################################################################
# subroutine: createReporterId($dbh, $csid):
# create a reporter_id for composite_sequence_id.
#
# parameters in: $dbh -- database handle
#                $csid -- the composite_sequence_id
# parameter out: $rid -- the new reporter_id.
# X. Jing Li: 11/10/2004
###########################################################################
sub createReporterId {
    my ($dbh, $csid) = @_;
    my ($sth, $rc, $sql, $rid);

    $sth = $dbh->prepare("select nextval('reporter_id_seq')") 
         or die "Can not prepare select nextval: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $rid = $sth->fetchrow;

    $sql = "insert into reporter (reporter_id, composite_sequence_id) values ($rid, $csid)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $rid;
}
 
###########################################################################
# subroutine: getSridDetail($dbh, $srid, $tissue_type_label_id)
# this subroutine get the exp_set_name, experiment_name, time_point, replicate
# for a specific sampling_replicate_id and create a html talbe row for it.
#
# parameters in: $dbh -- database handle
#                $srid -- the sampling_point_id
#                $tissue_type_label_id --
# parameter out: $html_table_row -- the msg in html talge row format.
# X. Jing Li: 12/21/2005
###########################################################################
sub getSridDetail {
    my ($dbh, $srid, $tissue_type_label_id) = @_;
    my ($sql, $sth, $rc, $es_name, $exp_name, $l_name, $replicate, $tissue_type,
        $html_table_row);

    $sql = "select ES.exp_set_name, E.exp_name, L.\"name\", SR.replicate from experiment_set ES, experiment E, sampling_point SP, sampling_replicate SR, label L where E.exp_set_id = ES.exp_set_id AND SP.experiment_id = E.experiment_id AND SP.label_id = L.label_id AND SP.sampling_point_id = SR.sampling_point_id AND SR.sampling_replicate_id = $srid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    ($es_name, $exp_name, $l_name, $replicate) = $sth->fetchrow();
    $sql = "select \"name\" from label where label_id = $tissue_type_label_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $tissue_type = $sth->fetchrow();

    $html_table_row = "<TR><TD><font color=\"#000000\">$es_name</font></TD><TD><font color=\"#000000\">$exp_name</font></TD><TD><font color=\"#000000\">$l_name</font></TD><TD><font color=\"#000000\">$replicate</font></TD><TD><font color=\"#000000\">$tissue_type</font></TD></TR>";
    $sth->finish;
    return $html_table_row;
}

###########################################################################
# subroutine: 
# getSridsDetail($dbh, $srids, $ptid, $extract_id, $tissue_type_label_id)
# this subroutine get the exp_set_name, experiment_name, time_point, replicate
# for a specific sampling_replicate_id and create a html talbe row for it.
#
# parameters in: $dbh -- database handle
#                $srid -- the sampling_point_id
#                $tissue_type_label_id --
# parameter out: $html_table_row -- the msg in html talge row format.
# X. Jing Li: 12/21/2005
###########################################################################
sub getSridsDetail {
    my ($dbh, $srids, $ptid, $extract_id, $tissue_type_label_id) = @_;
    my ($sql, $sth, $rc, $es_name, $exp_name, $l_name, $replicate, $tissue_type,
        $extract_name, $protocol_type_name, $msg);

    $sql = "select \"name\" from extract where extract_id = $extract_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $extract_name = $sth->fetchrow();
    $sql = "select \"name\" from label where label_id = $tissue_type_label_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $tissue_type = $sth->fetchrow();
    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $protocol_type_name = $sth->fetchrow();
    $msg = "For extraction \"$extract_name\", the following samples are already been normalized in DOME for tissue type \"$tissue_type\" and protocol \"$protocol_type_name\".<br><center><table border class=\"result-table\"><TR><th>Exp Set Name</th><th>Experiment Name</th><th>Time Point</th><th>Replicate</th></tr>";

    $sql = "select ES.exp_set_name, E.exp_name, L.\"name\", SR.replicate from experiment_set ES, experiment E, sampling_point SP, sampling_replicate SR, label L where E.exp_set_id = ES.exp_set_id AND SP.experiment_id = E.experiment_id AND SP.label_id = L.label_id AND SR.sampling_point_id = SP.sampling_point_id AND SR.sampling_replicate_id in ($srids)";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    while (($es_name, $exp_name, $l_name, $replicate) = $sth->fetchrow()) {
        $msg .= "<TR><TD><font color=\"#000000\">$es_name</font></TD><TD><font color=\"#000000\">$exp_name</font></TD><TD><font color=\"#000000\">$l_name</font></TD><TD><font color=\"#000000\">$replicate</font></TD></TR>";
    }
    $msg .= "</TABLE></center><br>If you press \"Continue\", the program will skip the ones that already in DOME. Otherwise you must delete the data from DOME before re-normalize the data set.<br>";

    $sth->finish;
    return $msg;
}
