#!/usr/local/bin/perl



use DBI;

use DOMELIB;

#use Statistics::R ;

use R;

use RReferences;

use strict "vars";



#===============================================================================

# This script will be called from Front-end and call R.limma library to do the

# loess normalization for each biological sample of cDNA microarray.

#

# author: X. Jing Li

# date: August 2, 2005

# Reversion:

#===============================================================================



my ($prefix, $srid1, $tissue_type_label_id1, $srid2, $tissue_type_label_id2, $ptid, $extract_id);

my ($database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc,

    $time, $count, $sridDetail, $exp_name, $exp_set_name, $replicate, $lname, 

    $extract_name, $filename, $file_location, $array_id,

    $sample_extract_id, $gpr_file, $gal_file, $array_batch_id, $method, 

    $script_name, $R, $rstr,

    $output_M, $output_A, $genelist, $fid, $rid, $block, $row, $column, $name,

    $tc, %fid_rids, %spot_fids, %rid_names, %fid_tcs, %rid_fids, %multi_rids,

    @contents, $line, $index, %index_keys, %index_gnames, %index_tcs, $value,

    %values_M, %values_A, $value_l, $value_A, $sql2, $mainsql, $multiple, $msg);



($prefix, $srid1, $tissue_type_label_id1, $srid2, $tissue_type_label_id2, $ptid, $extract_id) = @ARGV;

#print "ByLimma: @ARGV\n";



#== start time:

$time = TimeStamp();

print "Starting time = $time\n";



#== 1. connect to the database:

($database, $username, $password, $bnet_DB, $specieDB) = database_login2();

$dbh = DBI->connect($database,$username,$password) or die "Can't connect to $database:$DBI::errstr.";

$dbh->begin_work;

$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";

$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";



#== 2. check the existance of this arrays:

$sql = "select count(*) from \"ma_norm\" where \"protocol_type_id\" = $ptid AND \"sampling_replicate1_id\" = $srid1 and \"sampling_replicate_id\" = $srid2";

#print "$sql\n";

$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

$count = $sth->fetchrow();

if ($count > 0) {

    $msg = getErrorDetailForMaNorm($dbh, $srid1, $srid2, $ptid, $extract_id);

    print "$msg\n";

    $dbh->rollback;

    $sth->finish;

    $dbh->disconnect;

    $time = TimeStamp();

    print "Ending time = $time\n";

    exit 1;

}



#== 3. get the gpr file name:

#$sql = "select F.file_name, FL.location, H.array_id, SE.sample_extract_id from \"file\" F, \"file_location\" FL, \"file_location_link\" FLL, image_analysis IA, raw_data RD, hybridization H, labeled_extract LE, sample_extract SE, \"sample\" S, sampling_replicate SR where FLL.file_id = F.file_id AND FLL.file_location_id = FL.file_location_id AND FLL.file_location_link_id = IA.file_location_link_id AND IA.raw_data_id = RD.raw_data_id AND RD.hybridization_id = H.hybridization_id AND (H.labeled_extract1_id = LE.labeled_extract_id AND LE.sample_extract_id = SE.sample_extract_id AND SE.sample_id = S.sample_id AND S.sampling_replicate_id = $srid1) AND (H.labeled_extract2_id = LE.labeled_extract_id AND LE.sample_extract_id = SE.sample_extract_id AND SE.sample_id = S.sample_id AND S.sampling_replicate_id = $srid2)";

$sql = "select F.\"file_name\", FL.\"location\", H.\"array_id\", SE.\"sample_extract_id\" from \"file\" F, \"file_location\" FL, \"file_location_link\" FLL, \"image_analysis\" IA, \"raw_data\" RD, \"hybridization\" H, \"labeled_extract\" LE, \"sample_extract\" SE, \"sample\" S where FLL.\"file_id\" = F.\"file_id\" AND FLL.\"file_location_id\" = FL.\"file_location_id\" AND FLL.\"file_location_link_id\" = IA.\"file_location_link_id\" AND IA.\"raw_data_id\" = RD.\"raw_data_id\" AND RD.\"hybridization_id\" = H.\"hybridization_id\" AND H.\"labeled_extract2_id\" = LE.\"labeled_extract_id\" AND LE.\"sample_extract_id\" = SE.\"sample_extract_id\" AND SE.\"sample_id\" = S.\"sample_id\" AND S.\"sampling_replicate_id\" = $srid2";

#print "$sql\n";

$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

if ($sth->rows() == 0) {

    $msg = getErrorDetail($dbh, $srid1, $srid2, $extract_id);

    print $msg;

    $dbh->rollback;

    $sth->finish;

    $dbh->disconnect;

    print "Ending time = ". TimeStamp() . "\n";

    exit 1;

} else {

    ($filename, $file_location, $array_id, $sample_extract_id) = $sth->fetchrow();

    $gpr_file = $file_location . $filename;

    $gpr_file =~ s/\(/\\\(/g;

    $gpr_file =~ s/\)/\\\)/g;

}



#== 4. get gal file name:

$sql = "select F.\"file_name\", FL.\"location\", AB.\"array_batch_id\" from \"file\" F, \"file_location\" FL, \"file_location_link\" FLL, \"array\" A, \"array_batch\" AB where A.\"array_id\" = $array_id AND A.\"array_batch_id\" = AB.\"array_batch_id\" AND AB.\"file_location_link_id\" = FLL.\"file_location_link_id\" AND FLL.\"file_id\" = F.\"file_id\" AND FLL.\"file_location_id\" = FL.\"file_location_id\"";

#print "$sql\n";

$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

($filename, $file_location, $array_batch_id) = $sth->fetchrow();

$gal_file = $file_location . $filename;



#== 5. get the R options from database:

$sql = "select \"method_options\" from \"protocol_type\" where \"protocol_type_id\" = $ptid";

$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

$value = $sth->fetchrow();

if ($value =~ /SCRIPT/) {

    ($script_name, $method) = split /\#\#/, $value;

    $script_name =~ s/SCRIPT=|\"//g;

    $method =~ s/OPT=//g;

} else {

    print "This is not a R method. \n";

    $dbh->rollback;

    $sth->finish;

    $dbh->disconnect;

    $time = TimeStamp();

    print "Ending time = $time\n";

    exit 1;

}



#== 6. get the output file names:

$output_M = $prefix . "norm_out_M.txt";

$output_A = $prefix . "norm_out_A.txt";

$genelist = $prefix . "genelist.txt";



#== 7. do the cluster analysis by R:

#&R::initR("--silent");

&R::initR("--no-save", "--silent");

&R::library("RSPerl");

#print "Loaded RSPerl library\n";

#$R = Statistics::R->new();

#$R->startR;



$rstr = "library(limma);\n"

    . "RG <- read.maimages(\"" . $gpr_file . "\", source=\"genepix\");\n"

    . "RG\$genes <- readGAL(galfile=\"" . $gal_file . "\");\n"

    . "RG\$printer <- getLayout(RG\$genes);\n"

    . "MA.norm <- normalizeWithinArrays(RG, $method);\n"

    . "write.table(MA.norm\$M, \"$output_M\", sep=\"\t\");\n"

    . "write.table(MA.norm\$A, \"$output_A\", sep=\"\t\");\n"

    . "write.table(MA.norm\$gene, \"$genelist\", sep=\"\t\");\n";

#print "$rstr\n";

#$R->send(qq`$rstr`);

#$R->stopR();

&R::eval($rstr);



#== 8. get all reporter_ids for each gene in this array_batch:

$sql = "select F.\"feature_id\", F.\"reporter_id\", F.\"block\", F.\"row\", F.\"column\", CS.\"annotation\", CS.\"probe_name\" from \"feature\" F, \"reporter\" R, \"composite_sequence\" CS where F.\"array_batch_id\" = $array_batch_id and F.\"reporter_id\" = R.\"reporter_id\" and R.\"composite_sequence_id\" = CS.\"composite_sequence_id\"";

#print "$sql\n";

$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

while (($fid, $rid, $block, $row, $column, $name, $tc) = $sth->fetchrow()) {

    $fid_rids{$fid} = $rid;

    $spot_fids{($block."_".$row."_".$column)} = $fid;

    $rid_names{$rid} = $name;

    $fid_tcs{$fid} = $tc;

    if (! exists $rid_fids{$rid}) {

        $rid_fids{$rid} = $fid;

    } else {

        $multi_rids{$rid} += 1;

    }

}



#== 9. start to read the output files and parse the contents for database 

#== insertion:

#= 9.1 read the gene_list file:

open (GENE, "<$genelist") or die "Can not open gene list file $genelist.\n";

@contents = <GENE>;

close GENE;



$line = shift @contents;

foreach $line (@contents) {

    chomp $line;

    ($index, $block, $row, $column, $name, $tc) = split ("\t", $line);

    $index =~ s/\"//g;

    $name =~ s/\"//g;

    $tc =~ s/\"//g;

    $block =~ s/\s//g;

    $row =~ s/\s//g;

    $column =~ s/\s//g;

    $index_keys{$index} = $block."_".$row."_".$column;

    $index_gnames{$index} = $name;

    $index_tcs{$index} = $tc;

}

#= 9.2 read the norm_out_M file:

open (IN, "<$output_M") or die "Can not open normalized output data M file $output_M.$_\n";

@contents = <IN>;

close IN;

shift @contents;

foreach $line (@contents) {

    chomp $line;

    ($index, $value) = split /\t/, $line;

    $index =~ s/\"//g;

    $values_M{$index} = $value;

}

#= 9.3 read the norm_out_A file:

open (IN, "<$output_A") or die "Can not open normalized output data A file $output_A.\n";

@contents = <IN>;

close IN;

shift @contents;

foreach $line (@contents) {

    chomp $line;

    ($index, $value) = split /\t/, $line;

    $index =~ s/\"//g;

    $values_A{$index} = $value;

}



#== 10. start create the main sql for data insertion:

$mainsql = "";

$msg = "";

foreach $index (sort {$a <=> $b} keys %index_tcs) {

    $multiple = "false";

    $fid = $spot_fids{$index_keys{$index}};

    $rid = $fid_rids{$fid};

    if (exists $multi_rids{$rid}) {

        $multiple = "true";

    }

    if ($index_tcs{$index} ne $fid_tcs{$fid}) {

        $msg .= "index = $index, fid = $fid, fid_tcs = $fid_tcs{$fid}, index_tc = $index_tcs{$index}, these two does not match.\n";

        next;

    }

    $value_l = $values_M{$index};

    $value = 2**$value_l;

    $value_A = $values_A{$index};

    

    $sql = "insert into \"ma_norm\" (\"ma_norm_id\", \"sampling_replicate1_id\", \"reporter_id\", \"multiple_reporter\", \"protocol_type_id\", \"sample_extract_id\", \"feature_id\", \"sampling_replicate_id\"";

    $sql2 = " values (nextval('ma_norm_id_seq'), $srid1, $rid, \'$multiple\', $ptid, $sample_extract_id, $fid, $srid2";

    if ($value_l ne "NA"){

        $sql .= ", intensity_l, intensity";

        $sql2 .= ", $value_l, $value";

    }

    if ($value_A ne "NA"){

        $sql .= ", \"A\"";

        $sql2 .= ", $value_A";

    }

    $sql .= ")";

    $sql2 .= ");\n";

    $mainsql .= $sql . $sql2;

}

open (TMP, ">/export/home/xingjing/data/dome/temp/temp.txt");

print TMP $mainsql;

close TMP;



if ($msg ne "") {

    print $msg;

    $dbh->rollback;

    $sth->finish;

    $dbh->disconnect;

    unlink $genelist or warn "can not unlink $genelist\n";

    unlink $output_M or warn "can not unlink $output_M\n";

    unlink $output_A or warn "can not unlink $output_A\n";

    $time = TimeStamp();

    print "Ending time = $time\n";

    exit 1;

}

if ($mainsql ne "") {

    #print "time = " . TimeStamp() . "\n";

    #print "inserting data to \"ma_norm\".......\n";

    $sth = $dbh->prepare($mainsql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    if ($rc == 1) {

        $dbh->commit;

        $sth->finish;

        $dbh->disconnect;

        unlink $genelist or warn "can not unlink $genelist\n";

        unlink $output_M or warn "can not unlink $output_M\n";

        unlink $output_A or warn "can not unlink $output_A\n";

        print "Normalization data has been insert into database SUCCESSFULLY.\n";

        print "Ending time = " . TimeStamp() . "\n";

        exit 0;

    } else {

        $dbh->rollack;

        $sth->finish;

        $dbh->disconnect;

        unlink $genelist or warn "can not unlink $genelist\n";

        unlink $output_M or warn "can not unlink $output_M\n";

        unlink $output_A or warn "can not unlink $output_A\n";

        print "Normalization data has ERROR. No data has been inserted into database.\n";

        print "Ending time = " . TimeStamp() . "\n";

        exit 1;

    }

}



###########################################################################

# subroutine: getErrorDetailForMaNorm($dbh, $srid1, $srid2, $ptid, $extract_id)

#

# parameters in: $dbh --- the database handle,

#                $srid --- the sampling_replicate_id,

#                $ptid --- the statistical protocol_type_id,

#                $extract_id ---

# parameter out: $msg --- the message for this set of parameters.

#

# X. Jing Li, 8/18/2005

###########################################################################

sub getErrorDetailForMaNorm {

    my ($dbh, $srid1, $srid2, $ptid, $extract_id) = @_;

    my ($sth, $rc, $sql, $msg, $exp_name, $exp_set_name, $replicate, $lname, $pt_name);

    $sql = "select E.exp_name, ES.exp_set_name, SR.replicate, L.\"name\" from experiment E, experiment_set ES, \"sample\" S, sample_extract SE, label L, sampling_point SP, sampling_replicate SR where SE.extract_id = $extract_id and SE.sample_id = S.sample_id and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_replicate_id = $srid2 and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    ($exp_name, $exp_set_name, $replicate, $lname) = $sth->fetchrow();

    

    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $pt_name = $sth->fetchrow();

    $msg = "The protocol_type \"$pt_name\" for this sample has been calulated before. The experiment_set name is \"$exp_set_name\", the experiment name is \"$exp_name\", the time point is \"$lname\", the biological replicate is \"$replicate\". ";

    $sth->finish;

    return $msg;

}



###########################################################################

# subroutine: getErrorDetail($dbh, $srid1, $srid2, $extract_id)

#

# parameters in: $dbh --- the database handle,

#                $srid --- the sampling_replicate_id,

#                $extract_id ---

# parameter out: $exp_name -- the name of the experiment'

#                $exp_set_name -- experiment_set name,

#                $replicate -- the number of the biological replicate,

#                $lname -- the time point label name,

#                $extract_name -- the full name of the extraction.

#

# X. Jing Li, 9/6/2005

###########################################################################

sub getErrorDetail {

    my ($dbh, $srid1, $srid2, $extract_id) = @_;

    my ($sth, $rc, $sql, $exp_name, $exp_set_name, $replicate, $lname, $extract_name);



    #== get extract_name:

    $sql = "select \"name\" from extract where extract_id = $extract_id";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $extract_name = $sth->fetchrow();



    #== get channel info:

    $sql = "select E.exp_name, ES.exp_set_name, SR.replicate, L.\"name\" from experiment E, experiment_set ES, label L, sampling_point SP, sampling_replicate SR where SR.sampling_replicate_id = $srid1 and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    ($exp_name, $exp_set_name, $replicate, $lname) = $sth->fetchrow();

    $msg = "There is no original data from extract \"$extract_name\" for Channel 1 with experiment \"$exp_name\" of experiment_set \"$exp_set_name\" at time \"$lname\" of replicate number \"$replicate\". ";



    $sql = "select E.exp_name, ES.exp_set_name, SR.replicate, L.\"name\" from experiment E, experiment_set ES, label L, sampling_point SP, sampling_replicate SR where SR.sampling_replicate_id = $srid2 and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    ($exp_name, $exp_set_name, $replicate, $lname) = $sth->fetchrow();

    $msg .= " Channel 2 with experiment \"$exp_name\" of experiment_set \"$exp_set_name\" at time \"$lname\" of replicate number \"$replicate\". ";



    $msg .= "Check if you had uploaded the original data file to database. \n";

    

    $sth->finish;

    #print "In sub: " . $msg;

    return $msg;

}



