<?php
/**
* This script is a Front-End for normalizing Microarray data already added 
* in the DOME database.
* Date: 3 May 2005
* @author Aejaaz Kamal <aekamal@vt.edu>
*/


include_once("common.php");

$currPageName = 'NORMALIZE_MICROARRAY';
?>

<?php
$title = "Normalize Microarray Data";
include("header.php");
if (!(strtoupper($_SESSION['role']) == 'MANAGER' OR strtoupper($_SESSION['role']) == 'EDITOR'))
{	printError("You are not authorized to view this page. Your role level should be 'editor' or 'manager' to view this page.");	exit();}
?>

<p class="help-tips">Please provide information about which data sample you want to normalize and choose a protocol to be used for normalization. The data would be stored in the database after normalization to assist in further quering.</p>

<?php
if (isset($_REQUEST['technique_id']))
{
	$currPageNameUpper = strtoupper($currPageName);
	$_SESSION[$currPageNameUpper]['technique_id'] = $_REQUEST['technique_id'];
}

if ($_REQUEST['submit'])
{	processForm();	}
if ($errMsgs)
{  
	foreach ($errMsgs as $msg)
	{	printError($msg);	}
}
fillForm();
include_once("footer.php");
?>

<?php
//Functions
function fillForm()
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	$self = $_SERVER[PHP_SELF];
	print "<form name=\"init_form\" method=\"post\" action=\"$self\">\n";
	
	printMetaDataSelections("", $_SESSION[$currPageNameUpper]);
	
	//Print Normalization Method
	$sql = "SELECT \"protocol_type_id\", \"protocol_type_name\" FROM \"protocol_type\" 
			WHERE \"technique_id\" = '{$_SESSION[$currPageNameUpper]['technique_id']}' 
			AND UPPER(\"method\") = 'STATISTICAL' ORDER BY UPPER(\"protocol_type_name\")";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Protocol Types. " . $sth->getMessage());	return;	}
	print "\n<p><label>Normalization Method: </label>";
	print "<select name=\"protocol_type_id\">";
	
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['protocol_type_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
	
	
	//Print Extract.
	$sql = "SELECT \"extract_id\", \"name\" FROM \"extract\" WHERE \"technique_id\" = '{$_SESSION[$currPageNameUpper]['technique_id']}' ORDER BY UPPER(\"name\")";
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Extracts. " . $sth->getMessage());	return;	}
	print "<p><label>Extraction Method: </label>";
	print "<select name=\"extract_id\">";
	print "\n\t<option value=\"0\">--Select One--</option>";
	while ($row = $sth->fetchRow()) 
	{
		print "\n\t<option value=\"$row[0]\"";
		if ($_SESSION[$currPageNameUpper]['extract_id'] == $row[0])
		{	print " selected";	}
		print ">$row[1]</option>";
	}
	print "\n</select></p>";
	
	print "<input type=\"submit\" name=\"submit\" value=\"Normalize\" />";
	print "\n</form>";

}

/**============================
* Request Functions
*============================*/
function processForm()
{
	global $errMsgs, $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	
	$_SESSION[$currPageNameUpper]['experiment_set_id'] = $_REQUEST['experiment_set_id'];
	$_SESSION[$currPageNameUpper]['exp_name'] = $_REQUEST['exp_name'];
	$_SESSION[$currPageNameUpper]['sampling_point_label_id'] = $_REQUEST['sampling_point_label_id'];
	$_SESSION[$currPageNameUpper]['biological_replicate'] = $_REQUEST['biological_replicate'];
	$_SESSION[$currPageNameUpper]['tissue_type_label_id'] = $_REQUEST['tissue_type_label_id'];
	$_SESSION[$currPageNameUpper]['protocol_type_id'] = $_REQUEST['protocol_type_id'];
	$_SESSION[$currPageNameUpper]['extract_id'] = $_REQUEST['extract_id'];
	
	if (!$_SESSION[$currPageNameUpper]['technique_id'])
	{	$errMsgs[] = "technique_id not set";	}
	
	if (!$_SESSION[$currPageNameUpper]['experiment_set_id'])
	{	$errMsgs[] = "Please Select an Experiment Set.";	}
	
	if (!$_SESSION[$currPageNameUpper]['exp_name'])
	{	$errMsgs[] = "Please Select an Experiment.";	}
	
	if (!$_SESSION[$currPageNameUpper]['sampling_point_label_id'])
	{	$errMsgs[] = "Please Select a Sampling Point";	}
	
	if (!$_SESSION[$currPageNameUpper]['tissue_type_label_id'])
	{	$errMsgs[] = "Please Select a Tissue Type";	}
	
	if (!$_SESSION[$currPageNameUpper]['biological_replicate'])
	{	$errMsgs[] = "Please Select the biological replicate number.";	}
	
	if (!$_SESSION[$currPageNameUpper]['protocol_type_id'])
	{	$errMsgs[] = "Please Select a Normalization Method.";	}
	
	if (!$_SESSION[$currPageNameUpper]['extract_id'])
	{	$errMsgs[] = "Please Select extraction method.";	}
	
	if ($errMsgs)
	{	return 0;	}
	
	//Now find out sampling_replicate_id from the given info.
	$sql = "SELECT sr.\"sampling_replicate_id\" FROM \"sampling_replicate\" sr
					, \"sampling_point\" sp
					, \"experiment\" e
				WHERE e.\"exp_set_id\" = '{$_SESSION[$currPageNameUpper]['experiment_set_id']}'
				AND e.\"exp_name\" = '{$_SESSION[$currPageNameUpper]['exp_name']}'
				AND sp.\"experiment_id\" = e.\"experiment_id\"
				AND sp.\"label_id\" = '{$_SESSION[$currPageNameUpper]['sampling_point_label_id']}'
				AND sr.\"sampling_point_id\" = sp.sampling_point_id
				AND sr.\"replicate\" = '{$_SESSION[$currPageNameUpper]['biological_replicate']}'";
	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	
		$errMsgs[] = "Error retrieving Sampling Replicate. " . $sth->getMessage();	
		return 0;
	}
	
	if ($sth->numrows() > 0)
	{
		$row = $sth->fetchRow();
		$_SESSION[$currPageNameUpper]['sampling_replicate_id'] = $row[0];
		//print "Set sampling_replicate_id = {$_SESSION[$currPageNameUpper]['sampling_replicate_id']} ($n)";
	}
	else
	{	$errMsgs[] = "Sampling Replicate not found in the database for the information provided";	}
	
	
	$perlScriptFile = "normalizeCDnaByLimma.pl";
	$prefix = $_SESSION['BROWSER_TMP_FULL'] . session_id();
	$sampling_replicate_id = $_SESSION[$currPageNameUpper]['sampling_replicate_id'];
	$protocol_type_id = $_SESSION[$currPageNameUpper]['protocol_type_id'];
	$extract_id = $_SESSION[$currPageNameUpper]['extract_id'];
	$tissue_type_label_id = $_SESSION[$currPageNameUpper]['tissue_type_label_id'];
	
	$com = "export PATH; PATH=\$PATH:{$_SESSION['R_DIR']}; export R_HOME; R_HOME={$_SESSION['R_HOME']}; {$_SESSION['PERL_DIR']}perl $perlScriptFile \"$prefix\" $sampling_replicate_id $protocol_type_id $extract_id $tissue_type_label_id 2>&1;";
	$retString = exec($com, $retLines, $retValue);
	
	if ($retLines)
  	{
		foreach ($retLines as $retLine)
		{	$msg .= "$retLine <br />";	}
  	}
	if ($retValue == 0)	//success
  	{	
		if (!$msg)
	  	{	$msg = "System command to '$perlScriptFile' returned a successful value.";	}
	  	
	  	$_SESSION['Message'][] = $msg;
		//redirect
		print "<meta http-equiv=\"refresh\" content=\"0; URL=intro.php?stepno=1\">";
		exit();
	}
	else
	{	
	  	if (!$msg)
	  	{	$msg = "System command to '$perlScriptFile' failed. <br />Command was \"$com\"\nReturn: $retString RetValue = $retValue";	}
	  	$errMsgs[] = $msg;
	}
	
  	if ($errMsgs)
	{	return 0;	}
	return 1;
}

?>
