#!/usr/local/bin/perl

use GD;
use Math::Trig;
use POSIX;
use strict "vars";

#===============================================================================
# This script will color the map components by the intensity of the elements in
# the data file.
# rc file format:
# TYPENAME = AAAAA
# COLOR-AAAAA = RES UPPER LOWER MAXR MAXG MAXB MINR MING MINB
# TYPENAME = BBBBB
# COLOR-BBBBB = RES UPPER LOWER MAXR MAXG MAXB MINR MING MINB
# TYPENAME = CCCCC
# COLOR-CCCCC = RES UPPER LOWER MAXR MAXG MAXB MINR MING MINB
# 
# For experimental data file, we have the file format as 
# TYPE\tNAME\tVALUE\n
# AAAAA\tname1\tvalue1\n
# BBBBB\tname2\tvalue2\n
# CCCCC\tname3\tvalue3\n
# ...
#
# For the image_info file, we have format as:
# TYPE\tNAME\tSHAPE\tCOORD\n
# where TYPE is the same string as in TYPENAME in .rc file; NAME is the same as
# in experimental data file and SHAPEs are elipstic CIRC, rectangles RECT and
# polygons POLY for now. The coordinates will be X_Y_W_H for CIRC and RECT, 
# X1,Y1_X2,Y2_..._Xn,Yn for POLY.
#
# Author: X. Jing Li
# Date: Nov. 8, 2006
# Reversions: 
#===============================================================================

my ($input_file, $output_file, $rc_file, $image_file, $image_info) = @ARGV;
#open (TMP, ">./tmp-plots/temp.arg") || die "Can not open temp file. $!";
#print TMP "perl_brome.pl: input_file = $input_file\noutput_file = $output_file\nrc_file = $rc_file\nimage_file = $image_file\nimage_info = $image_info\n";


my (@lines, $line, $r_uppers, %uppers, $r_lowers, %lowers, $temp, $type, @types, $info, $resolution, %resolutions, $upper, $lower, $maxR, $maxG, $maxB, $minR, $minG, $minB, %maxRs, %maxGs, %maxBs, %minRs, %minGs, %minBs, $ref_AoA, %colors, @array, %bounds, $name, $value, $ref_array, %type_data_HoH, $shape, $coord, %name_types, %name_shapes, %name_coords, $ref_coords, $image_title,
    $legend_panel_w, $png_file, $orig_image, $orig_w, $orig_h, $orig_white, $orig_black, $new_image, $start_x, $start_y, $legend_box_w, $legend_box_h, $black, $white, $red, $type_text, $ref_color_array, @color_array, $i, $color, $r_res, $legend_text, $max, $min, $new_w, $new_h, $x, $y, $ref_hash, $name_type, $param, $circ_x, $circ_y, $circ_w, $circ_h, $rect_x, $rect_y, $rect_w, $rect_h, @points, $pt, $poly);

$legend_panel_w = 200;

#== 1. open the rc file to create data structure for type of elements and its 
#== resolution array:
open(RC, "<$rc_file") or die "Can not open the rc file $rc_file. $!";
@lines = <RC>;
close RC;

$r_uppers = ();
%uppers = %$r_uppers;
$r_lowers = ();
%lowers = %$r_lowers;
foreach $line (@lines) {
    chomp $line;
    if ($line =~ /^TYPENAME/) {
        ($temp, $type) = split /\=/, $line;
        push @types, $type;
    } elsif ($line =~ /^COLOR/) {
        ($type, $info) = split /\=/, $line;
        $type =~ s/COLOR\-//g;
        ($resolution, $upper, $lower, $maxR, $maxG, $maxB, $minR, $minG, $minB) = split /\ /, $info;
        $resolutions{$type} = $resolution;
        $uppers{$type} = $upper;
        $lowers{$type} = $lower;
        $maxRs{$type} = $maxR;
        $maxGs{$type} = $maxG;
        $maxBs{$type} = $maxB;
        $minRs{$type} = $minR;
        $minGs{$type} = $minG;
        $minBs{$type} = $minB;
    }
}

#== 2. create array to hold the discritezed colors
foreach $type (@types) {
    $resolution = $resolutions{$type};
    $ref_AoA = getColorArray($resolution, $maxRs{$type}, $maxGs{$type}, $maxBs{$type}, $minRs{$type}, $minGs{$type}, $minBs{$type});
    $colors{$type} = $ref_AoA;
    $upper = $uppers{$type};
    $lower = $lowers{$type};
    $ref_array = createBoundArray($resolution, $upper, $lower);
    $bounds{$type} = $ref_array;
}


#== 3. get experimental data from input_file:
#$input_file =~ s/\ /\\\ /g;
open (IN, "<$input_file") or die "Can not open input data file \"$input_file\". $!";
@lines = <IN>;
close IN;

my @temps = split /BROMETITLE/, $input_file;
$image_title = pop @temps;
$image_title =~ s/\.\w+//g;
#print TMP "image_title = $image_title\n";

foreach $line (@lines) {
    chomp $line;
    $line =~ s/\
//g;
    ($type, $name, $value) = split /\t/, $line;
    $name = uc $name;
    if (!exists $type_data_HoH{$type}) {
        $type_data_HoH{$type} = createHashRef($type);
    }
    $type_data_HoH{$type}->{$name} = $value;
}
### check the data in hash:
#my $msg = "";
#foreach $type (keys %type_data_HoH) {
#    $ref_hash = $type_data_HoH{$type};
#    $msg .= "type = $type, ref_hash = " . $ref_hash . "\n";
#    foreach $name (sort keys %$ref_hash) {
#        $msg .= "line 117: name = $name, value = " . $ref_hash->{$name}. "\n";
#   }
#}
#print TMP $msg;

#== 4. get the image info:
open (IN, "<$image_info") || die "Can not open image_info file $image_info. $!";
@lines = <IN>;
close IN;
foreach $line (@lines) {
    chomp $line;
    ($type, $name, $shape, $coord) = split /\t/, $line;
    $name = uc $name;
    $name_types{$name} = $type;
    $name_shapes{$name} = $shape;
    if (exists $name_coords{$name}) {
        $ref_coords = $name_coords{$name};
        $name_coords{$name} = updateArrayRef($ref_coords, $coord);
    } else {
        $name_coords{$name} = createArrayRef($coord);
    }
}
### check data in hash:
foreach $name (keys %name_types) {
    $type = $name_types{$name};
    $shape = $name_shapes{$name};
    $coord = $name_coords{$name};
#    print TMP "line 144: name = $name, type = $type, shape = $shape, coord_num = " . @$coord . ", ";
    @array = @$coord;
#    print TMP "coord = @array\n";
}

#== 5. load the image:
$png_file = $image_file;
if ($png_file =~ /png$/i) {
    $orig_image = newFromPng GD::Image($png_file) || die "Can not open the original file $png_file. $!";
} elsif ($png_file =~ /gif$/i) {
    $orig_image = newFromGif GD::Image($png_file) || die "Can not open the original file $png_file. $!";
} elsif ($png_file =~ /jpg$|jpeg$/i) {
    $orig_image = newFromJpeg GD::Image($png_file) || die "Can not open the original file $png_file. $!";
} elsif ($png_file =~ /xbm$/i) {
    open (XBM, "$png_file") || die "Can not open the original file $png_file. $!";
    $orig_image = newFromCbm GD::Image(\*XBM) || die;
} 
$orig_white = $orig_image->colorClosest(255, 255, 255);
$orig_black = $orig_image->colorClosest(0, 0, 0);
$orig_image->transparent($orig_white);
($orig_w, $orig_h) = $orig_image->getBounds();

#== 5.1 using the original image size as guide and create a new one with the same
#== height and $legend_w for the legend:
$start_x = 10;
$start_y = 10;
$legend_box_w = 30;
$legend_box_h = 15;
$new_w = $orig_w + $legend_panel_w;
$new_image = new GD::Image($new_w, $orig_h) || die "Can not create new image. $!";
$black = $new_image->colorAllocate(0, 0, 0);
$white = $new_image->colorAllocate(255, 255, 255);
$red = $new_image->colorAllocate(255, 0, 0);

#== 5.2 set the new_image backgroud as white:
$new_image->rectangle(0, 0, $new_w, $orig_h, $black);
$new_image->filledRectangle(0, 0, $new_w, $orig_h, $white);

#== 5.3 start painting the new_image:
#-- draw image_title:
$new_image->string(gdGiantFont, $start_x, $start_y, $image_title, $red);
$start_y += 2*$legend_box_h;

foreach $type (@types) {
    if (!exists $type_data_HoH{$type}) {
        next;
    }
    $type_text = "$type";
    $new_image->string(gdSmallFont, $start_x, $start_y, $type_text, $black);
    $ref_color_array = $colors{$type};
    @color_array = @$ref_color_array;
    $resolution = $resolutions{$type};

    #-- deal one resolution at a time:
    for($i = 0; $i<$resolution; $i++) {
        $x = $start_x;
        $y = $start_y + $legend_box_h * ($i+1);
        if ($i == 0) {
            $lower =  $bounds{$type}->[($resolution-1)-1];
            $upper = 10000000;
            $legend_text = "x > " . $lower;
        } elsif ($i == $resolution-1) {
            $lower = -10000000;
            $upper = $bounds{$type}->[0];
            $legend_text = "x <= "  . $upper;
        } else {
            $lower = $bounds{$type}->[($resolution-1) - ($i+1)];
            $upper = $bounds{$type}->[($resolution-1) - $i];
            $legend_text = $lower . " < x <= " . $upper; 
        }
        #-- get color
        $color = $new_image->colorAllocate($color_array[0]->[$i], $color_array[1]->[$i], $color_array[2]->[$i]);

        #-- paint legend box:
        $new_image->rectangle($x, $y, $x+$legend_box_w, $y+$legend_box_h, $black);
        $new_image->filledRectangle($x+1, $y+1, $x+$legend_box_w-1, $y+$legend_box_h-1, $color);
        $x += $legend_box_w+10;
        $new_image->string(gdSmallFont, $x, $y, $legend_text, $black);
        
        #-- paint the spots on the new_image:
        $ref_hash = $type_data_HoH{$type};
        foreach $name (keys %name_types) {
            $name_type = $name_types{$name};
            #print TMP "line 218: name = $name, type = $type, name_type = $name_type\n";
            
            if ($type eq $name_type) {
                $value = $ref_hash->{$name};
                #print TMP "line 222: name = $name, type = $type, value = $value\n";
                if ($value eq "") {
                    next;
                } 
                if ($value > $lower && $value <= $upper) {
                    $shape = $name_shapes{$name};
                    $coord = $name_coords{$name};
                    @array = @$coord;
                    foreach $param (@array) {
                        if ($shape eq "circ") {
                            ($circ_x, $circ_y, $circ_w, $circ_h) = split /\_/, $param;
                            #print TMP "line 236: name = $name, type = $type, value = $value, lower = $lower, upper = $upper, coord = $circ_x, $circ_y, $circ_w, $circ_h\n";
                            $new_image->filledEllipse($legend_panel_w+$circ_x, $circ_y, $circ_w, $circ_h, $color);
                        } elsif ($shape eq "rect") {
                            ($rect_x, $rect_y, $rect_w, $rect_h) = split /\_/, $param;
                            #print TMP "line 240: name = $name, type = $type, value = $value, lower = $lower, upper = $upper, coord = $rect_x, $rect_y, $rect_w, $rect_h\n";
                            #$new_image->rectangle($legend_panel_w+$rect_x, $rect_y, $legend_panel_w + $rect_x + $rect_w, $rect_y + $rect_h, $black);
                            $new_image->filledRectangle($legend_panel_w+$rect_x + 1, $rect_y + 1, $legend_panel_w + $rect_x + $rect_w - 1, $rect_y + $rect_h - 1, $color);
                        } elsif ($shape eq "poly") {
                            #print TMP "name = $name, type = $type, coordi = ";
                            @points = split /\_/, $param;
                            $poly = new GD::Polygon;
                            foreach $pt (@points) {
                                ($x, $y) = split /\,/, $pt;
                                #print "$x,$y ";
                                $poly->addPt($legend_panel_w+$x, $y);
                            }
                            #print "\n";
                            $new_image->filledPolygon($poly, $color);
                        }
                    }
                }
            }
        }
    }
    $start_y += $legend_box_h * ($resolution+1) + $legend_box_h;
}

#== merge orig_image and new_image then save to output_file:
$new_image->copyMerge($orig_image, $legend_panel_w, 0, 0, 0, $orig_w, $orig_h, 100);
open (OUT, ">$output_file") || die "can not open output file $output_file. $!";
print OUT $new_image->gif;
close OUT;

#close TMP;
exit 0;


###########################################################################
# subroutine: getColorArray($res, $maxR, $maxG, $maxB, $minR, $minG, $minB)
#
# parameters in: $res -- the resolution of the type
#                $maxR -- red component of the max color
#                $maxG -- green component of the max color
#                $maxB -- blue component of the max color
#                $minR -- red component of the min color
#                $minG -- green component of the min color
#                $minB -- blue component of the min color
# parameter out: $r_array -- the reference of array of array
#
# X. Jing Li, 11/2006
###########################################################################
sub getColorArray {
    my ($res, $maxR, $maxG, $maxB, $minR, $minG, $minB) = @_;
    my ($ref_red, $ref_green, $ref_blue, @array);
    #print "res = $res, maxR = $maxR, maxG = $maxG, maxB = $maxB, minR = $minR, minG = $minG, minB = $minB\n";
    $ref_red = createColorComponentArray($res, $maxR, $minR);
    $ref_green = createColorComponentArray($res, $maxG, $minG);
    $ref_blue = createColorComponentArray($res, $maxB, $minB);
    @array = ($ref_red, $ref_green, $ref_blue);
    return \@array;
}

###########################################################################
# subroutine: createHashRef($key, $value)
#
# parameters in: $key -- the key for the first pair of key-value of the hash
#                $value -- the value for the first pair of key-value of the hash
# parameter out: $r_hash -- the new reference of te hash
#
# X. Jing Li, 9/22/2006
###########################################################################
sub createHashRef {
    my ($key, $value) = @_;

    my $r_hash = ();
    return $r_hash;
}

###########################################################################
# subroutine: createColorComponentArray($res, $maxC, $minC)
#
# parameters in: $res -- the resolution 
# parameter out: $r_array -- the new reference of te array
#
# X. Jing Li, 9/22/2006
###########################################################################
sub createColorComponentArray {
    my ($res, $maxC, $minC) = @_;
    my ($comp, $color, @comps);
    #print "createColorComponentArray: res = $res, max = $maxC, min = $minC\n";

    $color = $maxC - $minC;
    if ($color > 0) {
        for($i=0; $i<$res; $i++) {
            $comp = $color/($res - 1)*($res-$i-1) + $minC;
            push @comps, $comp;
        }
    } elsif ($color < 0) {
        for($i=0; $i<$res; $i++) {
            $comp = abs($color) / ($res-1) * $i + $maxC;
            push @comps, $comp;
        }
    } else {
        for($i=0; $i<$res; $i++) {
            push @comps, $maxC;
        }
    }
    #print "comps = @comps\n";
    return \@comps;
}
            

###########################################################################
# subroutine: createArrayRef($ref_array)
#
# parameters in: $value -- the value to start an array
# parameter out: $r_array -- the new reference of te array
#
# X. Jing Li, 11/2006
###########################################################################
sub createArrayRef {
    my ($value) = @_;
    my @array = ($value);
    return \@array;
}

###########################################################################
# subroutine: updateArrayRef($r_array, $v)
#
# parameters in: $r_array -- the reference of an array
#                $v -- the value needed to be added into the array
# parameter out: $r_array -- the new reference of te array
#
# X. Jing Li, 11/2006
###########################################################################
sub updateArrayRef {
    my ($r_array, $v) = @_;
    my @array = @$r_array;
    push @array, $v;
    return \@array;
}

###########################################################################
# subroutine: createBoundArray($res, $upper, $lower)
#
# parameters in: $res -- the resolution of the data type
#                $upper -- the upper bound
#                $lower -- the lower bound
# parameter out: $r_array -- the new reference of te array
#
# X. Jing Li, 11/8/2006
###########################################################################
sub createBoundArray {
    my ($res, $upper, $lower) = @_;
    #print "res = $res, upper = $upper, lower = $lower\n";
    my (@array, $bound);

    for ($i=0; $i<$res-1; $i++) {
        $bound = ($upper - $lower) / ($res - 2) * $i + $lower;
        if (length ($bound) > 5) {
            $bound =~ /(\W?\d+\.\d{2})(\d{1})/;
            if ($2 >= 5){
                $bound = $1 + 0.01;
            } else {
                $bound = $1;
            }
        }
        push @array, $bound;
    }
    #print "array = @array\n";
    return \@array;
}


