<?php
/**
* This script helps the curator to update B-Net with 
* the updates sent in a file using the sendUpdates.php script.
* All Updates are stored in a hash.
* Every individual table belongs to a group.
* Groups are done in order to be able to retrieve entities
* while displaying updates on the Front-End.
* Hence this hash containing updates is referenced as follows
* $updatesHash[TABLE_GROUP][UNIQUE_KEY_FOR_THIS RECORD][TABLE_NAME]
* The location referenced above contains the new record (as an array
* mostly of fields other than the UNIQUE_KEY) when there is
* an Insert or Update. In case of a Delete it contains the value 'D'.
* Note that there can be an additional unique key for the Hash
* $updatesHash after the three keys described above in the cases
* where more than one update can be done on the same table for a particular
* unique group.
* 
* Date: 25 Oct 2004
* @author Aejaaz Kamal <aekamal@vt.edu>
*/

include_once("common.php");
include_once("workFlow.php");
//Parse Get or Post step number and set the current step number.
setStepNumber();

//Sets Environment Variable
setVariables($_SESSION['STEP_NUMBER']);
doControlBarRedirection(1);  
$title = "DOME : Process Updates";
include("header.php");
?>


<?php
if ($errMsgs)
{
	foreach ($errMsgs as $msg)
	{
		printError($msg);
	}
}

if (strtoupper($_SESSION['role']) == 'EDITOR' OR strtoupper($_SESSION['role']) == 'MANAGER')
{	fillForm();	}
else
{	printError("You are not authorized to view this page. Your role level should be 'editor' or 'manager' to view this page.");	}

include ("footer.php");
?>

<?php
//Functions
function fillForm()
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	$self = $_SERVER[PHP_SELF];
	print "<form name=\"init_form\" method=\"post\" action=\"$self\" enctype=\"multipart/form-data\">\n";
	
	//This MAX_SIZE is an advisory to the browser. (So that the user does not find
	// that his file is rejected by the php on the server after uploading it).
	print '<input type="hidden" name="MAX_FILE_SIZE" value="30000000" />';
	print "<p><label>Updates File: </label>";
	print '<input name="updatesFile" size="48" type="file" value="Select Updates File using browse"/>';
	print "</p>";
	printControlBar($_SESSION['STEP_NUMBER']);
	print "</form>";
}


/**============================
* Request Functions
*============================*/
function processForm($currPageName)
{
	global $errMsgs, $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	if ($_FILES['updatesFile']['name'] == "")
	{
		$errMsgs[] = "Please select the file to upload using the browse button";		
	}
	elseif (preg_match('/\'/', $_FILES['updatesFile']['name']))
	{	$errMsgs[] = "Name of the updates file contains an invalid character.";	}
	else
	{	
		$uploaddir = $_SESSION['BROWSER_TMP_RELATIVE'];
		$uploadfile = $uploaddir . $_FILES['updatesFile']['name'];
		if (move_uploaded_file($_FILES['updatesFile']['tmp_name'], $uploadfile)) 
		{
		   //File is valid, and was successfully uploaded
		   $_SESSION[$currPageNameUpper]['updatesFile'] = $uploadfile;
		   system("chmod 644 $uploadfile");
		   include_once($uploadfile);
		   if (!$updatesHash)
		   {	
			   $errMsgs[] = "Update file contains no data.";	
			   return 0;
		   }
		   //Sort Identifiers in each of the groups.
		   $groups = array_keys($updatesHash);
		   foreach ($groups as $group)
		   {
				ksort($updatesHash[$group]);   
		   }
		   
		   //This is the order in which the groups have to be processed
		   $_SESSION[$currPageNameUpper]['GROUPS'] = array('REFERENCE'
		   											, 'EXTERNAL_DATABASE'
		   											, 'EVIDENCE_CODE'
		   											, 'LOCATION'
		   											, 'EVENT_TYPE'
		   											, 'EC'
		   											, 'SPECIES'
		   											, 'COMPOUND'
		   											, 'GENE'
		   											, 'PROTEIN'
		   											, 'MRNA'
		   											, 'COMPLEX'
		   											, 'EVENT'
		   											, 'ACTIVITY'
		   											, 'PATHWAY'
		   											);
		   $_SESSION[$currPageNameUpper]['updatesHash'] = $updatesHash;
		   return 1;
		} 
		else 
		{
			$errMsgs[] = "Could not move the uploaded file to the Destination on the server";
		}
	}
}

?>
