<?php
/**
* This script is a Front-End for producing platform
* file to submit to GEO.
* Date: 27 Sep 2006
* @author Aejaaz Kamal <aekamal@vt.edu>
*/

include_once("common.php");

$currPageName = 'PLATFORM_FILE';
$currPageNameUpper = strtoupper($currPageName);

$title = "Platform File";
include("header.php");
if (strtoupper($_SESSION['role']) != 'MANAGER')
{	printError("You are not authorized to view this page. Your role level should be 'manager' to view this page.");	exit();}

fillForm();
include_once("footer.php");
?>

<?php
//Functions
function fillForm()
{
	global $errMsgs;
	$sid = session_id();
	$platformFileRelative = "{$_SESSION['BROWSER_TMP_RELATIVE']}platformFile-{$sid}.txt";
	$platformFileFull = "{$_SESSION['BROWSER_TMP_FULL']}platformFile-{$sid}.txt";
	
	
	//Get Platform Data from array_batch
	$sql = "SELECT \"array_batch_id\", \"platform_title\", \"platform_distribution\", 
		\"platform_technology\", \"platform_organism\", \"platform_manufacturer\", \"platform_manufacture_protocol\",
		\"platform_catalog_number\", \"platform_web_link\", \"platform_support\", \"platform_coating\",
		\"platform_description\", \"platform_contributor\", \"platform_pubmed_id\", \"platform_geo_accession\"
	FROM \"array_batch\" ORDER BY UPPER(\"platform_title\")";
	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Data from array_batch. " . $sth->getMessage());	return false;	}
	if ($sth->numRows() == 0)
	{	
		$errMsgs[] = "There are no defined platforms.";
		return false;
	}
	
	while ($row = $sth->fetchRow()) 
	{
		$platform_title = $row[1];
		fillPlatform($platforms, $platform_title, 'platform_distribution', $row[2], true);
		fillPlatform($platforms, $platform_title, 'platform_technology', $row[3], true);
		fillPlatformArray($platforms, $platform_title, 'platform_organism', $row[4], true);
		fillPlatform($platforms, $platform_title, 'platform_manufacturer', $row[5], true);
		fillPlatform($platforms, $platform_title, 'platform_manufacture_protocol', $row[6], true);
		fillPlatformArray($platforms, $platform_title, 'platform_catalog_number', $row[7], false);
		fillPlatformArray($platforms, $platform_title, 'platform_web_link', $row[8], false);
		fillPlatformArray($platforms, $platform_title, 'platform_support', $row[9], false);
		fillPlatformArray($platforms, $platform_title, 'platform_coating', $row[10], false);
		fillPlatformArray($platforms, $platform_title, 'platform_description', $row[11], false);
		fillPlatformArray($platforms, $platform_title, 'platform_contributor', $row[12], false);
		fillPlatformArray($platforms, $platform_title, 'platform_pubmed_id', $row[13], false);
		fillPlatform($platforms, $platform_title, 'platform_geo_accession', $row[14], false);
		fillPlatformTable($platforms, $row[0]);
	}
	if ($errMsgs)
	{  
		foreach ($errMsgs as $msg)
		{	printError($msg);	}
		printError("Platform file not produced due to the error(s) shown above.");
		print "<br /><br /><a href=\"GEOMenu.php\"><-Back to GEO Menu</a>";
		return;
	}
	//print_r($platforms);
	
	//Now produce the platform file.
	producePlatformFile($platforms, $platformFileFull);
	if ($errMsgs)
	{  
		foreach ($errMsgs as $msg)
		{	printError($msg);	}
		return;
	}
	else
	{	print "<br /><a href=\"$platformFileRelative\" target=\"platform_file\">Platform File</a>";	}
	print "<br /><br /><a href=\"GEOMenu.php\"><-Back to GEO Menu</a>";
}

function fillPlatform(&$platforms, $platform_title, $platform_field, $value, $is_required = false)
{
	global $errMsgs;
	$value = trim($value);
	if ($is_required AND !$value)
	{	$errMsgs[] = "Please specify '$platform_field' for platform with title '$platform_title'.";	}
	elseif (strlen($value))
	{	$platforms[$platform_title][$platform_field] = $value;	}
}

function fillPlatformArray(&$platforms, $platform_title, $platform_field, $value, $is_required = false)
{
	global $errMsgs;
	$value = trim($value);
	if ($is_required AND !$value)
	{	$errMsgs[] =  "Please specify at least one '$platform_field' for platform with title '$platform_title'.";	}
	else
	{	
		$values = split('::', $value);
		foreach ($values as $value)
		{	
			if (strlen($value))
			$platforms[$platform_title][$platform_field][] = trim($value);	
		}
	}
}

function fillPlatformTable(&$platforms, $array_batch_id)
{
	global $errMsgs;
	
	//Get Platform Table Data from feature, reporter and composite_sequence.
	$sql = "SELECT ab.\"platform_title\", f.\"feature_id\", cs.\"probe_name\", cs.\"annotation\", f.\"row\",
		f.\"column\", f.\"metarow\", f.\"metacolumn\", r.\"reporter_sequence\"
		FROM \"composite_sequence\" cs, \"feature\" f, \"reporter\" r, \"array_batch\" ab
		WHERE ab.\"array_batch_id\" = '$array_batch_id'
		AND f.array_batch_id = ab.\"array_batch_id\"
		AND f.\"reporter_id\" = r.\"reporter_id\"
		AND r.\"composite_sequence_id\" = cs.\"composite_sequence_id\"";

	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	printError("Error retrieving Table Data for platform_title '$row[0]'. " . $sth->getMessage());	return false;	}
	$i = 0;
	while ($row = $sth->fetchRow()) 
	{
		$platform_title = array_shift($row);
		$platforms[$platform_title]['TableData'][$i] = $row;
		$i++;
	}
	return true;
	
}

function producePlatformFile(&$platforms, $platformFile)
{
	global $errMsgs;
	$arrayPlatformFields = array('platform_organism', 'platform_catalog_number', 
		'platform_web_link', 'platform_support', 'platform_coating', 
		'platform_description', 'platform_contributor', 'platform_pubmed_id');
	//Open file to write
	if (!$dataFileHandle = fopen($platformFile, 'w'))
	{
		$errMsgs[] = "Cannot open Platform file ($platformFile)";
		return false;
	}
		
	foreach ($platforms as $platform_title=>$platform)
	{
		$data .= "^PLATFORM = $platform_title\n";
		$data .= "!Platform_title = $platform_title\n";
		foreach ($platform as $platform_field=>$field_value_or_array)
		{
			if ('TableData' == $platform_field)
			{	continue;	}
			if (in_array($platform_field, $arrayPlatformFields))
			{
				foreach ($field_value_or_array as $value)
				{	$data .= '!' . ucfirst(strtolower($platform_field)) . " = {$value}\n";	}
			}
			else
			{	$data .= '!' . ucfirst(strtolower($platform_field)) . " = {$field_value_or_array}\n";	}
		}
		if (isset($platforms[$platform_title]['TableData']))
		{
			$data .= '#ID =
#PROBE_NAME = 
#ANNOTATION = 
#ROW = Row within array
#COLUMN = Column within array
#METAROW = 
#METACOLUMN = 
#REPORTER_SEQUENCE = ' . "\n";
		}
		
		$data .= "!Platform_table_begin\n";
		if (isset($platforms[$platform_title]['TableData']))
		{
			$data .= "ID\tPROBE_NAME\tANNOTATION\tROW\tCOLUMN\tMETAROW\tMETACOLUMN\tREPORTER_SEQUENCE\n";
			foreach ($platforms[$platform_title]['TableData'] as $rowNo=>$dataRow)
			{	
				$feature_id = array_shift($dataRow);
				$data .= getIDREFforGEO($feature_id) . "\t" . join ("\t", $dataRow) . "\n";	
			}
		}
		$data .= "!Platform_table_end\n";
	}
	
	// Write $data to our opened file.
	if (!fwrite($dataFileHandle, $data))
	{
		$errMsgs[] =  "Cannot write to file ($platformFile)";
		return false;
	}
	fclose($dataFileHandle);	
	return true;
}

function getIDREFforGEO($ID)
{
	$nameParts = split(' ', $_SESSION['SPECIES_NAME']);
	foreach ($nameParts as $namePart)
	{	
		if (strlen($namePart) > 0)
		{	$speciesInitials .= strtoupper(chr(ord($namePart)));	}
	}
	return $speciesInitials . str_pad($ID, 7, "0", STR_PAD_LEFT);
}
?>