#!/usr/local/bin/perl



use DBI;

use DOMELIB;

use strict "vars";



#===============================================================================

# This script copy the cell file from tmp_plot to file_home and create records

# in tables file, file_location, array, sample, sample_extract, labeled_extract,

# hybridization, raw_data and image_analysis. There will be no data insertion to

# ma_norm until normalizing time.

#

# Author: X. Jing Li

# Date: August 15, 2005

# Reversions: 

#  11/28/2005: changed the script for technical_replicate will be given.

# 

#===============================================================================

my ($file_home, $raw_file, $srid, $extract_id, $array_batch_id, $hybridization_ptid, $scanning_ptid, $tissue_type_label_id, $person_id, $tech_repl, $sample_entity, $sample_title, $other_desc,);



my ($database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc,

    $array_name, $array_file_name, @temp, $array_id, $detail, $technique_id,

    $fname, $upload_time, $person, $file_type, $file_type_id, $desc, 

    $file_link_id, $tech_repl, $sample_id, $sample_extract_id, 

    $labeled_extract_id, $hybridization_id, $raw_data_id, $image_analysis_id);



($file_home, $raw_file, $srid, $extract_id, $array_batch_id, $hybridization_ptid, $scanning_ptid, $tissue_type_label_id, $person_id, $tech_repl, $sample_entity, $sample_title, $other_desc) = @ARGV; 

#print "file_home = $file_home, rawfile = $raw_file, srid = $srid, extract = $extract_id, array_batch_id = $array_batch_id, hybridization_ptid = $hybridization_ptid, scanning_ptid = $scanning_ptid, tissue_type_label_id = $tissue_type_label_id, person_id = $person_id, tech_repl = $tech_repl\n";



#== connect to the database:

($database, $username, $password, $bnet_DB, $specieDB) = database_login2();

$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";

$dbh->begin_work;

$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";

$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";



#== 1. check if this file set has been put into database:

$array_name = $raw_file;

@temp = split "/", $array_name;

$array_file_name = pop @temp;

($array_name, $file_type) = split /\./, $array_file_name;

$file_type = uc("." . $file_type);

$array_id = getArrayId($dbh, $array_batch_id, $array_name);

if($array_id ne "") {

    print "ERROR\tOriginal file $array_file_name is already in database\n";

    $detail = getArrayIdDetail($dbh, $array_id);

    print $detail;

    $sth->finish;

    $dbh->rollback;

    $dbh->disconnect;

    exit 1;

}



#== get the file_type_id for this file_type:

$sql = "select file_type_id from file_type where upper(file_suffix) = \'$file_type\'";

$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

$file_type_id = $sth->fetchrow();

if ($file_type_id eq "") {

    $desc = "Affymetrix data file";

    $file_type_id = createFileTypeId($dbh, $file_type, $desc);

}



#== check the file table to see if this file has been used:

$sql = "select F.file_name, F.time_uploaded, P.person_name from \"file\" F, person P where upper(F.file_name) = \'" . uc($array_file_name) . "\' and P.person_id = F.person_id";

#print "sql = $sql\n";

$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

($fname, $upload_time, $person) = $sth->fetchrow();

if($fname ne "") {

    $detail = "Data file \"$array_file_name\" has been uploaded by \"$person\" in \"$upload_time\" with file name \"$fname\".\n";

    print $detail;

    $sth->finish;

    $dbh->rollback;

    $dbh->disconnect;

    exit 1;

}

    



#== 2.If the code runs here that means this is a new array. get technique_id:

$sql = "select technique_id from extract where extract_id = $extract_id";

$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

$technique_id = $sth->fetchrow();



#== 3. get the technical_replicate:

#$sql = "select count(*) from hybridization where hybridization_protocol_id = $hybridization_ptid and labeled_extract1_id in (select labeled_extract_id from labeled_extract where sample_labeling_id = $sample_labeling_id and technique_id = $technique_id and sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and sample_id in (select sample_id from \"sample\" where sampling_replicate_id = $srid and label_id = $tissue_type_label_id)))";

#print "$sql\n";

#$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

#$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#$tech_repl = $sth->fetchrow();

#if ($tech_repl > 0 and $is_tech_repl == 0) {

#    print "There is an array exist already. Do you want to add it as a technical replicate of the array?\n";

#    $sth->finish;

#    $dbh->rollback;

#    $dbh->disconnect;

#    exit 99;

#} elsif ($tech_repl > 0 && $is_tech_repl == 1) {

#    $sql = "select max(technical_replicate) from sample_extract where extract_id = $extract_id and sample_id in (select sample_id from \"sample\" where label_id = $tissue_type_label_id and sampling_replicate_id = $srid)";

#    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

#    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#    $tech_repl = $sth->fetchrow();

#    $tech_repl ++;

#} else {

#    $tech_repl = 1;

#}



#== 4. create new records in sample, sample_extract, labeled_extract, array,

#== file, file_location_link, hybridization, raw_data, and image_analysis. Then

#== insert data into quantification:

$sample_id = createSampleId($dbh, $srid, $tissue_type_label_id);

$sample_extract_id = createSampleExtractId($dbh, $sample_id, $extract_id, $tech_repl);

$labeled_extract_id = createLabeledExtractId($dbh, $sample_extract_id, "", $technique_id);



$array_id = createArrayId($dbh, $array_batch_id, $array_name);

$hybridization_id = createHybridizationId($dbh, $labeled_extract_id, '', $array_id, $hybridization_ptid, $person_id, $sample_entity, $sample_title, $other_desc);

$file_link_id = createFileIds($dbh, $file_home, $raw_file, $file_type_id, $person_id);



$raw_data_id = createRawDataId($dbh, $hybridization_id, $scanning_ptid, '', '');

$image_analysis_id = createImageAnalysisId($dbh, $raw_data_id, $file_link_id);

if ($image_analysis_id ne "") {

    $dbh->commit;

    print "Array $array_name has been successfully uploaded into DOME.\n";

    $sth->finish;

    $dbh->disconnect;

    exit 0;

} else {

    $dbh->rollback;

    print "FAILED to upload the array $array_name into DOME.\n";

    $sth->finish;

    $dbh->disconnect;

    exit 1;

}



###########################################################################

# subroutine: getArrayId($dbh, $file_name)

#

# parameters in: $dbh --- the database handle,

#                $file_name --- the file name

# parameter out: $_id --- the id.

#

# X. Jing Li, 4/12/2005

###########################################################################

sub getArrayId {

    my ($dbh, $attay_batch_id, $array_name) = @_;

    my ($sql, $sth, $rc, $array_name, $array_id);



    $sql = "select array_id from \"array\" where array_batch_id = $array_batch_id and array_name = \'$array_name\'";

    #print "$sql\n";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $array_id = $sth->fetchrow;

    $sth->finish;



    return $array_id;

}



###########################################################################

# subroutine: getArrayIdDetail($dbh, $array_id)

#

# parameters in: $dbh --- the database handle,

#                $array_id --- the file name

# parameter out: $detail --- the exp_set_name, exp_name, label.name for this id

#

# X. Jing Li, 4/12/2005

###########################################################################

sub getArrayIdDetail {

    my ($dbh, $array_id) = @_;

    my ($sql, $sth, $rc, $a_name, $detail, $es_name, $e_name, $l_name, $t_name,

        $repl);



    $sql = "select array_name from \"array\" where array_id = $array_id";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $a_name = $sth->fetchrow();

    $detail = "The array with name \"$a_name\" is already in database. ";



    $sql = "select ES.exp_set_name, E.exp_name, L.name, T.technique_name, SR.replicate from experiment_set ES, experiment E, label L, technique T, sampling_point SP, sampling_replicate SR, sample S, sample_extract SE, labeled_extract LE, hybridization H where H.array_id = $array_id and H.labeled_extract1_id = LE.labeled_extract_id and LE.technique_id = T.technique_id and LE.sample_extract_id = SE.sample_extract_id and SE.sample_id = S.sample_id and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";

    #print "$sql\n";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    ($es_name, $e_name, $l_name, $t_name, $repl) = $sth->fetchrow;

    if ($es_name ne "" || $e_name ne "" || $l_name ne "" || $t_name ne "" ||

        $repl ne "" ) {

        $detail .= "The array is from experiment_set \"$es_name\", experiment \"$e_name\", replicate \"$repl\" at time \"$l_name\" with technique \"$t_name\";\n";

    }



    $sth->finish;

    return $detail;

}



###########################################################################

# subroutine: getFileLocationLinkId($dbh, $file):

#    this subroutine get the file_location_link_id for $file.

#

# parameters in: $dbh --- the database handle,

#                $file_namename --- the file name

# parameter out: $file_location_link_id --- the id.

#

###########################################################################

sub getFileLocationLinkId {

    my ($dbh, $file) = @_;

    my ($sql, $sth, $rc, $id);

    

    $sql = "select file_location_link_id from file_location_link where "

        . "file_id = (select file_id from file where file_name = \'$file\')";

#    print "$sql\n";

    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";

    $id = $sth->fetchrow;

    $sth->finish;



    return $id;

}







###########################################################################

# subroutine: createFileLocationId($dbh, $file_home)

#

# parameters in: $dbh --- the database handle,

#                $file_home --- the file home

# parameter out: $flid --- the new file_location_id.

#

# X. Jing Li, 4/29/2005

###########################################################################

sub createFileLocationId {

    my ($dbh, $file_home) = @_;

    my ($sql, $sth, $rc, $flid);



    $sql = "select nextval('file_location_id_seq')";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $flid = $sth->fetchrow();



    $sql = "insert into \"file_location\" (file_location_id, \"location\") values ($flid, \'$file_home\')";

    print "$sql\n";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";



    $sth->finish;

    return $flid;

}



###########################################################################

# subroutine: createHybridizationId($dbh, $labeled_extract_1_id, 

#               $labeled_extract_2_id, $array_id, $hybrid_protocol_id,

#               $person_id, $sample_entity, $sample_title, $other_desc):

#  this subroutine create a hybridization_id in HYBRIDIZATION table

# 

# parameters in: $dbh --- database handle;

#                $labeled_extract_1_id --- labeled_extract id for Cy3;

#                $labeled_extract_2_id --- labeled_extract id for Cy5;

#                $array_id --- array id;

# parameter out: $hybridization_id --- a new hybridization_id 

###########################################################################

sub createHybridizationId {

    my ($dbh, $lb_ext_id_1, $lb_ext_id_2, $aid, $hybrid_prot_id, $person, $sample_entity, $sample_title, $other_desc) = @_;

    my ($sql, $sth, $rc, $sql1, $sql2);

    my $hybridization_id;

    #print "$lb_ext_id_1, $lb_ext_id_2, $aid, $hybrid_prot_id, $person\n";



    $sth = $dbh->prepare("select nextval('hybridization_id_seq')") 

        or die "Can not prepare select nextval from dual: $DBI::errstr";

    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";

    $hybridization_id = $sth->fetchrow;



    $sql1 = "insert into hybridization (hybridization_id, ";

    $sql2 = "values (\'$hybridization_id\', ";

    if ($lb_ext_id_1 ne "") {

        $sql1 .= "labeled_extract1_id, ";

        $sql2 .= "\'$lb_ext_id_1\', ";

    }

    if ($lb_ext_id_2 ne "") {

        $sql1 .= "labeled_extract2_id, ";

        $sql2 .= "\'$lb_ext_id_2\', ";

    }

    if ($sample_entity ne "") {

        $sql1 .= "sample_entity, ";

        $sql2 .= "\'$sample_entity\', ";

    }

    if ($sample_title ne "") {

        $sql1 .= "sample_title, ";

        $sql2 .= "\'$sample_title\', ";

    }

    if ($other_desc ne "") {

        $sql1 .= "other_description, ";

        $sql2 .= "\'$other_desc\', ";

    }

    $sql1 .= "array_id, hybridization_protocol_id, person_id) ";

    $sql2 .= "\'$aid\', \'$hybrid_prot_id\', \'$person\')";

    $sql = $sql1 . $sql2;

    #print "$sql\n";

    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";



    $sth->finish;

    return $hybridization_id;

}

