#!/usr/local/bin/perl


use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script reads the cDNA data files and insert data into database DOME_DEV
# This script need the following information:
#
# basename -- the session_id for file names, need to get rid from file name 
#   before insert into database;
# file_home -- where to move the data file for their final destination
# extract_id
# sampling_replicate_1_id
# tissue_type_1_id
# sampling_replicate_2_id
# tissue_type_2_id
# technical_replicate
# gpr file name
# channel_1 tiff file name
# channel_2 tiff file name
# channel_1 name (sample_extract_1_id)
# channel_2 name (sample_extract_2_id)
# array_batch_id
# gpr_file_type_id
# cy3_file_type_id
# cy5_file_type_id
# hybrid_protocol_id
# scan_prot_id 
# person_id
#
# Author: X. Jing Li
# Date: April 5, 2005
# Reversions: 
#   9/20/2005: add code to delete previously uploaded dat and all related data
#     in ma_norm, sp_summary, and sp_comparison.
#===============================================================================

my ($inputfile, $force_add, $sample_entity, $sample_title, $other_desc, $outputfile, @parameters, $line, $temp, $error,
    $database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc,
    $file_home, $extract_id, $person_id, $gpr_file_name, $file_name, @temp, 
    $desc, $file_type_id, $type, $gpr_file_type_id, $tif_file_type_id, 
    $technical_replicate, $hybridization_protocol_id, $scanning_protocol_id,
    $gpr_file_link_id, 
    $srid1, $tissue_type_label_id1, $sample_labeling_id1, $sample_id1, 
    $sample_extract_id1, $labeled_extract_id1, $chnl1_file_name, $cy3_file_link_id,
    $srid2, $tissue_type_label_id2, $sample_labeling_id2, $sample_id2, 
    $sample_extract_id2, $labeled_extract_id2, $chnl2_file_name, $cy5_file_link_id,
    $array_batch_id, $array_id, $array_name, $technique_id,
    $hybridization_id, $raw_data_id, $image_analysis_id, $ok, $detail, $msg, $file_id);
    
#($inputfile, $force_add) = @ARGV;
($inputfile, $sample_entity, $sample_title, $other_desc) = @ARGV;

print "Starting time = " . TimeStamp() . "\n";

open (PAR, "<$inputfile") || die "Can not open the parameter file $inputfile\n";
@parameters = <PAR>;
close PAR;

foreach $line (@parameters) {
    chomp $line;
    #print "$line\n";
    if ($line =~ /FILEHOME/) {
        ($temp, $file_home) = split /=/, $line;
    } elsif ($line =~ /EXTRACT_ID/) {
        ($temp, $extract_id) = split /=/, $line;
    } elsif ($line =~ /PERSON_ID/) {
        ($temp, $person_id) = split /=/, $line;
    } elsif ($line =~ /DATA_FILE_NAME/) {
        ($temp, $gpr_file_name) = split /=/, $line;
    } elsif ($line =~ /SAMPLE_LABELING_ID_1/) {
        ($temp, $sample_labeling_id1) = split /=/, $line;
    } elsif ($line =~ /SAMPLING_REPLICATE_ID_1/) {
        ($temp, $srid1) = split /=/, $line;
    } elsif ($line =~ /TISSUE_TYPE_ID_1/) {
        ($temp, $tissue_type_label_id1) = split /=/, $line;
    } elsif ($line =~ /CHANNEL1_FILE_NAME/) {
        ($temp, $chnl1_file_name) = split /=/, $line;
    } elsif ($line =~ /SAMPLE_LABELING_ID_2/) {
        ($temp, $sample_labeling_id2) = split /=/, $line;
    } elsif ($line =~ /SAMPLING_REPLICATE_ID_2/) {
        ($temp, $srid2) = split /=/, $line;
    } elsif ($line =~ /TISSUE_TYPE_ID_2/) {
        ($temp, $tissue_type_label_id2) = split /=/, $line;
    } elsif ($line =~ /CHANNEL2_FILE_NAME/) {
        ($temp, $chnl2_file_name) = split /=/, $line;
    } elsif ($line =~ /SCANNING_PROTOCOL_ID/) {
        ($temp, $scanning_protocol_id) = split /=/, $line;
    } elsif ($line =~ /ARRAY_BATCH/) {
        ($temp, $array_batch_id) = split /=/, $line;
    } elsif ($line =~ /HYBRIDIZATION_PROTOCOL_ID/) {
        ($temp, $hybridization_protocol_id) = split /=/, $line;
    } else {
        print "unknown parameter $line\n";
    }
}

#print "file_home = $file_home\nextract_id = $extract_id\nperson_id = $person_id\ngpr_file_name = $gpr_file_name\ngpr_file_type_id = $gpr_file_type_id\nsample_labeling_id1 = $sample_labeling_id1\nsrid1 = $srid1\ntissue_type_label_id1 = $tissue_type_label_id1\nchnl1_file_name = $chnl1_file_name\ntif_file_type_id = $tif_file_type_id\nsample_labeling_id2 = $sample_labeling_id2\nsrid2 = $srid2\ntissue_type_label_id2 = $tissue_type_label_id2\nchnl2_file_name = $chnl2_file_name\narray_batch_id = $array_batch_id\nscanning_protocol_id = $scanning_protocol_id\nhybridization_protocol_id = $hybridization_protocol_id\n";

#== check all needed parameters:
if ($file_home eq "" ) {
    print "ERROR\tNo FILEHOME definded\n";
    exit 1;
} 
if($extract_id eq "") {
    print "ERROR\tNo EXTRACT_ID definded\n";
    exit 1;
}
if ($gpr_file_name eq "") {
    print "ERROR\tNo DATA_FILE_NAME definded\n";
    exit 1;
}
if ($chnl1_file_name eq "") {
    print "ERROR\tNo CHANNEL1_FILE_NAME definded\n";
    exit 1;
}
if ($chnl2_file_name eq "") {
    print "ERROR\tNo CHANNEL2_FILE_NAME definded\n";
    exit 1;
}
if ($srid1 eq "") {
    print "ERROR\tNo SAMPLING_REPLICATE_ID_1 definded\n";
    exit 1;
}
if ($tissue_type_label_id1 eq "") {
    print "ERROR\tNo TISSUE_TYPE_ID_1 definded\n";
    exit 1;
}
if ($sample_labeling_id1 eq "") {
    print "ERROR\tNo SAMPLE_LABELING_ID_1 definded\n";
    exit 1;
}
if ($srid2 eq "") {
    print "ERROR\tNo SAMPLING_REPLICATE_ID_2 definded\n";
    exit 1;
}
if ($tissue_type_label_id2 eq "") {
    print "ERROR\tNo TISSUE_TYPE_ID_2 definded\n";
    exit 1;
}
if ($sample_labeling_id2 eq "") {
    print "ERROR\tNo SAMPLE_LABELING_ID_2 definded\n";
    exit 1;
}
if ($hybridization_protocol_id eq "") {
    print "ERROR\tNo HYBRIDIZATION_PROTOCOL_ID definded\n";
    exit 1;
}
if ($array_batch_id eq "") {
    print "ERROR\tNo ARRAY_BATCH definded\n";
    exit 1;
}
if ($scanning_protocol_id eq "") {
    print "ERROR\tNo SCANNING_PROTOCOL_ID definded\n";
    exit 1;
}
#if ($gpr_file_type_id eq "") {
#    print "ERROR\tNo file_type_id is definded for GPR file.\n";
#    exit 1;
#}
#if ($tif_file_type_id eq "") {
#    print "ERROR\tNo file_type_id is definded for TIF file.\n";
#    exit 1;
#}

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#$sample_labeling_id1 = 1;
#$sample_labeling_id2 = 2;
#== get technique_id:
$sql = "select technique_id from extract where extract_id = $extract_id";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$technique_id = $sth->fetchrow();

#== check if this file has been uploaded before:
@temp = split "/", $gpr_file_name;
$file_name = pop @temp;
$sql = "select file_id from \"file\" where file_name = \'$file_name\'";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$file_id = $sth->fetchrow();
if ($file_id ne "" && $force_add eq "0") {
    print "The file \"$file_name\" has been uploaded before.\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}

#== 1. check if this array set has been put into database:
$array_name = $file_name;
@temp = split /\./, $array_name;
$array_name = shift @temp;
#print "file_name = $file_name, array_name = $array_name\n";
$array_id = getArrayId($dbh, $array_batch_id, $array_name);
if($array_id ne "" && $force_add eq "0") {
    print "ERROR\tGPR file $file_name is already in database\n";
    $detail = getArrayIdDetail($dbh, $array_id);
    print $detail;
    $sth->finish;
    $dbh->rollback;
    $dbh->disconnect;
    print "Ending time = " . TimeStamp() . "\n";
    exit 1;
}

#== check this set of parameters has been used before:
$sql = "select image_analysis_id from image_analysis where raw_data_id in (select raw_data_id from raw_data where scanning_protocol_id = $scanning_protocol_id and hybridization_id in (select hybridization_id from hybridization where hybridization_protocol_id = $hybridization_protocol_id and labeled_extract1_id in (select labeled_extract_id from labeled_extract where sample_labeling_id = $sample_labeling_id1 and technique_id = $technique_id and sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and sample_id in (select sample_id from sample where sampling_replicate_id = $srid1 and label_id = $tissue_type_label_id1))) AND labeled_extract2_id in (select labeled_extract_id from labeled_extract where sample_labeling_id = $sample_labeling_id2 and technique_id = $technique_id and sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and sample_id in (select sample_id from sample where sampling_replicate_id = $srid2 and label_id = $tissue_type_label_id2)))))";
#print "$sql\n";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$image_analysis_id = $sth->fetchrow();
#print "image_analysis_id = $image_analysis_id\n";
if ($image_analysis_id ne "") {
    print "This set of parameters has been used before. If you continue, the following data will be deleted.\n";
    $msg = getRelatedInfoForArray($dbh, $image_analysis_id);
    print $msg;
    print "Ending time = " . TimeStamp() . "\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    exit 99;
}
#} elsif ($image_analysis_id ne "" && $force_add eq "1") {
#    $msg = deleteRelatedDataForArray($dbh, $image_analysis_id);
#    if ($msg ne "SUCCESS") {
#        print "Trying to delete related data in DOME has FAILED. No data has been deleted.<br>$msg<br>\n";
#        print "Ending time = " . TimeStamp() . "\n";
#        $dbh->rollback;
#        $sth->finish;
#        $dbh->disconnect;
#        exit 1;
#    } 
#}    

#== if get here hten either array is a new one or old data has been deleted
#== successfully. create new records now.
#== get the file_type_ids for gpr and tif:
$sql = "select file_type_id, file_suffix from file_type where upper(file_suffix) like '%GPR' or upper(file_suffix) like '%TIF'";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($file_type_id, $type) = $sth->fetchrow()) {
    if ($type =~ /GPR/i) {
        $gpr_file_type_id = $file_type_id;
    } elsif ($type =~ /TIF/i) {
        $tif_file_type_id = $file_type_id;
    }
}
if ($gpr_file_type_id eq "") {
    $desc = "Microarray - GPR data file";
    $gpr_file_type_id = createFileTypeId($dbh, $gpr_file_name, $desc);
}
if ($tif_file_type_id eq "") {
    $desc = "TIF image ";
    $tif_file_type_id = createFileTypeId($dbh, $chnl1_file_name, $desc);
}

#== 3. get the technical_replicate:
#$sql = "select count(*) from hybridization where hybridization_protocol_id = $hybridization_protocol_id and labeled_extract1_id in (select labeled_extract_id from labeled_extract where sample_labeling_id = $sample_labeling_id1 and technique_id = $technique_id and sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and sample_id in (select sample_id from sample where sampling_replicate_id = $srid1 and label_id = $tissue_type_label_id1))) AND labeled_extract2_id in (select labeled_extract_id from labeled_extract where sample_labeling_id = $sample_labeling_id2 and technique_id = $technique_id and sample_extract_id in (select sample_extract_id from sample_extract where extract_id = $extract_id and sample_id in (select sample_id from sample where sampling_replicate_id = $srid2 and label_id = $tissue_type_label_id2)))";
#$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
#$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#$technical_replicate = $sth->fetchrow();
#$technical_replicate++;
$technical_replicate = 1;

#== 4. Create new records in sample, sample_extract, labeled_extract, array, 
#== file, file_location_link, hybridization, raw_data, and image_analysis. Then 
#== insert data into quantification:
$sample_id1 = createSampleId($dbh, $srid1, $tissue_type_label_id1);
$sample_extract_id1 = createSampleExtractId($dbh, $sample_id1, $extract_id, $technical_replicate);
$labeled_extract_id1 = createLabeledExtractId($dbh, $sample_extract_id1, $sample_labeling_id1, $technique_id);

$sample_id2 = createSampleId($dbh, $srid2, $tissue_type_label_id2);
$sample_extract_id2 = createSampleExtractId($dbh, $sample_id2, $extract_id, $technical_replicate);
$labeled_extract_id2 = createLabeledExtractId($dbh, $sample_extract_id2, $sample_labeling_id2, $technique_id);

$array_id = createArrayId($dbh, $array_batch_id, $array_name);
   
$hybridization_id = createHybridizationId($dbh, $labeled_extract_id1, $labeled_extract_id2, $array_id, $hybridization_protocol_id, $person_id, $sample_entity, $sample_title, $other_desc);
$gpr_file_link_id = createFileIds($dbh, $file_home, $array_name, $gpr_file_name, $gpr_file_type_id, $person_id);
$cy3_file_link_id = createFileIds($dbh, $file_home, $array_name, $chnl1_file_name, $tif_file_type_id, $person_id);
$cy5_file_link_id = createFileIds($dbh, $file_home, $array_name, $chnl2_file_name, $tif_file_type_id, $person_id);

$raw_data_id = createRawDataId($dbh, $hybridization_id, $scanning_protocol_id, $cy3_file_link_id, $cy5_file_link_id);
$image_analysis_id = createImageAnalysisId($dbh, $raw_data_id, $gpr_file_link_id);
$ok = insertMicroArrayData($dbh, $gpr_file_name, $image_analysis_id, $array_batch_id);

if ($ok eq "SUCCESS") {
    $dbh->commit;
    print "Array $array_name has been successfully insertred into DOME.\n";
    #print $msg;
    $sth->finish;
    $dbh->disconnect;
    print "Ending time = " . TimeStamp() . "\n";
    exit 0;        
} else {
    $dbh->rollback;
    print "FAILED to insert data into Quantification table.\n";
    $sth->finish;
    $dbh->disconnect;
    print "Ending time = " . TimeStamp() . "\n";
    exit 1;
}


###########################################################################
# subroutine: getArrayId($dbh, $file_name)
#
# parameters in: $dbh --- the database handle,
#                $file_name --- the file name
# parameter out: $_id --- the id.
#
# X. Jing Li, 4/12/2005
###########################################################################
sub getArrayId {
    my ($dbh, $attay_batch_id, $file_name) = @_;
    my ($sql, $sth, $rc, $array_name, $array_id);

    $sql = "select array_id from \"array\" where array_batch_id = $array_batch_id and array_name = \'$file_name\'";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $array_id = $sth->fetchrow;
    $sth->finish;

    return $array_id;
}

###########################################################################
# subroutine: getArrayIdDetail($dbh, $array_id)
#
# parameters in: $dbh --- the database handle,
#                $array_id --- the file name
# parameter out: $detail --- the exp_set_name, exp_name, label.name for this id
#
# X. Jing Li, 4/12/2005
###########################################################################
sub getArrayIdDetail {
    my ($dbh, $array_id) = @_;
    my ($sql, $sth, $rc, $a_name, $detail, $es_name, $e_name, $l_name, $t_name,
        $repl);

    $sql = "select array_name from \"array\" where array_id = $array_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $a_name = $sth->fetchrow();
    $detail = "The array with name \"$a_name\" is already in database. ";

    $sql = "select ES.exp_set_name, E.exp_name, L.name, T.technique_name, SR.replicate from experiment_set ES, experiment E, label L, technique T, sampling_point SP, sampling_replicate SR, sample S, sample_extract SE, labeled_extract LE, hybridization H where H.array_id = $array_id and H.labeled_extract1_id = LE.labeled_extract_id and LE.technique_id = T.technique_id and LE.sample_extract_id = SE.sample_extract_id and SE.sample_id = S.sample_id and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($es_name, $e_name, $l_name, $t_name, $repl) = $sth->fetchrow;
    if ($es_name ne "" || $e_name ne "" || $l_name ne "" || $t_name ne "" ||
        $repl ne "" ) {
        $detail .= "The channel 1 is from experiment_set \"$es_name\", experiment \"$e_name\", replicate \"$repl\" at time \"$l_name\" with technique \"$t_name\";\n";
    }

    $sql = "select ES.exp_set_name, E.exp_name, L.name, T.technique_name, SR.replicate from experiment_set ES, experiment E, label L, technique T, sampling_point SP, sampling_replicate SR, sample S, sample_extract SE, labeled_extract LE, hybridization H where H.array_id = $array_id and H.labeled_extract2_id = LE.labeled_extract_id and LE.technique_id = T.technique_id and LE.sample_extract_id = SE.sample_extract_id and SE.sample_id = S.sample_id and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($es_name, $e_name, $l_name, $t_name, $repl) = $sth->fetchrow;
    if ($es_name ne "" || $e_name ne "" || $l_name ne "" || $t_name ne "" ||
        $repl ne "" ) {
        $detail .= "The channel 2 is from experiment_set \"$es_name\", experiment \"$e_name\", replicate \"$repl\" at time \"$l_name\" with technique \"$t_name\".\n";
    }

    $sth->finish;
    return $detail;
}


###########################################################################
# subroutine: getFileLocationLinkId($dbh, $file):
#    this subroutine get the file_location_link_id for $file.
#
# parameters in: $dbh --- the database handle,
#                $file_namename --- the file name
# parameter out: $file_location_link_id --- the id.
#
###########################################################################
sub getFileLocationLinkId {
    my ($dbh, $file) = @_;
    my ($sql, $sth, $rc, $id);
    
    $sql = "select file_location_link_id from file_location_link where "
        . "file_id = (select file_id from file where file_name = \'$file\')";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
    $id = $sth->fetchrow;
    $sth->finish;

    return $id;
}

###########################################################################
# subroutine: getIdsBeforeLabeledExtract
#           ($dbh, $sampling_replicate_id, $tissue_type_id, $technical_replicate, 
#            $extract_id, $sample_labeling_id)
#
# parameters in: $dbh --- the database handle,
#                $file_namename --- the file name
# parameter out: $_id --- the id.
#
# X. Jing Li, 4/12/2005
###########################################################################
sub getIdsBeforeLabeledExtract {
    my ($dbh, $sampling_replicate_id, $tissue_type_id, $technical_replicate, 
        $extract_id, $sample_labeling_id) = @_;

    my ($sql, $sth, $rc, $sample_id, $sample_extract_id, $labeled_extract_id);

    $sql = "select SE.sample_id, LE.sample_extract_id, LE.labeled_extract_id from sample S, sample_extract SE, labeled_extract LE where S.label_id = $tissue_type_id and S.sampling_replicate_id = $sampling_replicate_id and SE.extract_id = $extract_id and SE.technical_replicate = $technical_replicate and SE.sample_id = S.sample_id and LE.sample_extract_id = SE.sample_extract_id and LE.sample_labeling_id = $sample_labeling_id and LE.technique_id = (select technique_id from extract where extract_id = $extract_id)";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    ($sample_id, $sample_extract_id, $labeled_extract_id) = $sth->fetchrow;
    $sth->finish;
    
    return $sample_id, $sample_extract_id, $labeled_extract_id;
}

###########################################################################
# subroutine: 
# getIdsAfterLabeledExtract($dbh, $labeled_extract_id1, $labeled_extract_id2,
#    $hybrid_prot_id, $scan_prot_id)
#
# parameters in: $dbh --- the database handle,
#                $file_namename --- the file name
# parameter out: $_id --- the id.
#
# X. Jing Li, 4/12/2005
###########################################################################
sub getIdsAfterLabeledExtract {
    my ($dbh, $labeled_extract_id1, $labeled_extract_id2, $hybrid_prot_id,
        $scan_prot_id) = @_;
    my ($sql, $sth, $rc, $hybrid_id, $array_id, $person_id, $raw_data_id,
        $cy3_file_link_id, $cy5_file_link_id,);

    $sql = "select H.hybridization_id, H.array_id, H.person_id, RD.raw_data_id, RD.Cy3_file_location_link_id, RD.Cy5_file_location_link_id from hybridization H, raw_data RD where H.labeled_extract1_id = $labeled_extract_id1 and H.labeled_extract2_id = $labeled_extract_id2 and H.hybridization_protocol_id = $hybrid_prot_id and RD.scanning_protocol_id = $scan_prot_id and H.hybridization_id = RD.hybridization_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($hybrid_id, $array_id, $person_id, $raw_data_id, $cy3_file_link_id, $cy5_file_link_id) = $sth->fetchrow();
    if ($hybrid_id ne "") {
        $sth->finish;
        return $hybrid_id, $array_id, $person_id, $raw_data_id, $cy3_file_link_id, $cy5_file_link_id;
    }
        
    $sql = "select hybridization_id, array_id, person_id from hybridization where labeled_extract1_id = $labeled_extract_id1 and labeled_extract2_id = $labeled_extract_id2 and hybridization_protocol_id = $hybrid_prot_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($hybrid_id, $array_id, $person_id)  = $sth->fetchrow();
    if ($hybrid_id ne "") {
        $sth->finish;
        return $hybrid_id, $array_id, $person_id, "", "", "";
    } else {
       $sth->finish;
       return "", "", "", "", "", "";
    } 
}

###########################################################################
# subroutine: getCDNASampleDetail
#
# parameters in: $dbh --- the database handle,
#                $raw_data_id --- the raw_data_id
# parameter out: $detail --- the string for all ids, 
#
# X. Jing Li, 4/12/2005
###########################################################################
sub getCDnaSampleDetail {
    my ($dbh, $raw_data_id) = @_;
    my ($sth, $rc, $sql, $leid1, $leid2, $array_name, $es_name, $e_name,
        $l_name, $t_name, $detail);

    $detail = "This sample has ";
    $sql = "select H.labeled_extract1_id, H.labeled_extract2_id, A.array_name from \"array\" A, hybridization H, raw_data RD where RD.raw_data_id = $raw_data_id and RD.hybridization_id = H.hybridization_id and H.array_id = A.array_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($leid1, $leid2, $array_name) = $sth->fetchrow;
    print "$leid1, $leid2, $array_name\n";
    $detail .= "array name \"$array_name\", ";
    
    $sql = "select ES.exp_set_name, E.exp_name, L.name, T.technique_name from experiment_set ES, experiment E, label L, technique T, sampling_point SP, sampling_replicate SR, sample S, sample_extract SE, labeled_extract LE where LE.labeled_extract_id = $leid1 and LE.technique_id = T.technique_id and LE.sample_extract_id = SE.sample_extract_id and SE.sample_id = S.sample_id and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($es_name, $e_name, $l_name, $t_name) = $sth->fetchrow;
    #print "$es_name, $e_name, $l_name, $t_name\n";
    if ($es_name ne "" && $e_name ne "" && $l_name ne "") {
        $detail .= "channel 1 from experiment set \"$es_name\", experiment \"$e_name\", at time \"$l_name\" with technique name \"$t_name\".\n";
    }

    $sql = "select ES.exp_set_name, E.exp_name, L.name, T.technique_name from experiment_set ES, experiment E, label L, technique T, sampling_point SP, sampling_replicate SR, sample S, sample_extract SE, labeled_extract LE where LE.labeled_extract_id = $leid2 and LE.technique_id = T.technique_id and LE.sample_extract_id = SE.sample_extract_id and SE.sample_id = S.sample_id and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($es_name, $e_name, $l_name, $t_name) = $sth->fetchrow;
    #print "$es_name, $e_name, $l_name, $t_name\n";
    if ($es_name ne "" && $e_name ne "" && $l_name ne "") {
        $detail .= "channel 2 is from experiment set \"$es_name\", experiment \"$e_name\", at time \"$l_name\" with technique name \"$t_name\".\n";
    }

    $sth->finish;
    return $detail;
}

###########################################################################
# subroutine: createFileIds($dbh, $file_home, $basename, $array_name, 
#             $file_name, $file_type_id
#
# parameters in: $dbh --- the database handle,
#                $file_namename --- the file name
# parameter out: $_id --- the id.
#
# X. Jing Li, 4/12/2005
###########################################################################
sub createFileIds {
    my ($dbh, $file_home, $array_name, $file_name, $file_type_id, $person_id) = @_;
    my ($sql, $sth, $rc, $flid, $fid, $fllid, @temp, $file_suffix, $fname,
        $new_fname, $cmd);

    #== get file_location_id for file_home:
    $sql = "select file_location_id from file_location where location = \'$file_home\'";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $flid = $sth->fetchrow;
#    if ($flid eq "") {
#        $sth->finish;
#        print "ERROR\tfile_home is not in table\"file_location\"\n";
#        $dbh->rollback;
#        dbh->disconnect;
#        exit 1;
#    }
    if ($flid eq "") {
        $flid = createFileLocationId($dbh, $file_home);
    }

    #== create file_id:
    @temp = split "/", $file_name;
    $fname = pop @temp;
    $fname = $array_name . "_" . $fname;

    $sql = "select nextval('file_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $fid = $sth->fetchrow;

    $sql = "insert into \"file\" (file_id, file_name, file_type_id, time_uploaded, person_id) values ($fid, \'$fname\', $file_type_id, (SELECT abstime(timestamp 'now')), $person_id)";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    #== create file_location_link_id:
    $sql = "select nextval('file_location_link_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $fllid = $sth->fetchrow;

    $sql = "insert into file_location_link (file_location_link_id, file_id, file_location_id) values ($fllid, $fid, $flid)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    if ($fid ne "" && $flid ne "" && $fllid ne "") {
        $new_fname = $file_home . $fname;
        $cmd = "cp \"$file_name\" \"$new_fname\"\n";
        #print "$cmd";
        system $cmd;
        #unlink ($file_name) or warn "Can not delete file $file_name from holding diretory. $!\n";
    }
    $sth->finish;
    return $fllid;
}

###########################################################################
# subroutine: createFileLocationId($dbh, $file_home)
#
# parameters in: $dbh --- the database handle,
#                $file_home --- the file home
# parameter out: $flid --- the new file_location_id.
#
# X. Jing Li, 4/29/2005
###########################################################################
sub createFileLocationId {
    my ($dbh, $file_home) = @_;
    my ($sql, $sth, $rc, $flid);

    $sql = "select nextval('file_location_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $flid = $sth->fetchrow();

    $sql = "insert into \"file_location\" (file_location_id, \"location\") values ($flid, $file_home)";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $sth->finish;
    return $flid;
}

###########################################################################
# subroutine:createtFileTypeId($dbh, $file_name, $desc)
#
# parameters in: $dbh --- the database handle,
#                $file_name --- the file home,
#                $desc -- description of the file type;
# parameter out: $ftid --- the new file_type_id;
#
# X. Jing Li, 9/14/2005
###########################################################################
sub createFileTypeId {
    my ($dbh, $file_name, $desc) = @_;
    my ($sql, $sth, $rc, @temp, $file_type, $ftid);

    @temp = split /\./, $file_name;
    $file_type = "." . pop @temp;
    $sql = "select nextval('file_type_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $ftid = $sth->fetchrow();

    $sql = "insert into\"file_type\"  (file_type_id, \"type\", \"file_suffix\") values ($ftid, \'$desc\', \'$file_type\')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $sth->finish;
    return $ftid;
}

###########################################################################
# subroutine:getRelatedInfoForArray
#  ($dbh, $image_analysis_id)
#
# parameters in: $dbh --- the database handle,
#                $image_analysis_id --- the iage_analysis_id,
# parameter out: $message -- the detailed info about the data for the array
#
# X. Jing Li, 9/22/2005
###########################################################################
sub getRelatedInfoForArray {
    my ($dbh, $image_analysis_id) = @_;
    my ($sql, $sth, $rc, $leid1, $leid2, $array_name, @ptids, $ptid, $pt_ids,
        $exp_set_name, $exp_name, $l_name, $t_name, $srid, $pt_name, 
        %ptid_names, $key, $protocol_id, $spid, %pid_spids, $process1_id,
        %pid_process_ids, @keys, $key, $message);
 
    #== 0. get detail about this array:
    $sql = "select H.labeled_extract1_id, H.labeled_extract2_id, A.array_name from \"array\" A, hybridization H, raw_data RD, image_analysis IA where IA.image_analysis_id = $image_analysis_id and IA.raw_data_id = RD.raw_data_id and RD.hybridization_id = H.hybridization_id and H.array_id = A.array_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    #print "$sql\n$rc\n";
    ($leid1, $leid2, $array_name) = $sth->fetchrow;
    $sql = "select ES.exp_set_name, E.exp_name, L.name, T.technique_name, SP.sampling_point_id, SR.sampling_replicate_id from experiment_set ES, experiment E, label L, technique T, sampling_point SP, sampling_replicate SR, \"sample\" S, sample_extract SE, labeled_extract LE where LE.labeled_extract_id in ($leid1, $leid2) and LE.technique_id = T.technique_id and LE.sample_extract_id = SE.sample_extract_id and SE.sample_id = S.sample_id and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    #print "$sql\n$rc\n";
    while (($exp_set_name, $exp_name, $l_name, $t_name, $spid, $srid) = $sth->fetchrow) {
        if ($l_name !~ /Reference/i) {
            last;
        }
    }

    #== 1. check normalization:
    $sql = "select distinct protocol_type_id from \"ma_norm\" where sample_extract_id in (select sample_extract_id from labeled_extract where (labeled_extract_id in (select labeled_extract1_id from hybridization where hybridization_id in (select hybridization_id from raw_data where raw_data_id in (select raw_data_id from image_analysis where image_analysis_id = $image_analysis_id)))) or (labeled_extract_id in (select labeled_extract2_id from hybridization where hybridization_id in (select hybridization_id from raw_data where raw_data_id in (select raw_data_id from image_analysis where image_analysis_id = $image_analysis_id))))) ";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    while ($ptid = $sth->fetchrow() ) {
        push @ptids, $ptid;
    }
    if (@ptids > 0) {
        #-- get the protocol_type names:
        $pt_ids = join ", ", @ptids;
        $sql = "select protocol_type_id, protocol_type_name from protocol_type where protocol_type_id in ($pt_ids)";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        while (($ptid, $pt_name) = $sth->fetchrow() ) {
            $ptid_names{$ptid} = $pt_name;
        }

        #== 2. check sp_summary:
        $sql = "select distinct protocol_id, sampling_point_id from sp_summary where sampling_point_id = $spid and protocol_id in (select protocol_id from protocol where protocol_type_id in ($pt_ids))";
        #print "$sql\n";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        #print "$sql\n$rc\n";
        while (($protocol_id, $spid) = $sth->fetchrow() ) {
            $key = $protocol_id . "_" . $spid;
            $pid_spids{$key} = getSamplingPointIdDetail($dbh, $protocol_id, $spid);
        }
        
        if (keys %pid_spids > 0) {
            my @keys = keys %pid_spids;
            ($protocol_id, $spid) = split /\_/, $keys[0];
            #== 3. check sp_comparison. All we need is to get all process1_ids from process1:
            $sql = "select distinct process1_id, protocol_id from sp_comparison where process1_id in (select process1_id from process1 where (sampling_point1_id = $spid or sampling_point2_id = $spid)) and protocol_id in (select protocol_id from protocol where protocol_type_id in ($pt_ids))";
            #print "$sql\n";
            $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
            $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
            while (($process1_id, $protocol_id) = $sth->fetchrow() ) {
                $key = $process1_id . "_" . $protocol_id;
                $pid_process_ids{$key} = getProcess1IdDetail($dbh, $protocol_id, $process1_id);
            }
        }
    }

    #== create the final message for all information we have now:
    if (@ptids == 0) {
        $message = "There is NO normalized data for this data set. Do you want replace the data in DOME with this set of data? If yes click \"Continue\" botton and the old data will be replaced.\n";
        $sth->finish;
        return $message;
    } else {
        $message = "Experiment set: $exp_set_name<br>";
        $message .= "Experiment: $exp_name<br>";
        $message .= "Sample point: $l_name<br><br>";

        $message .= "<CENTER><b>Normalized data</b><br><table border><TH>Protocol Type</TH>\n ";
        foreach $ptid (keys %ptid_names) {
            $message .= "<TR><TD>$ptid_names{$ptid} </TD></TR>";
        }
        $message .= "</table></CENTER><br>\n";
        if (keys %pid_spids > 0) {
            $message .= "<CENTER><b>Mean and standard-deviation</b><br>\n";
            $message .="<table border><TH>Protocol Type</TH>";
            foreach $key (keys %pid_spids) {
                $message .= $pid_spids{$key};
            }
            $message .= "</table><br></CENTER>\n";
        }

        if ((keys %pid_process_ids) > 0) {
            $message .= "<CENTER><b>Comparison data</b><br>";
            $message .= "<table border><TR><TH COLSPAN=3>Sample 1</TH><TH COLSPAN=3>Sample 2</TH><TH ROWSPAN=2>Protocol Name</TH><TH ROWSPAN=2>Test</TH></TR>\n";
            $message .="<TR><TH>Experiment Set</TH><TH>Experiment</TH><TH>Sampling Point</TH><TH>Experiment Set</TH><TH>Experiment</TH><TH>Sampling Point</TH></TR>\n";
            foreach $key (keys %pid_process_ids) {
                $message .= $pid_process_ids{$key};
            }
            $message .= "</table><br></CENTER>\n";
        }
    }
    $sth->finish;
    return $message;
} 

###########################################################################
# subroutine:getSamplingPointIdDetail($dbh, $protocol_id, $spid)
#
# parameters in: $dbh --- the database handle,
#                $protocol_id --- the protocol id for analysis,
#                $spid -- sampling_point_id;
# parameter out: $msg --- the protocol_name in html table row format;
#
# X. Jing Li, 9/27/2005
###########################################################################
sub getSamplingPointIdDetail {
    my ($dbh, $protocol_id, $spid) = @_;
    my ($sql, $sth, $rc, $pt_name, $prot_name, $msg);

    $sql = "select PT.protocol_type_name, P.protocol_name from protocol_type PT, protocol P where PT.protocol_type_id = P.protocol_type_id and P.protocol_id = $protocol_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($pt_name, $prot_name) = $sth->fetchrow();

    #$sql = "select ES.exp_set_name, E.exp_name, L.name from experiment_set ES, experiment E, label L, sampling_point SP where SP.sampling_point_id = $spid and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    #$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    #($exp_set_name, $exp_name, $l_name) = $sth->fetchrow();
    $msg = "<TR><TD>$pt_name</TD></TR>\n";

    $sth->finish;
    return $msg;
}

###########################################################################
# subroutine:getProcess1IdDetail($dbh, $protocol_id, $process1_id)
#
# parameters in: $dbh --- the database handle,
#                $protocol_id --- the protocol id,
#                $process1_id --- the ;
# parameter out: $msg --- the detailed message in html table row format;
#
# X. Jing Li, 9/27/2005
###########################################################################
sub getProcess1IdDetail {
    my ($dbh, $protocol_id, $process1_id) = @_;
    #print "$dbh, $protocol_id, $process1_id\n";
    my ($sql, $sth, $rc, $pt_name, $prot_name, $exp_set_name, $exp_name,
        $l_name, $msg);

    $sql = "select PT.protocol_type_name, P.protocol_name from protocol_type PT, protocol P where PT.protocol_type_id = P.protocol_type_id and P.protocol_id = $protocol_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($pt_name, $prot_name) = $sth->fetchrow();
    
    $sql = "select ES.exp_set_name, E.exp_name, L.name from experiment_set ES, experiment E, label L, sampling_point SP where SP.sampling_point_id in (select sampling_point1_id from process1 where process1_id = $process1_id) and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($exp_set_name, $exp_name, $l_name) = $sth->fetchrow();
    $msg = "<TR><TD>$exp_set_name</TD><TD>$exp_name</TD><TD>$l_name</TD>";
    
    $sql = "select ES.exp_set_name, E.exp_name, L.name from experiment_set ES, experiment E, label L, sampling_point SP where SP.sampling_point_id in (select sampling_point2_id from process1 where process1_id = $process1_id) and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($exp_set_name, $exp_name, $l_name) = $sth->fetchrow();
    $msg .= "<TD>$exp_set_name</TD><TD>$exp_name</TD><TD>$l_name</TD>";
    $msg .= "<TD>$pt_name</TD><TD>$prot_name</TD></TR>\n";
    
    $sth->finish;
    return $msg;
}

###########################################################################
# subroutine:deleteRelatedDataForArray($dbh, $image_analysis_id)
#
# parameters in: $dbh --- the database handle,
#                $image_analysis_id --- the iage_analysis_id,
# parameter out: $message -- the detailed information about the data for the array
#
# X. Jing Li, 9/28/2005
###########################################################################
sub deleteRelatedDataForArray {
    my ($dbh, $image_analysis_id) = @_;
    my ($sql, $sth, $rc, $sampling_point_id, $sampling_replicate_id, 
        $sample_id, $sample_extract_id, $labeled_extract_id, $hybridization_id,
        $array_id, $tissue_label_id, $raw_data_id, $cy3_file_location_link_id,
        $cy5_file_location_link_id, $gpr_file_location_link_id, 
        $tissue_id, $hid, $aid, $rdid, $cy3_id, $cy5_id, $gpr_id, $delete_sql,
        @spids, @srids, @seids, @ptids, $ptid, $ptids_str, $file_name,
        $seids_str, $spids_str, $err);

    #== get all related ids:
    $sql .= "select SP.sampling_point_id, SR.sampling_replicate_id, S.sample_id, S.label_id, SE.sample_extract_id, LE.labeled_extract_id, H.hybridization_id, H.array_id, RD.raw_data_id, RD.cy3_file_location_link_id, RD.cy5_file_location_link_id, IA.file_location_link_id from sampling_point SP, sampling_replicate SR, \"sample\" S, sample_extract SE, labeled_extract LE, hybridization H, raw_data RD, image_analysis IA where IA.image_analysis_id = $image_analysis_id and IA.raw_data_id = RD.raw_data_id and RD.hybridization_id = H.hybridization_id and (H.labeled_extract1_id = LE.labeled_extract_id or H.labeled_extract2_id = LE.labeled_extract_id) and LE.sample_extract_id = SE.sample_extract_id and SE.sample_id = S.sample_id and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_point_id = SP.sampling_point_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    #print "$sql\n$rc\n";
    while (($sampling_point_id, $sampling_replicate_id, $sample_id, $tissue_label_id, $sample_extract_id, $labeled_extract_id, $hybridization_id, $array_id, $raw_data_id, $cy3_file_location_link_id, $cy5_file_location_link_id, $gpr_file_location_link_id) = $sth->fetchrow()) {
        push @spids, $sampling_point_id;
        push @srids, $sampling_replicate_id;
        push @seids, $sample_extract_id;
        $tissue_id = $tissue_label_id;
        $hid = $hybridization_id;
        $aid = $array_id;
        $rdid = $raw_data_id;
        $cy3_id = $cy3_file_location_link_id;
        $cy5_id = $cy5_file_location_link_id;
        $gpr_id = $gpr_file_location_link_id;
        #== delete from sample, sample_extract, labeled_extract, hybridization,
        #== raw_data, image_analysis, quantification, file, file_location_link,
        #== array,
        $delete_sql .= "delete from \"sample\" where sample_id = $sample_id;\n";
        $delete_sql .= "delete from sample_extract where sample_extract_id = $sample_extract_id;\n";
        $delete_sql .= "delete from labeled_extract where labeled_extract_id = $labeled_extract_id;\n";
        
    }
    $delete_sql .= "delete from hybridization where hybridization_id = $hid;\n";
    $delete_sql .= "delete from \"array\" where array_id = $aid;\n";
    $delete_sql .= "delete from raw_data where raw_data_id = $rdid;\n";
    $delete_sql .= "delete from image_analysis where image_analysis_id = $image_analysis_id;\n";
    $file_name = getFileName($dbh, $cy3_id);
    #unlink $file_name or $err .= "Can not delete old file $file_name.<br>\n";
    $delete_sql .= "delete from \"file\" where file_id = (select file_id from file_location_link where file_location_link_id = $cy3_id);\n";
    $delete_sql .= "delete from file_location_link where file_location_link_id = $cy3_id;\n";
    $file_name = getFileName($dbh, $cy5_id);
    #unlink $file_name or $err .= "Can not delete old file $file_name.<br>\n";
    $delete_sql .= "delete from \"file\" where file_id = (select file_id from file_location_link where file_location_link_id = $cy5_id);\n";
    $delete_sql .= "delete from file_location_link where file_location_link_id = $cy5_id;\n";
    $file_name = getFileName($dbh, $gpr_id);
    #unlink $file_name or $err .= "Can not delete old file $file_name.<br>\n";
    $delete_sql .= "delete from \"file\" where file_id = (select file_id from file_location_link where file_location_link_id = $gpr_id);\n";
    $delete_sql .= "delete from file_location_link where file_location_link_id = $gpr_id;\n";
    $delete_sql .= "delete from quantification where image_analysis_id = $image_analysis_id;\n";

    $seids_str = join ", ", @seids;
    $sql = "select distinct protocol_type_id from \"ma_norm\" where sample_extract_id in ($seids_str) ";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    while ($ptid = $sth->fetchrow() ) {
        push @ptids, $ptid;
    }
    $ptids_str = join ", ", @ptids;
    $spids_str = join ", ", @spids;

    #== delete records in ma_norm, sp_summary, sp_comparison:
    $delete_sql .= "delete from ma_norm where sample_extract_id in ($seids_str);\n";
    if ($ptids_str ne "") {
        $delete_sql .= "delete from sp_summary where sampling_point_id in ($spids_str) and label_id = $tissue_id and protocol_id in (select protocol_id from protocol where protocol_type_id in ($ptids_str));\n";
        $delete_sql .= "delete from sp_comparison where process1_id in (select process1_id from process1 where sampling_point1_id in ($spids_str) or sampling_point2_id in ($spids_str)) and protocol_id in (select protocol_id from protocol where protocol_type_id in ($ptids_str));\n";
    }
    #print $delete_sql . "\n";
    if ($delete_sql ne "") {
        $sth = $dbh-> prepare($delete_sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        #print "rc = $rc\n";
        $sth->finish;
        return "SUCCESS";
    } else {
        return "FAILED";
    }
}

###########################################################################
# subroutine: getFileName($dbh, $file_location_link_id)
#
# parameters in: $dbh --- the database handle,
#                $file_location_link_id --- the id
# parameter out: $file_name --- the file name with full path.
#
# X. Jing Li, 9/29/2005
###########################################################################
sub getFileName {
    my ($dbh, $file_location_link_id) = @_;
    my ($sql, $sth, $rc, $file_name, $file_location);
    
    $sql = "select F.file_name, FL.\"location\" from \"file\" F, file_location FL, file_location_link FLL where FLL.file_location_link_id = $file_location_link_id and FLL.file_id = F.file_id and FL.file_location_id = FLL.file_location_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($file_name, $file_location) = $sth->fetchrow();
    $file_name = $file_location . $file_name;
    #print "file_name = $file_name\n";
    
    $sth->finish;
    return $file_name;
}

###########################################################################
# subroutine: createHybridizationId($dbh, $labeled_extract_1_id, 
#               $labeled_extract_2_id, $array_id, $hybrid_protocol_id,
#               $person_id, $sample_entity, $sample_title, $other_desc):
#  this subroutine create a hybridization_id in HYBRIDIZATION table
# 
# parameters in: $dbh --- database handle;
#                $labeled_extract_1_id --- labeled_extract id for Cy3;
#                $labeled_extract_2_id --- labeled_extract id for Cy5;
#                $array_id --- array id;
# parameter out: $hybridization_id --- a new hybridization_id 
###########################################################################
sub createHybridizationId {
    my ($dbh, $lb_ext_id_1, $lb_ext_id_2, $aid, $hybrid_prot_id, $person, $sample_entity, $sample_title, $other_desc) = @_;
    my ($sql, $sth, $rc, $sql1, $sql2);
    my $hybridization_id;
    #print "$lb_ext_id_1, $lb_ext_id_2, $aid, $hybrid_prot_id, $person\n";

    $sth = $dbh->prepare("select nextval('hybridization_id_seq')") 
        or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $hybridization_id = $sth->fetchrow;

    $sql1 = "insert into hybridization (hybridization_id, ";
    $sql2 = "values (\'$hybridization_id\', ";
    if ($lb_ext_id_1 ne "") {
        $sql1 .= "labeled_extract1_id, ";
        $sql2 .= "\'$lb_ext_id_1\', ";
    }
    if ($lb_ext_id_2 ne "") {
        $sql1 .= "labeled_extract2_id, ";
        $sql2 .= "\'$lb_ext_id_2\', ";
    }
    if ($sample_entity ne "") {
        $sql1 .= "sample_entity, ";
        $sql2 .= "\'$sample_entity\', ";
    }
    if ($sample_title ne "") {
        $sql1 .= "sample_title, ";
        $sql2 .= "\'$sample_title\', ";
    }
    if ($other_desc ne "") {
        $sql1 .= "other_description, ";
        $sql2 .= "\'$other_desc\', ";
    }
    $sql1 .= "array_id, hybridization_protocol_id, person_id) ";
    $sql2 .= "\'$aid\', \'$hybrid_prot_id\', \'$person\')";
    $sql = $sql1 . $sql2;
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $hybridization_id;
}
