#!/usr/local/bin/perl



use DBI;

use DOMELIB;

use strict "vars";



#===============================================================================

# This script will read proteomics data file, checking for each 

# protein spot name for existance. If all names are existed in "reference_spot"

# then insert data into the table "sample", "sample_extract",

# "table_name", "proteomics_data". If any names are not in "rference_spot" 

# add the spot name in "reference_spot" and continue.

#

# The format for data file is:

# Name\tpI\tMW\tIntensity\tNorm_intensity\n

# name_1\tpi_1\tmw_1\tv1\tnv1\n

# namem_2\tpi_2\tmw_2\tv2\tnv2\n

# ...

# name_M\pi_M\mw_M\vM\nvM\n

# where vj, nvj, j = 1, ...M are the intensity and normalized intensity for

# proteomics.

# 

# arguments passed in to this script are:

# proc_data_file -- the full name of the input processed data file;

# raw_file -- the full name of the input raw data file;

# file_home -- the location where all files permenantly reside;

# technique_id -- for proteomics or other technique;

# sampling_replicate_id -- 

# technical _replicate -- 

# person_id -- who did the measurement

# extract_id --

# protocol_id -- for experimental protocol for proteomics 

# protocol_type_id -- protocol type for future analytical;

# tissue_type-id --

#

# Author: X. Jing Li

# Date: July 25, 2005

#  11/10/2005: change the code to fit the new schema for proteomics.

#==============================================================================



my ($proc_data_file, $raw_file, $file_home, $technique_id, $srid, $tech_repl,

    $person_id, $extract_id, $protocol_id, $s_ptid, $tissue_type_label_id,

    $kw_table, $kwid, 

    $database, $username, $password, $bnet_DB, $specieDB, $sql, $dbh, $sth, $rc,

    @temps, $proc_file_name, $raw_file_name, $pmid, $proc_file_id, $raw_file_id,

    @contents, $line, @fields, $i, %spots, %pIs, %MWs, %values, %norm_values,

    %file_spots, %found_spots, $all_names, $not_found_spots, $count_not_found_spots, $spot, $pi, $mw, 

    $intensity, $intensity_l, $norm_intensity, $norm_intensity_l, $ok,

    $proc_file_location_link_id, $proc_file_type_id, $raw_file_location_link_id, 

    $raw_file_type_id, $sample_id, $sample_extract_id, $sql1, $sql2 

    );

    

#== get the arguments:

($proc_data_file, $raw_file, $file_home, $technique_id, $srid, $tech_repl, $person_id, $extract_id, $protocol_id, $tissue_type_label_id) = @ARGV;

#print "proc_data_file = $proc_data_file, raw_file = $raw_file, file_home = $file_home, technique_id = $technique_id, srid = $srid, tech_repl = $tech_repl, person_id = $person_id, extract_id = $extract_id, protocol_id = $protocol_id, ttlid = $tissue_type_label_id\n";



#== global parameters:

$kw_table = "REFERENCE_SPOT";



#== 1. connect to the database:

($database, $username, $password, $bnet_DB, $specieDB) = database_login2();

$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";

$dbh->begin_work;

$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";

$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";



#== 2. get the kw_table_id for kw_table:

$sql = "select kw_table_id from ref_kw_table where upper(kw_table) = \'$kw_table\'";

$sth = $dbh->prepare($sql) or die "Can not prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can not execute: $DBI::errstr.";

$kwid = $sth->fetchrow();



#== get protocol_type_id for statistical:

$sql = "select \"protocol_type_id\" from \"protocol_type\" where \"technique_id\" = (select \"technique_id\" from \"extract\" where \"extract_id\" = $extract_id) AND \"method\" = \'statistical\'";

$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

$s_ptid = $sth->fetchrow();

 



#== 3. check if the files have been uploaded before:

@temps = split /\//, $proc_data_file;

$proc_file_name = pop @temps;

@temps = split /\//, $raw_file;

$raw_file_name = pop @temps;

#print "proc_file = $proc_file_name, raw_file_name = $raw_file_name\n";

$sql = "select proteomics_measurement_id from proteomics_measurement where proc_file_location_link_id in (select file_location_link_id from file_location_link where file_id in (select file_id from \"file\" where file_name = \'$proc_file_name\') and file_location_id in (select file_location_id from file_location where \"location\" = \'$file_home\')) or raw_file_location_link_id in (select file_location_link_id from file_location_link where file_id in (select file_id from \"file\" where file_name = \'$raw_file_name\') and file_location_id in (select file_location_id from file_location where \"location\" = \'$file_home\'))";

#print "$sql\n";

$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

$pmid = $sth->fetchrow();

if ($pmid ne "") {

    $sql = "select file_id from \"file\" where file_name = \'$proc_file_name\'";

    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $proc_file_id = $sth->fetchrow();

    $sql = "select file_id from \"file\" where file_name = \'$raw_file_name\'";

    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $raw_file_id = $sth->fetchrow();

    if ($raw_file_id ne "" && $proc_file_name ne "") {

        print "ERROR\tFiles $proc_file_name and $raw_file_name have been uploaded before\n";

        $dbh->rollback;

        $sth->finish;

        $dbh->disconnect;

        exit 1;

    } elsif ( $raw_file_id ne "") {

        print "ERROR\tFile $raw_file_name has been uploaded before\n";

        $dbh->rollback;

        $sth->finish;

        $dbh->disconnect;

        exit 1;

    } elsif ( $proc_file_id ne "") {

        print "ERROR\tFile $proc_file_name has been uploaded before\n";

        $dbh->rollback;

        $sth->finish;

        $dbh->disconnect;

        exit 1;

    }

}



#== if get here then the sample is a new one. start to parse the proc_file:

#== 4. read the input and output files:

open (IN, "<$proc_data_file") || die "Can not open the input file $proc_data_file.\n";

@contents = <IN>;

close IN;



#= 5. parse the lines and save the info in hashes:

$all_names = "";

for ($i = 1; $i<@contents; $i++) {

    $line = $contents[$i];

    $line =~ s/[\n\r]//g;

    chomp $line;

    @fields = split /\t/, $line;

    $spot = shift @fields;

    $spot =~ s/\'/\'\'/g;

    $spots{$i} = $spot;

    $file_spots{$spot} = $i;

    $pIs{$i} = shift @fields;

    $MWs{$i} = shift @fields;

    $values{$i} = shift @fields;

    $norm_values{$i} = shift @fields;

    $all_names .= ", \'" . $spot . "\'";

}

$all_names = substr($all_names, 2);



#== 7. check each spot_name in data file, if anyone is not in DOME.reference_spot then exit the script:

$sql = "select reference_spot_id from reference_spot where reference_spot_id in ($all_names)";

$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";

$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

while ($spot = $sth->fetchrow()) {

    $found_spots{$spot} = 1;

}

$not_found_spots = "";

$count_not_found_spots = 0;

foreach $spot (keys %file_spots) {

    if (! exists $found_spots{$spot}) {

        $not_found_spots .= ", $spot";

        $count_not_found_spots++;

    }

}

if ($not_found_spots ne "") {

    $not_found_spots = substr($not_found_spots, 2);

    if ($count_not_found_spots == 1) {

        print "The reference spot \"$not_found_spots\" is not in the catalog reference spot file.\n";

    } else {

        print "The reference spots \"$not_found_spots\" are not in the catalog reference spot file.\n";

    }

    $dbh->rollback;

    $sth->finish;

    $dbh->disconnect;

    exit 1;

}



#== 8. all names are in database. insert new records into "proteomics_data"

#= 8.1 copy the files into file_location, create file_id, file_location_link_id

#= for both raw_file and proc_file:

$proc_file_type_id = getFileTypeId($dbh, $proc_file_name);

if($proc_file_type_id eq '') {

    $proc_file_type_id = createFileTypeId($dbh, $proc_file_name, 'Proteomics processed data file');

    #print "proc_file_type_id = $proc_file_type_id\n";

}

$proc_file_location_link_id = createFileIds($dbh, $file_home, $proc_data_file, $proc_file_type_id, $person_id);

if ($raw_file_name ne "") {

    $raw_file_type_id = getFileTypeId($dbh, $raw_file_name);

    if($raw_file_type_id eq '') {

        $raw_file_type_id = createFileTypeId($dbh, $raw_file_name, 'Proteomics raw data file');

    }

    $raw_file_location_link_id = createFileIds($dbh, $file_home, $raw_file, $raw_file_type_id, $person_id);

}



#= 8.2 a new record in sample and sample_extract:

$sample_id = getSampleIdFromSampleExtract($dbh, $srid, $extract_id, $tissue_type_label_id);

if ($sample_id eq "") {

    $sample_id = createSampleId($dbh, $srid, $tissue_type_label_id);

    if ($sample_id eq "") {

        $dbh->rollback;

        $sth->finish;

        $dbh->disconnect;

        print "Failed to create \"sample_id\"\n";

        exit 1;

    }

}



#= 8.3 a new record in sample_extract:

#print "sid = $sample_id, extract_id = $extract_id, tech_repl = $tech_repl\n";

$sample_extract_id = getSampleExtractId($dbh, $sample_id, $extract_id, $tech_repl);

if($sample_extract_id eq "") {

    $sample_extract_id = createSampleExtractId($dbh, $sample_id, $extract_id, $tech_repl);

    if ($sample_extract_id eq "") {

        $dbh->rollback;

        $sth->finish;

        $dbh->disconnect;

        print "Failed to create \"sample_extract_id\"\n";

        exit 1;

    }

}



#= 8.4 new proteomics_measurement_ids:

$pmid = getProteomicsMeasurementId($dbh, $sample_extract_id, $protocol_id);

if ($pmid ne"") {

    $ok = updateProteomicsMeasurementWithFileLocationLinkIds($dbh, $pmid, $protocol_id, $proc_file_location_link_id, $raw_file_location_link_id);

} else {

    $pmid = createProteomicsMeasurementId($dbh, $sample_extract_id, $protocol_id, $proc_file_location_link_id, $raw_file_location_link_id);

    if ($pmid eq "") {

        $dbh->rollback;

        $sth->finish;

        $dbh->disconnect;

        print "Failed to create \"proteomics_measurement_id\"\n";

        exit 1;

    }

}



#= 8.5 insert data into "proteomics_data" table:

$sql = "";

foreach $i (sort {$a <=> $b} keys %spots) {

    $spot = $spots{$i};

    $intensity = $values{$i};

    $pi = $pIs{$i};

    $mw = $MWs{$i};

    $norm_intensity = $norm_values{$i};

    if ($intensity eq "" && $norm_intensity eq "") {

        next;

    } else {

        $sql1 = "insert into \"proteomics_data\" (proteomics_data_id, proteomics_measurement_id, reference_spot_id, protocol_type_id"; 

        $sql2 = "values (nextval('proteomics_data_id_seq'), $pmid, $spot, $s_ptid";

    }

    if ($intensity ne "") {

      $sql1 .= ", intensity";

      $sql2 .= ", $intensity";

    }

    if ($norm_intensity ne "") {

        $sql1 .= ", normalized_intensity";

        $sql2 .= ", $norm_intensity";

    }

    if ($intensity > 0) {

        $intensity_l = log($intensity) / log(2);

        $sql1 .= ", intensity_l";

        $sql2 .= ", $intensity_l";

    }

    if ($norm_intensity > 0) {

        $norm_intensity_l = log($norm_intensity) / log(2);

        $sql1 .= ", normalized_intensity_l";

        $sql2 .= ", $norm_intensity_l";

    }

    if ($pi ne "") {

        $sql1 .= ", \"pI\"";

        $sql2 .= ", $pi";

    }

    if ($mw ne "") {

        $sql1 .= ", \"MW\"";

        $sql2 .= ", $mw";

    }

    $sql1 .= ") ";

    $sql2 .= ");\n";

    $sql .= $sql1 . $sql2;

}

#open (TMP, ">/export/home/xingjing/data/dome/temp/readDataFromProt.log") or die "can not open temp file.\n";

#print TMP $sql;

#close TMP;



if ($sql ne "") {

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    print "Inserting proteomics data to DOME is successful.\n";

    $dbh->commit;

} else {

    print "No data has been inserted into DOME.\n";

    $dbh->rollback;

}

$sth->finish;

$dbh->disconnect;



exit;





###########################################################################

# subroutine: createReferenceSporId($dbh, $spot_name)

#

# parameters in: $dbh --- the database handle,

#                $spot_name --- the file home

# parameter out: $spot_id --- the new reference_spot_id.

#

# X. Jing Li, 7/27/2005

###########################################################################

sub createReferenceSpotId {

    my ($dbh, $spot_name) = @_;

    my ($sql, $sth, $rc, $spot_id);



    $sql = "select nextval('reference_spot_id_seq')";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $spot_id = $sth->fetchrow();



    $sql = "insert into reference_spot (reference_spot_id, \"name\") values ($spot_id, \'$spot_name\')";

    #print "$sql\n";

    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";



    $sth->finish;

    return $spot_id;

}



###########################################################################

# subroutine: getSampleIdFromSampleExtract($dbh, $srid):

# Get the sample_id from database.

#

# parameters in: $dbh --- the database handle;

#                $srid --- sampling_replicate_id

# parameter out: $sample_id --- the sample_id associated with the 

#                    sampling_replicate_id..

# X. Jing Li. 8/31/2004

###########################################################################

sub getSampleIdFromSampleExtract {

    my ($dbh, $srid, $extract_id, $tissie_type_label_id) = @_;

    my ($sql, $sth, $rc, $sample_id);



    $sql = "select sample_id from sample_extract where extract_id = $extract_id and sample_id in (select sample_id from sample where sampling_replicate_id = $srid and label_id = $tissie_type_label_id)";

    $sth = $dbh->prepare($sql) or die "Can't prepare select: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";

    $sample_id = $sth->fetchrow;

    $sth->finish;



    return $sample_id;

}



###########################################################################

# getProteomicsMeasurementId($dbh, $sample_extract_id, $protocol_id):

#  this subroutine create a proteomics_measurement_id in proteomics_measurement

#  table

# 

# parameters in: $dbh --- database handle;

#                $sample_extract_id --- ;

#                $protocol_id --- the protocol_id for experimental protocol_type_id for proteomics;

# parameter out: $pmid --- the new proteomics_measurement_id; 

# X. Jing Li, 11/10/2005.

###########################################################################

sub getProteomicsMeasurementId {

    my ($dbh, $sample_extract_id, $protocol_type_id) = @_;

    my ($sql, $sth, $rc, $pmid);



    $sql = "select proteomics_measurement_id from proteomics_measurement where sample_extract_id = $sample_extract_id AND protocol_type_id = (select protocol_type_id from protocol where protocol_id = $protocol_id)";

#    print "$sql\n";

    $sth = $dbh->prepare($sql) or die "Can't prepare select: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";

    $pmid = $sth->fetchrow;

    $sth->finish;



    return $pmid;

}



###########################################################################

# updateProteomicsMeasurementWithFileLocationLinkIds 

#   ($dbh, $proc_file_location_link_id, $raw_file_location_link_id):

#  this subroutine update a proteomics_measurement record in 

# proteomics_measurement table with file_location_link_ids for processed data

# file and raw data file.

# 

# parameters in: $dbh --- database handle;

#                $proc_file_licationlink_id --- ;

#                $raw_file_location_link_id ---

# parameter out: $ok --- 1 for success and other numbers for failed; 

# X. Jing Li, 11/10/2005.

###########################################################################

sub updateProteomicsMeasurementWithFileLocationLinkIds {

    my ($dbh, $pmid, $protocol_id, $proc_file_location_link_id, $raw_file_location_link_id) = @_;

    my ($sql, $sth, $rc, $ok);



    $sql = "update proteomics_measurement set ";

    if ($proc_file_location_link_id ne "") {

        $sql .= "proc_file_location_link_id = $proc_file_location_link_id";

        if ($raw_file_location_link_id ne "") {

            $sql .= ", raw_file_location_link_id = $raw_file_location_link_id ";

        }

    } elsif ($raw_file_location_link_id ne "") {

        $sql .= "raw_file_location_link_id = $raw_file_location_link_id ";

    } else {

        return 99;

    }

    $sql .= " where proteomics_measurement_id = $pmid and protocol_type_id = (select protocol_type_id from protocol where protocol_id = $protocol_id)";

#    print "$sql\n";

    $sth = $dbh->prepare($sql) or die "Can't prepare select: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";

    $ok = $rc;

    $sth->finish;



    return $ok;

}



###########################################################################

# subroutine: createPrteomicsMeasurementId

# ($dbh, $sample_extract_id, $protocol_id, $proc_file_location_link_id, ):

#  this subroutine create a proteomics_measurement_id in proteomics_measurement

#  table

# 

# parameters in: $dbh --- database handle;

#                $sample_extract_id --- ;

#                $protocol_id --- protocol_id for experimental protocol_type_id

#                $proc_file_location_link_id ---

#                $raw__file_location_link_id --- the replicate number;

# parameter out: $proteomics_measurement_id --- the new proteomics_measurement

#                                               _id; 

# X. Jing Li, 7/27/2005.

###########################################################################

sub createProteomicsMeasurementId {

    my ($dbh, $sample_extract_id, $protocol_id, $proc_file_location_link_id, $raw_file_location_link_id) = @_;

    my ($sql, $sql2, $sth, $rc, $protocol_type_id, $proteomics_measurement_id);



    $sth = $dbh->prepare("select nextval('proteomics_measurement_id_seq')") 

        or die "Can not prepare select nextval from dual: $DBI::errstr";

    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";

    $proteomics_measurement_id = $sth->fetchrow;



    $sql = "select protocol_type_id from protocol where protocol_id = $protocol_id";

    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $protocol_type_id = $sth->fetchrow();



    $sql = "insert into proteomics_measurement (proteomics_measurement_id, sample_extract_id, protocol_type_id, proc_file_location_link_id";

    $sql2 = " values ($proteomics_measurement_id, $sample_extract_id, $protocol_type_id, $proc_file_location_link_id";

    if ($raw_file_location_link_id ne "") {

        $sql .= ", raw_file_location_link_id";

        $sql2 .= ", $raw_file_location_link_id";

    }

    $sql .= ")" . $sql2 . ")";

    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";

    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";



    $sth->finish;

    return $proteomics_measurement_id;

}

###########################################################################

# subroutine:  createCpdNameByMWPI($prefix, $ret_time, $peak_ion)

#   This subroutine will create a system name for BNET and DOME according to

# the retention time and peak ion. The prefix is formatted in the way of 

# XX -- the PI or Lab's first two initials. ex. LS for Lloyd Sumner.

# YY -- the technique for extraction. ex. G1 for GC/MS with settings from ROBLE

# Pn -- 

#

# The molecular weight and ion are reformatted as the following:

# AAAAAABBB -- the molecular weight padded with 0s. AAAAAA is the integer part

#     and BBB is the fractional part. ex. 12345.78 will like 012345780.

# CCDDD -- ion padded with 0s. CC is the integer part and DDD is the decimal.

#

# parameters in: $prefix -- the prefix;

#                $mw -- the molecular weight;

#                $pi -- the ion;

# parameter out: $sysname -- BNET system name for unknowns.

#

##############################################################################

sub createProteinNameByMWPI {

    my ($prefix, $mw, $pi) = @_;

    my ($int, $dec, $sysname);



    ($int, $dec) = split /\./, $pi;

    $int = prepad($int, 2);

    $dec = postpad($dec, 3);

    $pi = $int . $dec;

    ($int, $dec) = split /\./, $mw;

    $int = prepad($int, 6);

    $dec = postpad($dec, 3);

    $mw = $int . $dec;

    $sysname = $prefix . "-" . $mw . "-" . $pi;

    return $sysname;

}



###########################################################################

# subroutine: prepad($str, $digit):

#  this subroutine padding 0s in front of the string

# 

# parameters in: $str --- the str need padding;

#                $digit --- the length of finished str;

# parameter out: $str --- the str after padding with 0s in front. 

#

###########################################################################

sub prepad {

    my ($str, $digit) = @_;



    while (length $str < $digit) {

        $str = "0" . $str;

    }

    return $str;

}



###########################################################################

# subroutine: postpad($str, $digit):

#  this subroutine padding 0s at the end of the string

# 

# parameters in: $str --- the str need padding;

#                $digit --- the length of finished str;

# parameter out: $str --- the str after padding with 0s at the end. 

#

###########################################################################

sub postpad {

    my ($str, $digit) = @_;



    while (length $str < $digit) {

        $str = $str . "0";

    }

    return $str;

}

