<?php
/**
* This script provides the user with a file containing 
* updates to B-NET between a specified range of dates.
* All Updates are stored in a hash.
* Every individual table belongs to a group.
* Groups are done in order to be able to retrieve entities
* while displaying updates on the Front-End.
* Hence this hash containing updates is referenced as follows
* $updatesHash[TABLE_GROUP][UNIQUE_KEY_FOR_THIS RECORD][TABLE_NAME]
* The location referenced above contains the new record (as an array
* mostly of fields other than the UNIQUE_KEY) when there is
* an Insert or Update. In case of a Delete it contains the value 'D'.
* Note that there can be an additional unique key for the Hash
* $updatesHash after the three keys described above in the cases
* where more than one update can be done on the same table for a particular
* unique group.
* 
* Date: 19 Oct 2004
* @author Aejaaz Kamal <aekamal@vt.edu>
*/

include_once("common.php");
include_once("workFlow.php");
//Parse Get or Post step number and set the current step number.
setStepNumber();

//Sets Environment Variable
setVariables($_SESSION['STEP_NUMBER']);
doControlBarRedirection(1);  
$currPageNameUpper = strtoupper($currPageName);
$title = "DOME : Send Updates";
include("header.php");
?>
	

<?php
if ($errMsgs)
{
	foreach ($errMsgs as $msg)
	{
		printError($msg);
	}
}

fillForm();	
include ("footer.php");
?>


<?php
//Functions

function fillForm()
{
	global $currPageName;
	$currPageNameUpper = strtoupper($currPageName);
	print "\n<form name=\"searchCpd\" method=\"post\" action=\"{$_SERVER['PHP_SELF']}\" >\n";
	
	//Print Updates From 
	print "<p><label>Updates From Date and Time (yyyy-mm-dd HH:MM:SS): </label>";
	print "<input type=\"text\" name=\"updatesFrom\" value=\"{$_SESSION[$currPageNameUpper]['updatesFrom']}\" size=\"19\"></p>";
	
	//Print Updates To 
	print "<p><label>Updates To Date and Time (yyyy-mm-dd HH:MM:SS): </label>";
	print "<input type=\"text\" name=\"updatesTo\" value=\"";
	print date('Y-m-d H:i:s');	//Default Current Date
	print "\" size=\"19\"></p>";
	
	//Print Update Button
	print "\n<input type=\"hidden\" name=\"submit\" value=\"NEXT >\" />";
	print "\n<p><input type=\"submit\" name=\"sub\" value=\"Get Updates File\"></p>";
	print "</form>";
}




/*==========================================
Request Functions
============================================*/
function processForm($currPageName = "")
{
	global $errMsgs;
	if ($_REQUEST['sub'] == 'Get Updates File')
	{
		$_SESSION[$currPageNameUpper]['updatesFrom'] = $_REQUEST['updatesFrom'];
		$_SESSION[$currPageNameUpper]['updatesTo'] = $_REQUEST['updatesTo'];
	
		if (!$_SESSION[$currPageNameUpper]['updatesFrom'])
		{
			$errMsgs[] = "Please Enter the date to process updates from.";
			return 0;	
		}
		if (!$_SESSION[$currPageNameUpper]['updatesTo'])
		{
			$errMsgs[] = "Please Enter the date to process updates to.";
			return 0;	
		}
		$ret = fillUpdatesHash($updatesHash, $_SESSION[$currPageNameUpper]['updatesFrom'], $_SESSION[$currPageNameUpper]['updatesTo']);
		
		header('Content-Disposition: attachment; filename="updates.txt"');
		print writeUpdatesFile($updatesHash, $_SESSION[$currPageNameUpper]['updatesFrom'], $_SESSION[$currPageNameUpper]['updatesTo']);
		exit();
	}
}

function fillUpdatesHash(&$updatesHash, $from, $to)
{
	global $errMsgs;
	
	$sql = "SELECT u.kw_table_id, u.record_id, UPPER(r.kw_table), UPPER(u.operation), u.identifier FROM update u, ref_kw_table r
		WHERE time >= '$from' AND time <= '$to'
		AND r.kw_table_id = u.kw_table_id 
		ORDER BY time";

	//print $sql;
	$sth = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sth) ) 
	{	
		$errMsgs[] = "Error retrieving updates from 'update' table. (" . $sth->getMessage() . ")";
		return 0;
	}
	if ($sth->numrows() == 0)
	{
		$errMsgs[] = "No updates Found between the timestamps '$from' AND '$to'";
		return 0;	
	}
	while ($row = $sth->fetchRow()) 
	{
		switch($row[2])
		{
		case 'ACTIVITY':
			fillHashForActivity($row, $updatesHash);
			break;
		case 'ALLNAMES':
			fillHashForAllNames($row, $updatesHash);
			break;
		case 'COMPLEX':
			fillHashForComplex($row, $updatesHash);
			break;
		case 'COMPLEX_MEMBER':
			fillHashForComplexMember($row, $updatesHash);
			break;
		case 'COMPOUND':
			fillHashForCompound($row, $updatesHash);
			break;
		case 'CPD_DERIVATIVE':
			fillHashForCompoundDer($row, $updatesHash);
			break;
		case 'EC':
			fillHashForEC($row, $updatesHash);
			break;
		case 'EVENT':
			fillHashForEvent($row, $updatesHash);
			break;
		case 'EVENT_COMPONENT':
			fillHashForEventComponent($row, $updatesHash);
			break;
		case 'EVENT_EC':
			fillHashForEventEC($row, $updatesHash);
			break;
		case 'EVENT_TYPE':
			fillHashForEventType($row, $updatesHash);
			break;
		case 'EVIDENCE':
			fillHashForEvidence($row, $updatesHash);
			break;
		case 'EVIDENCE_CODE':
			fillHashForEvidenceCode($row, $updatesHash);
			break;
		case 'EXTERNAL_DATABASE':
			fillHashForExternalDatabase($row, $updatesHash);
			break;
		case 'EXTERNAL_LINKS':
			fillHashForExternalLinks($row, $updatesHash);
			break;
		case 'GENE':
			fillHashForGene($row, $updatesHash);
			break;
		case 'GO_ACCESSION':
			break;
		case 'LOCATION':
			fillHashForLocation($row, $updatesHash);
			break;
		case 'MRNA':
			fillHashForMrna($row, $updatesHash);
			break;
		case 'PATHWAY':
			break;
		case 'PATHWAY_EVENT':
			break;
		case 'PROTEIN':
			fillHashForProtein($row, $updatesHash);
			break;
		case 'PROTEIN_EC':
			fillHashForProteinEC($row, $updatesHash);
			break;
		case 'REFERENCE':
			fillHashForReference($row, $updatesHash);
			break;
		case 'SPECIE_COMPOUND':
			fillHashForSpecieCompound($row, $updatesHash);
			break;
		case 'SPECIES':
			fillHashForSpecies($row, $updatesHash);
			break;
		}
	}
	return 1;
}


function fillHashForActivity(&$row, &$updatesHash)
{
	global $errMsgs;
	
	//processing event_id
	$sql = SELECT an.aname FROM allnames an, activity a
		   	WHERE a.activity_id = '$row[1]'
		   	AND an.record_id = a.event_id
			AND an.kw_table_id = '{$_SESSION['TableID']['EVENT']}'
			AND UPPER(an.is_primary) = 'Y';
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving event information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{	$identifier = $rowR[0];	}
	else
	{	$identifier = 'NULL';	}
	
	//processing ec_id
	$sql = SELECT an.aname FROM allnames an, activity a
		   	WHERE a.activity_id = '$row[1]'
		   	AND an.record_id = a.ec_id
			AND an.kw_table_id = '{$_SESSION['TableID']['EC']}'
			AND UPPER(an.is_primary) = 'Y';
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving ec information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{	$identifier .= ":$rowR[0];	}
	else
	{	$identifier .= ':NULL';	}
				
	//processing protein_state_id
	$sql = SELECT an.aname, s.type FROM 
		allnames an, protein_state ps, state s, activity a
		    		WHERE a.activity_id = '$row[1]'
		    		AND ps.protein_state_id = a.protein_state_id
		    		AND an.record_id = ps.protein_id
					AND an.kw_table_id = '{$_SESSION['TableID']['PROTEIN']}'
					AND UPPER(an.is_primary) = 'Y'
					AND s.state_id = ps.state_id;
	
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving protein_state information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{	$identifier .= "::$rowR[0]:$rowR[1]::";	}
	else
	{	$identifier .= "::NULL::";	}
				
	if ($row[3] == 'I' || $row[3] == 'U')
	{			
		$updatesHash[$row[2]][$identifier][$row[2]] = array();
	}
	elseif ($row[3] == 'D')
	{
		$updatesHash[$row[2]][$row[4]][$row[2]] = 'D';
	}
}

function fillHashForAllNames(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an2.aname as primary_name, UPPER(r.kw_table), an1.aname, UPPER(an1.is_primary)
		 FROM  allnames an1, allnames an2, ref_kw_table r
		WHERE an1.name_id = '$row[1]'
		AND an1.kw_table_id = r.kw_table_id
		AND an2.record_id = an1.record_id
		AND an2.kw_table_id = an1.kw_table_id
		AND UPPER(an2.is_primary) = 'Y'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			if ($rowR[3] == 'N')
			{
				$updatesHash[$rowR[1]][$rowR[0]][$row[2]][$rowR[2]] = array($rowR[3]);
			}
		}
	}
	elseif ($row[3] == 'D')
	{	
		$rowR = split(':', $row[4]);
		if ($rowR[3] == 'N')
		{
			$updatesHash[$rowR[1]][$rowR[0]][$row[2]][$rowR[2]] = 'D';	
		}
	}
}

function fillHashForComplex(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an.aname, l.location_name FROM complex c, allnames an 
		WHERE c.complex_id = '$row[1]'
		AND c.location_id = l.location_id
		AND an.record_id = '$row[1]'
		AND an.kw_table_id = '{$_SESSION['TableID']['COMPLEX']}'
		AND UPPER(an.is_primary) = 'Y'";

	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash['COMPLEX'][$rowR[0]][$row[2]] = array($rowR[1]);
		}
	}
	elseif ($row[3] == 'D')
	{	$updatesHash['COMPLEX'][$row[4]][$row[2]] = 'D';	}
}

function fillHashForComplexMember(&$row, &$updatesHash)
{
	global $errMsgs;
	//Belongs to 'COMPLEX' group. Will have more than one members,
	//Hence, we use another hash with primary name of 
	//the complex_member as key of this hash
	//(instead of a single value) to store in the
	//main ($updatesHash) data structure.
	$sql = "SELECT an1.aname as complex_pn, UPPER(r.kw_table), an2.aname as complex_member_pn,
		 cm.stoichiometry
		 FROM  complex_member cm, allnames an1, allnames an2, ref_kw_table r
		WHERE cm.complex_member_id = '$row[1]'
		AND cm.kw_table_id = r.kw_table_id
		AND an1.record_id = cm.complex_id
		AND an1.kw_table_id = '{$_SESSION['TableID']['COMPLEX']}'
		AND UPPER(an1.is_primary) = 'Y'
		AND an2.record_id = cm.record_id
		AND an2.kw_table_id = cm.kw_table_id
		AND UPPER(an2.is_primary) = 'Y'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$unique = "$rowR[1]:$rowR[2]";
			$updatesHash['COMPLEX'][$rowR[0]][$row[2]][$unique] = array($rowR[3]);
		}
	}
	elseif ($row[3] == 'D')
	{
		$rowR = split(':', $row[4]);
		$unique = "$rowR[1]:$rowR[2]";
		$updatesHash['COMPLEX'][$rowR[0]][$row[2]][$rowR[1]] = 'D';	
	}
}

function fillHashForCompound(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an.aname, c.cpd_formula, c.mol_wt FROM compound c, allnames an 
		WHERE c.cpd_id = '$row[1]'
		AND an.record_id = '$row[1]'
		AND an.kw_table_id = '{$_SESSION['TableID']['COMPOUND']}'
		AND UPPER(an.is_primary) = 'Y'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash['COMPOUND'][$rowR[0]][$row[2]] = array($rowR[1], $rowR[2]);
		}
	}
	elseif ($row[3] == 'D')
	{	$updatesHash['COMPOUND'][$row[4]][$row[2]] = 'D';	}
}

function fillHashForCompoundDer(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an.aname, cd.name FROM cpd_derivative cd, allnames an 
		WHERE cd.cpd_derivative_id = '$row[1]'
		AND an.record_id = cd.cpd_id
		AND an.kw_table_id = '{$_SESSION['TableID']['COMPOUND']}'
		AND UPPER(an.is_primary) = 'Y'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash['COMPOUND'][$rowR[0]][$row[2]][$rowR[1]] = array();
		}
	}
	elseif ($row[3] == 'D')
	{
		$rowR = split(':', $row[4]);
		$updatesHash['COMPOUND'][$rowR[0]][$row[2]][$rowR[1]] = 'D';
	}
}

function fillHashForEC(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an.aname, ec.ec_name FROM ec, allnames an
		WHERE ec.ec_id = '$row[1]'
		AND an.record_id = '$row[1]'
		AND an.kw_table_id = '{$_SESSION['TableID']['EC']}'
		AND UPPER(an.is_primary) = 'Y'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash[$row[2]][$rowR[0]][$row[2]] = array($rowR[1]);
		}
	}
	elseif ($row[3] == 'D')
	{
		$rowR = split(':', $row[4]);
		$updatesHash[$row[2]][$rowR[0]][$row[2]] = 'D';
	}
}

function fillHashForEvent(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an.aname, et.event_type FROM event e, allnames an, event_type et
		WHERE e.event_id = '$row[1]'
		AND an.record_id = '$row[1]'
		AND an.kw_table_id = '{$_SESSION['TableID']['EVENT']}'
		AND UPPER(an.is_primary) = 'Y'
		AND e.event_type_id = et.event_type_id";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash[$row[2]][$rowR[0]][$row[2]] = array($rowR[1]);
		}
	}
	elseif ($row[3] == 'D')
	{
		$rowR = split(':', $row[4]);
		$updatesHash[$row[2]][$rowR[0]][$row[2]] = 'D';
	}
}

function fillHashForEventComponent(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an1.aname as event_pn, UPPER(r.kw_table), an2.aname as ec_pn, UPPER(ec.is_substrate), 
		ec.stoichiometry, l.location_name
		FROM event_component ec, allnames an1, allnames an2, ref_kw_table r, location l
		WHERE ec.event_component_id = '$row[1]'
		AND ec.location_id = l.location_id
		AND ec.kw_table_id = r.kw_table_id
		AND an2.record_id = ec.record_id
		AND an2.kw_table_id = ec.kw_table_id
		AND UPPER(an2.is_primary) = 'Y'
		AND an1.record_id = ec.event_id
		AND an1.kw_table_id = '{$_SESSION['TableID']['EVENT']}'
		AND UPPER(an1.is_primary) = 'Y'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$unique = "$rowR[1]:$rowR[2]:$rowR[3]";
			$updatesHash['EVENT'][$rowR[0]][$row[2]][$unique] = array($rowR[4], $rowR[5]);
		}
	}
	elseif ($row[3] == 'D')
	{
		
		$rowR = split(':', $row[4]);
		$unique = "$rowR[1]:$rowR[2]:$rowR[3]";
		$updatesHash['EVENT'][$rowR[0]][$row[2]][$unique] = 'D';
	}
}

function fillHashForEventEC(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an.aname, ec.ec_name FROM event_ec eec, allnames an, ec
		WHERE eec.event_ec_id = '$row[1]'
		AND ec.ec_id = eec.ec_id
		AND an.record_id = eec.event_id
		AND an.kw_table_id = '{$_SESSION['TableID']['EVENT']}'
		AND UPPER(an.is_primary) = 'Y'";
		
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash['EVENT'][$rowR[0]][$row[2]][$rowR[1]] = array();
		}
	}
	elseif ($row[3] == 'D')
	{
		$rowR = split(':', $row[4]);
		$updatesHash['EVENT'][$rowR[0]][$row[2]][$rowR[1]] = 'D';
	}
}
	
function fillHashForEventType(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT et.event_type_name FROM event_type et
		WHERE et.event_type_id = '$row[1]'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash[$row[2]][$rowR[0]][$row[2]] = array();
		}
	}
	elseif ($row[3] == 'D')
	{
		$rowR = split(':', $row[4]);
		$updatesHash[$row[2]][$rowR[0]][$row[2]] = 'D';
	}
}

function fillHashForEvidence(&$row, &$updatesHash)
{
	global $errMsgs;
	
	$sql = "SELECT ev.kw_table_id FROM evidence ev WHERE ev.evidence_id = '$row[1]'";
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($rowR[0] == $_SESSION['TableID']['SPECIE_COMPOUND'])
	{
		$sql = "SELECT an.aname as pn, UPPER(r1.kw_table), 
			ref.locator, ec.evidence_code, UPPER(r2.kw_table), 
			ec.technique, ec.strength
			 FROM  evidence ev, allnames an, specie_compound  sc, reference ref, 
			 evidence_code ec, ref_kw_table r1, ref_kw_table r2
			WHERE ev.evidence_id = '$row[1]'
			AND ev.reference_id = ref.reference_id
			AND ev.evidence_code_id = ec.evidence_code_id
			AND ev.kw_table_id = r1.kw_table_id
			AND ec.kw_table_id = r2.kw_table_id
			AND sc.specie_cpd_id = ev.record_id
			AND an.record_id = sc.cpd_id
			AND an.kw_table_id = '{$_SESSION['TableID']['COMPOUND']}'
			AND UPPER(an.is_primary) = 'Y'";
	}
	else
	{
		$sql = "SELECT an.aname as pn, UPPER(r1.kw_table), 
			ref.locator, ec.evidence_code, UPPER(r2.kw_table), 
			ec.technique, ec.strength
			 FROM  evidence ev, allnames an, reference ref, 
			 evidence_code ec, ref_kw_table r1, ref_kw_table r2
			WHERE ev.evidence_id = '$row[1]'
			AND ev.reference_id = ref.reference_id
			AND ev.evidence_code_id = ec.evidence_code_id
			AND ev.kw_table_id = r1.kw_table_id
			AND ec.kw_table_id = r2.kw_table_id
			AND an.record_id = ev.record_id
			AND an.kw_table_id = ev.kw_table_id
			AND UPPER(an.is_primary) = 'Y'";
	}
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	//Evidences pointing to SPECIE_COMPOUND are to be treated in the
	//COMPOUND group.

	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($rowR[1] == 'SPECIE_COMPOUND')
		{	$rowR[1] = 'COMPOUND';	}
		$unique = "$rowR[2]::$rowR[3]:$rowR[4]:$rowR[5]:$rowR[6]::";
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash[$rowR[1]][$rowR[0]][$row[2]][$unique] = array();
		}
	}
	elseif ($row[3] == 'D')
	{
		$records = split('::', $row[4]);
		$rowR = split(':', $records[0]);
		
		//since locator may contain the separator character ':'
		$locator = join(':', array_slice($rowR, 2));	
		
		if ($rowR[1] == 'SPECIE_COMPOUND')
		{	$rowR[1] = 'COMPOUND';	}
		
		$unique = "$locator::$records[1]::$records[2]";
		$updatesHash[$rowR[1]][$rowR[0]][$row[2]][$unique] = 'D';
	}
}

function fillHashForEvidenceCode(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT ec.evidence_code, UPPER(r.kw_table), ec.technique, ec.strength 
		FROM evidence_code ec, ref_kw_table r
		WHERE ec.evidence_code_id = '$row[1]'
		AND ec.kw_table_id = r.kw_table_id";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		$unique = "$rowR[0]:$rowR[1]:$rowR[2]:$rowR[3]";
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash[$row[2]][$unique][$row[2]] = array();
		}
	}
	elseif ($row[3] == 'D')
	{
		
		/*$rowR = split(':', $row[4]);
		$unique = "$rowR[0]:$rowR[1]:$rowR[2]:$rowR[3]";*/
		$updatesHash[$row[2]][$row[4]][$row[2]] = 'D';
	}
}

function fillHashForExternalDatabase(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT ed.full_db_name, ed.database_name, ed.url
		FROM external_database ed
		WHERE ed.external_database_id = '$row[1]'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash[$row[2]][$rowR[0]][$row[2]] = array($rowR[1], $rowR[2]);
		}
	}
	elseif ($row[3] == 'D')
	{
		
		$rowR = split(':', $row[4]);
		$updatesHash[$row[2]][$rowR[0]][$row[2]] = 'D';
	}
}

function fillHashForExternalLinks(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an.aname as el_pn, UPPER(r.kw_table), ed.full_db_name,
		el.external_table_name, el.external_field_name, el.internal_field_name, 
		el.external_id
		FROM external_links el, allnames an, ref_kw_table r, external_database ed
		WHERE el.external_links_id = '$row[1]'
		AND ed.external_database_id = el.external_database_id
		AND el.kw_table_id = r.kw_table_id
		AND an.record_id = el.record_id
		AND an.kw_table_id = el.kw_table_id
		AND UPPER(an.is_primary) = 'Y'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash[$rowR[1]][$rowR[0]][$row[2]][$rowR[2]] = array($rowR[3], $rowR[4], $rowR[5], $rowR[6]);
		}
	}
	elseif ($row[3] == 'D')
	{
		
		$rowR = split(':', $row[4]);
		$updatesHash[$rowR[1]][$rowR[0]][$row[2]][$rowR[2]] = 'D';
	}
}

function fillHashForGene(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an1.aname, an2.aname FROM gene g, allnames an1, allnames an2
		WHERE g.protein_id = '$row[1]'
		AND an1.record_id = '$row[1]'
		AND an1.kw_table_id = '{$_SESSION['TableID']['GENE']}'
		AND UPPER(an1.is_primary) = 'Y'
		AND an2.record_id = g.specie_id
		AND an2.kw_table_id = '{$_SESSION['TableID']['SPECIES']}'
		AND UPPER(an2.is_primary) = 'Y'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash[$row[2]][$rowR[0]][$row[2]] = array($rowR[1]);
		}
	}
	elseif ($row[3] == 'D')
	{	$updatesHash['GENE'][$row[4]][$row[2]] = 'D';	}
}

function fillHashForLocation(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT l.location_name FROM location l
		WHERE l.location_id = '$row[1]'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash[$row[2]][$rowR[0]][$row[2]] = array();
		}
	}
	elseif ($row[3] == 'D')
	{
		$rowR = split(':', $row[4]);
		$updatesHash[$row[2]][$rowR[0]][$row[2]] = 'D';
	}
}

function fillHashForMrna(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an1.aname, an2.aname FROM mrna m, allnames an1, allnames an2, allnames an3
		WHERE m.mrna_id = '$row[1]'
		AND an1.record_id = '$row[1]'
		AND an1.kw_table_id = '{$_SESSION['TableID']['MRNA']}'
		AND UPPER(an1.is_primary) = 'Y'
		AND an2.record_id = m.gene_id
		AND an2.kw_table_id = '{$_SESSION['TableID']['GENE']}'
		AND UPPER(an2.is_primary) = 'Y'
		AND an3.record_id = m.protein_id
		AND an3.kw_table_id = '{$_SESSION['TableID']['PROTEIN']}'
		AND UPPER(an3.is_primary) = 'Y'";
		
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash['MRNA'][$rowR[0]][$row[2]] = array($rowR[1], $rowR[2]);
		}
	}
	elseif ($row[3] == 'D')
	{
		$rowR = split(':', $row[4]);
		$updatesHash['MRNA'][$rowR[0]][$row[2]] = 'D';
	}
}

function fillHashForProtein(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an1.aname, an2.aname FROM protein p, allnames an1, allnames an2
		WHERE p.protein_id = '$row[1]'
		AND an1.record_id = '$row[1]'
		AND an1.kw_table_id = '{$_SESSION['TableID']['PROTEIN']}'
		AND UPPER(an1.is_primary) = 'Y'
		AND an2.record_id = p.specie_id
		AND an2.kw_table_id = '{$_SESSION['TableID']['SPECIES']}'
		AND UPPER(an2.is_primary) = 'Y'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash['PROTEIN'][$rowR[0]][$row[2]] = array($rowR[1]);
		}
	}
	elseif ($row[3] == 'D')
	{	$updatesHash['PROTEIN'][$row[4]][$row[2]] = 'D';	}
}

function fillHashForProteinEC(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an.aname, ec.ec_name FROM protein_ec pec, allnames an, ec
		WHERE pec.protein_ec_id = '$row[1]'
		AND ec.ec_id = pec.ec_id
		AND an.record_id = pec.protein_id
		AND an.kw_table_id = '{$_SESSION['TableID']['PROTEIN']}'
		AND UPPER(an.is_primary) = 'Y'";
		
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash['PROTEIN'][$rowR[0]][$row[2]][$rowR[1]] = array();
		}
	}
	elseif ($row[3] == 'D')
	{
		$rowR = split(':', $row[4]);
		$updatesHash['PROTEIN'][$rowR[0]][$row[2]][$rowR[1]] = 'D';
	}
}

function fillHashForReference(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT ref.locator, ref.authors, ref.year, ref.title, 
		ref.pubmed_accession 
		FROM reference ref
		WHERE ref.reference_id = '$row[1]'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash[$row[2]][$rowR[0]][$row[2]] = array($rowR[1], $rowR[2], $rowR[3], $rowR[4]);
		}
	}
	elseif ($row[3] == 'D')
	{
		$rowR = split(':', $row[4]);
		$updatesHash[$row[2]][$rowR[0]][$row[2]] = 'D';
	}
}

function fillHashForSpecieCompound(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an1.aname as cpd_pn, an2.aname as specie_pn
			FROM  allnames an1, allnames an2, specie_compound sc
		WHERE sc.specie_cpd_id = '$row[1]'
		AND an1.record_id = sc.cpd_id
		AND an1.kw_table_id = '{$_SESSION['TableID']['COMPOUND']}'
		AND UPPER(an1.is_primary) = 'Y'
		AND an2.record_id = sc.specie_id
		AND an2.kw_table_id = '{$_SESSION['TableID']['SPECIES']}'
		AND UPPER(an2.is_primary) = 'Y'";

	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash['COMPOUND'][$rowR[0]][$row[2]][$rowR[1]] = array();
		}
	}
	elseif ($row[3] == 'D')
	{
		$rowR = split(':', $row[4]);
		$updatesHash['COMPOUND'][$rowR[0]][$row[2]][$rowR[1]] = 'D';
	}
}

function fillHashForSpecies(&$row, &$updatesHash)
{
	global $errMsgs;
	$sql = "SELECT an.aname as species_pn, s.taxon_ncbi_accession 
		FROM  allnames an, species s
		WHERE s.specie_id = '$row[1]'
		AND an.record_id = '$row[1]'
		AND an.kw_table_id = '{$_SESSION['TableID']['SPECIES']}'
		AND UPPER(an.is_primary) = 'Y'";
		
	$sthR = $_SESSION['dbh']->query($sql);
	if ( DB::isError($sthR) ) 
	{	
		$errMsgs[] = "Error retrieving information for '$row[2]' with ID '$row[1]'. (" . $sthR->getMessage() . ")";
		return 0;
	}
	$rowR = $sthR->fetchRow();
	if ($sthR->numRows() > 0)
	{ 
		if ($row[3] == 'I' || $row[3] == 'U')
		{
			$updatesHash[$row[2]][$rowR[0]][$row[2]] = array($rowR[1]);
		}
	}
	elseif ($row[3] == 'D')
	{	$updatesHash['SPECIES'][$row[4]][$row[2]] = 'D';	}
}

function writeUpdatesFile(&$updatesHash, $from, $to)
{
	$exportable = var_export($updatesHash, true);
	$contents = '<?php' . "\n\n/**Updates sent by:\n*User: {$_SESSION['verified_user']}\n*From: $from\n*To: $to\n*/\n";
	$contents .=  ' $' . updatesHash . ' = ' . $exportable . ';' . "\n\n" . '?>';
	return $contents;
}

?>
