#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script will get data from user created data file and insert data into
# sp_comparison.
#
# author: X. Jing Li
# date: Oct. 18, 2005
# reversion:
# 8/3/2006: add ratio_of_medians, ratio_of_medians_l, difference_of_medians, 
#   difference_of_medians_l in uploading file.
#===============================================================================

my ($input_file, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2,
    $extract_id, $protocol_id, $is_logged, $force_add,
    $database, $username, $password, $bnet_DB, $specieDB, $dbh, $sth, $rc, 
    $sql, $sql1, $sql2, $main_sql, $kw_table_id, $count, @lines, $line, $i,
    $process1_id, $rid, $name, $search_name, %rid_names, @temps, %headers,
    $ratio_means, $pvalue, $diff_means, $ratio_medians, $diff_medians);

#== get parameters from ARGV:
($input_file, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2, $extract_id, $protocol_id, $is_logged) = @ARGV;
open (TMP, ">./tmp-plots/temp.txt") || die "Can not open tmp file.$!";
print TMP "@ARGV\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== start
print "Starting time = " . TimeStamp() . "\n";

#== get kw_table_id:
$kw_table_id = getKwTableIdByProtocolId($dbh, $protocol_id);
if ($kw_table_id eq "") {
    print "Unknown entity from analysis method.";
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}

#== get the process1_id:
$process1_id = getProcess1ID($dbh, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2);
if ($process1_id ne "") {
    #== check if the data has been inserted before:
    $sql = "select count(*) from sp_comparison where process1_id = $process1_id and protocol_id = $protocol_id and kw_table_id = $kw_table_id";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $count = $sth->fetchrow();
    if ($count > 0) {
        print "There is data for these parameter set in DOME.\n";
        $sth->finish;
        $dbh->disconnect;
        exit 1;
    }
} elsif ($process1_id eq "") {
    $process1_id = createProcess1Id($dbh, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2);
}

$sql = "create TEMP table temp_names (tname varchar(500), \"search_name\" varchar(500)) ON COMMIT DROP;\n";
open (IN, "<$input_file") or die "Can not open the input data file $input_file.\n";
@lines = <IN>;
shift @lines;
foreach $line (@lines) {
    chomp $line;

    @temps = split /\t/, $line;
    $name = $temps[0];
    $search_name = getSearchName($name);
    $sql .= "insert into temp_names (tname, search_name) values (\'$name\', \'$search_name\');\n";
}
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== get record_ids for all names:
if ($kw_table_id == 33) {
    #-- microarray:
    $sql = "select R.reporter_id, CS.probe_name from reporter R, composite_sequence CS, temp_names TN where R.composite_sequence_id = CS.composite_sequence_id and CS.probe_name = TN.tname";
} elsif ($kw_table_id = 11) {
    #-- metabolite
    $sql = "select A.record_id, TN.tname from allnames A, temp_names TN where upper(A.search_name) = upper(TN.search_name)";
}
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($rid, $name) = $sth->fetchrow() ) {
    $rid_names{$name} = $rid;
}
 
#== read data from input file:
open (IN, "<$input_file") or die "Can not open the input data file $input_file.\n";
@lines = <IN>;
close IN;

#== get the first line:
$line = shift @lines;
#print TMP "header = $line";
@temps = split /\t/, $line;
for($i=0; $i<@temps; $i++) {
    if ($temps[$i] =~ /NAME/i) {
        $headers{"NAME"} = $i;
    }
    if ($temps[$i] =~ /RATIO_MEANS/i) {
        $headers{"RATIO_MEANS"} = $i;
    }
    if ($temps[$i] =~ /VALUE/i) {
        $headers{"VALUE"} = $i;
    }
    if ($temps[$i] =~ /DIFF_MEANS/i) {
        $headers{"DIFF_MEANS"} = $i;
    }
    if ($temps[$i] =~ /RATIO_MEDIANS/i) {
        $headers{"RATIO_MEDIANS"} = $i;
    }
    if ($temps[$i] =~ /DIFF_MEDIANS/i) {
        $headers{"DIFF_MEDIANS"} = $i;
    }
}

foreach $line (@lines) {
    chomp $line;

    @temps = split /\t/, $line;
    $rid = $rid_names{$temps[$headers{"NAME"}]};
    $ratio_means = "";
    $pvalue = "";
    $diff_means = "";
    $ratio_medians = "";
    $diff_medians = "";
    if ($rid eq "") {
        print "Name " .$temps[$headers{"NAME"}] . " is not in DOME.\n";
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        print "Ending time: " . TimeStamp() . "<br>";
        exit 1;
    }
    if ($headers{"RATIO_MEANS"} ne "") {
        $ratio_means = $temps[$headers{"RATIO_MEANS"}];
    }
    if ($headers{"VALUE"} ne "") {
        $pvalue =  $temps[$headers{"VALUE"}];
    }
    if ($headers{"DIFF_MEANS"} ne "") {
        $diff_means =  $temps[$headers{"DIFF_MEANS"}];
    }
    if ($headers{"RATIO_MEDIANS"} ne "") {
        $ratio_medians = $temps[$headers{"RATIO_MEDIANS"}];
    }
    if ($headers{"DIFF_MEDIANS"} ne "") {
        $diff_medians =  $temps[$headers{"DIFF_MEDIANS"}];
    }
 
    if ($ratio_means ne "" ||$pvalue ne "" || $diff_means ne "" ||$ratio_medians ne "" || $diff_medians ne "") {
        $sql1 = "insert into sp_comparison (sp_comparison_id, process1_id, protocol_id, kw_table_id, record_id, extract_id";
        $sql2 = " values(nextval(\'sp_comparison_id_seq\'), $process1_id, $protocol_id, $kw_table_id, $rid, $extract_id";
        if ($is_logged) {
            if ($ratio_means ne "") {
                $sql1 .= ", ratio_of_means_l";
                $sql2 .= ", $ratio_means";
            }
            if ($pvalue ne "") {
                $sql1 .= ", p_value_l";
                $sql2 .= ", $pvalue";
            }
            if ($diff_means ne "") {
                $sql1 .= ", difference_of_means_l";
                $sql2 .= ", $diff_means";
            }
            if ($ratio_medians ne "") {
                $sql1 .= ", ratio_of_medians_l";
                $sql2 .= ", $ratio_medians";
            }
            if ($diff_medians ne "") {
                $sql1 .= ", difference_of_medians_l";
                $sql2 .= ", $diff_medians";
            }
        } else {
            if ($ratio_means ne "") {
                $sql1 .= ", ratio_of_means";
                $sql2 .= ", $ratio_means";
            }
            if ($pvalue ne "") {
                $sql1 .= ", p_value";
                $sql2 .= ", $pvalue";
            }
            if ($diff_means ne "") {
                $sql1 .= ", difference_of_means";
                $sql2 .= ", $diff_means";
            }
            if ($ratio_medians ne "") {
                $sql1 .= ", ratio_of_medians";
                $sql2 .= ", $ratio_medians";
            }
            if ($diff_medians ne "") {
                $sql1 .= ", difference_of_medians";
                $sql2 .= ", $diff_medians";
            }
        }
        $sql1 .= ")";
        $sql2 .= ");\n";
    }
    $main_sql .= $sql1 . $sql2;
}
#print TMP $main_sql;
#close TMP;

if ($main_sql ne "") {
    $sth = $dbh->prepare($main_sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    #print "rc = $rc\n";
    if ($rc == 1) {
        print "Data inseertion is SUCCESSFUL<br>";
        $dbh->commit;
        $sth->finish;
        $dbh->disconnect;
        print "Ending time: " . TimeStamp() . "<br>";
        exit 0;
    } else {
        print "Data inseertion has FAILED<br>";
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        print "Ending time: " . TimeStamp() . "<br>";
        exit 1;
     }
}
print "Data inseertion has FAILED<br>";
$dbh->rollback;
$sth->finish;
$dbh->disconnect;
print "Ending time: " . TimeStamp() . "<br>";
exit 1;

###########################################################################
# sub getProcess1ID($dbh, $spid1, $label_id1, $spid2, $label_id2):
#  Get the process1_id from process1.
#
#  parameter in: $dbh -- database handle,
#                $spid1 -- sampling_point_id for numeritor,
#                $label_id1 -- the tissue_type_label_id for numeritor,
#                $spid2 -- sampling_point_id for denominator,
#                $label_id1 -- the tissue_type_label_id for denominator,
#  parameter out:$process1_id -- process1_id for spid1, spid2.
#
# X. Jing Li, 10/19/2005.
###########################################################################
sub getProcess1ID {
    my ($dbh, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2) = @_;
    my ($sql, $sth, $rc, $process1_id);

    $sql = "select process1_id from process1 where sampling_point1_id = $spid1 AND sampling_point2_id = $spid2 AND sp1_tissue_type_label_id = $tissue_type_label_id1 AND sp2_tissue_type_label_id = $tissue_type_label_id2";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $process1_id = $sth->fetchrow();

    $sth->finish;
    return $process1_id;
}

###########################################################################
# subroutine: createProcess1Id($dbh, $spid1, $tissue_type_label_id1, $spid2, 
#         $tissue_type_label_id2):
# This subroutine create a process1_id. 
#
# parameters in: $dbh -- the database handle,
#                $spid1 -- sampling_point_id for the one to be compared;
#                $spid2 -- sampling_point_id for the sample to be compared with,
#                $tissue_type_label_id1 -- the tissue type for spid1,
#                $tissue_type_label_id2 -- the tissue type for spid2,
# parameter out: $process1_id -- the new process1_id;
#
# X. Jing Li, 5/24/2005
###########################################################################
sub createProcess1Id {
    my ($dbh, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2) = @_;
    my ($sql, $sth, $rc, $process1_id);

    $sql = "select nextval('process1_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't preparet: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $process1_id = $sth->fetchrow();

    $sql = "insert into process1 (process1_id, sampling_point1_id, sp1_tissue_type_label_id, sampling_point2_id, sp2_tissue_type_label_id) values ($process1_id, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2)";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't preparet: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";

    $sth->finish;
    return $process1_id;
}
