#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script will get data from user created data file and insert data into
# sp_comparison.
#
# author: X. Jing Li
# date: Oct. 18, 2005
# reversion:
#===============================================================================

my ($input_file, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2,
    $extract_id, $protocol_id, $is_logged, $force_add,
    $database, $username, $password, $bnet_DB, $specieDB, $dbh, $sth, $rc, 
    $sql, $sql1, $sql2, $main_sql, $kw_table_id, $count, @lines, $line, $i,
    $process1_id, $rid, $probe_name, %rid_pnames, @temps, %headers,
    $ratio, $pvalue, $diff);

#== get parameters from ARGV:
($input_file, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2, $extract_id, $protocol_id, $is_logged) = @ARGV;
print "@ARGV\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== start
#print "Starting time = " . TimeStamp() . "\n";

#== get kw_table_id:
$kw_table_id = getKwTableId($dbh, $protocol_id);
if ($kw_table_id eq "") {
    print "Unknown entity from analysis method.";
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}

#== get the process1_id:
$process1_id = getProcess1ID($dbh, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2);
if ($process1_id ne "") {
    #== check if the data has been inserted before:
    $sql = "select count(*) from sp_comparison where process1_id = $process1_id and protocol_id = $protocol_id and kw_table_id = $kw_table_id";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $count = $sth->fetchrow();
    if ($count > 0) {
        print "There is data for these parameter set in DOME.\n";
        $sth->finish;
        $dbh->disconnect;
        exit 1;
    }
} elsif ($process1_id eq "") {
    $process1_id = createProcess1Id($dbh, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2);
}

#== get kw_table_id and reporter_ids for all probe_names:
$sql = "select R.reporter_id, CS.probe_name from reporter R, composite_sequence CS where R.composite_sequence_id = CS.composite_sequence_id";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($rid, $probe_name) = $sth->fetchrow()) {
    $rid_pnames{$probe_name} = $rid;
}

#== read data from input file:
open (IN, "<$input_file") or die "Can not open the input data file $input_file.\n";
@lines = <IN>;
close IN;

#== get the first line:
$line = shift @lines;
@temps = split /\t/, $line;
for($i=0; $i<@temps; $i++) {
    if ($temps[$i] =~ /NAME/i) {
        $headers{"PROBE_NAME"} = $i;
    }
    if ($temps[$i] =~ /RATIO/i) {
        $headers{"RATIO"} = $i;
    }
    if ($temps[$i] =~ /VALUE/i) {
        $headers{"VALUE"} = $i;
    }
    if ($temps[$i] =~ /DIFF/i) {
        $headers{"DIFF"} = $i;
    }
}

foreach $line (@lines) {
    chomp $line;

    @temps = split /\t/, $line;
    $rid = $rid_pnames{$temps[$headers{"PROBE_NAME"}]};
    $ratio = "";
    $pvalue = "";
    $diff = "";
    if ($rid eq "") {
        print "Probe_name " .$temps[$headers{"PROBE_NAME"}] . " is not in DOME.\n";
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        print "Ending time: " . TimeStamp() . "<br>";
        exit 1;
    }
    if ($headers{"RATIO"} ne "") {
        $ratio = $temps[$headers{"RATIO"}];
    }
    if ($headers{"VALUE"} ne "") {
        $pvalue =  $temps[$headers{"VALUE"}];
    }
    if ($headers{"DIFF"} ne "") {
        $diff =  $temps[$headers{"DIFF"}];
    }
    if ($ratio ne "" ||$pvalue ne "" || $diff ne "") {
        $sql1 = "insert into sp_summary (sp_summary_id, process1_id, protocol_id, kw_table_id, record_id, extract_id";
        $sql2 = " values(nextval(\'sp_summary_id_seq\'), $process1_id, $protocol_id, $kw_table_id, $rid, $extract_id";
        if ($is_logged) {
            if ($ratio ne "") {
                $sql1 .= ", ratio_of_means_l";
                $sql2 .= ", $ratio";
            }
            if ($pvalue ne "") {
                $sql1 .= ", pvalue_l";
                $sql2 .= ", $pvalue";
            }
            if ($diff ne "") {
                $sql1 .= ", difference_of_ratio_l";
                $sql2 .= ", $diff";
            }
        } else {
            if ($ratio ne "") {
                $sql1 .= ", ratio_of_means";
                $sql2 .= ", $ratio";
            }
            if ($pvalue ne "") {
                $sql1 .= ", pvalue";
                $sql2 .= ", $pvalue";
            }
            if ($diff ne "") {
                $sql1 .= ", difference_of_ratio";
                $sql2 .= ", $diff";
            }
        }
        $sql1 .= ")";
        $sql2 .= ");\n";
    }
    $main_sql .= $sql1 . $sql2;
}
print $main_sql;

if ($main_sql ne "") {
    $sth = $dbh->prepare($main_sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    #print "rc = $rc\n";
    if ($rc == 1) {
        print "Data inseertion is SUCCESSFUL<br>";
        $dbh->commit;
        $sth->finish;
        $dbh->disconnect;
        print "Ending time: " . TimeStamp() . "<br>";
        exit 0;
    } else {
        print "Data inseertion has FAILED<br>";
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        print "Ending time: " . TimeStamp() . "<br>";
        exit 1;
     }
}
print "Data inseertion has FAILED<br>";
$dbh->rollback;
$sth->finish;
$dbh->disconnect;
print "Ending time: " . TimeStamp() . "<br>";
exit 1;

###########################################################################
# sub getProcess1ID($dbh, $spid1, $label_id1, $spid2, $label_id2):
#  Get the process1_id from process1.
#
#  parameter in: $dbh -- database handle,
#                $spid1 -- sampling_point_id for numeritor,
#                $label_id1 -- the tissue_type_label_id for numeritor,
#                $spid2 -- sampling_point_id for denominator,
#                $label_id1 -- the tissue_type_label_id for denominator,
#  parameter out:$process1_id -- process1_id for spid1, spid2.
#
# X. Jing Li, 10/19/2005.
###########################################################################
sub getProcess1ID {
    my ($dbh, $spid1, $tissue_type_label_id1, $spid2, $tissue_type_label_id2) = @_;
    my ($sql, $sth, $rc, $process1_id);

    $sql = "select process1_id from process1 where sampling_point1_id = $spid1 AND sampling_point2_id = $spid2 AND sp1_tissue_type_label_id = $tissue_type_label_id1 AND sp2_tissue_type_label_id = $tissue_type_label_id2";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $process1_id = $sth->fetchrow();

    $sth->finish;
    return $process1_id;
}
