#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script will get data from user created data file and insert data into 
# sp_summary.
#
# author: X. Jing Li
# date: Oct. 7, 2005
# reversion: 
#===============================================================================

my ($input_file, $spid, $tissue_type_label_id, $extract_id, $protocol_type_id, $is_logged, 
    $database, $username, $password, $bnet_DB, $specieDB, $dbh, $sth, $rc, $sql, $sql1, $sql2, $main_sql, 
    $i, $kw_table_id, $protocol_type_name, $count, $protocol_id, $rid, $name, $search_name, %rid_names, @lines, $line,  @temps, %headers, $mean, $median, $stdev, $n_data); 

#== get parameters from ARGV:
($input_file, $spid, $tissue_type_label_id, $extract_id, $protocol_type_id, $is_logged) = @ARGV;
#print "argv = @ARGV\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== start 
print "Starting time = " . TimeStamp() . "\n";

#== get the protocol_id for the means:
$sql = "select \"protocol_id\" from \"protocol\" where \"protocol_name\" = \'Mean\' and \"protocol_type_id\" = $protocol_type_id";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$protocol_id = $sth->fetchrow();
if ($protocol_id eq "") {
    $sql = "select \"protocol_type_name\" from \"protocol_type\" where \"protocol_type_id\" = $protocol_type_id";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $protocol_type_name = $sth->fetchrow();
    print "There is no protocol_id for Mean of protocol_type \"$protocol_type_name\" in DOME.<br>\n";
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    #print "Ending time: " . TimeStamp() . "<br>";
    exit 1;
}

#== get kw_table_id:
$kw_table_id = getKwTableId($dbh, $protocol_type_id);
if ($kw_table_id eq "") {
    print "Unknown entity from extraction method.";
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}

#== check if the data has been inserted before:
$sql = "select count(*) from \"sp_summary\" where \"sampling_point_id\" = $spid and \"protocol_id\" in (select \"protocol_id\" from \"protocol\" where \"protocol_type_id\" = $protocol_type_id) and \"kw_table_id\" = $kw_table_id";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$count = $sth->fetchrow();
if ($count > 0) {
    print "There is data for these parameter set in DOME.\n";
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}

$sql = "create TEMP table temp_names (tname varchar(500), \"search_name\" varchar(500)) ON COMMIT DROP;\n";
open (IN, "<$input_file") or die "Can not open the input data file $input_file.\n";
@lines = <IN>;
print "there are " . @lines . " lines in input file.\n";

shift @lines;
foreach $line (@lines) {
    chomp $line;

    @temps = split /\t/, $line;
    $name = $temps[0];
    $search_name = getSearchName($name);
    $sql .= "insert into temp_names (tname, search_name) values (\'$name\', \'$search_name\');\n";
}
#print $sql;
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== get record_ids for all names:
if ($kw_table_id == 33) {
    $sql = "select R.\"reporter_id\", CS.\"probe_name\" from \"reporter\" R, \"composite_sequence\" CS, temp_names TN where R.\"composite_sequence_id\" = CS.\"composite_sequence_id\" and CS.probe_name = TN.tname";
} elsif ($kw_table_id = 11) {
    $sql = "select A.record_id, TN.tname from allnames A, temp_names TN where upper(A.search_name) = upper(TN.search_name)";
}
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#print "there are " . $sth->rows() . " rows of records come back\n";
while (($rid, $name) = $sth->fetchrow()) {
    $rid_names{$name} = $rid;
}
print "There are " . (keys %rid_names) . " records in hash rid_names\n";

#== read data from input file:
open (IN, "<$input_file") or die "Can not open the input data file $input_file.\n";
@lines = <IN>;
close IN;

#== get the first line:
$line = shift @lines;
chomp $line;
$line =~ s/\"//g;
@temps = split /\t/, $line;
for($i=0; $i<@temps; $i++) {
    if ($temps[$i] =~ /NAME/i) {
        $headers{"NAME"} = $i;
    } elsif ($temps[$i] =~ /VALUE/i || $temps[$i] =~ /MEAN/i) {
        $headers{"MEAN"} = $i;
    } elsif ($temps[$i] =~ /STDEV|STDDEV/i) {
        $headers{"STDEV"} = $i;
    } elsif ($temps[$i] =~ /NUMBER_OF_DATA/i) {
        $headers{"NDATA"} = $i;
    } elsif ($temps[$i] =~ /MEDIAN/i) {
        $headers{"MEDIAN"} = $i;
    }

}

foreach $line (@lines) {
    chomp $line;
    $line =~ s/\"|\n|\r//g;
    
    @temps = split /\t/, $line;
    $rid = $rid_names{$temps[$headers{"NAME"}]};
    if ($rid eq "") {
        print "In line:\n$line\nName \"" .$temps[$headers{"NAME"}] . "\" is not in DOME.\n";
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        print "Ending time: " . TimeStamp() . "<br>";
        exit 1;
    }
    $mean = $temps[$headers{"MEAN"}];
    if ($headers{"MEAN"} ne "") {
        $mean = $temps[$headers{"MEAN"}];
    }
    if ($headers{"STDEV"} ne "") {
        $stdev = $temps[$headers{"STDEV"}];
    }
    if ($headers{"NDATA"} ne "") {
        $n_data = $temps[$headers{"NDATA"}];
    }
    if ($headers{"MEDIAN"} ne "") {
        $median = $temps[$headers{"MEDIAN"}];
    }
    if ($mean ne "" || $stdev ne "" || $n_data ne "" || $median ne "") {
        $sql1 = "insert into sp_summary (sp_summary_id, sampling_point_id, protocol_id, kw_table_id, record_id, label_id, extract_id";
        $sql2 = " values(nextval(\'sp_summary_id_seq\'), $spid, $protocol_id, $kw_table_id, $rid,  $tissue_type_label_id, $extract_id";
#        if ($spid2 ne "") {
#            $sql1 .= ", sampling_point_id2, label_id2";
#            $sql2 .= ", $spid2, $tissue_type_label_id2";
#        }
        if ($is_logged) {
            if ($mean ne "") {
                $sql1 .= ", mean_l";
                $sql2 .= ", $mean";
            }
            if ($stdev ne "") {
                $sql1 .= ", std_dev_l";
                $sql2 .= ", $stdev";
            }
            if ($median ne "") {
                $sql1 .= ", median_l";
                $sql2 .= ", $median";
            }
        } else {
            if ($mean ne "") {
                $sql1 .= ", mean";
                $sql2 .= ", $mean";
            }
            if ($stdev ne "") {
                $sql1 .= ", std_dev";
                $sql2 .= ", $stdev";
            }
            if ($median ne "") {
                $sql1 .= ", median";
                $sql2 .= ", $median";
            }
        }
        if ($n_data ne "") {
            $sql1 .= ", \"N_data\"";
            $sql2 .= ", $n_data";
        }
        $sql1 .= ")";
        $sql2 .= ");\n";
        $main_sql .= $sql1 . $sql2;
    }
}
#open (TMP, ">./tmp-plots/temp.sql");
#print TMP $main_sql;
#close TMP;

if ($main_sql ne "") {
    $sth = $dbh->prepare($main_sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    #print "rc = $rc\n";
    if ($rc == 1) {
        print "Data insertion is SUCCESSFUL<br>";
        $dbh->commit;
        $sth->finish;
        $dbh->disconnect;
        print "Ending time: " . TimeStamp() . "<br>";
        exit 0;
    } else {
        print "Data inseertion has FAILED<br>";
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        print "Ending time: " . TimeStamp() . "<br>";
        exit 1;
     }
}
print "Data inseertion has FAILED<br>";
$dbh->rollback;
$sth->finish;
$dbh->disconnect;
print "Ending time: " . TimeStamp() . "<br>";
exit 1;

