#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script will get data from user created data file and insert data into 
# table "ma_norm" as user normalized microarray data.
# The data file format as the following:
# probeset_id\tvalue1...\n
# id_1\tv1...\n
# ...
#
# The values could be intensity or log_intensity or log_of total_intensity.
#
# author: X. Jing Li
# date: May 23, 2006
# reversion: 
#===============================================================================
my ($input_file, $srid, $replicate, $tissue_type_label_id, $extract_id, $protocol_type_id,
    $database, $username, $password, $bnet_DB, $specieDB, $dbh, $sth, $rc, $sql, $sql1, $sql2, $mainsql,
    $count, $exp_set_name, $exp_name, $time, $repl, $pt_name, $extract_name, $msg, $msg2, $rid, $pname, %pname_rids, @lines, $line, @headers, $header, $index, %headers, @probe_ids, $probe_id, @values, %probe_intens, %probe_intens_l, @missing_pnames, $seid, $value, $value_l);

#== get parameters from ARGV:
#($input_file, $srid, $replicate, $tissue_type_label_id, $extract_id, $protocol_type_id, $is_logged, $force_add) = @ARGV;
($input_file, $srid, $replicate, $tissue_type_label_id, $extract_id, $protocol_type_id) = @ARGV;
#print "argv = @ARGV\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== start 
#print "Starting time = " . TimeStamp() . "\n";

#== check if the data has been inserted before:
$sql = "select count(*) from \"ma_norm\" where \"sampling_replicate_id\" = $srid and \"protocol_type_id\" = $protocol_type_id";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$count = $sth->fetchrow();
if ($count > 0 ) {
    ($exp_set_name, $exp_name, $time, $repl, $pt_name, $extract_name)
        = getDetail($dbh, $srid, $tissue_type_label_id, $protocol_type_id, $extract_id);
    $msg2 = getRelatedInfoFromSummaryComparison($dbh, $srid, $replicate, $tissue_type_label_id, $protocol_type_id, $extract_id);
 
    $msg = "The sample of extraction = \"$extract_name\" with experiment_name = \"$exp_set_name\", experiment_name = \"$exp_name\", at time point \"$time\", biological replicate = \"$repl\", technical replicate = \"$replicate\" using normalizing protocol \"$pt_name\" has been inserted to DOME before. If you want to replace the data in DOME with this set of data, please delete the previous uploaded data and all its related data first.";
    print $msg2 . $msg;
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}

#== get all reporter_ids for probe_id:
$sql = "select R.reporter_id, CS.probe_name from reporter R, composite_sequence CS where CS.composite_sequence_id = R.composite_sequence_id";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($rid, $pname) = $sth->fetchrow()) {
    $pname_rids{$pname} = $rid;
}

#== read data from input file:
open (IN, "<$input_file") or die "Can not open the input data file $input_file. $!\n";
@lines = <IN>;
close IN;

#== get the headers:
$line = shift @lines;
chomp $line;
$line =~ s/\
|\"//g;
@headers = split /\t/, $line;
$index = 0;
foreach $header (@headers) {
    if (uc ($header) eq "PROBE_SET_NAME") {
        $headers{"PNAME"} = $index;
    } elsif (uc ($header) eq "INTENSITY") {
        $headers{"INTENSITY"} = $index;
    } elsif (uc ($header) eq "LOG_INTENSITY") {
        $headers{"LOG_INTENSITY"} = $index;
    } else {
        $msg = "Undefined column header \"$header\". Please name your columns following the examples in the help page.";
        print $msg;
        $sth->finish;
        $dbh->disconnect;
        exit 1;
    }
    $index++;
}
#foreach $header (keys %headers) {
#    print "header = $header, index = " . $headers{$header} . "\n";
#}

foreach $line (@lines) {
    chomp $line;
    if ($line !~ /\w/) {
        next;
    }

    $line =~ s/\
|\"//g;
    @values = split /\t/, $line;
#    print "probe_id = $probe_id, value = $value\n";
    $probe_id = $values[$headers{"PNAME"}];
    push @probe_ids, $probe_id;
    if (exists$headers{"INTENSITY"}) {
        $probe_intens{$probe_id} = $values[$headers{"INTENSITY"}];
    }
    if (exists$headers{"LOG_INTENSITY"}) {
        $probe_intens_l{$probe_id} = $values[$headers{"LOG_INTENSITY"}];
    }
    if (!exists $pname_rids{$probe_id}) {
        push @missing_pnames, $probe_id;
    }
}
if (@missing_pnames > 0) {
    if (@missing_pnames == 1) {
        $msg = "The following probe_set name \"@missing_pnames\" is not in database DOME. Please add it to DOME first.";
    } else {
        $msg = "The following probe_set names \"" . (join ", ", @missing_pnames) . "\" are not in database DOME. Please add them to DOME first.";
    }
    print $msg;
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}

#== get the sample_extract_id:
$sql = "select sample_extract_id from sample_extract where technical_replicate = $replicate and extract_id = $extract_id and sample_id in (select sample_id from \"sample\" where sampling_replicate_id = $srid)";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$seid = $sth->fetchrow();
if ($seid eq "") {
    ($exp_set_name, $exp_name, $time, $repl, $pt_name, $extract_name)
        = getDetail($dbh, $srid, $tissue_type_label_id, $protocol_type_id, $extract_id);
    $msg = "There is no sample in DOME labeled with extraction = \"$extract_name\" with experiment_set_name = \"$exp_set_name\", experiment_name = \"$exp_name\", at time point = \"$time\", biology replicate = \"$repl\", technical replicate = \"$replicate\". Please check your pameters and try again.";
    print $msg;
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}

$mainsql = "";
foreach $probe_id (@probe_ids) {
    $rid = $pname_rids{$probe_id};
    $value = $probe_intens{$probe_id};
    $value_l = $probe_intens_l{$probe_id};
    if ($value eq "" && $value_l ne "") {
        $value = 2 ** $value_l;
    } elsif ($value_l eq "" && $value > 0) {
        $value_l = log($value) / log(2);
    } elsif ($value_l eq "" && $value eq "") {
        next;
    }
    $sql1 = "insert into \"ma_norm\" (ma_norm_id, sampling_replicate_id, sample_extract_id, reporter_id, protocol_type_id, intensity";
    $sql2 = "values (nextval('ma_norm_id_seq'), $srid, $seid, $rid, $protocol_type_id, $value";
    if($value_l ne "") {
        $sql1 .= ", intensity_l";
        $sql2 .= ", $value_l";
    }
    $sql1 .= ") ";
    $sql2 .= ");\n";
#    $mainsql .= "/* $probe_id, reporter_id = $rid */\n";
    $mainsql .= $sql1 . $sql2;
}

#print $mainsql;

$sth = $dbh->prepare($mainsql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$dbh->commit;
$sth->finish;
$dbh->disconnect;
exit;

###########################################################################
# Subroutine getDetail($dbh, $srid, $tissue_type_label_id, $ptid, $extract_id):
# This subroutine will return the experiment_set_name, experiemnt_name, name for
# time point, biological replicate, protocol_type name and extraction name.
#             
# parameters in: $dbh -- database handle
#                $sampling_replicate_id -- 
#                $tissue_type_label_id --
#                $ptid -- protocol_type_id
#                $extract_id --
#           out: $exp_set_name -- experiment_set name,
#                $exp_name -- experiment rname,
#                $time -- the label name for time point,
#                $bio_repl -- biological replicate number,
#                $pt_name -- the protocol_type name,
#                $extravt_name -- the name for the extract_id
#
# X. Jing Li: 5/24/2006
###########################################################################
sub getDetail {
    my ($dbh, $srid, $tissue_type_label_id, $ptid, $extract_id) = @_;
    my ($sql, $sth, $rc, $exp_set_name, $exp_name, $time, $bio_repl, $pt_name, $extract_name);

    #-- get exp_set_name, exp_name, time, bio-replicate:
    $sql = "select ES.exp_set_name, E.exp_name, L.\"name\", SR.replicate from experiment_set ES, experiment E, sampling_point SP, sampling_replicate SR, label L where SR.sampling_replicate_id = $srid and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($exp_set_name, $exp_name, $time, $bio_repl) = $sth->fetchrow();

    $sql = "select protocol_type_name from protocol_type where protocol_type_id = $ptid";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $pt_name = $sth->fetchrow();

    $sql = "select \"name\" from extract where extract_id = $extract_id";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $extract_name = $sth->fetchrow();

    $sth->finish;
    return $exp_set_name, $exp_name, $time, $bio_repl, $pt_name, $extract_name;
}

###########################################################################
# Subroutine getRelatedInfoFromSummaryComparison
#   ($dbh, $srid, $repl, $tissue_type_label_id, $ptid, $extract_id):
# This subroutine will return the experiment_set_name, experiemnt_name, name for
# time point, biological replicate, protocol_type name and extraction name.
#             
# parameters in: $dbh -- database handle
#                $sampling_replicate_id -- 
#                $repl -- technical replicate number
#                $tissue_type_label_id --
#                $ptid -- protocol_type_id
#                $extract_id --
#           out: $message -- the html table format for information in sp_summary
#                            and sp_comparison
#
# X. Jing Li: 6/1/2006
###########################################################################
sub getRelatedInfoFromSummaryComparison {
    my ($dbh, $srid, $repl, $ttl_id, $ptid, $extract_id) = @_;
    my ($sth, $rc, $sql, $exp_set_name, $exp_name, $lname, $extract_name, $spid, $process1_id, $protocol_id, $html1, $message, $key, %pid_process_ids);

    #== 1. get detail about this array:
    $sql = "select ES.exp_set_name, E.exp_name, L.name, EX.\"name\", SP.sampling_point_id from experiment_set ES, experiment E, label L, extract EX, sampling_point SP, sampling_replicate SR, \"sample\" S, sample_extract SE where EX.extract_id = SE.extract_id and SR.sampling_replicate_id = $srid and S.label_id = $ttl_id and SE.extract_id = $extract_id and SE.technical_replicate = $repl and SE.sample_id = S.sample_id and S.sampling_replicate_id = SR.sampling_replicate_id and SR.sampling_point_id = SP.sampling_point_id and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($exp_set_name, $exp_name, $lname, $extract_name, $spid) = $sth->fetchrow();

#    $sql = "select distinct protocol_id from sp_summary where sampling_point_id = $spid and protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid)";
#    #print $sql."\n";
#    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
#    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    #print "$sql\n$rc\n";
#    ($protocol_id, $spid) = $sth->fetchrow();
    $sql = "select distinct protocol_id, sampling_point_id from sp_summary where protocol_id in (select protocol_id from protocol where protocol_type_id = $ptid) and sampling_point_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) and extract_id = $extract_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#    $count = $sth->fetchrow();
    ($protocol_id, $spid) = $sth->fetchrow();
    if ($protocol_id ne "") {
        $html1 = getSamplingPointIdDetail($dbh, $protocol_id, $spid);
    }

    #== 3. check sp_comparison. All we need is to get all process1_ids from process1:
    $sql = "select distinct process1_id, protocol_id from sp_comparison where process1_id in (select process1_id from process1 where (sampling_point1_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid) or sampling_point2_id = (select sampling_point_id from sampling_replicate where sampling_replicate_id = $srid))) and protocol_id in (select protocol_id from protocol where protocol_type_id =$ptid)";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    while (($process1_id, $protocol_id) = $sth->fetchrow() ) {
        $key = $process1_id . "_" . $protocol_id;
        #print "key = $key\n";
        $pid_process_ids{$key} = getProcess1IdDetail($dbh, $protocol_id, $process1_id);
    }
    
     #== create the final message for all information we have now:
    $message = "Experiment set: $exp_set_name<br>";
    $message .= "Experiment: $exp_name<br>";
    $message .= "Sample point: $lname<br><br>";
    if ($html1 ne "") {
        $message .= $html1;
    }
    if ((keys %pid_process_ids) > 0) {
        $message .= "<CENTER><b>Comparison data</b><br>";
        $message .= "<table border><TR><TH COLSPAN=3>Sample 1</TH><TH COLSPAN=3>Sample 2</TH><TH ROWSPAN=2>Protocol Name</TH><TH ROWSPAN=2>Test</TH></TR>\n";
        $message .="<TR><TH>Experiment Set</TH><TH>Experiment</TH><TH>Sampling Point</TH><TH>Experiment Set</TH><TH>Experiment</TH><TH>Sampling Point</TH></TR>\n";
        foreach $key (keys %pid_process_ids) {
            $message .= $pid_process_ids{$key};
        }
        $message .= "</table><br></CENTER>\n";
    }
    $sth->finish;
    return $message;
}

###########################################################################
# subroutine:getSamplingPointIdDetail($dbh, $protocol_id, $spid)
#
# parameters in: $dbh --- the database handle,
#                $protocol_id --- the protocol id for analysis,
#                $spid -- sampling_point_id;
# parameter out: $msg --- the protocol_name in html table row format;
#
# X. Jing Li, 9/27/2005
###########################################################################
sub getSamplingPointIdDetail {
    my ($dbh, $protocol_id, $spid) = @_;
    my ($sql, $sth, $rc, $pt_name, $prot_name, $msg);

    $msg = "<center><b>Calculated Summary Data</b><br><table border = 1><TR><TH>Protocol_type Name</TH><TH>Protocol Nmae</TH></TR>";
    $sql = "select PT.protocol_type_name, P.protocol_name from protocol_type PT, protocol P where PT.protocol_type_id = P.protocol_type_id and P.protocol_id = $protocol_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    while (($pt_name, $prot_name) = $sth->fetchrow()) {
        $msg .= "<TR><TD>$pt_name</TD><TD>$prot_name</TD</TR>\n";
    }
    $msg .= "</table></center><br>\n";
    $sth->finish;
    return $msg;
}

###########################################################################
# subroutine:getProcess1IdDetail($dbh, $protocol_id, $process1_id)
#
# parameters in: $dbh --- the database handle,
#                $protocol_id --- the protocol id,
#                $process1_id --- the ;
# parameter out: $msg --- the detailed message in html table row format;
#
# X. Jing Li, 9/27/2005
###########################################################################
sub getProcess1IdDetail {
    my ($dbh, $protocol_id, $process1_id) = @_;
    #print "$dbh, $protocol_id, $process1_id\n";
    my ($sql, $sth, $rc, $pt_name, $prot_name, $exp_set_name, $exp_name,
        $l_name, $msg);

    $sql = "select PT.protocol_type_name, P.protocol_name from protocol_type PT, protocol P where PT.protocol_type_id = P.protocol_type_id and P.protocol_id = $protocol_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($pt_name, $prot_name) = $sth->fetchrow();
    
    $sql = "select ES.exp_set_name, E.exp_name, L.name from experiment_set ES, experiment E, label L, sampling_point SP where SP.sampling_point_id in (select sampling_point1_id from process1 where process1_id = $process1_id) and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($exp_set_name, $exp_name, $l_name) = $sth->fetchrow();
    $msg = "<TR><TD>$exp_set_name</TD><TD>$exp_name</TD><TD>$l_name</TD>";
    
    $sql = "select ES.exp_set_name, E.exp_name, L.name from experiment_set ES, experiment E, label L, sampling_point SP where SP.sampling_point_id in (select sampling_point2_id from process1 where process1_id = $process1_id) and SP.label_id = L.label_id and SP.experiment_id = E.experiment_id and E.exp_set_id = ES.exp_set_id";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    ($exp_set_name, $exp_name, $l_name) = $sth->fetchrow();
    $msg .= "<TD>$exp_set_name</TD><TD>$exp_name</TD><TD>$l_name</TD>";
    $msg .= "<TD>$pt_name</TD><TD>$prot_name</TD></TR>\n";
    
    $sth->finish;
    return $msg;
}
