#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script will read data from infile, get information from column 3, 4, 5 10
# and treat the new genes and old genes as the following stratage:
# 1. If the new gene is a TC, get the gene_id for that TC and update all related
#  gene_id for old TC to new gene_id.
# 2. If new gene and a geneBank id, create new gene_id, allnames_id as primary,
#  allnales_id for annotation, evidence_id, external_link_id for this gene and
#  update all related gene_id for old TC to new gene.
# 3. parse the go_accession information. Create go_accession_id if new info is
#  encounterd. update or create evidence to new go_accession_id.
#
# Author: X. Jing Li
# Date: July 12, 2006
# Reversions: 
#============================================================================== 
my ($database, $username, $password, $bnet_DB, $specieDB, $dbh, $sql, $sth, $rc,
    $mainsql, $infile, $kw_table_id, $oldTC_DB_id, $newTC_DB_id, @lines, $line, $index, @terms, $newTC, $oldTC, $annot, $go_info, %oldTC_newTCs, %newTC_oldTCs, %newTC_annots, %newTC_go, $r_tcs, @old_TCs, $rid, $unikey, $is_primary, %tc_rids, %tc_unikeys, %tc_is_primary, $is_diff, $tc, $newTC_rid, $newTC_unikey, $newTC_is_primary, $oldTC_rid, $oldTC_unikey, $oldTC_is_primary, $temp, $unikeyTC, $paramSettings, $specie_name, $specie_id, $reference_id, $taxon_id, $evidence_code_id, $msg, $evidence_id, $external_links_id, $external_db_id, $newTC_aid, $annot_aid, $newTC_evi_id, $TIGR_DB_id, $TIGR_reference_id, $genbank_DB_id, $genbank_ref_id
    );

print "Starting time = " . TimeStamp() . ".\n";
$kw_table_id = 10;
$TIGR_DB_id = 2;
$genbank_DB_id = 3;
open (TEMP, ">./tmp-plots/updateTCs_temp.txt") or die "Can not open temp file.\n";

#== connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die " Can't connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== open the file contains the old and new TCs from TIGR
$infile = "/export/home/xingjing/data/dome/microarray/16k_oligo_TC_mapping.txt";
open (IN, "<$infile") or die "Can not open the input file $infile. $!";
@lines = <IN>;
close IN;
print "There are " . @lines . " lines in input file.\n";
foreach $line (@lines) {
    chomp $line;
    if ($line !~ /\w/) {
        print $line;
        next;
    }

    @terms = split /\t/, $line;
    $oldTC = $terms[2];
    $newTC = $terms[3];
    $annot = $terms[4];
    $annot =~ s/^\"|^\s+|\s+$|\"$//g;
    $go_info = $terms[9];
    $go_info =~ s/^\"|^\s+|\s+$|\"$//g;
#    print "newTC = $newTC, oldTC = $oldTC\n";
    if (!exists $oldTC_newTCs{$oldTC}) {
        $oldTC_newTCs{$oldTC} = $newTC;
        $newTC_annots{$newTC} = $annot;
        $newTC_go{$newTC} = $go_info;
    }
    if (!exists $newTC_oldTCs{$newTC}) {
        $newTC_oldTCs{$newTC} = createValueArray($oldTC);
    } else {
        $newTC_oldTCs{$newTC} = updateValueArray($newTC_oldTCs{$newTC}, $oldTC);
    }
}
print "There are " . (keys %oldTC_newTCs) . " keys in oldTC hash and " . (keys %newTC_oldTCs) . " in newTC hash\n";

foreach $newTC (sort keys %newTC_oldTCs) {
    $r_tcs = $newTC_oldTCs{$newTC};
    @old_TCs = @$r_tcs;
    if (@old_TCs > 1) {
        print TEMP "new_tc = $newTC, old_tcs = (@old_TCs)\n";
    }
}

#== create a temp table for newTC naems and synonyms:
$sql = "create TEMP table temp_names (oldTC varchar(50), newTC varchar(50)) ON COMMIT DROP;\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== insert all pairs into temp table:
$sql = "";
$index = 0;
foreach $oldTC (keys %oldTC_newTCs) {
    $newTC = $oldTC_newTCs{$oldTC};
    $sql .= "insert into temp_names (newTC, oldTC) values (\'$newTC\', \'$oldTC\');\n";
}
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#$sth = $dbh->prepare("select count(*) from temp_names") or die "Can't prepare: $DBI::errstr.";
#$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#print "There are " . $sth->fetchrow() . " records in temp_names\n";

#== get gene_id, unikey and is_primary for all newTC names:
$sql = "select A.record_id, A.unikey, A.is_primary, TN.newTC from allnames A, temp_names TN where A.kw_table_id = $kw_table_id and A.aname = TN.newTC";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "C
an't execute: $DBI::errstr.";
print "There are " . ($sth->rows()) . " gene_id for newTC\n";
while (($rid, $unikey, $is_primary, $newTC) = $sth->fetchrow()) {
    #print TEMP "$newTC, $rid, $unikey, $is_Primary\n";
    if (!exists $tc_rids{$newTC}) {
        $tc_rids{$newTC} = $rid;
        $tc_unikeys{$newTC} = $unikey;
        $tc_is_primary{$newTC} = $is_primary;
    } else {
        if ($rid ne $tc_rids{$newTC}) {
            print "exists\t$newTC\t".$tc_rids{$newTC}."\t$rid\t$unikey\t$is_primary\t".$tc_rids{$newTC}."\t".$tc_unikeys{$newTC}."\t".$tc_is_primary{$newTC}."\n";
        }
    }
}
print "There are " . (keys %tc_rids) . " record_ids for newTCs.\n";

#== print out the new gene with no gene_ids:
#open (NORID, ">./tmp-plots/updateTCs_no_geneid.txt") or die "Can not open no_gene_id file. $!.\n";
#$index = 0;
#foreach $newTC (sort keys %newTC_oldTCs) {
#    if ($tc_rids{$newTC} eq "") {
#        $index++;
#        print NORID "$index\t$newTC\t" . (@$newTC_oldTCs{$newTC}) . "\n";
#    }
#}
#close NORID;

#== get all gene_id for oldTC names:
$sql = "select A.record_id, A.unikey, A.is_primary, TN.oldTC from allnames A, temp_names TN where A.kw_table_id = $kw_table_id and A.aname = TN.oldTC";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
print "There are " . ($sth->rows()) . " rows in join of temp table and allnames for oldTC\n";
while (($rid, $unikey, $is_primary, $oldTC) = $sth->fetchrow()) {
    #print TEMP "$oldTC, $rid, $unikey, $is_primary\n";
    $tc_rids{$oldTC} = $rid;
    $tc_unikeys{$oldTC} = $unikey;
    $tc_is_primary{$oldTC} = $is_primary;
}
print "There are " . (keys %tc_rids) . " record_ids for newTC and oldTCs.\n";

#== print out the new_tc and old_tcs
foreach $newTC (sort keys %newTC_oldTCs) {
    $r_tcs = $newTC_oldTCs{$newTC};
    @old_TCs = @$r_tcs;
    $is_diff = 0;
    $oldTC = $old_TCs[0];
    if ($tc_rids{$oldTC} ne $tc_rids{$newTC}) {
        print TEMP "new\t$newTC\t$tc_rids{$newTC}\t$tc_unikeys{$newTC}\t$tc_is_primary{$newTC}\t";
        print TEMP "old\t$oldTC\t$tc_rids{$oldTC}\t$tc_unikeys{$oldTC}\t$tc_is_primary{$oldTC}\tDIFF";
        $is_diff = 1;
    }
    foreach $tc (@old_TCs) {
        if ($tc ne $oldTC && $tc_rids{$tc} ne $tc_rids{$newTC}) {
            if ($is_diff == 0) {
                print TEMP "new\t$newTC\t$tc_rids{$newTC}\t$tc_unikeys{$newTC}\t$tc_is_primary{$newTC}\told\t$tc\t$tc_rids{$tc}\t$tc_unikeys{$tc}\t$tc_is_primary{$tc}\tDIFF";
                $is_diff = 1;
            } else {
                print TEMP "\told\t$tc\t$tc_rids{$tc}\t$tc_unikeys{$tc}\t$tc_is_primary{$tc}\tDIFF";
                $is_diff = 1;
            }
        }
    }
    if ($is_diff == 1) {
        print TEMP "\n";
    }
}
#== get all newTCs with oldTC as synonym. If an oldTc is a synonym of newTC,
#== delete oldTc info from hashes %tc_rids, %tc_unikeys, %tc_is_primary:
$index = 0;
$sql = "select A.aname, TN.oldTC from allnames A, allnames AA, temp_names TN where A.kw_table_id = $kw_table_id and A.is_primary != 'A' and A.record_id = AA.record_id and AA.kw_table_id = $kw_table_id and AA.aname = TN.oldTC ";
#print "$sql\n";
$sth = $dbh-> prepare($sql) or die "Can't prepare select: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute the select: $DBI::errstr.";
#$rows = $sth->rows();
print "There are " . ($sth->rows()) . " oldTCs in allnames are the synonyms of newTC.\n";
print TEMP "Index\tNEWTCS\tTC\tGene_id\tUnikey\tis_primary\tOLDTCS\tTC\tGene_id\tUnikey\tis_primary\n";
while (($newTC, $oldTC) = $sth->fetchrow()) {
    if (exists $oldTC_newTCs{$oldTC} && exists $newTC_oldTCs{$newTC}) {
        $index++;
        #print TEMP "$index\tNEW\t$newTC\t".$tc_rids{$newTC}."\t".$tc_unikeys{$newTC}."\t".$tc_is_primary{$newTC}."\tOLD\t".$oldTC."\t".$tc_rids{$oldTC}."\t".$tc_unikeys{$oldTC}."\t".$tc_is_primary{$oldTC}."\n";;
        if ($tc_rids{$newTC} ne $tc_rids{$oldTC}) {
            $index++;
            print TEMP "$index\tNEW\t$newTC\t".$tc_rids{$newTC}."\t".$tc_unikeys{$newTC}."\t".$tc_is_primary{$newTC}."\tOLD\t".$oldTC."\t".$tc_rids{$oldTC}."\t".$tc_unikeys{$oldTC}."\t".$tc_is_primary{$oldTC}."\n";
        } else {
            print TEMP "DELETE $index\tNEW\t$newTC\t".$tc_rids{$newTC}."\t".$tc_unikeys{$newTC}."\t".$tc_is_primary{$newTC}."\tOLD\t".$oldTC."\t".$tc_rids{$oldTC}."\t".$tc_unikeys{$oldTC}."\t".$tc_is_primary{$oldTC}."\n";
            delete $oldTC_newTCs{$oldTC};
            delete $tc_rids{$oldTC};
            delete $tc_unikeys{$oldTC};
            delete $tc_is_primary{$oldTC};
        }
    }
}
print "There are " . (keys %oldTC_newTCs) . " items left after delete the existing synonyms.\n";
close TEMP;

my $new_file = "./tmp-plots/updateTCs_new_old.txt";
open (NEW, ">$new_file") or warn "Can not open new file $new_file. $!";
print NEW "newTC\tnewTC Gene_id\tnewTC unikey\tnewTC isPrimary\toldTC\toldTC gene_id\toldTC unikey\toldTC isPrimary\tgo_info\n";
#== list all newTCs and oldTCs and go_info which need updated:
foreach $newTC (sort keys %newTC_oldTCs) {
    $newTC_rid = $tc_rids{$newTC};
    $newTC_unikey = $tc_unikeys{$newTC};
    $newTC_is_primary = $tc_is_primary{$newTC};
    $go_info = $newTC_go{$newTC};
    $r_tcs = $newTC_oldTCs{$newTC};
    @old_TCs = @$r_tcs;
    if (@old_TCs == 1 && exists $oldTC_newTCs{$old_TCs[0]}){
        $oldTC = $old_TCs[0];
        $oldTC_rid = $tc_rids{$oldTC};
        $oldTC_unikey = $tc_unikeys{$oldTC};
        $oldTC_is_primary = $tc_is_primary{$oldTC};

        ($temp, $unikeyTC) = split /\_/, $oldTC_unikey;
        if (exists $newTC_oldTCs{$unikeyTC}) {
            print NEW "old $newTC\t$newTC_rid\t$newTC_unikey\t$newTC_is_primary\t$oldTC\t$oldTC_rid\t$oldTC_unikey\t$oldTC_is_primary\t$unikeyTC\t$go_info\n";
        } else {
            print NEW "$newTC\t$newTC_rid\t$newTC_unikey\t$newTC_is_primary\t$oldTC\t$oldTC_rid\t$oldTC_unikey\t$oldTC_is_primary\t$unikeyTC\t$go_info\n";
        }
    } else {
        foreach $oldTC (@old_TCs) {
            if (exists $oldTC_newTCs{$oldTC}){
                $oldTC_rid = $tc_rids{$oldTC};
                $oldTC_unikey = $tc_unikeys{$oldTC};
                $oldTC_is_primary = $tc_is_primary{$oldTC};
                ($temp, $unikeyTC) = split /\_/, $oldTC_unikey;

                if (exists $newTC_oldTCs{$unikeyTC}) {
                    print NEW "old $newTC\t$newTC_rid\t$newTC_unikey\t$newTC_is_primary\t$oldTC\t$oldTC_rid\t$oldTC_unikey\t$oldTC_is_primary\t$unikeyTC\t$go_info\n";
                } else {
                    print NEW "new $newTC\t$newTC_rid\t$newTC_unikey\t$newTC_is_primary\t$oldTC\t$oldTC_rid\t$oldTC_unikey\t$oldTC_is_primary\t$unikeyTC\t$go_info\n";
                }
            }
        }
    }
}
close NEW;

#== get specie reference_id, specie_id, taxon_id and evidence_code_id:
$paramSettings = "./paramSettings.php";
open (PAR, "<$paramSettings") or die "Can not open paremeter file.";
@lines = <PAR>;
close PAR;
foreach $line (@lines) {
    chomp $line;
    if ($line =~ /\'SPECIES_NAME\'/) {
        ($temp, $specie_name) = split /=/, $line;
        $specie_name =~ s/^\s+|\'+|\;|\s+$//g;
    }
}
#$sql = "select \"value\" from \"default\" where \"name\" = 'GENE_REFERENCE'";
#$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
#$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
#$reference_id = $sth->fetchrow();
#print "reference_id = $reference_id\n";
$TIGR_reference_id = 895;
$genbank_ref_id = 3;

$sql = "select record_id from \"allnames\" where upper(aname) = upper(\'$specie_name\') and kw_table_id = (select kw_table_id from \"ref_kw_table\" where kw_table = \'SPECIES\')";
#print "$sql\n";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$specie_id = $sth->fetchrow();
print "specie_id = $specie_id\n";

$sql = "select taxon_ncbi_accession from \"species\" where specie_id = $specie_id";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
$taxon_id = $sth->fetchrow();
$evidence_code_id = 14;  # IEP for "GENE" table
print "taxon_id = $taxon_id\n";

#== There are three different cases for the newTC and oldTCs:
#== 1. new TC has no gene_id
#== 2. Both newTC and oldTc are primary names and have different gene_ids;
#== 3. oldTC is a synonym and both newTC and oldTC have different gene_ids.
open (TEMP, ">./tmp-plots/updateTcs_new_update_records.log") or die "Can not open log file for create records. $!\n";
open (TMP, ">./tmp-plots/updateTCs_oldTC_N.txt")  or die "Can not open log file for oldTC is not primary. $!\n";
print TMP "oldTC\toldTC_rid\toldTC_is_priamry\toldTC_unikey\tnewTC\tnewTC_rid\tnewTC_is_primary\tneewTC_unikey\n";
$oldTC_DB_id = $TIGR_DB_id;
foreach $oldTC (sort keys %oldTC_newTCs) {
    $newTC = $oldTC_newTCs{$oldTC};
    $annot = $newTC_annots{$newTC};
    $newTC_rid = $tc_rids{$newTC};
    $newTC_unikey = $tc_unikeys{$newTC};
    $newTC_is_primary = $tc_is_primary{$newTC};
    if ($newTC =~ /^TC/){
        $newTC_DB_id = $TIGR_DB_id;
        $reference_id = $TIGR_reference_id;
    } else {
        $newTC_DB_id = $genbank_DB_id;
        $reference_id = $genbank_ref_id;
    }
    $go_info = $newTC_go{$newTC};
    $oldTC_rid = $tc_rids{$oldTC};
    $oldTC_unikey = $tc_unikeys{$oldTC};
    $oldTC_is_primary = $tc_is_primary{$oldTC};
    
    if ($newTC_rid eq "") {
        #-- create new ids for the newTC:
        $newTC_rid = createGeneId($dbh, $specie_id);
        $evidence_id = createEvidenceId($dbh, $evidence_code_id, $reference_id, $kw_table_id, $newTC_rid);
        $external_links_id = createExternalLinksId($dbh, $newTC_DB_id, $kw_table_id, $newTC_rid, "gene_id", $newTC);
        $newTC_unikey = $taxon_id . "_" . $newTC;
        $newTC_aid = createAllnameId($dbh, $newTC, $kw_table_id, $newTC_rid, "Y", $newTC_unikey);
        if ($annot ne "-") {
            $annot_aid = createAllnameId($dbh, $annot, $kw_table_id, $newTC_rid, "A", $newTC_unikey);
        }
        if ($go_info ne "[]") {
            $msg = updateGoAccId($dbh, $evidence_id, $go_info);
            print TEMP "$oldTC\t$oldTC_rid\t$oldTC_is_primary\t$newTC\t$newTC_rid\t$evidence_id\t$msg\n";
        }
        #-- update the oldTC to newTC
        $msg = updateSynonymInAllnames($dbh, $kw_table_id, $oldTC_rid, "N", $newTC_rid, "N", $newTC_unikey);
        print TEMP "$msg\n";
        $msg = deleteFromAllnames($dbh, $kw_table_id, $oldTC_rid, "A");
        print TEMP "$msg\n";
        $msg = updateEvidence($dbh, $kw_table_id, $oldTC_rid, $newTC_rid, $evidence_code_id, $reference_id);
        print TEMP "$msg\n";
        $msg = updateExternalLinks($dbh, $kw_table_id, $oldTC_rid, $oldTC_DB_id, $newTC_rid, $newTC_DB_id);
        print TEMP "$msg\n";
        $msg = updateMapComponent($dbh, $kw_table_id, $oldTC_rid, $newTC_rid, $newTC, $specie_id);
        print TEMP "$msg\n";
        $msg = updateMRna($dbh, $oldTC_rid, $newTC_rid);
        print TEMP "$msg\n";
        $msg = updateCompositeSequence($dbh, $oldTC_rid, $newTC_rid);
        print TEMP "$msg\n";
        $msg = deleteFromGene($dbh, $oldTC_rid, $specie_id);
        print TEMP "$msg\n";
        $msg = updateSynonymInAllnames($dbh, $kw_table_id, $oldTC_rid, "Y", $newTC_rid, "N", $newTC_unikey);
        print TEMP "$msg\n";
    } elsif ($newTC_rid ne $oldTC_rid) {
        if ($oldTC_is_primary eq "Y") {
            $newTC_evi_id = getEvidenceId($dbh, $kw_table_id, $newTC_rid, $evidence_code_id, $reference_id); 
            print "from getEvidenceId: newTC = $newTC, newTC_rid = $newTC_rid, newTC_evi_id = $newTC_evi_id\n";
            $msg = updateSynonymInAllnames($dbh, $kw_table_id, $oldTC_rid, "N", $newTC_rid, "N", $newTC_unikey);
            print TEMP "$msg\n";
            $msg = deleteFromAllnames($dbh, $kw_table_id, $oldTC_rid, "A");
            print TEMP "$msg\n";
            if ($go_info ne "[]") {
                $msg = updateGoAccId($dbh, $newTC_evi_id, $go_info);
                print TEMP "$oldTC\t$oldTC_rid\t$oldTC_is_primary\t$newTC\t$newTC_rid\t$evidence_id\t$msg\n";
            } 
            $msg = updateEvidence($dbh, $kw_table_id, $oldTC_rid, $newTC_rid, $evidence_code_id, $reference_id); 
            print TEMP "$msg\n";
            $msg = updateExternalLinks($dbh, $kw_table_id, $oldTC_rid, $oldTC_DB_id, $newTC_rid, $newTC_DB_id);
            print TEMP "$msg\n";
            $msg = updateMapComponent($dbh, $kw_table_id, $oldTC_rid, $newTC_rid, $newTC, $specie_id);
            print TEMP "$msg\n";
            $msg = updateMRna($dbh, $oldTC_rid, $newTC_rid);
            print TEMP "$msg\n";
            $msg = updateCompositeSequence($dbh, $oldTC_rid, $newTC_rid);
            print TEMP "$msg\n";
            $msg = deleteFromGene($dbh, $oldTC_rid, $specie_id);
            print TEMP "$msg\n";
            $msg = updateSynonymInAllnames($dbh, $kw_table_id, $oldTC_rid, "Y", $newTC_rid, "N", $newTC_unikey);
            print TEMP "$msg\n";
        } elsif ($oldTC_is_primary eq "N") {
            #-- newTC and oldTC have different gene_ids and oldTC is not primary name:
            print TMP "$oldTC\t$oldTC_rid\t$oldTC_is_primary\t$oldTC_unikey\t$newTC\t$newTC_rid\t$newTC_is_primary\t$newTC_unikey\n";
        }
    }
}
#close TEMP;
close TMP;
print "Ending time = " . TimeStamp() . "\n";
$sth->finish;
$dbh->rollback;
$dbh->disconnect;
exit;

###########################################################################
# subroutine: createValueArray($v):
# This subroutine create a anynomouse array. 
#
# parameters in: $v --- the first value for the anynomous array
# parameter out: $ref_array --- the reference of the anynomous array
#
###########################################################################
sub createValueArray {
    my ($v) = @_;
    my ($ref_array, @array);

    $ref_array = [];
    @array = @$ref_array;
    push @array, $v;
    return \@array;
}

###########################################################################
# subroutine: updateValueArray($rv1, $v2)
#
# parameters in: $rv1 --- the original arry reference
#                $v2 --- the new array item
# parameter out: $ref_array --- reference of the new array 
#
###########################################################################
sub updateValueArray {
    my ($rv1, $v2) = @_;
    my (@value1, $ref_array, @array);

    $ref_array = [];
    @array = @$ref_array;
    @value1 = @$rv1;
    push @array, @value1;
    push @array, $v2;
    return \@array;
}

###########################################################################
# subroutine: updateSymonymInAllnames
# ($dbh, $kwid, $old_rid, $old_type, $new_rid, $new_type, $new_unikey)
# This subroutine updates allnames table and make one record to a synonym of
# another record.
#
# parameters in: $dbh -- database handle
#                $kwid -- kw_table_id
#                $old_rid -- old record_id
#                $old_type -- old record is_primary type
#                $new_rid -- the new record_id 
#                $new_type -- the new type
#                $new_unikey -- the new unikey
# parameter out: $sql -- last action for database
#                $rc -- database execution result
#
# X. Jing Li, 9/7/2006
###########################################################################
sub updateSynonymInAllnames {
    my ($dbh, $kwid, $old_rid, $old_type, $new_rid, $new_type, $new_unikey) = @_;
    my ($sql, $sth, $rc, $message);

    $sql = "update allnames set record_id = $new_rid, unikey = \'$new_unikey\', is_primary = \'$new_type\' where kw_table_id = $kwid and record_id = $old_rid and is_primary = \'$old_type\'";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;
    $message = "$rc\t$sql";
    return $message;
}

###########################################################################
# subroutine: updateSymonymInAllnames
# ($dbh, $kwid, $old_rid, $old_type, $new_rid, $new_type, $new_unikey)
# This subroutine updates allnames table and make one record to a synonym of
# another record.
#
# parameters in: $dbh -- database handle
#                $kwid -- kw_table_id
#                $rid -- record_id
#                $name_type -- is_primary type
# parameter out: $message -- the return message
#
# X. Jing Li, 9/7/2006
###########################################################################
sub deleteFromAllnames {
    my ($dbh, $kwid, $rid, $name_type) = @_;
    my ($sql, $sth, $rc, $message);

    $sql = "delete from allnames where kw_table_id = $kwid and record_id = $rid and is_primary = \'$name_type\'";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;
    $message = "$rc\t$sql";
    return $message;
}

###########################################################################
# subroutine: updateEvidence
# ($dbh, $kwid, $rid, $evidence_code_id, $reference_id)
# This subroutine get the evidence_id from evidence table.
#
# parameters in: $dbh -- database handle
#                $kwid -- kw_table_id
#                $rid -- record_id
#                $evidence_code_id --
#                $reference_id
# parameter out: $evidence_id
#
# X. Jing Li, 9/14/2006
###########################################################################
sub getEvidenceId {
    my ($dbh, $kwid, $rid, $evidence_code_id, $reference_id) = @_;
    my ($sql, $sth, $rc, $evi_id);

    $sql = "select evidence_id from evidence where evidence_code_id = $evidence_code_id and reference_id = $reference_id and kw_table_id = $kwid and record_id = $rid";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $evi_id = $sth->fetchrow();

    $sth->finish;
    return $evi_id;
}


###########################################################################
# subroutine: updateEvidence
# ($dbh, $kwid, $old_rid, $new_rid, $evidence_code_id, $reference_id)
# This subroutine updates evidence table.
#
# parameters in: $dbh -- database handle
#                $kwid -- kw_table_id
#                $old_rid -- old record_id
#                $new_rid -- the new record_id 
#                $evidence_code_id --
#                $reference_id
# parameter out: $message -- the return message
#
# X. Jing Li, 9/8/2006
###########################################################################
sub updateEvidence {
    my ($dbh, $kwid, $old_rid, $new_rid, $evidence_code_id, $reference_id) = @_;
    my ($sql, $sth, $rc, $message);

    #== if there are any records in evidence_go_accession for the old rid,
    #== delete them first. 
    $sql = "delete from evidence_go_accession where evidence_id in (select evidence_id from evidence wherekw_table_id = $kwid and record_id = $old_rid)";
    $sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $message = "$rc\t$sql\n";

    #== if there is no evidence_id for the new record_id for the same 
    #== evidence_code_id and reference_id, update the evidence table directly
    #== for the record_id. If there is already have a record_id for the new rid
    #== with the same evidence_code_id and reference_id, delete the record for
    #== the old rid with evidence_code_id and reference_id, then update the
    #== other record(s) for the old rid to new rid.
    $sql = "select evidence_id from evidence where evidence_code_id = $evidence_code_id and reference_id = $reference_id and kw_table_id = $kwid and record_id = $new_rid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() == 0) {
        $sql = "update evidence set record_id = $new_rid where kw_table_id = $kwid and record_id = $old_rid and reference_id = $reference_id and evidence_code_id = $evidence_code_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    } else {
        $sql = "delete from evidence where evidence_code_id = $evidence_code_id and reference_id = $reference_id and kw_table_id = $kwid and record_id = $old_rid;\n";
        $sql .= "update evidence set record_id = $new_rid where kw_table_id = $kwid and record_id = $old_rid";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    }
    $message .= "$rc\t$sql\n";
    $sth->finish;
    return $message;
}

###########################################################################
# subroutine: updateExternalLinks
#           ($dbh, $kwid, $old_rid, $old_db_id, $new_rid, $new_db_id)
# This subroutine updates External_links table.
#
# parameters in: $dbh -- database handle
#                $kwid -- kw_table_id
#                $old_rid -- old record_id
#                $old_db_id -- old database id
#                $new_rid -- the new record_id 
#                $new_db_id -- the new database id
# parameter out: $message
#
# X. Jing Li, 9/8/2006
###########################################################################
sub updateExternalLinks {
    my ($dbh, $kwid, $old_rid, $old_db_id, $new_rid, $new_db_id) = @_;
    my ($sql, $sth, $rc, $message);

    $sql = "select external_links_id from external_links where kw_table_id = $kwid and record_id = $new_rid and external_database_id = $new_db_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    if ($sth->rows() > 0) {
        #-- there is a external_link for the new_id. Delete the old link
        $sql = "delete from external_links where kw_table_id = $kwid and record_id = $old_rid and external_database_id = $old_db_id";
    } elsif ($sth->rows() == 0 ){
        #-- here is no record is the same as the new_id, update the record for
        #-- old_id to new_id:
        $sql = "update external_links set record_id = $new_rid, external_database_id = $new_db_id where kw_table_id = $kwid and record_id = $old_rid and external_database_id = $old_db_id";
    }
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;
    $message = "$rc\t$sql\n";
    return $message;
}

###########################################################################
# subroutine: updateMapComponent($dbh, $kwid, $old_rid, $new_rid, $specie_id)
# This subroutine updates allnames table and make one record to a synonym of
# another record.
#
# parameters in: $dbh -- database handle
#                $kwid -- kw_table_id
#                $old_rid -- old record_id
#                $new_rid -- the new record_id 
#                $specie_id
# parameter out: $message
#
# X. Jing Li, 9/8/2006
###########################################################################
sub updateMapComponent {
    my ($dbh, $kwid, $old_rid, $new_rid, $new_name, $specie_id) = @_;
    my ($sql, $sth, $rc, $message);

    $new_name =~ s/\'/\'\'/g;
    $sql = "update map_component set record_id = $new_rid, component_name = \'$new_name\' where kw_table_id = $kwid and record_id = $old_rid and specie_id = $specie_id";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;
    $message = "$rc\t$sql\n";
    return $message;
}

###########################################################################
# subroutine: updateMRna($dbh, $old_rid, $new_rid)
# This subroutine updates allnames table and make one record to a synonym of
# another record.
#
# parameters in: $dbh -- database handle
#                $old_rid -- old record_id
#                $new_rid -- the new record_id 
# parameter out: $message
#
# X. Jing Li, 9/8/2006
###########################################################################
sub updateMRna {
    my ($dbh, $old_rid, $new_rid) = @_;
    my ($sql, $sth, $rc, $message);

    $sql = "update mrna set gene_id = $new_rid where gene_id = $old_rid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;
    $message = "$rc\t$sql\n";
    return $message;
}

###########################################################################
# subroutine: updateCompositeSequence($dbh, $old_rid, $new_rid)
# This subroutine updates allnames table and make one record to a synonym of
# another record.
#
# parameters in: $dbh -- database handle
#                $old_rid -- old record_id
#                $new_rid -- the new record_id 
# parameter out: $message -- the return message
#
# X. Jing Li, 9/8/2006
###########################################################################
sub updateCompositeSequence {
    my ($dbh, $old_rid, $new_rid)= @_;
    my ($sql, $sth, $rc, $message);

    $sql = "update composite_sequence set gene_id = $new_rid where gene_id = $old_rid";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;
    $message = "$rc\t$sql\n";
    return $message;
}

###########################################################################
# subroutine: deleteFromGene($dbh, $gene_id, $specie_id)
# This subroutine delete record from the gene table
#
# parameters in: $dbh -- database handle
#                $gene_id -- the record needs to be deleted
#                $specie_id -- the specie_id 
# parameter out: $message -- the return message 
#
# X. Jing Li, 9/14/2006
###########################################################################
sub deleteFromGene {
    my ($dbh, $gene_id, $specieid) = @_;
    my ($sql, $sth, $rc, $message);

    $sql = "delete from gene where specie_id = $specie_id and gene_id = $gene_id";
    $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
    $sth->finish;
    $message = "$rc\t$sql\n";
    return $message;
}

###########################################################################
# subroutine: updateGoAccId($dbh, $evidence_id, $go_info)
# This subroutine updates go_accession table if the go_id is new. It also create
# new records in evience_go_accession table to link the evidence_id and 
# go_accession_id.
#
# parameters in: $dbh -- database handle
#                $evidence_id -- 
#                $go_info -- GO names, ids and types
# parameter out: $message -- the return message
#
# X. Jing Li, 9/13/2006
###########################################################################
sub updateGoAccId {
    my ($dbh, $evidence_id, $go_info) = @_;
    my ($sth, $rc, $sql, @items, $item, $go_name, $go_id, $go_type, $go_acc_id, $evi_goAcc_id, @evi_goAcc_ids, $message);

    #== parse the go_info:
    $go_info =~ s/^\[|\]$//g;
    @items = split /\]\[/, $go_info;
    foreach $item (@items) {
        if ($item !~ /\|GO\:/) {
            next;
        }
        
        ($go_name, $go_id, $go_type) = split /\|/, $item;
        if ($go_type eq "func") {
            $go_type = "function";
        } elsif ($go_type eq "comp") {
            $go_type = "component";
        } elsif ($go_type eq "proc") {
            $go_type = "process";
        } else {
            print "Unknown go_type \"$go_type\" in $go_info.\n";
        }
        $sql = "select go_accession_id from go_accession where go_id = \'$go_id\' and go_type = \'$go_type\'";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        $go_acc_id = $sth->fetchrow();
        if ($go_acc_id eq "") {
            $go_acc_id = createGoAccessionId($dbh, $go_name, $go_id, $go_type);
            $message .= "go_name = $go_name, go_id = $go_id, go_tyep = $go_type, go_acc_id = $go_acc_id\n";
        }
        $evi_goAcc_id = createEvidenceGoAccId($dbh, $evidence_id, $go_acc_id);
        push @evi_goAcc_ids, $evi_goAcc_id;
    }
    $sth->finish;
    $message = join ", ", @evi_goAcc_ids;
    return $message;
}

###########################################################################
# subroutine: createGoAccssionId($dbh, $go_name, $go_id, $go_type)
# This subroutine create new record in go_accession table.
#
# parameters in: $dbh -- database handle
#                $go_name -- 
#                $go_id --
#                $go_type -- GO types. function, component or process
# parameter out: $go_acc_id -- go_accession_id
#
# X. Jing Li, 9/13/2006
###########################################################################
sub createGoAccessionId {
    my ($dbh, $go_name, $go_id, $go_type) = @_;
    my ($sql, $sth, $rc, $go_acc_id);

    $go_name =~ s/\'/\'\'/g;
    $sth = $dbh->prepare("select nextval('go_accession_id_seq')") or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $go_acc_id = $sth->fetchrow;

    $sql = "insert into go_accession (go_accession_id, go_name, go_id, go_type) values (\'$go_acc_id\', \'$go_name\', \'$go_id\', \'$go_type\')";
#    print "createGoAccessionId: $sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $sth->finish;
    return $go_acc_id;
}

###########################################################################
# subroutine: createEvidenceGoAccId($dbh, $evidence_id, $go_accession_id)
# This subroutine create links between evidence and go_accession tables.
#
# parameters in: $dbh -- database handle
#                $evidence_id -- 
#                $go_accession_id --
# parameter out: $evi_go_acc_id -- new evidence_go_acc_id
#
# X. Jing Li, 9/13/2006
###########################################################################
sub createEvidenceGoAccId {
    my ($dbh, $evidence_id, $go_accession_id) = @_;
    my ($sql, $sth, $rc, $evi_go_acc_id);

    $sth = $dbh->prepare("select nextval('evidence_go_acc_id_seq')") or die "Can not prepare select nextval from dual: $DBI::errstr";
    $rc = $sth->execute or die "Can not execute select: $DBI::errstr.";
    $evi_go_acc_id = $sth->fetchrow;

    $sql = "insert into evidence_go_accession (evidence_go_acc_id, evidence_id, go_accession_id) values (\'$evi_go_acc_id\', \'$evidence_id\', \'$go_accession_id\')";
#    print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $evi_go_acc_id;
}
