#!/usr/local/bin/perl

use DBI;
use DOMELIB;
use strict "vars";

#===============================================================================
# This script let user uoload map from frontend and insert map components into 
# bnet. The user will supply image_file, coordinates of the image components and
# the map name, map_type. User will also supply the elements names corresponding
# to the names in hte map incase the names for the same map component are diff.
# i.e. map component named '1.2.3.4' are actually 'TC111111', 'TC222222', ...
#
# The map_type can only for the following map_types:
# "Metabolite neighborhood", "Signaling pathways", "Gene network", 
# "Metabolic Pathways", "Generic"
#
# The file format is as following:
# Identifier\tshape\tcoordinates\n
# The identifier is for the map_component name which may not be the name in the
# data file. For the shape we can only take the following shapes with parameters
# defined as:
# circle -- including ovals.
#   shape - circ
#   Params - centercoordinate x_y_width_height
# rectangle -- incluing squares.
#   shape - rect
#   Params -- upper left cornor coordinate x_y_width_height
# polygon -- number of vertices as the number of parameters. coordinate of each 
#   vertice is one parameter and (x, y) denoted as x_y.
#   shape - poly
#   params - x1,y1_x2,y2_..._xN,yN
# random -- maybe later.
#
# The names file contains three columns:
# Type\Identifer\tID\n
# Type is for the component type in the data. It must match the types in Bnet.
# Identifier is for the names in map_info file. ID is the names for experiment
# elements.
#
# Author: X. Jing Li
# Date: Jan. 15, 2007
# Reversions: 
#===============================================================================

my ($image, $image_info, $map_name, $map_type_id, $map_comp_names, $outputfile, $file_home, $person_id, $is_kegg, $force_add,
    $database, $username, $password, $bnet_DB, $specieDB, $dbh, $sth, $rc, $sql,
    $kw_name, $kwid, %kw_id_names, $temp, $file_name, $map_id, $map_msg, @lines, $line, %comp_names, $comp_name, $comp_name_kw, $i, $name, $type, $shape, $coord, %coords, $comp_search_name, %unique_comp_names, $ambig_names, %correspond_names, %names, %name_types, $search_name, %search_names, %types, $name_kw, %unique_types, %shapes, $r_shapes, %name_rids, $rid, $no_cpd_id_names,  $missing_names, $missing_types, $msg, $comp_name, $coord, $map_comp_id, %map_comp_ids, %map_comp_id_names, %map_comp_id_shapes, %map_comp_id_coords, $r_comp_name, @comp_names, );

($image, $image_info, $map_name, $map_type_id, $map_comp_names, $outputfile, $file_home, $person_id, $is_kegg, $force_add) = @ARGV;
open (TMP, ">./tmp-plots/uploadMapToBnet.log");
print TMP "image = $image, image_info = $image_info, map_name = $map_name, map_type_id = $map_type_id, map_comp_names = $map_comp_names, outputfile = $outputfile, file_home = $file_home, person_id = $person_id, is_kegg = $is_kegg, force_add = $force_add\n";
#close TMP;

#== 1. connect to the database:
($database, $username, $password, $bnet_DB, $specieDB) = database_login2();
$dbh = DBI->connect($database,$username,$password) or die "Can\'t connect to $database:$DBI::errstr.";
$dbh->begin_work;
$sql = "set search_path to \"$bnet_DB\", \"$specieDB\"";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";

#== get all kw_table_ids:
$sql = "select kw_table, kw_table_id from ref_kw_table where in_bnet = 'TRUE'";
$sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($kw_name, $kwid) = $sth->fetchrow()) {
    $kw_id_names{$kwid} = $kw_name;
}

#== check the map table to see if this map has been uploaded before:
($temp, $file_name) = split /\_NEWMAP\_/, $image;
($map_id, $map_msg) = getMapId($dbh, $file_name, $map_name, $map_type_id);
#print "map_id = $map_id, msg = $map_msg\n";
if ($is_kegg eq "N") {
    if ($map_id ne "" && ($map_msg eq "EXIST/SAME_FILE" || $map_msg eq "EXIST/DIFF_FILE") && $force_add == 0) {
        $sql = "select MC.component_name, MC.shape, MC.coordinates, PI.kw_table_id, PI.name from map_component MC, pathway_info PI where MC.map_id = $map_id AND MC.map_component_id = PI.map_component_id order by MC.component_name";
        #print "$sql\n";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        $msg = "<CENTER><table border=1><TR><TH>Type</TH><TH>Identifier</TH><TH>Name</TH><TH>Shape</TH><TH>Coordinates</TH></TR>";
        while (($comp_name, $shape, $coord, $kwid, $name) = $sth->fetchrow()) {
            $type = $kw_id_names{$kwid};
            $msg .= "<TR><TD>$type</TD><TD>$comp_name</TD><TD>$name</TD><TD>$shape</TD><TD>$coord</TD></TR>";
        }
        $msg .= "</TABLE></CENTER>";
    
        print "The map with name \"$map_name\" has been uploaded before. It contains the following components information saved in the DOME and BNet. Please check them with the map information you want to upload now. If they are the same then press \"Cancel\" to go back the previours page. Otherwise the map data in the databases will be deleted and the new data will be uploaded. Press \"Continue\" to replace the old data with these new data. Note the order of the information may not have the same order as it been uploaded. <br \> $msg <br \>";
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 99;
    } else {
        #== map_id == "" or map_msg == "NEWMAP"
        #== open the image_info file and check the names. If two names only differ by case, space or comma, hyphen, we will let users to know that we think them are the same for the first time. When force_add == 1 then we do not show it to users.
        #print "image_info = $image_info\n";
        open (IN, "<$image_info") or die "Can not open the image info file $image_info. $!";
        @lines = <IN>;
        close IN;
        foreach $line (@lines) {
            #print $line;
            if ($line !~ /circ|rect|poly/i || $line !~ /\w/) {
                next;
            }
            chomp $line;
            $line =~ s/\
|\n|\r//g;
            
            ($comp_name, $shape, $coord) = split /\t/, $line;
            $comp_search_name = uc getSearchName($comp_name);
            if (!exists $unique_comp_names{$comp_search_name}) {
                $unique_comp_names{$comp_search_name} = $comp_name;
            } else {
                if ($unique_comp_names{$comp_search_name} ne $comp_name) {
                    $ambig_names .= "\"$comp_name\" and \"" . $unique_comp_names{$comp_search_name} . "\" \n";
                }
            }
            $comp_names{$comp_name} = $comp_name;
            $shapes{$comp_name}{$coord} = $shape;
        }
        if ($ambig_names ne "" && $force_add == 0) {
            print "The following names are very similar and BNet considers them as the same. If they are not the same please rename them so they are distinct.\n$ambig_names";
            $dbh->rollback;
            $sth->finish;
            $dbh->disconnect;
            exit 99;
        }
    }
} elsif ($map_id ne "" && $is_kegg eq "Y") {
    if ($map_msg eq "NEWMAP") {
        $sql = "select distinct map_component_id, component_name, shape, coordinates from map_component where map_id = $map_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        while (($map_comp_id, $comp_name, $shape, $coord) = $sth->fetchrow()) {
            $map_comp_id_names{$map_comp_id} = $comp_name;
            $map_comp_id_shapes{$map_comp_id} = $shape;
            $map_comp_id_coords{$map_comp_id} = $$coord;
        }
    } else {
        $sql = "SELECT DISTINCT MC.component_name, MC.shape, MC.coordinates, PI.kw_table_id, PI.name FROM map_component MC, pathway_info PI WHERE MC.map_component_id = PI.map_component_id AND MC.map_id = $map_id ORDER by MC.compomemt_name";
        $sth = $dbh-> prepare($sql) or die "Can't prepare: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute: $DBI::errstr.";
        $msg = "<CENTER><table border=1><TR><TH>TYPE</TH><TH>Identifier</TH><TH>Name</TH><TH>Shape</TH><TH>Coordinates</TH></TR>";
        while (($comp_name, $shape, $coord, $kwid, $name) = $sth->fetchrow()) {
            $type = $kw_id_names{$kwid};
            $msg .= "<TR><TD>$type</TD><TD>$comp_name</TD><TD>$name</TD><TD>$shape</TD><TD>$coord</TD></TR>";
        }
        $msg .= "</TABLE></CENTER>";
        print "The map with name \"$map_name\" has been uploaded before. It contains the following components information saved in the BNET. Please check them with the map information you want to upload now. If they are the same then press \"Cancel\" to go back the previours page. Otherwise the map data in the database will be deleted and the new data will be uploaded into BNET. Press \"Continue\" to replace the old data with these new data. Note the order of the information may not have the same order as it been uploaded. <br \> $msg <br \>";
        $sth->finish;
        $dbh->rollback;
        $dbh->disconnect;
        exit 99;
    }
}

foreach $map_comp_id (keys %map_comp_id_names) {
    $comp_name = $map_comp_id_names{$map_comp_id};
    $shape = $map_comp_id_shapes{$map_comp_id};
    $coord = $map_comp_id_coords{$map_comp_id};
    print "map_comp_id = $map_comp_id, comp_name = $comp_name, shape = $shape, coord = $coord\n";
}
    
#foreach $comp_name (keys %shapes) {
#    $shape = $shapes{$comp_name};
#    $coord = $coords{$comp_name};
#    print "comp_name = $comp_name, shape = $shape, coord = $coord\n";
#}

#== get all component_names and corresponding map_comp_names:
open (IN, "<$map_comp_names") || die "Can not open map component name file $map_comp_names. $!";
@lines = <IN>;
close IN;
$line = shift @lines;

$i = 1;
foreach $line (@lines) {
    if ($line =~ /^type/i || $line !~ /\w/) {
        $i++;
        next;
    }
    chomp $line;
    $line =~ s/\
|\n|\r//g;
    ($type, $comp_name, $name) = split /\t/, $line;
    $names{$i} = $name;
    $types{$name} = $type;
    if (exists $correspond_names{$name}) {
        $correspond_names{$name} = updateArrayRef($correspond_names{$name}, $comp_name);
    } else {
        $correspond_names{$name} = createArrayRef($comp_name);
    }
    $search_name = uc getSearchName($name);
    $search_name =~ s/\'/\'\'/g;    
    $search_names{$name} = $search_name;
    $i++;
}

#== get all names with the record_id by quary allnames: If a comp_name is not in
#== BNet we will just skip it. The component will not be painted ever
$sql = "create TEMP table temp_names (name varchar(500), search_name varchar(500)) ON COMMIT DROP;\n";
foreach $i (sort {$a <=> $b} keys %names) {
    $name = $names{$i};
    $search_name = $search_names{$name};
    $name =~ s/\'/\'\'/;
    $search_name =~ s/\'/\'\'/;
    $sql .= "insert into \"temp_names\" values (\'$name\', \'$search_name\');\n";
}

#== select record_id for compound:
$sql .= "select TN.name, TN.search_name, A.kw_table_id, A.record_id from temp_names TN, allnames A where TN.search_name = upper(A.search_name) and A.kw_table_id = 11";
#print "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($name, $search_name, $kwid, $rid) = $sth->fetchrow()) {
    if ($kw_id_names{$kwid} eq $types{$name}) {
        $name_rids{$name} = $rid;
        $name_types{$name} = 11;
    }
}
#== select record_id from reporter:
$sql = "select TN.name, TN.search_name, R.reporter_id from temp_names TN, allnames A, composite_sequence CS, reporter R where TN.search_name = upper(A.search_name) and A.kw_table_id = 10 AND A.record_id = CS.gene_id and CS.composite_sequence_id = R.composite_sequence_id";
print TMP "$sql\n";
$sth = $dbh->prepare($sql) or die "Can't prepare: $DBI::errstr.";
$rc = $sth->execute or die "Can't execute: $DBI::errstr.";
while (($name, $search_name, $rid) = $sth->fetchrow()) {
    $name_rids{$name} = $rid;
    $name_types{$name} = 33;
}
#== select record_id for protein:
$sql = "select TN.name, TN.search_name, PS.protein_id from temp_names TN, allnames A, ref_spot_protein_state RSPS, protein_state PS where TN.search_name = upper(A.search_name) and A.kw_table_id = 13 AND A.record_id = PS.protein_id AND PS.protein_state_id = RSPS.protein_state_id AND RSPS.reference_spot_id = RS.reference_spot_id";
while (($name, $search_name, $rid) = $sth->fetchrow()) {
    $name_rids{$name} = $rid;
    $name_types{$name} = 13;
}
#foreach $name (sort keys %name_rids) {
#    $rid = $name_rids{$name};
#    $type = $name_types{$name};
#    print "name = $name, rid = $rid, type = $type\n";
#}

#== check if any name are not in bnet;
$no_cpd_id_names = "";
$missing_names = "";
foreach $i (sort {$a <=> $b} keys %names) {
    $name = $names{$i};
    if ($name eq "") {
        next;
    }
    $type = $types{$name};
    print TMP "name = $name, type = $type\n";
    if ($type eq "COMPOUND" && !exists $name_rids{$name}) {
        $no_cpd_id_names .= "NOTFOUND\t$name\n";
    } elsif (!exists $name_rids{$name}) {
        $missing_names .= "Line $i: name \"$name\" is unknown.\n";
    }
}
#print TMP $no_cpd_id_names, $missing_names;

if ($force_add == 0 && $no_cpd_id_names ne "") {
    open (OUT, ">$outputfile") or warn "Can not open output data file $outputfile\n";
    print OUT $no_cpd_id_names;
    print $no_cpd_id_names;
    close OUT;
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    exit 99;
}
if ($force_add == 0 && $missing_names ne "") {
    if($missing_names ne "") {
        print "The following names are unknown by Bnet:\n$missing_names\n";
    }
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    exit 99;
}

#== If get here then all names are legal. If there is no map_id, create one.
#== Otherwise, delete the all data in map_component for the map_id and reinsert
#== new data:
if ($map_id ne "" && $is_kegg eq "N" && $force_add == 1) {
    #$sql = "delete from map_component where map_id = $map_id and specie_id = $specie_id";
    $sql = "DELETE FROM \"pathway_info\" WHERE \"map_component_id\" IN (SELECT \"map_component_id\" FROM \"map_component\" WHERE \"map_id\" = $map_id)";
    $sth = $dbh->prepare($sql) or die "Can't prepare. $!";
    $rc = $sth->execute or die "Can't execute \"$sql\". $!";
    if ($map_msg =~ /EXIST/) {
        #-- update the image file in table "map":
        $sql = "DELETE FROM \"map_component\" WHERE \"map_id\" = $map_id;\n";
        
        $sql .= "update map set map_number = \'$file_name\' where map_id = $map_id";
        $sth = $dbh->prepare($sql) or die "Can't prepare. $!";
        $rc = $sth->execute or die "Can't execute \"$sql\". $!";
        foreach $comp_name (sort keys %comp_names) {
            $r_shapes = $shapes{$comp_name};
            foreach $coord (sort keys %$r_shapes){
                $shape = $r_shapes->{$coord};
                $map_comp_id = createMapComponentId($dbh, $map_id, $comp_name, $shape, $coord);
                $map_comp_ids{$coord} = $map_comp_id;
            }
        }
    }
} elsif ($map_id eq "") {
    $map_id = createMapId($dbh, $image, $map_name, $map_type_id);
    foreach $comp_name (sort keys %comp_names) {
        $r_shapes = $shapes{$comp_name};
        foreach $coord (sort keys %$r_shapes){
            $shape = $r_shapes->{$coord};
            $map_comp_id = createMapComponentId($dbh, $map_id, $comp_name, $shape, $coord);
            $map_comp_ids{$coord} = $map_comp_id;
        }
    }
}

#== insert map_info into map_component and map_:
$sql = "";
foreach $i (sort {$a <=> $b} keys %names) {
    $name = $names{$i};
    $r_comp_name = $correspond_names{$name};
    @comp_names = @$r_comp_name;
    $kwid = $name_types{$name};
    $rid = $name_rids{$name};
    foreach $comp_name (@comp_names) {
        $r_shapes = $shapes{$comp_name};
        foreach $coord (keys %$r_shapes) {
            $shape = $r_shapes->{$coord};
            $map_comp_id = $map_comp_ids{$coord};
            print TMP "i = $i, name = $name, comp_name = $comp_name, map_comp_id = $map_comp_id, kwid = $kwid, rid = $rid, shape = $shape, coord = $coord\n";
            if ($map_comp_id eq "" || $kwid eq "" || $rid eq "" || $comp_name eq "") {
                next;
            }
            $sql .= "insert into pathway_info (pathway_info_id, map_component_id, kw_table_id, record_id, name) values (nextval('pathway_info_id_seq'), $map_comp_id, $kwid, $rid, \'$name\');\n";
        }
    }
}

my $temp_file = "./tmp-plots/uploadMapComp_temp.sql";
print TMP $sql;
close TMP;
 
if ($sql ne "") {
    $sth = $dbh->prepare($sql) or die "Can not prepare: $DBI::errstr.";
    $rc = $sth->execute or die "Can not execute: $DBI::errstr.";
    if ($rc != 1) {
        print "Failed to upload Pathway/Image information into DOME. \n";
        $dbh->rollback;
        $sth->finish;
        $dbh->disconnect;
        exit 1;
    } else{
        if ($is_kegg eq "N"){
            $file_name = $file_home . $file_name;
            my $cmd = "cp $image $file_name";
            system $cmd;
            unlink $image or warn "can not delete $image. $!";;
            unlink $image_info || warn "can not delete $image_info. $!";
            unlink $map_comp_names || warn "can not delete $map_comp_names. $!";
            print "Uploading image and its information have been successful.";
        } else {
            print "Uploading pathway \"$map_name\" is successful.";
        }
        $dbh->commit;
        $sth->finish;
        $dbh->disconnect;
        exit 0;
    }
} else {
    $dbh->rollback;
    $sth->finish;
    $dbh->disconnect;
    exit 1;
}

###########################################################################
# subroutine: getMapId($dbh, $map_name, $map_type_id);
#  This subroutine create a record in table map for a new image map.
# 
# parameters in: $dbh -- database handle
#                $map_name -- unique name for the image map
#                $map_type_id -- image catogory id
# parameter out: $str --- the str after padding with 0s in front. 
# X. Jing Li
# 11/07/2006
###########################################################################
sub getMapId {
    my ($dbh, $image_file, $map_name, $map_type_id) = @_;
    my ($sth, $rc, $sql, @terms, $file_name, $map_id, $map_number, $count, $msg, $map_type);
    
    #== check map table for uniqueness:
    if ($image_file eq "") {
        $sql = "select map_id from map where upper(map_name) = upper(\'$map_name\') and map_type_id = $map_type_id";
        $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
        $map_id = $sth->fetchrow(); 
        if ($map_id ne "") {
            $sql = "select count(*) from pathway_info where map_component_id in (select distinct map_component_id from map_component where map_id = $map_id)";
            $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
            $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
            $count = $sth->fetchrow();
            if ($count > 0) {
                $msg = "EXIST/SAME_FILE";
            } else {
                $msg = "NEWMAP";
            }
        } else {
            $msg = "NEWMAP";
        }
    } else {
        @terms = split /\//, $image_file;
        $file_name = pop @terms;
        $sql = "select map_id from map where map_number = \'$file_name\' and upper(map_name) = upper(\'$map_name\') and map_type_id = $map_type_id";
        #print "$sql\n";
        $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
        $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
        $map_id = $sth->fetchrow();
        if ($map_id ne "") {
            $sql = "select count(*) from pathway_info where map_component_id in (select distinct map_component_id from map_component where map_id = $map_id)";
            $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
            $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
            $count = $sth->fetchrow();
            if ($count > 0) {
                $msg = "EXIST/SAME_FILE";
            } else {
                $msg = "NEWMAP";
            }
        } else {
            $sql = "select map_id, map_type_id from map where upper(map_name) = upper(\'$map_name\')";
            $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
            $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
            ($map_id, $map_type) = $sth->fetchrow();
            if ($map_id ne "" && $map_type ne $map_type_id) {
                $msg = "EXIST/DIFF_TYPE";
            } else {
                $msg = "NEWMAP";
            }
        }
    }
    $sth->finish;
    return $map_id, $msg;
}

###########################################################################
# subroutine: createMapId($dbh, $map_name, $map_type_id);
#  This subroutine create a record in table map for a new image map.
# 
# parameters in: $dbh -- database handle
#                $map_name -- unique name for the image map
#                $map_type_id -- image catogory id
# parameter out: $str --- the str after padding with 0s in front. 
# X. Jing Li
# 2/13/2007
###########################################################################
sub createMapId {
    my ($dbh, $image_file, $map_name, $map_type_id) = @_;
    my ($sth, $rc, $sql, @terms, $file_name, $map_id, $map_number);

    #-- get file name:
    ($temp, $file_name) = split /\_NEWMAP\_/, $image_file;

    $sql = "select nextval('map_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $map_id = $sth->fetchrow();
    
    $sql = "insert into map (map_id, map_name, map_type_id, map_number) values ($map_id, \'$map_name\', $map_type_id, \'$file_name\')";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $map_id;
}

###########################################################################
# subroutine: prepad($str, $digit):
#  this subroutine padding 0s in front of the string
# 
# parameters in: $str --- the str need padding;
#                $digit --- the length of finished str;
# parameter out: $str --- the str after padding with 0s in front. 
#
###########################################################################
sub prepad {
    my ($str, $digit) = @_;

    while (length $str < $digit) {
        $str = "0" . $str;
    }
    return $str;
}

###########################################################################
# subroutine: createMapComponentId($dbh, $map_id, $comp_name, $shape, $coord);
#  This subroutine create a record in table map_component for a new image map.
# 
# parameters in: $dbh -- database handle
#                $map_id -- map_id for the image map
#                $comp_name -- the map_component name
#                $shape -- the shape of the component. It can be circ, rect or
#                          poly.
#                $coord -- the coordinates of the component.
# parameter out: $map_comp_id -- the map_component_id. 
# X. Jing Li
# 3/22/2007
###########################################################################
sub createMapComponentId {
    my ($dbh, $map_id, $comp_name, $shape, $coord) = @_;
    my ($sth, $rc, $sql, $map_comp_id);

    $sql = "select nextval('map_component_id_seq')";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";
    $map_comp_id = $sth->fetchrow();
    
    $sql = "insert into map_component (map_component_id, map_id, component_name, shape, coordinates) values ($map_comp_id, $map_id, \'$comp_name\', \'$shape\', \'$coord\')";
    #print "$sql\n";
    $sth = $dbh-> prepare($sql) or die "Can't prepare insert: $DBI::errstr.";
    $rc = $sth->execute or die "Can't execute the insert: $DBI::errstr.";

    $sth->finish;
    return $map_comp_id;
}

sub createArrayRef {
    my ($item) = @_;
    my @array = ($item);
    return \@array;
}

sub updateArrayRef {
    my ($r_array, $item) = @_;
    my @array = @$r_array;
    push @array, $item;
    $r_array = \@array;
    return $r_array;
}
